Package: CodelistGenerator
Title: Identify Relevant Clinical Codes and Evaluate Their Use
Version: 4.1.0
Authors@R: c(
    person("Edward", "Burn", email = "edward.burn@ndorms.ox.ac.uk", 
    role = c("aut", "cre"),
    comment = c(ORCID = "0000-0002-9286-1128")),
    person( "Marta", "Alcalde-Herraiz", email = "marta.alcaldeherraiz@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0009-0002-4405-1814")),
    person("Martí", "Català", email = "marti.catalasabate@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0003-3308-9905")),
    person("Xihang", "Chen", email = "xihang.chen@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0009-0001-8112-8959")),
    person("Nuria", "Mercade-Besora", email = "nuria.mercadebesora@ndorms.ox.ac.uk", 
    role = c("aut"), comment = c(ORCID = "0009-0006-7948-3747")),
    person("Mike", "Du", email = "mike.du@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0002-9517-8834")),
    person("Danielle", "Newby", email = "danielle.newby@ndorms.ox.ac.uk", 
    role = c("aut"), 
    comment = c(ORCID = "0000-0002-3001-1478"))
    )
Description: Generate a candidate code list for the Observational Medical Outcomes Partnership (OMOP) common data model based on string matching. For a given search strategy, a candidate code list will be returned.
License: Apache License (>= 2)
Encoding: UTF-8
Depends: R (>= 4.1)
Imports: cli (>= 3.1.0), clock, DBI (>= 1.1.0), dplyr (>= 1.1.0), glue
        (>= 1.5.0), lifecycle, omopgenerics (>= 1.3.4), PatientProfiles
        (>= 1.4.4), purrr, rlang (>= 1.0.0), stringi (>= 1.8.1),
        stringr (>= 1.6.0), tidyr (>= 1.2.0), vctrs
Suggests: omock, covr, duckdb, CDMConnector (>= 2.1.0), visOmopResults
        (>= 1.4.0), CohortConstructor (>= 0.5.0), jsonlite, knitr,
        rmarkdown, testthat (>= 3.0.0), RPostgres, odbc, openxlsx,
        spelling, tibble, gt, flextable, tictoc
Config/testthat/edition: 3
Config/testthat/parallel: true
VignetteBuilder: knitr
URL: https://darwin-eu.github.io/CodelistGenerator/
BugReports: https://github.com/darwin-eu/CodelistGenerator/issues
Language: en-US
LazyData: true
Config/roxygen2/version: 8.0.0
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-08-01 14:22:10 UTC; orms0426
Author: Edward Burn [aut, cre] (ORCID: <https://orcid.org/0000-0002-9286-1128>),
  Marta Alcalde-Herraiz [aut] (ORCID:
    <https://orcid.org/0009-0002-4405-1814>),
  Martí Català [aut] (ORCID: <https://orcid.org/0000-0003-3308-9905>),
  Xihang Chen [aut] (ORCID: <https://orcid.org/0009-0001-8112-8959>),
  Nuria Mercade-Besora [aut] (ORCID:
    <https://orcid.org/0009-0006-7948-3747>),
  Mike Du [aut] (ORCID: <https://orcid.org/0000-0002-9517-8834>),
  Danielle Newby [aut] (ORCID: <https://orcid.org/0000-0002-3001-1478>)
Maintainer: Edward Burn <edward.burn@ndorms.ox.ac.uk>
Repository: CRAN
Date/Publication: 2026-08-01 15:10:02 UTC

# Generated by roxygen2: do not edit by hand

S3method(asCodelist,candidate_codes)
S3method(asCodelist,code_search)
S3method(asCodelist,codelist)
S3method(asCodelist,codelist_with_details)
S3method(asCodelist,concept_set_expression)
S3method(asCodelistWithDetails,candidate_codes)
S3method(asCodelistWithDetails,code_search)
S3method(asCodelistWithDetails,codelist)
S3method(asCodelistWithDetails,codelist_with_details)
S3method(asConceptSetExpression,code_search)
S3method(asConceptSetExpression,codelist)
S3method(asConceptSetExpression,codelist_with_details)
S3method(print,code_search)
export(addConcepts)
export(asCodelist)
export(asCodelistWithDetails)
export(asConceptSetExpression)
export(associatedConceptClassIds)
export(associatedDomains)
export(associatedDoseForms)
export(associatedDoseUnits)
export(associatedDrugIngredients)
export(associatedRelationshipIds)
export(associatedRouteCategories)
export(associatedVocabularies)
export(availableATC)
export(availableConceptClassIds)
export(availableDomains)
export(availableDoseForms)
export(availableDoseUnits)
export(availableDrugIngredients)
export(availableRelationshipIds)
export(availableRouteCategories)
export(availableVocabularies)
export(benchmarkCodelistGenerator)
export(codesFromCohort)
export(codesFromConceptSet)
export(cohortCodelist)
export(compareCodelists)
export(emptyCodeSearch)
export(excludeConcepts)
export(exportCodeSearch)
export(exportCodelist)
export(exportConceptSetExpression)
export(getATCCodes)
export(getCandidateCodes)
export(getDescendants)
export(getDrugIngredientCodes)
export(getMappings)
export(importCodeSearch)
export(importCodelist)
export(importConceptSetExpression)
export(intersectCodelists)
export(mockVocabRef)
export(newCodelist)
export(newCodelistWithDetails)
export(newConceptSetExpression)
export(searchStrategy)
export(stratifyByBrand)
export(stratifyByConcept)
export(stratifyByDomain)
export(stratifyByDoseForm)
export(stratifyByDoseUnit)
export(stratifyByRouteCategory)
export(stratifyByVocabulary)
export(subsetOnDomain)
export(subsetOnDoseForm)
export(subsetOnDoseUnit)
export(subsetOnIngredientRange)
export(subsetOnRouteCategory)
export(subsetOnVocabulary)
export(subsetToCodesInUse)
export(summariseAchillesCodeUse)
export(summariseCodeUse)
export(summariseCohortCodeUse)
export(summariseOrphanCodes)
export(tableAchillesCodeUse)
export(tableCodeUse)
export(tableCohortCodeUse)
export(tableOrphanCodes)
export(unionCodelists)
export(vocabularyVersion)
importFrom(lifecycle,deprecated)
importFrom(omopgenerics,cohortCodelist)
importFrom(omopgenerics,exportCodelist)
importFrom(omopgenerics,exportConceptSetExpression)
importFrom(omopgenerics,importCodelist)
importFrom(omopgenerics,importConceptSetExpression)
importFrom(omopgenerics,newCodelist)
importFrom(omopgenerics,newCodelistWithDetails)
importFrom(omopgenerics,newConceptSetExpression)
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,.env)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# CodelistGenerator <img src="man/figures/logo.png" align="right" height="200"/>

<!-- badges: start -->

[![CRAN
status](https://www.r-pkg.org/badges/version/CodelistGenerator)](https://CRAN.R-project.org/package=CodelistGenerator)
[![codecov.io](https://codecov.io/github/darwin-eu/CodelistGenerator/coverage.svg?branch=main)](https://app.codecov.io/github/darwin-eu/CodelistGenerator?branch=main)
[![R-CMD-check](https://github.com/darwin-eu/CodelistGenerator/workflows/R-CMD-check/badge.svg)](https://github.com/darwin-eu/CodelistGenerator/actions)
[![Lifecycle:stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
<!-- badges: end -->

## Installation

You can install CodelistGenerator from CRAN

``` r
install.packages("CodelistGenerator")
```

Or you can also install the development version of CodelistGenerator

``` r
install.packages("remotes")
remotes::install_github("darwin-eu/CodelistGenerator")
```

## Example usage

``` r
library(dplyr)
library(CDMConnector)
library(CodelistGenerator)
```

For this example we’ll use the Eunomia dataset (which only contains a
subset of the OMOP CDM vocabularies)

``` r
requireEunomia()
db <- DBI::dbConnect(duckdb::duckdb(), dbdir = eunomiaDir())
cdm <- cdmFromCon(db, 
                  cdmSchema = "main", 
                  writeSchema = "main", 
                  writePrefix = "cg_")
```

## Exploring the OMOP CDM Vocabulary tables

OMOP CDM vocabularies are frequently updated, and we can identify the
version of the vocabulary of our Eunomia data

``` r
vocabularyVersion(cdm = cdm)
#> [1] "v5.0 18-JAN-19"
```

## Vocabulary based codelists using CodelistGenerator

CodelistGenerator provides functions to extract code lists based on
vocabulary hierarchies. One example is \`getDrugIngredientCodes, which
we can use, for example, to get the concept IDs used to represent
aspirin and diclofenac.

``` r
ing <- getDrugIngredientCodes(cdm = cdm, 
                       name = c("aspirin", "diclofenac"),
                       nameStyle = "{concept_name}")
ing
#> 
#> - aspirin (2 codes)
#> - diclofenac (1 codes)
ing$aspirin
#> [1]  1112807 19059056
ing$diclofenac
#> [1] 1124300
```

## Systematic search using CodelistGenerator

CodelistGenerator can also support systematic searches of the vocabulary
tables to support codelist development. A little like the process for a
systematic review, the idea is that for a specified search strategy,
CodelistGenerator will identify a set of concepts that may be relevant,
with these then being screened to remove any irrelevant codes by
clinical experts.

We can do a simple search for asthma

``` r
asthma_codes1 <- getCandidateCodes(
  cdm = cdm,
  keywords = "asthma",
  domains = "Condition"
) 
asthma_codes1 |> 
  glimpse()
#> Rows: 2
#> Columns: 6
#> $ concept_id       <int> 317009, 4051466
#> $ found_from       <chr> "From initial search", "From initial search"
#> $ concept_name     <chr> "Asthma", "Childhood asthma"
#> $ domain_id        <chr> "Condition", "Condition"
#> $ vocabulary_id    <chr> "SNOMED", "SNOMED"
#> $ standard_concept <chr> "S", "S"
```

But perhaps we want to exclude certain concepts as part of the search
strategy, in this case we can add these like so

``` r
asthma_codes2 <- getCandidateCodes(
  cdm = cdm,
  keywords = "asthma",
  exclude = "childhood",
  domains = "Condition"
) 
asthma_codes2 |> 
  glimpse()
#> Rows: 1
#> Columns: 6
#> $ concept_id       <int> 317009
#> $ found_from       <chr> "From initial search"
#> $ concept_name     <chr> "Asthma"
#> $ domain_id        <chr> "Condition"
#> $ vocabulary_id    <chr> "SNOMED"
#> $ standard_concept <chr> "S"
```

## Summarising code use

As well as functions for finding codes, we also have functions to
summarise their use. Here for

``` r
library(flextable)
asthma_code_use <- summariseCodeUse(list("asthma" = asthma_codes1$concept_id) |> 
                                           newCodelist(),
  cdm = cdm
)
tableCodeUse(asthma_code_use, type = "flextable", style = "darwin")
```

<img src="man/figures/README-unnamed-chunk-8-1.png" width="100%" />

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
library(DBI)
library(dplyr)
library(CodelistGenerator)
library(CDMConnector)

## ----eval=FALSE---------------------------------------------------------------
#  # example with postgres database connection details
#  db <- DBI::dbConnect(RPostgres::Postgres(),
#    dbname = Sys.getenv("server"),
#    port = Sys.getenv("port"),
#    host = Sys.getenv("host"),
#    user = Sys.getenv("user"),
#    password = Sys.getenv("password")
#  )
#  
#  # create cdm reference
#  cdm <- CDMConnector::cdm_from_con(
#    con = db,
#    cdm_schema = Sys.getenv("vocabulary_schema")
#  )

## ----eval=FALSE---------------------------------------------------------------
#  getVocabVersion(cdm = cdm)

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
vocabVersion <- load(system.file("introVocab.RData", 
                 package = "CodelistGenerator"))
vocabVersion

## ----eval=FALSE---------------------------------------------------------------
#  codesFromDescendants <- tbl(
#    db,
#    sql(paste0(
#      "SELECT * FROM ",
#      vocabularyDatabaseSchema,
#      ".concept_ancestor"
#    ))
#  ) %>%
#    filter(ancestor_concept_id == "4182210") %>%
#    select("descendant_concept_id") %>%
#    rename("concept_id" = "descendant_concept_id") %>%
#    left_join(tbl(db, sql(paste0(
#      "SELECT * FROM ",
#      vocabularyDatabaseSchema,
#      ".concept"
#    )))) %>%
#    select(
#      "concept_id", "concept_name",
#      "domain_id", "vocabulary_id"
#    ) %>%
#    collect()

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
codesFromDescendants <- readRDS(system.file("introData01.RData", 
                                            package = "CodelistGenerator"))

## ----message=FALSE, warning=FALSE---------------------------------------------
codesFromDescendants %>% 
  glimpse()

## ----eval=FALSE---------------------------------------------------------------
#  dementiaCodes1 <- getCandidateCodes(
#    cdm = cdm,
#    keywords = "dementia",
#    domains = "Condition",
#    includeDescendants = TRUE
#  )

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
dementiaCodes1 <- readRDS(system.file("introData02.RData", 
                                            package = "CodelistGenerator"))

## ----message=FALSE, warning=FALSE---------------------------------------------
dementiaCodes1%>% 
  glimpse()

## ----eval=FALSE---------------------------------------------------------------
#  codeComparison <- compareCodelists(
#    codesFromDescendants,
#    dementiaCodes1
#  )

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
codeComparison <- readRDS(system.file("introData03.RData", 
                                            package = "CodelistGenerator"))

## ----message=FALSE, warning=FALSE---------------------------------------------
codeComparison %>%
  group_by(codelist) %>%
  tally()

## ----message=FALSE, warning=FALSE---------------------------------------------
codeComparison %>%
  filter(codelist == "Only codelist 2") %>% 
  glimpse()

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
icdMappings <- readRDS(system.file("introData04.RData", 
                                            package = "CodelistGenerator"))

## ----eval=FALSE---------------------------------------------------------------
#  icdMappings <- getMappings(
#    cdm = cdm,
#    candidateCodelist = dementiaCodes1,
#    nonStandardVocabularies = "ICD10CM"
#  )

## ----message=FALSE, warning=FALSE---------------------------------------------
icdMappings %>% 
  glimpse()

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
readMappings <- readRDS(system.file("introData05.RData", 
                                            package = "CodelistGenerator"))

## ----eval=FALSE---------------------------------------------------------------
#  readMappings <- getMappings(
#    cdm = cdm,
#    candidateCodelist = dementiaCodes1,
#    nonStandardVocabularies = "Read"
#  )

## ----message=FALSE, warning=FALSE---------------------------------------------
readMappings %>% 
  glimpse()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
library(DBI)
library(dplyr)
library(CodelistGenerator)

## ----eval=FALSE---------------------------------------------------------------
#  # postgres database connection details
#  serverDbi <- Sys.getenv("server")
#  user <- Sys.getenv("user")
#  password <- Sys.getenv("password")
#  port <- Sys.getenv("port")
#  host <- Sys.getenv("host")
#  
#  db <- DBI::dbConnect(RPostgres::Postgres(),
#    dbname = serverDbi,
#    port = port,
#    host = host,
#    user = user,
#    password = password
#  )
#  
#  # name of vocabulary schema
#  vocabularyDatabaseSchema <- "vocabulary"

## ----eval=FALSE---------------------------------------------------------------
#  library(dplyr)
#  library(CodelistGenerator)

## ----eval=FALSE---------------------------------------------------------------
#  acetaminophen1 <- getCandidateCodes(
#    cdm = cdm,
#    keywords = "acetaminophen",
#    domains = "drug",
#    standardConcept = "standard",
#    includeDescendants = TRUE
#  )

## ----message=FALSE, warning=FALSE, echo=FALSE---------------------------------
acetaminophen1 <- readRDS(system.file("medData01.RData", 
                                            package = "CodelistGenerator"))

## ----message=FALSE, warning=FALSE---------------------------------------------
acetaminophen1 %>% dplyr::glimpse()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
library(DBI)
library(dplyr)
library(CodelistGenerator)
library(CDMConnector)

## ----eval=FALSE---------------------------------------------------------------
#  library(DBI)
#  library(RPostgres)

## ----eval=FALSE---------------------------------------------------------------
#  # postgres database connection details
#  serverDbi <- Sys.getenv("server")
#  user <- Sys.getenv("user")
#  password <- Sys.getenv("password")
#  port <- Sys.getenv("port")
#  host <- Sys.getenv("host")
#  
#  db <- dbConnect(RPostgres::Postgres(),
#    dbname = serverDbi,
#    port = port,
#    host = host,
#    user = user,
#    password = password
#  )
#  
#  # name of vocabulary schema
#  vocabularyDatabaseSchema <- "vocabulary"
#  
#  # create cdm reference
#  cdm <- CDMConnector::cdm_from_con(
#    con = db,
#    cdm_schema = vocabularyDatabaseSchema
#  )

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
oaCodes1 <- readRDS(system.file("optionsData01.RData", 
                                            package = "CodelistGenerator"))

## ----eval=FALSE---------------------------------------------------------------
#  oaCodes1 <- getCandidateCodes(
#    cdm = cdm,
#    keywords = "osteoarthritis",
#    domains = "Condition",
#    searchInSynonyms = FALSE,
#    searchNonStandard = FALSE,
#    exclude = c(
#      "post-infection",
#      "post-traumatic"
#    ),
#    includeDescendants = FALSE,
#    includeAncestor = FALSE
#  )

## ----message=FALSE, warning=FALSE---------------------------------------------
oaCodes1 %>% 
  glimpse()

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
oaCodes2 <- readRDS(system.file("optionsData02.RData", 
                                            package = "CodelistGenerator"))

## ----eval=FALSE---------------------------------------------------------------
#  oaCodes2 <- getCandidateCodes(
#    cdm = cdm,
#    keywords = "osteoarthritis",
#    domains = "Condition",
#    searchInSynonyms = FALSE,
#    searchNonStandard = FALSE,
#    exclude = c(
#      "post-infection",
#      "post-traumatic"
#    ),
#    includeDescendants = TRUE,
#    includeAncestor = FALSE
#  )

## ----message=FALSE, warning=FALSE---------------------------------------------
newCodes1To2 <- compareCodelists(oaCodes1, oaCodes2) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To2 %>% 
  glimpse()

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
oaCodes3 <- readRDS(system.file("optionsData03.RData", 
                                            package = "CodelistGenerator"))

## ----eval=FALSE---------------------------------------------------------------
#  oaCodes3 <- getCandidateCodes(
#    cdm = cdm,
#    keywords = "osteoarthritis",
#    domains = c("Condition", "Observation"),
#    searchInSynonyms = FALSE,
#    searchNonStandard = FALSE,
#    exclude = c(
#      "post-infection",
#      "post-traumatic"
#    ),
#    includeDescendants = FALSE,
#    includeAncestor = FALSE
#  )

## ----message=FALSE, warning=FALSE---------------------------------------------
newCodes1To3 <- compareCodelists(oaCodes1, oaCodes3) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To3 %>% 
  glimpse()

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
oaCodes4 <- readRDS(system.file("optionsData04.RData", 
                                            package = "CodelistGenerator"))

## ----eval=FALSE---------------------------------------------------------------
#  oaCodes4 <- getCandidateCodes(
#    cdm = cdm,
#    keywords = "osteoarthritis",
#    domains = "Condition",
#    searchInSynonyms = TRUE,
#    searchNonStandard = FALSE,
#    exclude = c(
#      "post-infection",
#      "post-traumatic"
#    ),
#    includeDescendants = FALSE,
#    includeAncestor = FALSE
#  )

## ----message=FALSE, warning=FALSE---------------------------------------------
newCodes1To4 <- compareCodelists(oaCodes1, oaCodes4) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To4 %>% 
  glimpse()

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
oaCodes5 <- readRDS(system.file("optionsData05.RData", 
                                            package = "CodelistGenerator"))

## ----eval=FALSE---------------------------------------------------------------
#  oaCodes5 <- getCandidateCodes(
#    cdm = cdm,
#    keywords = "osteoarthritis",
#    domains = "Condition",
#    searchInSynonyms = FALSE,
#    searchNonStandard = TRUE,
#    exclude = c(
#      "post-infection",
#      "post-traumatic"
#    ),
#    includeDescendants = FALSE,
#    includeAncestor = FALSE
#  )

## ----message=FALSE, warning=FALSE---------------------------------------------
newCodes1To5 <- compareCodelists(oaCodes1, oaCodes5) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To5 %>% 
  glimpse()

## ----message=FALSE, warning=FALSE,echo=FALSE----------------------------------
oaCodes8 <- readRDS(system.file("optionsData07.RData", 
                                            package = "CodelistGenerator"))

## ----eval=FALSE---------------------------------------------------------------
#  oaCodes8 <- getCandidateCodes(
#    cdm = cdm,
#    keywords = "osteoarthritis",
#    domains = "Condition",
#    searchInSynonyms = FALSE,
#    searchNonStandard = FALSE,
#    exclude = c(
#      "post-infection",
#      "post-traumatic"
#    ),
#    includeDescendants = FALSE,
#    includeAncestor = TRUE
#  )

## ----message=FALSE, warning=FALSE---------------------------------------------
newCodes1To8 <- compareCodelists(oaCodes1, oaCodes8) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To8 %>% 
  glimpse()

## ----include = FALSE----------------------------------------------------------
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN
)

## ----warning=FALSE, message=FALSE---------------------------------------------
# library(DBI)
# library(duckdb)
# library(dplyr)
# library(CDMConnector)
# library(CodelistGenerator)

## ----warning=FALSE------------------------------------------------------------
# requireEunomia()
# db <- dbConnect(duckdb(), dbdir = eunomiaDir())
# cdm <- cdmFromCon(db,
#                   cdmSchema = "main",
#                   writeSchema = "main",
#                   writePrefix = "cg_")
# cdm

## -----------------------------------------------------------------------------
# cdm$concept |> glimpse()
# cdm$concept_relationship |> glimpse()
# cdm$concept_ancestor |> glimpse()
# cdm$concept_synonym |> glimpse()
# cdm$drug_strength |> glimpse()

## -----------------------------------------------------------------------------
# vocabularyVersion(cdm)

## -----------------------------------------------------------------------------
# availableVocabularies(cdm)

## ----eval = FALSE-------------------------------------------------------------
# library(readr)
# library(DBI)
# library(duckdb)
# library(omopgenerics)
# library(here)
# 
# vocab_folder <- here() # add path to directory
# 
# # read in files
# concept <- read_delim(here(vocab_folder, "CONCEPT.csv"),
#                       "\t",
#                       escape_double = FALSE, trim_ws = TRUE
# )
# concept_relationship <- read_delim(here(vocab_folder, "CONCEPT_RELATIONSHIP.csv"),
#                                    "\t",
#                                    escape_double = FALSE, trim_ws = TRUE
# )
# concept_ancestor <- read_delim(here(vocab_folder, "CONCEPT_ANCESTOR.csv"),
#                                "\t",
#                                escape_double = FALSE, trim_ws = TRUE
# )
# concept_synonym <- read_delim(here(vocab_folder, "CONCEPT_SYNONYM.csv"),
#                               "\t",
#                               escape_double = FALSE, trim_ws = TRUE
# )
# vocabulary <- read_delim(here(vocab_folder, "VOCABULARY.csv"), "\t",
#                          escape_double = FALSE, trim_ws = TRUE
# )
# 
# # write to duckdb
# db <- dbConnect(duckdb(), here(vocab_folder,"vocab.duckdb"))
# dbWriteTable(db, "concept", concept, overwrite = TRUE)
# dbWriteTable(db, "concept_relationship", concept_relationship, overwrite = TRUE)
# dbWriteTable(db, "concept_ancestor", concept_ancestor, overwrite = TRUE)
# dbWriteTable(db, "concept_synonym", concept_synonym, overwrite = TRUE)
# dbWriteTable(db, "vocabulary", vocabulary, overwrite = TRUE)
# # add empty person and observation period tables
# person_cols <- omopColumns("person")
# person <- data.frame(matrix(ncol = length(person_cols), nrow = 0))
# colnames(person) <- person_cols
# dbWriteTable(db, "person", person, overwrite = TRUE)
# observation_period_cols <- omopColumns("observation_period")
# observation_period <- data.frame(matrix(ncol = length(observation_period_cols), nrow = 0))
# colnames(observation_period) <- observation_period_cols
# dbWriteTable(db, "observation_period", observation_period, overwrite = TRUE)
# dbDisconnect(db)

## ----eval = FALSE-------------------------------------------------------------
# db <- dbConnect(duckdb(), here(vocab_folder,"vocab.duckdb"))
# cdm <- cdmFromCon(db, "main", "main", cdmName = "vocabularise", .softValidation = TRUE)

## ----include = FALSE----------------------------------------------------------
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN
)

## -----------------------------------------------------------------------------
# library(omopgenerics, warn.conflicts = FALSE)
# library(dplyr, warn.conflicts = FALSE)
# library(CodelistGenerator)

## -----------------------------------------------------------------------------
# codelist <- list("codes1" = c(1L, 2L, 3L),
#                  "codes2" = c(4L, 5L, 10L))
# codelist <- newCodelist(codelist)
# 
# codelist

## -----------------------------------------------------------------------------
# attributes(codelist)

## -----------------------------------------------------------------------------
# empty_codelist  <- emptyCodelist()
# 
# empty_codelist

## -----------------------------------------------------------------------------
# codelist_with_details <- list("codes1" = tibble("concept_id" = c(1L, 2L, 3L)),
#                               "codes2" = tibble("concept_id" = c(4L, 5L, 10L)))
# codelist_with_details <- newCodelistWithDetails(codelist_with_details)
# 
# codelist_with_details

## -----------------------------------------------------------------------------
# codelist_with_details <- list(
#   "codes1" = tibble("concept_id" = c(1L, 2L, 3L),
#                     "concept_name" = c("Musculoskeletal disorder", "Osteoarthrosis", "Arthritis"),
#                     "domain_id" = c("Condition", "Condition", "Condition"),
#                     "vocabulary_id" = c("SNOMED","SNOMED","SNOMED"),
#                     "standard_concept" = c("S","S","S")),
#   "codes2" = tibble("concept_id" = c(4L, 5L, 10L),
#                     "concept_name" = c("Osteoarthritis of knee", "Osteoarthritis of hip", "Adalimumab"),
#                     "domain_id" = c("Condition", "Condition", "Drug"),
#                     "vocabulary_id" = c("SNOMED","SNOMED","RxNorm"),
#                     "standard_concept" = c("S","S","S")))
# codelist_with_details <- newCodelistWithDetails(codelist_with_details)
# 
# codelist_with_details

## -----------------------------------------------------------------------------
# attributes(codelist_with_details)

## -----------------------------------------------------------------------------
# empty_codelist_with_details <- emptyCodelistWithDetails()
# 
# empty_codelist_with_details

## -----------------------------------------------------------------------------
# concept_set_expression <- list(
#   "codes1" = tibble("concept_id" = c(1L, 2L, 3L),
#                     "excluded" = c(FALSE, FALSE, FALSE),
#                     "descendants" = c(TRUE, FALSE, FALSE),
#                     "mapped" = c(TRUE, TRUE, TRUE)),
#   "codes2" = tibble("concept_id" = c(4L, 5L, 10L),
#                     "excluded" = c(FALSE, FALSE, FALSE),
#                     "descendants" = c(FALSE, FALSE, FALSE),
#                     "mapped" = c(TRUE, TRUE, TRUE))
# )
# 
# concept_set_expression <- newConceptSetExpression(concept_set_expression)
# 
# concept_set_expression

## -----------------------------------------------------------------------------
# attributes(concept_set_expression)

## -----------------------------------------------------------------------------
# empty_concept_set_expression <- emptyConceptSetExpression()
# 
# empty_concept_set_expression

## -----------------------------------------------------------------------------
# cdm <- mockVocabRef()
# codelist_to_codelist_with_details <- asCodelistWithDetails(codelist, cdm)
# codelist_to_codelist_with_details
# 
# codelist_to_concept_set_expression <- asCodelistWithDetails(codelist, cdm)
# codelist_to_concept_set_expression
# 

## -----------------------------------------------------------------------------
# 
# codelist_to_concept_set_expression <- asConceptSetExpression(codelist)
# codelist_to_concept_set_expression
# 
# codelist_with_details_to_concept_set_expression <- asConceptSetExpression(codelist_with_details)
# codelist_with_details_to_concept_set_expression

## -----------------------------------------------------------------------------
# codelist_with_details_to_codelist <- asCodelist(codelist_with_details)
# codelist_with_details_to_codelist
# 

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%"
)

## ----message=FALSE, warning=FALSE---------------------------------------------
library(dplyr)
library(CodelistGenerator)

cdm <- mockVocabRef()

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/1.png")

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/2.png")

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition", 
  standardConcept = "Standard",
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)

getCandidateCodes(
  cdm = cdm,
  keywords = "Disorder musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)

## ----message=FALSE------------------------------------------------------------
candidate_codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)

searchStrategy(candidate_codes)

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/3.png")

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  standardConcept = c("Non-standard", "Standard"),
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/4.png")

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = c(
    "Musculoskeletal disorder",
    "arthritis"
  ),
  domains = "Condition",
  standardConcept = c("Standard"),
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/5.png")

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/6.png")

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  exclude = c("Osteoarthrosis", "knee"),
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  exclude = c("knee osteoarthritis"),
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----message=FALSE------------------------------------------------------------
# No exclusion:
getCandidateCodes(
    cdm = cdm,
    keywords = "Knee",
    domains = "Condition",
    exclude = NULL,
    standardConcept = c("Standard", "Non-standard"),
    includeDescendants = TRUE,
    searchInSynonyms = FALSE,
    searchNonStandard = FALSE,
    includeAncestor = FALSE
)

# Exclusion looking for terms:
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("knee osteoarthritis"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# Exclusion looking for partial matching terms (without word boundaries)
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/knee osteoarthritis/"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# Exclusion looking for partial matching terms (without word boundaries)
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/e osteoarthritis/"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/\bKnee osteoarthritis/\b"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# We will now only search for "ee osteoarthritis" to show that 
# "knee osteoarthritis" won't be excluded:
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/\bee osteoarthritis/\b"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/7.png")

## ----message=FALSE------------------------------------------------------------
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Osteoarthritis of knee",
  includeAncestor = TRUE,
  domains = "Condition",
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
)

codes

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/8.png")

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthrosis",
  domains = "Condition",
  searchInSynonyms = TRUE,
  standardConcept = "Standard",
  includeDescendants = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/9.png")

## ----message=FALSE------------------------------------------------------------
getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthrosis",
  domains = "Condition",
  searchInSynonyms = TRUE,
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/10.png")

## ----message=FALSE------------------------------------------------------------
codes1 <- getCandidateCodes(
  cdm = cdm,
  keywords = "Degenerative",
  domains = "Condition",
  standardConcept = "Standard",
  searchNonStandard = TRUE,
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  includeAncestor = FALSE
)
codes1

## ----echo=FALSE---------------------------------------------------------------
knitr::include_graphics("Figures/11.png")

## ----message=FALSE------------------------------------------------------------
codes2 <- getCandidateCodes(
  cdm = cdm,
  keywords = "Degenerative",
  domains = "Condition",
  standardConcept = c("Non-standard", "Standard"),
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  includeAncestor = FALSE
)
codes2

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)

## ----include = FALSE----------------------------------------------------------
CDMConnector::requireEunomia("synpuf-1k", "5.3")

## ----message=FALSE, warning=FALSE---------------------------------------------
library(DBI)
library(duckdb)
library(dplyr)
library(CDMConnector)
library(CodelistGenerator)
library(visOmopResults)

# Connect to the database and create the cdm object
con <- dbConnect(duckdb(), 
                      eunomiaDir("synpuf-1k", "5.3"))
cdm <- cdmFromCon(con = con, 
                  cdmName = "Eunomia Synpuf",
                  cdmSchema   = "main",
                  writeSchema = "main", 
                  achillesSchema = "main")

timings <- benchmarkCodelistGenerator(cdm)

visOmopTable(timings,
             hide = c("variable_name", "variable_level", "strata_name", "strata_level"),
             groupColumn = "task")

## ----include = FALSE----------------------------------------------------------
  NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
  
  knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN)

## ----include = FALSE----------------------------------------------------------
#   CDMConnector::requireEunomia("synpuf-1k", "5.3")

## ----warning=FALSE, message=FALSE---------------------------------------------
# library(DBI)
# library(duckdb)
# library(dplyr)
# library(CDMConnector)
# library(CodelistGenerator)
# 
# # Download mock database
# requireEunomia(datasetName = "synpuf-1k", cdmVersion = "5.3")
# 
# # Connect to the database and create the cdm object
# con <- dbConnect(duckdb(), eunomiaDir("synpuf-1k", "5.3"))
# cdm <- cdmFromCon(con = con,
# cdmName = "Eunomia Synpuf",
# cdmSchema   = "main",
# writeSchema = "main",
# achillesSchema = "main")

## ----warning=FALSE, message=FALSE---------------------------------------------
# acetaminophen <- getDrugIngredientCodes(cdm,
#                                         name = "acetaminophen",
#                                         nameStyle = "{concept_name}",
#                                         type = "codelist")

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_drug <- subsetOnDomain(acetaminophen,
#                                      cdm,
#                                      domain = "Drug")
# 
# acetaminophen_drug

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_no_drug <- subsetOnDomain(acetaminophen,
#                                         cdm,
#                                         domain = "Drug",
#                                         negate = TRUE)
# 
# acetaminophen_no_drug

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_rxnorm <- subsetOnVocabulary(acetaminophen_drug,
#                                            cdm,
#                                            c("RxNorm"))
# acetaminophen_rxnorm

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_mg_unit <- subsetOnDoseUnit(acetaminophen_rxnorm,
#                                           cdm,
#                                           c("milligram", "unit"))
# acetaminophen_mg_unit

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_ingredient <- subsetOnIngredientRange(acetaminophen_drug,
#                                                 cdm,
#                                                 ingredientRange = c(3, 30))
# 
# acetaminophen_ingredient

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_route <- subsetOnRouteCategory(acetaminophen_mg_unit,
#                                              cdm,
#                                              c("transmucosal_rectal","unclassified_route"),
#                                              negate = TRUE)
# acetaminophen_route

## ----warning=FALSE, messages=FALSE--------------------------------------------
# # First, check which dose forms are available in our codelist
# acetaminophen_drug |>
#   associatedDoseForms(cdm)
# acetaminophen_oral <- subsetOnDoseForm(acetaminophen_drug,
#                                             cdm,
#                                             c("Oral Solution","Oral Capsule"))
# acetaminophen_oral

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_doses <- stratifyByDoseUnit(acetaminophen, cdm, keepOriginal = TRUE)
# 
# acetaminophen_doses

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_routes <- stratifyByRouteCategory(acetaminophen, cdm)
# 
# acetaminophen_routes

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_dose_forms <- stratifyByDoseForm(acetaminophen, cdm)
# 
# acetaminophen_dose_forms

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_routes1 <- addConcepts(acetaminophen_routes,
#                                      cdm,
#                                      concepts = c(1125315L))
# acetaminophen_routes1

## ----warning=FALSE, messages=FALSE--------------------------------------------
# x <- getDescendants(cdm = cdm, conceptId = c(1125315L))
# acetaminophen_routes2 <- addConcepts(acetaminophen_routes,
#                                      cdm,
#                                      concepts = x$concept_id,
#                                      codelistName = "acetaminophen_unclassified_route_category")
# acetaminophen_routes2

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen_routes3 <- excludeConcepts(acetaminophen_routes,
#                                          cdm,
#                                          concepts = x$concept_id,
#                                          codelistName = "acetaminophen_inhalable")
# acetaminophen_routes3

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen <- getDrugIngredientCodes(cdm,
#                                         name = "acetaminophen",
#                                         nameStyle = "{concept_name}",
#                                         type = "codelist")
# 
# new_codelist <- acetaminophen |>
#   addConcepts(cdm,
#               concepts = c(1L, 2L, 3L)) |>
#   subsetOnDomain(cdm,
#                  domain = "Drug") |>
#   stratifyByDoseUnit(cdm = cdm) |>
#   excludeConcepts(cdm,
#                   concepts = c(1127898))
# 
# new_codelist

## ----warning=FALSE, messages=FALSE--------------------------------------------
# acetaminophen <- getDrugIngredientCodes(cdm,
#                                         name = "acetaminophen",
#                                         nameStyle = "{concept_name}",
#                                         type = "codelist_with_details")
# hydrocodone <- getDrugIngredientCodes(cdm,
#                                       name = "hydrocodone",
#                                       doseUnit = "milligram",
#                                       nameStyle = "{concept_name}",
#                                       type = "codelist_with_details")

## -----------------------------------------------------------------------------
# comparison <- compareCodelists(acetaminophen,
#                                hydrocodone)
# 
# comparison |> glimpse()
# 
# comparison |> filter(codelist == "Both")

## ----include = FALSE----------------------------------------------------------
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN
)

CDMConnector::requireEunomia("synpuf-1k", "5.3")

## ----message=FALSE, warning=FALSE---------------------------------------------
# library(DBI)
# library(duckdb)
# library(dplyr)
# library(CDMConnector)
# library(CodelistGenerator)
# 
# # Connect to the database and create the cdm object
# con <- dbConnect(duckdb(),
#                  eunomiaDir("synpuf-1k", "5.3"))
# cdm <- cdmFromCon(con = con,
#                   cdmName = "Eunomia Synpuf",
#                   cdmSchema = "main",
#                   writeSchema = "main",
#                   achillesSchema = "main")

## -----------------------------------------------------------------------------
# availableDrugIngredients(cdm) |> glimpse()

## -----------------------------------------------------------------------------
# acetaminophen_codes <- getDrugIngredientCodes(
#   cdm = cdm,
#   name = c("acetaminophen")
# )
# 
# acetaminophen_codes

## -----------------------------------------------------------------------------
# acetaminophen_codes <- getDrugIngredientCodes(
#   cdm = cdm,
#   name = 1125315
# )
# 
# acetaminophen_codes

## -----------------------------------------------------------------------------
# acetaminophen_two_or_more_ingredients <- getDrugIngredientCodes(
#   cdm = cdm,
#   name = "acetaminophen",
#   ingredientRange = c(2, Inf),
#   type = "codelist_with_details"
# )
# 
# acetaminophen_two_or_more_ingredients
# 
# acetaminophen_two_or_more_ingredients[[1]] |>
#   pull("concept_name") |>
#   head(n = 5) # Only the first five will be shown

## -----------------------------------------------------------------------------
# acetaminophen_one_ingredient <- getDrugIngredientCodes(
#   cdm = cdm,
#   name = "acetaminophen",
#   ingredientRange = c(1, 1),
#   type = "codelist_with_details"
# )
# 
# acetaminophen_one_ingredient
# 
# acetaminophen_one_ingredient[[1]] |>
#   pull("concept_name") |>
#   head(n = 5) # Only the first five will be shown

## -----------------------------------------------------------------------------
# availableDoseForms(cdm) |> glimpse()

## -----------------------------------------------------------------------------
# acetaminophen_injections <- getDrugIngredientCodes(
#   cdm = cdm,
#   name = "acetaminophen",
#   doseForm = "injection",
#   type = "codelist_with_details"
# )
# 
# acetaminophen_injections[[1]] |>
#   pull("concept_name") |>
#   head(n = 5)

## -----------------------------------------------------------------------------
# availableDoseUnits(cdm) |> glimpse()

## -----------------------------------------------------------------------------
# acetaminophen_miligram <- getDrugIngredientCodes(
#   cdm = cdm,
#   name = "acetaminophen",
#   doseUnit = "milligram",
#   type = "codelist_with_details"
# )
# 
# acetaminophen_miligram[[1]] |>
#   pull("concept_name") |>
#   head(n = 5)

## -----------------------------------------------------------------------------
# availableRouteCategories(cdm) |> glimpse()

## -----------------------------------------------------------------------------
# acetaminophen_inhalable <- getDrugIngredientCodes(
#   cdm = cdm,
#   name = "acetaminophen",
#   routeCategory = "inhalable",
#   type = "codelist_with_details"
# )
# 
# acetaminophen_inhalable[[1]] |>
#   pull("concept_name") |>
#   head(n = 5)

## -----------------------------------------------------------------------------
# acetaminophen_heparin_codes <- getDrugIngredientCodes(
#   cdm = cdm,
#   name = c("acetaminophen", "heparin")
#   )
# 
# acetaminophen_heparin_codes

## -----------------------------------------------------------------------------
# availableATC(cdm, level = c("ATC 1st")) |> glimpse()
# availableATC(cdm, level = c("ATC 2nd")) |> glimpse()
# availableATC(cdm, level = c("ATC 3rd")) |> glimpse()
# availableATC(cdm, level = c("ATC 4th")) |> glimpse()
# availableATC(cdm, level = c("ATC 5th")) |> glimpse()

## -----------------------------------------------------------------------------
# atc_codelist <- getATCCodes(
#   cdm = cdm,
#   level = "ATC 1st",
#   name = "alimentary tract and metabolism"
# )
# 
# atc_codelist

## ----include = FALSE----------------------------------------------------------
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN)

## ----include = FALSE----------------------------------------------------------
# CDMConnector::requireEunomia("synpuf-1k", "5.3")

## ----message=FALSE, warning=FALSE---------------------------------------------
# library(DBI)
# library(duckdb)
# library(dplyr)
# library(CDMConnector)
# library(CodelistGenerator)
# library(CohortConstructor)
# library(omopgenerics)
# 
# # Connect to the database and create the cdm object
# con <- dbConnect(duckdb(),
#                       eunomiaDir("synpuf-1k", "5.3"))
# cdm <- cdmFromCon(con = con,
#                   cdmName = "Eunomia Synpuf",
#                   cdmSchema   = "main",
#                   writeSchema = "main",
#                   achillesSchema = "main")
# 
# # Create a codelist for depression
# depression <- getCandidateCodes(cdm,
#                                 keywords = "depression")
# depression <- newCodelist(list("depression" = depression$concept_id))

## ----message=FALSE, warning=FALSE---------------------------------------------
# achilles_code_use <- summariseAchillesCodeUse(depression,
#                                               cdm,
#                                               countBy = c("record", "person"))

## ----message=FALSE, warning=FALSE---------------------------------------------
# tableAchillesCodeUse(achilles_code_use,
#                      type = "gt")

## ----message=FALSE, warning=FALSE---------------------------------------------
# code_use <- summariseCodeUse(depression,
#                              cdm,
#                              countBy = c("record", "person"),
#                              byYear  = FALSE,
#                              bySex   = FALSE,
#                              ageGroup =  list("<=50" = c(0,50), ">50" = c(51,Inf)),
#                              dateRange = as.Date(c("2010-01-01", "2020-01-01")))
# 
# tableCodeUse(code_use, type = "gt")

## ----message=FALSE, warning=FALSE---------------------------------------------
# orphan <- summariseOrphanCodes(depression, cdm)
# tableOrphanCodes(orphan, type = "gt")

## ----message=FALSE, warning=FALSE---------------------------------------------
# cdm[["depression"]] <- conceptCohort(cdm,
#                                      conceptSet = depression,
#                                      name = "depression")

## ----message=FALSE, warning=FALSE---------------------------------------------
# cohort_code_use <- summariseCohortCodeUse(cdm,
#                                           cohortTable = "depression",
#                                           countBy = c("record", "person"))
# tableCohortCodeUse(cohort_code_use)

## ----message=FALSE, warning=FALSE---------------------------------------------
# cohort_code_use <- summariseCohortCodeUse(cdm,
#                                           cohortTable = "depression",
#                                           countBy = c("record", "person"),
#                                           timing = "entry")
# tableCohortCodeUse(cohort_code_use)

## ----message=FALSE, warning=FALSE---------------------------------------------
# anxiety <- getCandidateCodes(cdm,
#                              keywords = "anxiety")
# anxiety <- newCodelist(list("anxiety" = anxiety$concept_id))
# 
# cohort_code_use <- summariseCohortCodeUse(cdm,
#                                           cohortTable = "depression",
#                                           x = anxiety,
#                                           countBy = c("record", "person"),
#                                           timing = "entry")
# tableCohortCodeUse(cohort_code_use)

## ----message=FALSE, warning=FALSE---------------------------------------------
# cohort_code_use <- summariseCohortCodeUse(cdm = cdm,
#                                           cohortTable = "depression",
#                                           countBy = c("record", "person"),
#                                           byYear = FALSE,
#                                           bySex = TRUE,
#                                           ageGroup = NULL)
# tableCohortCodeUse(cohort_code_use)

getConceptIDName <- function(omopTableName){
  if(omopTableName == "visit_occurrence") {
    return("visit_concept_id")
  }
  if(omopTableName == "visit_detail") {
    return("visit_detail_concept_id")
  }
  if(omopTableName == "condition_occurrence") {
    return("condition_concept_id")
  }
  if(omopTableName == "drug_exposure") {
    return("drug_concept_id")
  }
  if(omopTableName == "procedure_occurrence") {
    return("procedure_concept_id")
  }
  if(omopTableName == "device_exposure") {
    return("device_concept_id")
  }
  if(omopTableName == "measurement") {
    return("measurement_concept_id")
  }
  if(omopTableName == "observation") {
    return("observation_concept_id")
  }
  if(omopTableName == "drug_strength") {
    return("drug_concept_id")
  }
  if(omopTableName == "concept_relationship"){
    return("concept_id_1")
  }
}

subsetOmopTable <- function(cdm, newOmopTable, omopTable, x){

  if(!is.null(x)){
    if(inherits(x, "codelist_with_details")){
      x <- as.numeric(unlist(purrr::map(x, \(x) x |> dplyr::pull("concept_id"))))
    } else {
      x <- as.numeric(unlist(x))
    }

    tableCodelist <- paste0(omopgenerics::uniqueTableName(),
                            omopgenerics::uniqueId())

    cdm <- omopgenerics::insertTable(cdm = cdm,
                                     name = tableCodelist,
                                     table = dplyr::tibble("concept_id" = x),
                                     overwrite = TRUE,
                                     temporary = FALSE)

    cdm[[newOmopTable]] <- cdm[[omopTable]] |>
      dplyr::rename("concept_id" = !!getConceptIDName(omopTable)) |>
      dplyr::inner_join(
        cdm[[tableCodelist]],
        by = "concept_id"
      ) |>
      dplyr::compute(temporary = FALSE, name = newOmopTable)

    omopgenerics::dropSourceTable(cdm, name = tableCodelist)

  } else {
    cdm[[newOmopTable]] <- cdm[[omopTable]] |>
      dplyr::rename("concept_id" = !!getConceptIDName(omopTable)) |>
      dplyr::compute(name = newOmopTable, temporary = FALSE)
  }

  return(cdm)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get available relationships between concepts
#'
#' @inheritParams cdmDoc
#' @param standardConcept1  Character vector with one or more of "Standard",
#' "Classification", and "Non-standard". These correspond to the flags used
#' for the standard_concept field in the concept table of the cdm.
#' @param standardConcept2  Character vector with one or more of "Standard",
#' "Classification", and "Non-standard". These correspond to the flags used
#' for the standard_concept field in the concept table of the cdm.
#' @param domains1 Character vector with one or more of the OMOP CDM domain. If NULL, all domains are considered.
#' @param domains2 Character vector with one or more of the OMOP CDM domain. If NULL, all domains are considered.
#'
#' @return A character vector with unique concept relationship values.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all relationship ids in the CDM between `Condition` and `Standard` concepts.
#' availableRelationshipIds(cdm = cdm,
#'                          standardConcept1 = "Standard",
#'                          standardConcept2 = "Standard",
#'                          domains1 = "Condition",
#'                          domains2 = "Condition")
#'
#' }
availableRelationshipIds <- function(cdm,
                            standardConcept1 = "Standard",
                            standardConcept2 = "Standard",
                            domains1 = "Condition",
                            domains2 = "Condition") {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  domains1 <- assertDomain(domains1, cdm)
  standardConcept1 <- assertStandardConcept(standardConcept1)
  domains2 <- assertDomain(domains2, cdm)
  standardConcept2 <- assertStandardConcept(standardConcept2)

  relationshipIds <- relationshipIdsInternal(cdm,
                                               newOmopTable = "concept_relationship",
                                               standardConcept1 = standardConcept1,
                                               standardConcept2 = standardConcept2,
                                               domains1 = domains1,
                                               domains2 = domains2,
                                               xx = NULL)

  return(relationshipIds)
}

#' Get all relationships types that exist in the OMOP vocabulary `concept_relationship`
#' table for a given set of concepts in a codelist.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @param standardConcept1  Character vector with one or more of "Standard",
#' "Classification", and "Non-standard". These correspond to the flags used
#' for the standard_concept field in the concept table of the cdm.
#' @param standardConcept2  Character vector with one or more of "Standard",
#' "Classification", and "Non-standard". These correspond to the flags used
#' for the standard_concept field in the concept table of the cdm.
#' @param domains1 Character vector with one or more of the OMOP CDM domain. If NULL, all domains are considered.
#' @param domains2 Character vector with one or more of the OMOP CDM domain. If NULL, all domains are considered.
#'
#' @return A character vector with unique concept relationship values.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Create codelist
#' codelist <- newCodelist(list("codes1" = c(8479L, 4117795L, 44022939L),
#'                              "codes2" = c(8480L, 8600L, 8481L, 4189167L, 40371897L)))
#'
#' # You can optionally restrict to only relationships between concepts that are
#' # "Standard" and "Non-standard". For example:
#' relationships <- associatedRelationshipIds(x = codelist,
#'                                            cdm = cdm,
#'                                            standardConcept1 = "Standard",
#'                                            standardConcept2 = "Non-standard")
#' relationships
#'
#' # It returns the relationships between concepts where:
#' #  - concept_id_1 is 'Standard'
#' #  - concept_id_2 is 'Non-standard'
#' # Similarly, we can obtain the relationships restricting by domain:
#' relationships <- associatedRelationshipIds(x = codelist,
#'                                            cdm = cdm,
#'                                            domains1 = c("Drug", "Condition"),
#'                                            domains2 = c("Drug", "Condition"))
#' relationships
#' }
associatedRelationshipIds <- function(x,
                                      cdm,
                                      standardConcept1 = c("Standard", "Non-standard", "Classification"),
                                      standardConcept2 = c("Standard", "Non-standard", "Classification"),
                                      domains1 = NULL,
                                      domains2 = NULL) {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  checkCodelist(x, allowConceptSetExpression = FALSE)
  domains1 <- assertDomain(domains1, cdm)
  standardConcept1 <- assertStandardConcept(standardConcept1)
  domains2 <- assertDomain(domains2, cdm)
  standardConcept2 <- assertStandardConcept(standardConcept2)

  if(inherits(x, "codelist_with_details")){
    x <- asCodelist(x)
  }

  if(inherits(x, "codelist")){
    newOmopTable <- paste0(omopgenerics::uniqueTableName(),
                           omopgenerics::uniqueId())
    codesTable <- paste0(omopgenerics::uniqueTableName(),
                         omopgenerics::uniqueId())
    cdm <- subsetOmopTable(cdm, newOmopTable, "concept_relationship", x)

    relationshipIds <- list()
    for( i in seq_along(x) ){
      relationshipIds[[names(x)[[i]]]] <- relationshipIdsInternal(cdm,
                                                                  newOmopTable = newOmopTable,
                                                                  codesTable = codesTable,
                                                                  standardConcept1 = standardConcept1,
                                                                  standardConcept2 = standardConcept2,
                                                                  domains1 = domains1,
                                                                  domains2 = domains2,
                                                                  xx = x[[i]])
    }
    omopgenerics::dropSourceTable(cdm, name = c(newOmopTable))
  }

  return(relationshipIds)
}

relationshipIdsInternal <- function(cdm, newOmopTable, codesTable, standardConcept1, standardConcept2,
                                    domains1, domains2, xx){

  relationshipIds <- cdm[[newOmopTable]]

  if(!is.null(xx)){
    cdm <- omopgenerics::insertTable(
      cdm, name = codesTable, dplyr::tibble(concept_id_1 = xx))
    relationshipIds <- relationshipIds |>
      dplyr::rename("concept_id_1" = "concept_id")  |>
      dplyr::inner_join(cdm[[codesTable]], by = "concept_id_1")
  }

  relationshipIds <- relationshipIds |>
    dplyr::left_join(
      cdm[["concept"]] |>
        dplyr::select(
          "concept_id_1" = "concept_id",
          "domain_id_1" = "domain_id",
          "standard_concept_1" = "standard_concept"
        ),
      by = c("concept_id_1")
    ) |>
    dplyr::left_join(
      cdm[["concept"]] |>
        dplyr::select(
          "concept_id_2" = "concept_id",
          "domain_id_2" = "domain_id",
          "standard_concept_2" = "standard_concept"
        ),
      by = c("concept_id_2")
    ) |>
    dplyr::mutate_at(c("domain_id_1", "domain_id_2"),
                     ~ tolower(.)) |>
    dplyr::mutate_at(c("standard_concept_1", "standard_concept_2"),
                     ~ dplyr::case_when(
                       is.na(.) ~ "non-standard",
                       . == "C" ~ "classification",
                       . == "S" ~ "standard"
                     )) |>
    dplyr::filter(.data$standard_concept_1 %in% .env$standardConcept1,
                  .data$standard_concept_2 %in% .env$standardConcept2,
                  tolower(.data$domain_id_1) %in% .env$domains1,
                  tolower(.data$domain_id_2) %in% .env$domains2)

  relationshipIds <- relationshipIds |>
    dplyr::select("relationship_id") |>
    dplyr::distinct() |>
    dplyr::pull() |>
    sort()

  return(relationshipIds)
}

test_that("tests with mock db", {
  skip_on_cran()
  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])
    namesInitial <- names(cdm)
    codelist_with_details <- list("codes" = cdm[["concept"]] |>
                                    dplyr::filter(concept_id %in% c(10L, 15L)) |>
                                    dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
                                    dplyr::mutate("ancestor_concept_id" = 999L) |>
                                    dplyr::collect()) |>
      omopgenerics::newCodelistWithDetails()

    codelist <- omopgenerics::newCodelist(list("codes1" = c(3L, 6L), "codes2" = c(24L)))

    # Available relationships ID ----
    expect_equal(sort(cdm$concept_relationship |>
                        dplyr::select("relationship_id") |>
                        dplyr::distinct() |>
                        dplyr::pull()),
                 availableRelationshipIds(cdm,
                                 standardConcept1 = c("Standard", "Non-standard", "Classification"),
                                 standardConcept2 = c("Standard", "Non-standard", "Classification"),
                                 domains1 = c("Condition", "Drug"),
                                 domains2 = c("Condition", "Drug")))

    expect_equal("Mapped from",
                 availableRelationshipIds(cdm,
                                 standardConcept1 = "Standard",
                                 standardConcept2 = "Non-standard",
                                 domains1 = "Condition",
                                 domains2 = "Condition"))
    expect_equal("Maps to",
                 availableRelationshipIds(cdm,
                                 standardConcept1 = "Non-standard",
                                 standardConcept2 = "Standard",
                                 domains1 = "Condition",
                                 domains2 = "Condition"))

    # codelist
    expect_no_error(x <- associatedRelationshipIds(cdm, x = codelist))
    expect_equal(names(x), c("codes1", "codes2"))
    expect_true(x$codes1 == "Due to of")
    expect_true(length(x$codes2) == 0)
    expect_equal(namesInitial, names(cdm))

    expect_no_error(x <- associatedRelationshipIds(cdm,
                                         x = codelist_with_details,
                                         standardConcept1 = c("Standard", "Non-standard"),
                                         standardConcept2 = c("Standard", "Non-standard"),
                                         domains1 = c("Condition", "Drug"),
                                         domains2 = c("Condition", "Drug")))
    expect_equal(x$codes, c("Has brand name", "RxNorm has dose form", "Subsumes"))

    # expected errors
    expect_error(availableRelationshipIds("cdm"))
    expect_error(availableRelationshipIds(cdm,
                                 standardConcept1 = "Standard",
                                 standardConcept2 = "Something else"))
    expect_error(availableRelationshipIds(cdm,
                                 domains1 = 22,
                                 domains2 = "Condition"))
    expect_error(x <- availableRelationshipIds(cdm, x = list("codes" = c(3L, 6L))))


    expect_error(associatedRelationshipIds("cdm"))
    expect_error(associatedRelationshipIds(cdm, x = 123))
    expect_error(associatedRelationshipIds(cdm,
                                          standardConcept1 = "Standard",
                                          standardConcept2 = "Something else"))
    expect_error(associatedRelationshipIds(cdm,
                                          domains1 = 22,
                                          domains2 = "Condition"))
    expect_error(x <- associatedRelationshipIds(cdm, x = list("codes" = c(3L, 6L))))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Format the result of summariseCohortCodeUse into a table.
#'
#' @param result A `<summarised_result>` with results of the type "cohort_code_use".
#' @inheritParams typeTableDoc
#' @inheritParams tableStyleDoc
#' @inheritParams headerStrataDoc
#' @inheritParams groupColumnStrataDoc
#' @inheritParams hideStrataDoc
#' @inheritParams .optionsDoc
#'
#' @return A table with a formatted version of the summariseCohortCodeUse
#' result.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' con <- DBI::dbConnect(duckdb::duckdb(),
#'                       dbdir = CDMConnector::eunomiaDir())
#' cdm <- CDMConnector::cdmFromCon(con,
#'                                   cdmSchema = "main",
#'                                   writeSchema = "main")
#' cdm <- CDMConnector::generateConceptCohortSet(cdm = cdm,
#' conceptSet = list(a = 260139,
#'                   b = 1127433),
#'                   name = "cohorts",
#'                   end = "observation_period_end_date",
#'                   overwrite = TRUE)
#'
#'results_cohort_mult <-
#'summariseCohortCodeUse(list(cs = c(260139,19133873)),
#'                       cdm = cdm,
#'                       cohortTable = "cohorts",
#'                       timing = "entry")
#'
#'tableCohortCodeUse(results_cohort_mult)
#'CDMConnector::cdmDisconnect(cdm)
#'}
#'
tableCohortCodeUse <- function(result,
                               type = NULL,
                               header = c("cdm_name", "estimate_name"),
                               groupColumn = character(),
                               hide = c("timing"),
                               .options = list(),
                               style = NULL) {

  rlang::check_installed("visOmopResults", version = "1.4.0")

  # checks
  result <- omopgenerics::validateResultArgument(result)

  # empty result object
  if(nrow(result) == 0){
    cli::cli_warn("`result` object is empty")
    return(visOmopResults::emptyTable(type = type))
  }

  # no cohort_code_use
  result <- result |>
    visOmopResults::filterSettings(.data$result_type == "cohort_code_use")
  if(nrow(result) == 0){
    cli::cli_warn("No code use results found in result object")
    return(visOmopResults::emptyTable(type = type))
  }

  x <- internalTableCodeUse(
    result = result,
    resultType = "cohort_code_use",
    type = type,
    header = header,
    groupColumn = groupColumn,
    hide = hide,
    style = style,
    .options = .options
  )

  return(x)
}

test_that("table code use expcted columns", {
  skip_on_cran()

  testthat::local_mocked_bindings(
    system_fonts = function(...) data.frame(family = "sans", path = ""),
    .package = "systemfonts"
  )

  con <- DBI::dbConnect(duckdb::duckdb(), dbdir = CDMConnector::eunomiaDir())
  cdm <- CDMConnector::cdmFromCon(con, cdmSchema = "main", writeSchema = "main")

  acetiminophen <- c(1125315,  1127433, 40229134,
                     40231925, 40162522, 19133768,  1127078) |>
    as.integer()
  poliovirus_vaccine <- c(40213160L)
  cs <- omopgenerics::newCodelist(list(acetiminophen = acetiminophen,
                                       poliovirus_vaccine = poliovirus_vaccine))

  # Code use
  results <- summariseCodeUse(cs,
                              cdm = cdm,
                              byYear = TRUE,
                              bySex = TRUE,
                              ageGroup = list(c(0,17),
                                              c(18,65),
                                              c(66, 100)))
  expect_no_error(tableCodeUse(result = results))
  expect_no_error(tableCodeUse(result = results,
                               style = "darwin"))
  expect_no_error(tableCodeUse(result = results,
                               type = "gt",
                               header = c("cdm_name", "estimate_name"),
                               groupColumn = NULL,
                               hide = character(),
                               .options = list()))
  expect_no_error(tableCodeUse(result = results,
                               type = "tibble",
                               header = c("cdm_name", "estimate_name"),
                               groupColumn = NULL,
                               hide = "source_concept_id",
                               .options = list()))
  expect_no_error(tableCodeUse(result = results,
                               type = "flextable",
                               header = c("cdm_name", "estimate_name"),
                               groupColumn = "domain_id",
                               .options = list()))

  # Cohort code use
  pharyngitis <- c(4112343L)
  cdm <- CDMConnector::generateConceptCohortSet(cdm = cdm,
                                                conceptSet = list(pharyngitis = pharyngitis),
                                                name = "pharyngitis",
                                                end = "observation_period_end_date",
                                                overwrite = TRUE)

  results_cohort <- summariseCohortCodeUse(omopgenerics::newCodelist(list(cs = 4134304L)),
                                           cdm = cdm,
                                           cohortTable = "pharyngitis",
                                           timing = "any")

  tableCohortCodeUse(
    result = results_cohort,
    type = "flextable",
    header = c("cdm_name", "estimate_name"),
    groupColumn = NULL,
    hide = c("source_concept_id", "year"),
    .options = list()
  )

  # group name
  tableCohortCodeUse(
    result = results_cohort,
    type = "flextable",
    header = c("cdm_name", "estimate_name"),
    groupColumn = list("group" = c("cohort_name", "codelist_name")),
    .options = list()
  )

  # timing in combination with concept and source arguments
  tableCohortCodeUse(
    result = results_cohort,
    type = "gt",
    header = c("cdm_name", "estimate_name"),
    groupColumn = list("group" = c("cohort_name", "codelist_name")),
    hide = character(),
    .options = list()
  )

  CDMConnector::cdmDisconnect(cdm)
})

test_that("table code use output formats", {
  skip_on_cran()

  testthat::local_mocked_bindings(
    system_fonts = function(...) data.frame(family = "sans", path = ""),
    .package = "systemfonts"
  )

  con <- DBI::dbConnect(duckdb::duckdb(), dbdir = CDMConnector::eunomiaDir())
  cdm <- CDMConnector::cdmFromCon(con, cdmSchema = "main", writeSchema = "main")

  acetiminophen <- c(1125315,  1127433, 40229134,
                     40231925, 40162522, 19133768,  1127078) |>
    as.integer()
  poliovirus_vaccine <- c(40213160L)
  cs <- omopgenerics::newCodelist(list(acetiminophen = acetiminophen,
                                       poliovirus_vaccine = poliovirus_vaccine))

  results <- summariseCodeUse(cs,
                              cdm = cdm,
                              byYear = TRUE,
                              bySex = TRUE,
                              ageGroup = list(c(0,17),
                                              c(18,65),
                                              c(66, 100)))
  expect_no_error(tableCodeUse(result = results,
                               type = "flextable",
                               header = c("cdm_name", "estimate_name"),
                               groupColumn = "codelist_name",
                               style = "darwin",
                               .options = list()))



  CDMConnector::cdmDisconnect(cdm)
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Perform a systematic search to identify a candidate codelist using the OMOP
#' CDM vocabulary tables.
#'
#' @description
#' Based on the given search strategy, this function identifies a set of codes
#' that may represent a clinical event of interest in data mapped to the OMOP
#' CDM. These codes can then be considered for creating a study phenotype.
#'
#' @inheritParams cdmDoc
#' @param keywords Character vector of terms to search for.
#' * The search performed is broad, matching the provided keywords as substrings
#' anywhere within concept names. For example, `keywords = c("sep")` will
#' find "sepsis", "aseptic necrosis", and "acquired cardiac septal defect"
#' (along with many more).
#' * Where more than one word is given, all combinations of those words will be
#' identified. For example, `keywords = c("knee osteoarthritis")` will identify
#' a concept with the name "osteoarthritis of knee".
#' * Multiple keywords can be provided. For example,
#' `keywords = c("knee osteoarthritis", "hip osteoarthritis")` would identify
#' both "osteoarthritis of knee" and "osteoarthritis of hip"
#' @param exclude  Character vector of words to identify concepts to exclude.
#' For example, `getCandidateCodes(cdm,  keywords = "septic", exclude = "aseptic",  domains = "condition")`
#' would remove concepts "aseptic" when searching for concepts with "septic" in
#' their name.
#' * When one term contains multiple words (e.g., "knee osteoarthritis"),
#' each word will be search individually, so that "osteoarthritis of knee" would also be excluded.
#' If you only want to exclude partial matching terms, please add "/" at the beginning and the end of each term
#' (e.g., `"/knee osteoarthritis/`"). Notice that, with this options,
#' concepts like "right knee osteoarthritis" will also be excluded (as this is a partial match), but
#' "osteoarthritis of knee" won't be excluded. Different terms can have different rules (e.g.,
#' c("hip osteoarthritis", "/knee osteoarthritis/")).
#' * With multiple words, if we want exact matches accounting for word boundaries, we need to use
#' `/\b` at the beginning and at the end of each expression. In the previous example, using
#' `"/bknee osteoarthritis/\b"`, "right knee osteoarthritis" won't be excluded, but
#' "History of knee osteoarthritis" will be excluded.
#' @param domains Character vector with one or more of the OMOP CDM domain for
#' which to search within. If NULL, all domains are included in the search. Use
#' `availableDomains(cdm = cdm)` to identify available domains to search within.
#' @param vocabularyId Character vector with one or more vocabulary ID to
#' restrict results to. If NULL, all vocabularies will be included in the
#' search. Use `availableVocabularies(cdm)` to identify available vocabularies
#' to restrict search to.
#' @inheritParams standardConceptDoc
#' @param searchInSynonyms Either TRUE or FALSE. If TRUE the code will also
#' search using both the primary name in the concept table and synonyms from
#' the concept synonym table.
#' @param searchNonStandard Either TRUE or FALSE. If TRUE the code will also
#' search via non-standard concepts.
#' @inheritParams includeDescendantsDoc
#' @param includeAncestor Either TRUE or FALSE.
#' If TRUE the direct ancestor concepts of identified concepts
#'  will be included in the candidate codelist.
#' @return A "candidate_codes" object. This includes a tibble with the potential
#' codes of interest, along with an attribute containing the search strategy.
#' @importFrom rlang .data
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' cdm <- mockVocabRef()
#' getCandidateCodes(
#'   cdm = cdm,
#'   keywords = "osteoarthritis"
#'  )
#' }
getCandidateCodes <- function(cdm,
                              keywords,
                              exclude = NULL,
                              domains = "Condition",
                              vocabularyId = NULL,
                              standardConcept = "Standard",
                              searchInSynonyms = FALSE,
                              searchNonStandard = FALSE,
                              includeDescendants = TRUE,
                              includeAncestor = FALSE) {
  start <- Sys.time()

  ## checks for standard types of user error
  omopgenerics::assertCharacter(keywords, null = FALSE, na = FALSE)
  domains <- assertDomain(domains, cdm)
  omopgenerics::assertChoice(standardConcept, choices = c("Standard", "Classification", "Non-standard"))
  vocabularyId <- assertVocab(vocabularyId, domain = domains, cdm, standardConcept = standardConcept)
  omopgenerics::assertLogical(searchInSynonyms)
  omopgenerics::assertLogical(searchNonStandard)
  omopgenerics::assertLogical(includeDescendants)
  omopgenerics::assertLogical(includeAncestor)

  requiredTables <- c("concept", "concept_relationship", "concept_ancestor",
                      "concept_synonym", "vocabulary")
  if("Drug" %in% stringr::str_to_sentence(domains)){
    requiredTables <- append(requiredTables, "drug_strength")
  }
  omopgenerics::validateCdmArgument(cdm, requiredTables = requiredTables)

  # run search by domain
  searchSpecs <- data.frame(
    id = seq_along(domains),
    domain = domains
  )
  searchSpecs <- split(
    searchSpecs,
    searchSpecs[, c("id")]
  )

  searchResults <- runSearch(keywords,
                             cdm = cdm,
                             exclude = exclude,
                             domains = domains,
                             vocabularyId = vocabularyId,
                             standardConcept = standardConcept,
                             searchInSynonyms = searchInSynonyms,
                             searchNonStandard = searchNonStandard,
                             includeDescendants = includeDescendants,
                             includeAncestor = includeAncestor
  ) |>
    dplyr::arrange(.data$concept_id)

  if (nrow(searchResults) == 0) {
    cli::cli_inform(c("!" = "No codes found for the given search strategy"))
  } else {
    cli::cli_alert_success(
      "{nrow(searchResults)} candidate concept{?s} identified"
    )
  }

  duration <- abs(as.numeric(Sys.time() - start, units = "secs"))
  cli::cli_inform(
    "Time taken: {floor(duration/60)} minutes and {duration %% 60 %/% 1} seconds"
  )

  searchResults |>
    dplyr::mutate(
      vocabulary_version = vocabularyVersion(cdm = cdm),
    ) |>
    newCodeSearch(searchStrategy = searchStrategyAttr(
      function_name = "getCandidateCodes",
      cdm = "cdm",
      keywords = cast(keywords),
      exclude = cast(exclude),
      domains = cast(domains),
      vocabularyId = cast(vocabularyId),
      standardConcept = cast(standardConcept),
      searchInSynonyms = cast(searchInSynonyms),
      searchNonStandard = cast(searchNonStandard),
      includeDescendants = cast(includeDescendants),
      includeAncestor = cast(includeAncestor)
    ))
}

newCandidateCodesClass <- function(x){

  x <- validateCandidateCodes(x)
  x <- addClass(x, "candidate_codes")

  return(x)
}

addClass <- function(x, value) {
  if (any(value %in% class(x))) x <- removeClass(x, value)
  base::class(x) <- c(value, base::class(x))

  return(x)
}

removeClass <- function(x, value) {
  if(value == "candidate_codes" | value == "candidateCodes"){
    attr(x, "search_strategy") <- NULL
  }
  base::class(x) <- base::class(x)[!(base::class(x) %in% value)]
  if(value == "candidate_codes"){
    attr(x, "search_strategy") <- NULL
  }
  return(x)
}

validateCandidateCodes <- function(x, call = parent.frame()) {
  # Check column presence
  x <- x |>
    omopgenerics::assertTable(
      class = c("tbl", "data.frame"),
      allowExtraColumns = FALSE,
      null = FALSE,
      columns = c("concept_id", "found_from", "concept_name", "domain_id",
                  "vocabulary_id", 'standard_concept')
    )

  # Check column types
  required_types <- list(
    "concept_id"       = "integer",
    "found_from"       = "character",
    "concept_name"     = "character",
    "domain_id"        = "character",
    "vocabulary_id"    = "character",
    "standard_concept" = "character"
  )

  # Check column types and warn
  for(i in names(required_types)) {
    actual <- class(x[[i]])
    expected <- required_types[[i]]
    if (actual != expected) {
      cli::cli_warn("Column {.val {colnames(x)[i]}} is {.val {actual}} but should be {.val {expected}}. It will be coerced.")
    }
  }

  x <- x |>
    dplyr::mutate(
      "concept_id" = as.integer(.data$concept_id),
      "found_from" = as.character(.data$found_from),
      "concept_name" = as.character(.data$concept_name),
      "domain_id"  = as.character(.data$domain_id),
      "vocabulary_id" = as.character(.data$vocabulary_id),
      "standard_concept" = as.character(.data$standard_concept)
    )

  # sort by concept_id and found_from
  found_from_values <- append(
    unique(x$found_from)[stringr::str_ilike(unique(x$found_from), pattern = "from initial search")],
    unique(x$found_from)[!stringr::str_ilike(unique(x$found_from), pattern = "from initial search")]
  )
  x <- x |>
    dplyr::mutate("found_from" = factor(.data$found_from, .env$found_from_values)) |>
    dplyr::arrange(.data$concept_id, .data$found_from) |>
    dplyr::mutate("found_from" = as.character(.data$found_from))

  # Add attribute
  attr(x, "search_strategy") <- dplyr::tibble(
    "vocabulary_version" = character(),
    "keywords" = character(),
    "exclude"  = character(),
    "domains"  = character(),
    "standard_concept" = character(),
    "search_in_synonyms" = logical(),
    "search_non_standard" = logical(),
    "include_descendants" = logical(),
    "include_ancestor" = logical())

  return(x)
}

addAttribute <- function(x, cdm, keywords, exclude, domains, standardConcept,
                        searchInSynonyms, searchNonStandard, includeDescendants,
                        includeAncestor){
  attr(x, "search_strategy") <- dplyr::tibble(
    "cdm_name" = omopgenerics::cdmName(cdm),
    "vocabulary_version" = vocabularyVersion(cdm),
    "keywords" = paste0('"', keywords, '"', collapse = ", "),
    "exclude"  = paste0('"', exclude, '"', collapse = ", "),
    "domains"  = paste0('"', domains, '"', collapse = ", "),
    "standard_concept" = paste0('"', standardConcept, '"', collapse = ", "),
    "search_in_synonyms" = searchInSynonyms,
    "search_non_standard" = searchNonStandard,
    "include_descendants" = includeDescendants,
    "include_ancestor" = includeAncestor) |>
    dplyr::mutate("exclude" = dplyr::na_if(.data$exclude, ""))

  return(x)
}

test_that("test table orphan codes work", {
  skip_on_cran()

  testthat::local_mocked_bindings(
    system_fonts = function(...) data.frame(family = "sans", path = ""),
    .package = "systemfonts"
  )

  cdm <- mockVocabRef("database")

  codes <- getCandidateCodes(
    cdm = cdm,
    keywords = "Musculoskeletal disorder",
    domains = "Condition",
    includeDescendants = FALSE
  )
  expect_warning(
    # some cdm tables not present
  orphan_codes <- summariseOrphanCodes(x = omopgenerics::newCodelist(list("msk" = codes$concept_id)),
                                       cdm = cdm)
  )

  expect_no_error(tableOrphanCodes(orphan_codes))
  expect_no_error(tableOrphanCodes(orphan_codes, style = "darwin"))

  # empty result
  expect_warning(
    # some cdm tables not present
  result <- summariseOrphanCodes(omopgenerics::newCodelist(list(a = 99999L)),
                                 cdm = cdm)
  )
  expect_warning(
    # empty result
    tableOrphanCodes(result))

  # not an orphan code use result result
  cond <- dplyr::tibble("person_id" = 1,
                        "condition_occurrence_id" = 1,
                        "condition_start_date" = as.Date("2000-01-01"),
                        "condition_end_date" = as.Date("2000-01-01"),
                        "condition_type_concept_id" = 1L,
                        "condition_concept_id" = 4,
                        "condition_source_concept_id" = 1,
                        "condition_source_value" = "a")
  cdm <- omopgenerics::insertTable(cdm,
                                   name = "condition_occurrence",
                                   table = cond)
  result <- summariseCodeUse(omopgenerics::newCodelist(list(knee_oa = 4L, hip_oa = 5L)),
                             cdm = cdm)
  expect_warning(
    # empty result
    tableOrphanCodes(result))

  # With source data
  con <- DBI::dbConnect(duckdb::duckdb(),
                        dbdir = CDMConnector::eunomiaDir())
  cdm <- CDMConnector::cdmFromCon(con,
                                  cdmSchema = "main",
                                  writeSchema = "main")
  cdm <- buildAchillesTables(cdm)

  x <- getCandidateCodes(cdm, keywords = "o")
  x <- omopgenerics::newCodelistWithDetails(list("codes" = x))
  expect_warning(result <- summariseOrphanCodes(x, cdm))
  expect_no_error(tab <- tableOrphanCodes(result))
  expect_true("Standard concept ID" %in% colnames(tab$`_data`))
  expect_true("Standard concept name" %in% colnames(tab$`_data`))
  CDMConnector::cdmDisconnect(cdm)
})

# Testing against different database platforms

test_that("redshift", {
  skip_on_cran()

  testthat::skip_if(Sys.getenv("CDM5_REDSHIFT_DBNAME") == "")
  skip_if_offline()

  db <-  DBI::dbConnect(RPostgres::Redshift(),
                        dbname   = Sys.getenv("CDM5_REDSHIFT_DBNAME"),
                        host     = Sys.getenv("CDM5_REDSHIFT_HOST"),
                        port     = Sys.getenv("CDM5_REDSHIFT_PORT"),
                        user     = Sys.getenv("CDM5_REDSHIFT_USER"),
                        password = Sys.getenv("CDM5_REDSHIFT_PASSWORD"))

  cdm <- CDMConnector::cdmFromCon(con = db,
                                  cdmSchema = Sys.getenv("CDM5_REDSHIFT_CDM_SCHEMA"),
                                  writeSchema = Sys.getenv("CDM5_REDSHIFT_SCRATCH_SCHEMA"),
                                  cdmVersion = "5.3")


  expect_no_error(routeCat <- availableRouteCategories(cdm))
  expect_true(all(routeCat %in%
                    c(doseFormToRoute$route_category, "unclassified_route")))
  # alphabetical order
  expect_identical(routeCat,
                   sort(availableRouteCategories(cdm)))

  cdm$concept <- cdm$concept |>
    dplyr::mutate(concept_id = as.integer64(concept_id)) |>
    dplyr::compute()

  # candidate code search
  expect_no_error(asthma <- getCandidateCodes(cdm,
                                              keywords = c("asthma",
                                                           "irritable airways",
                                                           "lung disease",
                                                           "respiratory abnormalities",
                                                           "sleep apnea",
                                                           "chronic obstructive lung disease",
                                                           "chronic obstructive lung disease"),

                                              domains = c("condition", "observation"),
                                              exclude = c("childhood", "juvenile"),
                                              searchInSynonyms = TRUE,
                                              searchNonStandard = TRUE,
                                              includeDescendants = TRUE,
                                              includeAncestor = TRUE))
  expect_true(nrow(asthma) > 0)

  # drug ingredients
  expect_no_error(metformin <- getDrugIngredientCodes(cdm, "metformin",
                                                      nameStyle = "{concept_name}"))
  expect_no_error(metformin <- getDrugIngredientCodes(cdm, 1503297,
                                                      nameStyle = "{concept_name}"))

  expect_true(inherits(metformin, "codelist"))
  expect_true("metformin" %in% names(metformin))

  expect_no_error(metformin_2 <- getDrugIngredientCodes(cdm, "metformin",
                                                        nameStyle = "{concept_code}"))
  expect_true("6809" %in% names(metformin_2))

  expect_no_error(metformin_3 <- getDrugIngredientCodes(cdm, "metformin",
                                                        nameStyle = "{concept_code}_{concept_name}"))
  expect_true("6809_metformin" %in% names(metformin_3))

  expect_error(getDrugIngredientCodes(cdm, "metformin",
                                      nameStyle = "something else"))


  # achilles
  cdm$achilles_results <- cdm$condition_occurrence |>
    dplyr::group_by(condition_concept_id) |>
    dplyr::tally(name = "count_value") |>
    dplyr::rename("stratum_1" = "condition_concept_id") |>
    dplyr::mutate(stratum_2 = NA,
                  stratum_3 = NA,
                  analysis_id = 401) |>
    dplyr::compute()

  asthma <- list(asthma = c(317009, 257581))
  result_achilles <- summariseAchillesCodeUse(asthma,
                                              cdm = cdm)
  result_cdm <- summariseCodeUse(asthma, cdm = cdm)

  expect_equal(result_achilles |>
                 dplyr::filter(stringr::str_detect(variable_level, "317009"),
                               estimate_name == "record_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(variable_level == 317009,
                               estimate_name == "record_count") |>
                 dplyr::pull("estimate_value"))

  expect_equal(result_achilles |>
                 dplyr::filter(stringr::str_detect(variable_level, "257581"),
                               estimate_name == "record_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(variable_level == 257581,
                               estimate_name == "record_count") |>
                 dplyr::pull("estimate_value"))


  cdm$achilles_results <- cdm$condition_occurrence |>
    dplyr::group_by(person_id, condition_concept_id) |>
    dplyr::tally() |>
    dplyr::ungroup() |>
    dplyr::group_by(condition_concept_id) |>
    dplyr::tally(name = "count_value") |>
    dplyr::rename("stratum_1" = "condition_concept_id") |>
    dplyr::mutate(stratum_2 = NA,
                  stratum_3 = NA,
                  analysis_id = 400) |>
    dplyr::compute()

  asthma <- list(asthma = c(317009, 257581))
  result_achilles <- summariseAchillesCodeUse(asthma,
                                              cdm = cdm)
  result_cdm <- summariseCodeUse(asthma, cdm = cdm)

  expect_equal(result_achilles |>
                 dplyr::filter(stringr::str_detect(variable_level, "317009"),
                               estimate_name == "person_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(variable_level == 317009,
                               estimate_name == "person_count") |>
                 dplyr::pull("estimate_value"))

  expect_equal(result_achilles |>
                 dplyr::filter(stringr::str_detect(variable_level, "257581"),
                               estimate_name == "person_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(variable_level == 257581,
                               estimate_name == "person_count") |>
                 dplyr::pull("estimate_value"))


  # edge cases
  # concept id not in achilles
  expect_message(summariseAchillesCodeUse(list(asthma = 123),
                                          cdm = cdm))

  # expected errors
  expect_error(summariseAchillesCodeUse(123, #not a named list
                                        cdm = cdm))
  expect_error(summariseAchillesCodeUse(asthma,
                                        cdm = "cdm")) # not a cdm
  expect_error(summariseAchillesCodeUse(asthma,
                                        cdm = cdm,
                                        countBy = "not an option"))

  # working concept set example with mock
  x <- codesFromConceptSet(
    cdm = cdm, path =  system.file(package = "CodelistGenerator",
                                   "concepts_dbms")
  )
  expect_true(x$oa_no_desc == 4079750)
  expect_true(!761485 %in% x$oa_no_desc)
  expect_true(761485 %in% x$oa_desc)


  CDMConnector::cdmDisconnect(cdm)
})

test_that("snowflake", {
  skip_on_cran()
  testthat::skip_if(Sys.getenv("SNOWFLAKE_SERVER") == "")
  skip_if_offline()

  con <- DBI::dbConnect(odbc::odbc(),
                        SERVER = Sys.getenv("SNOWFLAKE_SERVER"),
                        UID = Sys.getenv("SNOWFLAKE_USER"),
                        PWD = Sys.getenv("SNOWFLAKE_PASSWORD"),
                        DATABASE = Sys.getenv("SNOWFLAKE_DATABASE"),
                        WAREHOUSE = Sys.getenv("SNOWFLAKE_WAREHOUSE"),
                        DRIVER = Sys.getenv("SNOWFLAKE_DRIVER"))

  cdm_schema <- strsplit(Sys.getenv("SNOWFLAKE_CDM_SCHEMA"), "\\.")[[1]]
  write_schema <- strsplit(Sys.getenv("SNOWFLAKE_SCRATCH_SCHEMA"), "\\.")[[1]]

  cdm <- CDMConnector::cdmFromCon(con = con,
                                  cdmSchema = cdm_schema,
                                  writeSchema = write_schema,
                                  cdmName = "snowflake")


  # candidate code search
  expect_no_error(asthma<-getCandidateCodes(cdm,
                                            keywords = c("asthma",
                                                         "irritable airways",
                                                         "lung disease",
                                                         "respiratory abnormalities",
                                                         "sleep apnea",
                                                         "chronic obstructive lung disease",
                                                         "chronic obstructive lung disease"),

                                            domains = c("condition", "observation"),
                                            exclude = c("childhood", "juvenile"),
                                            searchInSynonyms = TRUE,
                                            searchNonStandard = TRUE,
                                            includeDescendants = TRUE,
                                            includeAncestor = TRUE))
  expect_true(nrow(asthma) > 0)

  # drug ingredients
  expect_no_error(getDrugIngredientCodes(cdm, "metformin"))

  # achilles
  cdm$achilles_results <- cdm$condition_occurrence |>
    dplyr::group_by(condition_concept_id) |>
    dplyr::tally(name = "count_value") |>
    dplyr::rename("stratum_1" = "condition_concept_id") |>
    dplyr::mutate(stratum_2 = NA,
                  stratum_3 = NA,
                  analysis_id = 401) |>
    dplyr::compute()

  asthma <- list(asthma = c(317009, 257581))
  result_achilles <- summariseAchillesCodeUse(asthma,
                                              cdm = cdm)
  result_cdm <- summariseCodeUse(asthma, cdm = cdm)

  expect_equal(result_achilles |>
                 dplyr::filter(variable_level == "317009",
                               variable_name == "record_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(variable_level == "317009",
                               variable_name == "record_count ") |>
                 dplyr::pull("estimate_value"))

  expect_equal(result_achilles |>
                 dplyr::filter(variable_level == "257581",
                               variable_name == "record_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(variable_level == "257581",
                               variable_name == "record_count ") |>
                 dplyr::pull("estimate_value"))

  cdm$achilles_results <- cdm$condition_occurrence |>
    dplyr::group_by(person_id, condition_concept_id) |>
    dplyr::tally() |>
    dplyr::ungroup() |>
    dplyr::group_by(condition_concept_id) |>
    dplyr::tally(name = "count_value") |>
    dplyr::rename("stratum_1" = "condition_concept_id") |>
    dplyr::mutate(stratum_2 = NA,
                  stratum_3 = NA,
                  analysis_id = 400) |>
    dplyr::compute()

  asthma <- list(asthma = c(317009, 257581))
  result_achilles <- summariseAchillesCodeUse(asthma,
                                              cdm = cdm)
  result_cdm <- summariseCodeUse(asthma, cdm = cdm)

  expect_equal(result_achilles |>
                 dplyr::filter(variable_level == "317009",
                               variable_name == "record_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(variable_level == "317009",
                               variable_name == "record_count ") |>
                 dplyr::pull("estimate_value"))



  expect_equal(result_achilles |>
                 dplyr::filter(group_level == "317009",
                               variable_name == "person_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(group_level ==  "317009",
                               variable_name == "person_count") |>
                 dplyr::pull("estimate_value"))
  expect_equal(result_achilles |>
                 dplyr::filter(group_level == "257581",
                               variable_name == "person_count") |>
                 dplyr::pull("estimate_value"),
               result_cdm |>
                 dplyr::filter(group_level ==  "257581",
                               variable_name == "person_count") |>
                 dplyr::pull("estimate_value"))

  # edge cases
  # concept id not in achilles
  expect_message(summariseAchillesCodeUse(list(asthma = 123),
                                          cdm = cdm))

  # expected errors
  expect_error(summariseAchillesCodeUse(123, #not a named list
                                        cdm = cdm))
  expect_error(summariseAchillesCodeUse(asthma,
                                        cdm = "cdm")) # not a cdm
  expect_error(summariseAchillesCodeUse(asthma,
                                        cdm = cdm,
                                        countBy = "not an option"))

  CDMConnector::cdmDisconnect(cdm)
})

test_that("postgres", {
  skip_on_cran()
  testthat::skip_if(Sys.getenv("CDM5_POSTGRESQL_DBNAME") == "")
  skip_if_offline()

  db <- DBI::dbConnect(RPostgres::Postgres(),
                       dbname = Sys.getenv("CDM5_POSTGRESQL_DBNAME"),
                       host = Sys.getenv("CDM5_POSTGRESQL_HOST"),
                       user = Sys.getenv("CDM5_POSTGRESQL_USER"),
                       password = Sys.getenv("CDM5_POSTGRESQL_PASSWORD"))
  cdm <- CDMConnector::cdmFromCon(
    con = db,
    cdmSchema = Sys.getenv("CDM5_POSTGRESQL_CDM_SCHEMA"),
    writeSchema = Sys.getenv("CDM5_POSTGRESQL_SCRATCH_SCHEMA"),
    writePrefix = "incp_",
    achillesSchema = Sys.getenv("CDM5_POSTGRESQL_CDM_SCHEMA")
  )

  # check orphan code use
  expect_no_error(summariseOrphanCodes(list("asthma"=317009L), cdm))

  asthma <- getCandidateCodes(cdm, "asthma", domains = c("condition",
                                                         "observation"))

  expect_identical(subsetOnDomain(list("asthma" = as.integer(asthma$concept_id)),
                                  cdm = cdm,
                                  domain = "condition"),
                   omopgenerics::newCodelist(list("asthma" = sort(as.integer(asthma |>
                                                                               dplyr::filter(domain_id == "Condition") |>
                                                                               dplyr::pull("concept_id"))))))
  expect_identical(subsetOnDomain(list("asthma" = as.integer(asthma$concept_id)),
                                  cdm = cdm,
                                  domain = c("condition", "observation")),
                   omopgenerics::newCodelist(list("asthma" = sort(as.integer(asthma |>
                                                                               dplyr::filter(domain_id %in% c("Condition", "Observation")) |>
                                                                               dplyr::pull("concept_id"))))))

  expect_identical(
    subsetOnDomain(list("asthma" = as.integer(asthma$concept_id)),
                   cdm = cdm,
                   domain = c("procedure")),
    omopgenerics::emptyCodelist())

  asthma_orphan <- summariseOrphanCodes(list(asthma = asthma$concept_id),
                                        cdm, domain = c("condition"))

  # limit orphan codes to a domain
  # we won't have any orphan codes from drug
  asthma_oc <- summariseOrphanCodes(list("asthma"= c(317009)),
                                    cdm,
                                    domain = "drug")
  expect_true(nrow(asthma_oc) == 0)


  codes <- getDrugIngredientCodes(cdm, "metformin")
  codes[["asthma"]] <- 317009

  expect_no_error(stratifyByDoseUnit(codes, cdm))
  expect_no_error(stratifyByDoseUnit(codes, cdm, keepOriginal = TRUE))
  expect_no_error(stratifyByRouteCategory(codes, cdm))
  expect_no_error(stratifyByRouteCategory(codes, cdm, keepOriginal = TRUE))


  drug_codes <- getDrugIngredientCodes(cdm, name = c("metformin",
                                                     "diclofenac"))

  # we can stratify by each concept contained
  drug_codes_by_concept <- stratifyByConcept(drug_codes,
                                             cdm = cdm)

  drug_codes_by_concept_used <- subsetToCodesInUse(drug_codes_by_concept, cdm)

  # if we subset to oral both should still have codes
  expect_true(length(subsetOnRouteCategory(drug_codes, cdm,
                                           routeCategory = "oral")) == 2)
  # only metformin has injectable route
  expect_true(length(subsetOnRouteCategory(drug_codes, cdm,
                                           routeCategory = "injectable")) == 1)
  # we can put multiple route categories
  # and we should get the same result if we subset up front or later
  drug_codes2 <- getDrugIngredientCodes(cdm,
                                        routeCategory = c("injectable",
                                                          "oral"),
                                        name = c("metformin","diclofenac"))
  drug_codes_subset <-  getDrugIngredientCodes(cdm,
                                               routeCategory = c("injectable",
                                                                 "oral"),
                                               name = c("metformin","diclofenac"))


  expect_true(length(drug_codes_subset) == 2)
  expect_identical(drug_codes_subset, drug_codes2)

  # can stratify by route
  expect_no_error(drug_codes_stratified_by_route <-  getDrugIngredientCodes(cdm,
                                                                            type = "codelist_with_details",
                                                                            name = c("metformin","diclofenac")) |>
                    stratifyByRouteCategory(cdm = cdm))
  expect_true(inherits(drug_codes_stratified_by_route, "codelist_with_details"))



  # can subset and stratify by dose unit
  expect_no_error(availableDoseUnits(cdm))
  drugs <- getDrugIngredientCodes(cdm,
                                  name = c("metformin","diclofenac"))
  expect_no_error(subsetOnDoseUnit(drugs, cdm, c("milligram")))

  expect_no_error(drug_codes_stratified_by_unit <-  getDrugIngredientCodes(cdm,
                                                                           name = c("metformin","diclofenac")) |>
                    stratifyByDoseUnit(cdm = cdm))

  # we can also stratify by both route and unit
  expect_no_error(drug_codes_stratified_by_route_and_unit <-  getDrugIngredientCodes(cdm,
                                                                                     name = c("metformin","diclofenac")) |>
                    stratifyByRouteCategory(cdm = cdm) |>
                    stratifyByDoseUnit(cdm = cdm))

  drugs_milligram_transdermal <-  getDrugIngredientCodes(cdm,
                                                         name = c("metformin","diclofenac"),
                                                         doseUnit = "milligram",
                                                         routeCategory = "transdermal")

  drugs_milligram_transdermal_2 <- getDrugIngredientCodes(cdm,
                                                          name = c("metformin","diclofenac")) |>
    subsetOnRouteCategory(cdm, "transdermal") |>
    subsetOnDoseUnit(cdm, "milligram")
  expect_identical(drugs_milligram_transdermal,
                   drugs_milligram_transdermal_2)


  atc <- getATCCodes(cdm, name = "alimentary tract and metabolism")
  expect_no_error(atc |>
                    stratifyByRouteCategory(cdm = cdm) |>
                    stratifyByDoseUnit(cdm = cdm))

  # make sure no extra domains added to the results
  codes <- getCandidateCodes(
    cdm = cdm,
    keywords = c("at") ,
    domains = c("Condition", "Observation"),
    standardConcept = "Standard",
    searchInSynonyms = FALSE,
    searchNonStandard = FALSE,
    includeDescendants = TRUE,
    includeAncestor = FALSE
  )
  expect_true(length(unique(codes$domain_id)) <= 2)

  CDMConnector::cdmDisconnect(cdm)
})

test_that("sql server", {
  skip_on_cran()
  testthat::skip_if(Sys.getenv("CDM5_SQL_SERVER_SERVER") == "")
  testthat::skip_if(Sys.getenv("SQL_SERVER_DRIVER") == "")
  skip_if_offline()

  db <- DBI::dbConnect(odbc::odbc(),
                       Driver   = Sys.getenv("SQL_SERVER_DRIVER"),
                       Server   = Sys.getenv("CDM5_SQL_SERVER_SERVER"),
                       Database = Sys.getenv("CDM5_SQL_SERVER_CDM_DATABASE"),
                       UID      = Sys.getenv("CDM5_SQL_SERVER_USER"),
                       PWD      = Sys.getenv("CDM5_SQL_SERVER_PASSWORD"),
                       TrustServerCertificate="yes",
                       Port     = Sys.getenv("CDM5_SQL_SERVER_PORT"))
  cdm <- CDMConnector::cdmFromCon(db,
                                  cdmSchema = c("CDMV54", "dbo"),
                                  achillesSchema = c("CDMV54", "dbo"),
                                  writeSchema = c("ohdsi", "dbo"))

  # check orphan code use
  expect_no_error(summariseOrphanCodes(list("asthma"=317009L) |> omopgenerics::newCodelist(),
                                       cdm))

  asthma <- getCandidateCodes(cdm, "asthma", domains = c("condition",
                                                         "observation"))

  expect_identical(subsetOnDomain(list("asthma" = as.integer(asthma$concept_id)) |>
                                    omopgenerics::newCodelist(),
                                  cdm = cdm,
                                  domain = "condition"),
                   omopgenerics::newCodelist(list("asthma" = sort(as.integer(asthma |>
                                                                               dplyr::filter(domain_id == "Condition") |>
                                                                               dplyr::pull("concept_id"))))))
  expect_identical(subsetOnDomain(list("asthma" = as.integer(asthma$concept_id)) |>
                                    omopgenerics::newCodelist(),
                                  cdm = cdm,
                                  domain = c("condition", "observation")),
                   omopgenerics::newCodelist(list("asthma" = sort(as.integer(asthma |>
                                                                               dplyr::filter(domain_id %in% c("Condition", "Observation")) |>
                                                                               dplyr::pull("concept_id"))))))

  expect_identical(
    subsetOnDomain(list("asthma" = as.integer(asthma$concept_id)) |>
                     omopgenerics::newCodelist(),
                   cdm = cdm,
                   domain = c("procedure")),
    omopgenerics::emptyCodelist())

  asthma_orphan <- summariseOrphanCodes(list(asthma = asthma$concept_id) |>
                                          omopgenerics::newCodelist(),
                                        cdm, domain = c("condition"))

  # limit orphan codes to a domain
  # we won't have any orphan codes from drug
  asthma_oc <- summariseOrphanCodes(list("asthma"= c(317009)) |>
                                      omopgenerics::newCodelist(),
                                    cdm,
                                    domain = "drug")
  expect_true(nrow(asthma_oc) == 0)


  codes <- getDrugIngredientCodes(cdm, "metformin")
  codes[["asthma"]] <- 317009

  expect_no_error(stratifyByDoseUnit(codes, cdm))
  expect_no_error(stratifyByDoseUnit(codes, cdm, keepOriginal = TRUE))
  expect_no_error(stratifyByRouteCategory(codes, cdm))
  expect_no_error(stratifyByRouteCategory(codes, cdm, keepOriginal = TRUE))


  drug_codes <- getDrugIngredientCodes(cdm, name = c("metformin",
                                                     "diclofenac"))

  # we can stratify by each concept contained
  drug_codes_by_concept <- stratifyByConcept(drug_codes,
                                             cdm = cdm)

  drug_codes_by_concept_used <- subsetToCodesInUse(drug_codes_by_concept, cdm)

  # if we subset to oral both should still have codes
  expect_true(length(subsetOnRouteCategory(drug_codes, cdm,
                                           routeCategory = "oral")) == 2)
  # only metformin has injectable route
  expect_true(length(subsetOnRouteCategory(drug_codes, cdm,
                                           routeCategory = "injectable")) == 1)
  # we can put multiple route categories
  # and we should get the same result if we subset up front or later
  drug_codes2 <- getDrugIngredientCodes(cdm,
                                        routeCategory = c("injectable",
                                                          "oral"),
                                        name = c("metformin","diclofenac"))
  drug_codes_subset <-  getDrugIngredientCodes(cdm,
                                               routeCategory = c("injectable",
                                                                 "oral"),
                                               name = c("metformin","diclofenac"))


  expect_true(length(drug_codes_subset) == 2)
  expect_identical(drug_codes_subset, drug_codes2)

  # can stratify by route
  expect_no_error(drug_codes_stratified_by_route <-  getDrugIngredientCodes(cdm,
                                                                            type = "codelist_with_details",
                                                                            name = c("metformin","diclofenac")) |>
                    stratifyByRouteCategory(cdm = cdm))
  expect_true(inherits(drug_codes_stratified_by_route, "codelist_with_details"))



  # can subset and stratify by dose unit
  expect_no_error(availableDoseUnits(cdm))
  drugs <- getDrugIngredientCodes(cdm,
                                  name = c("metformin","diclofenac"))
  expect_no_error(subsetOnDoseUnit(drugs, cdm, c("milligram")))

  expect_no_error(drug_codes_stratified_by_unit <-  getDrugIngredientCodes(cdm,
                                                                           name = c("metformin","diclofenac")) |>
                    stratifyByDoseUnit(cdm = cdm))

  # we can also stratify by both route and unit
  expect_no_error(drug_codes_stratified_by_route_and_unit <-  getDrugIngredientCodes(cdm,
                                                                                     name = c("metformin","diclofenac")) |>
                    stratifyByRouteCategory(cdm = cdm) |>
                    stratifyByDoseUnit(cdm = cdm))

  drugs_milligram_transdermal <-  getDrugIngredientCodes(cdm,
                                                         name = c("metformin","diclofenac"),
                                                         doseUnit = "milligram",
                                                         routeCategory = "transdermal")

  drugs_milligram_transdermal_2 <- getDrugIngredientCodes(cdm,
                                                          name = c("metformin","diclofenac")) |>
    subsetOnRouteCategory(cdm, "transdermal") |>
    subsetOnDoseUnit(cdm, "milligram")
  expect_identical(drugs_milligram_transdermal,
                   drugs_milligram_transdermal_2)


  atc <- getATCCodes(cdm, name = "alimentary tract and metabolism")
  expect_no_error(atc |>
                    stratifyByRouteCategory(cdm = cdm) |>
                    stratifyByDoseUnit(cdm = cdm))

  # make sure no extra domains added to the results
  codes <- getCandidateCodes(
    cdm = cdm,
    keywords = c("at") ,
    domains = c("Condition", "Observation"),
    standardConcept = "Standard",
    searchInSynonyms = FALSE,
    searchNonStandard = FALSE,
    includeDescendants = TRUE,
    includeAncestor = FALSE
  )
  expect_true(length(unique(codes$domain_id)) <= 2)

  CDMConnector::cdmDisconnect(cdm)
})

test_that("stratify by route works", {
  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backends[[i]])
    ing_codes <- getDrugIngredientCodes(cdm)

    ing_codes_str <- stratifyByRouteCategory(ing_codes, cdm)

     # all will are unclassified
    expect_true( all(stringr::str_detect(names(ing_codes_str),
                                        "injectable")))

    # can also keep the original
    ing_codes_str_all <- stratifyByRouteCategory(ing_codes, cdm,
                                                 keepOriginal = TRUE)
    expect_true(length(ing_codes_str_all) == 4)


    # if concepts are not from the drug domain we should an unclassified codelist
    oa <- getCandidateCodes(cdm = cdm, "osteoarthritis")
    oa_str <- stratifyByRouteCategory(omopgenerics::newCodelist(list(oa = oa$concept_id)),
                                      cdm, keepOriginal = FALSE)
    expect_true(names(oa_str)=="oa_unclassified_route_category")

    oa_str <- stratifyByRouteCategory(omopgenerics::newCodelistWithDetails(list(oa = oa)),
                                      cdm, keepOriginal = FALSE)
    expect_true(names(oa_str)=="oa_unclassified_route_category")
    expect_true(inherits(oa_str, "codelist_with_details"))

    # expected errors
    expect_error(stratifyByRouteCategory(list("a" = 1L),  cdm))
    expect_error(stratifyByRouteCategory(ing_codes,  "a"))
    expect_error(stratifyByRouteCategory(ing_codes,  cdm, "a"))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

test_that("searchStrategy works", {
  cdm <- mockVocabRef()
  x <- getCandidateCodes(cdm, keywords = "a")
  expect_no_error(ss <- searchStrategy(x))
  expect_equal(attr(x, "search_strategy"), ss)

  # expect_identical(
  #   searchStrategy(x),
  #   dplyr::tibble(
  #     "vocabulary_version" = "v5.0 22-JUN-22",
  #     "keywords" = "Arthritis, Knee",
  #     "exclude"  = "Childhood asthma",
  #     "domains"  = "Condition, Observation",
  #     "standard_concept" = "Standard",
  #     "search_in_synonyms" = TRUE,
  #     "search_non_standard" = TRUE,
  #     "include_descendants" = TRUE,
  #     "include_ancestor" = TRUE
  #   )
  # )
  expect_identical(
    searchStrategy(x),
    attr(x, "search_strategy")
  )

  # Not a candidate_codes
  expect_error(searchStrategy(list("a" = 1L)))
  expect_error(ss <- searchStrategy(list("a")))
})

test_that("tests with mock db", {
  skip_on_cran()
  # mock db
  cdm <- mockVocabRef("database")

  codes <- getCandidateCodes(
    cdm = cdm,
    keywords = "Musculoskeletal disorder",
    domains = "Condition",
    includeDescendants = FALSE
  )

  expect_warning( # not all tables in cdm
  orphan_codes <- summariseOrphanCodes(
    x = omopgenerics::newCodelist(list("msk" = codes$concept_id)),
    cdm = cdm)
  )

  # we should pick up knee osteoarthritis from our achilles tables
  expect_true(all(stringr::str_detect(orphan_codes |>
                                        dplyr::pull("variable_level"), c("4", "5", "4", "5"))))
  expect_equal(orphan_codes |>
                 dplyr::pull("estimate_value"),
               c("400", "200", "400", "200"))
  settings <- omopgenerics::settings(orphan_codes)
  expect_true(all(settings$result_type == "orphan_code_use"))

  #expected error
  expect_error(summariseOrphanCodes(x = "a", cdm = cdm))
  expect_error(summariseOrphanCodes(x = omopgenerics::newCodelist(list("msk" = codes$concept_id)),
                                    cdm = "a"))

  # requires achilles
  cdm$achilles_results <- NULL
  expect_error(summariseOrphanCodes(x = list("msk" = codes$concept_id),
                                    cdm = cdm))

  CDMConnector::cdmDisconnect(cdm)

  cdm <- omock::mockCdmReference()
  expect_error(summariseOrphanCodes(x = list("a" = 1), cdm))
})

test_that("test with synthea on sql server", {
  skip_if(Sys.getenv("darwinDbDatabaseServer") == "")

  db <-DBI::dbConnect(odbc::odbc(),
                      Driver   = Sys.getenv("SQL_SERVER_DRIVER"),
                      # Driver   = "ODBC Driver 17 for SQL Server", #asdf
                      Server   = Sys.getenv("CDM5_SQL_SERVER_SERVER"),
                      Database = Sys.getenv("CDM5_SQL_SERVER_CDM_DATABASE"),
                      UID      = Sys.getenv("CDM5_SQL_SERVER_USER"),
                      PWD      = Sys.getenv("CDM5_SQL_SERVER_PASSWORD"),
                      TrustServerCertificate="yes",
                      Port     = Sys.getenv("CDM5_SQL_SERVER_PORT"))

  cdm <- CDMConnector::cdmFromCon(
    con = db,
    cdmSchema = strsplit(Sys.getenv("CDM5_SQL_SERVER_CDM_SCHEMA"), "\\.")[[1]],
    writeSchema =  strsplit(Sys.getenv("CDM5_SQL_SERVER_SCRATCH_SCHEMA"), "\\.")[[1]])

  vocabVersion <- vocabularyVersion(cdm = cdm)
  expect_true(length(vocabVersion) == 1)
  expect_true(is.character(vocabVersion))

  # search in database
  asthma <- getCandidateCodes(
    cdm = cdm,
    keywords = "asthma",
    domains = "Condition"
  )
  expect_true(all(c(
    "concept_id", "concept_name",
    "domain_id", "vocabulary_id"
  ) %in%
    names(asthma)))
  expect_true(nrow(asthma) > 0)

  asthmaIcdMappings <- getMappings(
    cdm = cdm,
    candidateCodelist = asthma,
    nonStandardVocabularies = "ICD10CM"
  )
  expect_true(all(c(
    "standard_concept_id",
    "standard_concept_name",
    "standard_vocabulary_id",
    "non_standard_concept_id",
    "non_standard_concept_name",
    "non_standard_concept_code",
    "non_standard_vocabulary_id"
  ) %in%
    names(asthmaIcdMappings)))
  expect_true(nrow(asthmaIcdMappings) > 0)

  dbDisconnect(db)
})

test_that("tests with mock", {
  skip_on_cran()
  backends <- c("database")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])

    # tests
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "Musculoskeletal disorder",
      domains = "Condition",
      includeDescendants = TRUE
    )
    mappings <- getMappings(
      cdm = cdm,
      candidateCodelist = codes,
      nonStandardVocabularies = "READ"
    )
    expect_true(
      any(mappings$standard_concept_name %in% "Osteoarthrosis")
    )
    expect_true(
      any(mappings$non_standard_concept_name %in% "Degenerative arthropathy")
    )
    expect_true(
      any(mappings$standard_concept_name %in% "Osteoarthritis of knee")
    )
    expect_true(
      any(mappings$non_standard_concept_name %in% "Knee osteoarthritis")
    )

    expect_true(all(c(
      "standard_concept_id",
      "standard_concept_name",
      "standard_vocabulary_id",
      "non_standard_concept_id",
      "non_standard_concept_name",
      "non_standard_concept_code",
      "non_standard_vocabulary_id"
    ) %in%
      names(mappings)))

    # expect error if not a cdm reference
    expect_error(getMappings(
      cdm = "Not a cdm",
      candidateCodelist = codes,
      nonStandardVocabularies = "READ"
    ))

    # expect error if nonStandardVocabularies does not exist
    # expect works
    mappings <- getMappings(
      cdm = cdm,
      candidateCodelist = codes,
      nonStandardVocabularies = "READ"
    )
    # expect error
    expect_error(getMappings(
      cdm = cdm,
      candidateCodelist = codes,
      nonStandardVocabularies = "READX"
    ))
    expect_error(getMappings(
      cdm = cdm,
      candidateCodelist = codes,
      nonStandardVocabularies = c("Read", "READX")
    ))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

test_that("asCodelistWithDetails", {
  skip_on_cran()
  backends <- c("database", "data_frame")
  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backends[[i]])

    c <- omopgenerics::newCodelist(list("codes" = c(12L,13L),
                                        "codes_2" = c(12L)))

    # codelist to codelist with details
    expect_no_error(c_1 <- c |>
                      asCodelistWithDetails(cdm))
    expect_identical(names(c), names(c_1))
    expect_equal(c(12L, 13L),
      sort(c_1[[1]] |> dplyr::pull("concept_id")))
    expect_equal(c(12L),
                 sort(c_1[[2]] |> dplyr::pull("concept_id")))

    # candidate_codes to codelist_with_details
    a <- getCandidateCodes(cdm, keywords = "osteo")
    c <- asCodelistWithDetails(a)
    expect_true(inherits(c, "codelist_with_details"))
    expect_true(!"candidate_codes" %in% class(c))

    expect_no_error(omopgenerics::emptyCodelist() |>
      asCodelistWithDetails(cdm))
    expect_no_error(omopgenerics::emptyCodelistWithDetails() |>
                      asCodelistWithDetails())

  }
})

test_that("comparing two codelists", {
  skip_on_cran()
  backends <- c("database")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])

    # tests
    codes1 <- getCandidateCodes(
      cdm = cdm,
      keywords = "Arthritis",
      domains = "Condition",
      includeDescendants = TRUE
    )

    codes2 <- getCandidateCodes(
      cdm = cdm,
      keywords = c("knee osteoarthritis", "arthrosis"),
      domains = "Condition",
      includeDescendants = TRUE
    )

    # candidate_class codelist
    # codesCompared <- compareCodelists(
    #   codelist1 = codes1,
    #   codelist2 = codes2
    # )

    # compare codelist with details
    codesCompared2 <- compareCodelists(
      codelist1 = omopgenerics::newCodelistWithDetails(list("codelist1" = codes1)),
      codelist2 = omopgenerics::newCodelistWithDetails(list("codelist2" = codes2))
    )

    #expect_identical(codesCompared, codesCompared2)

    expect_true(all(c(
      "concept_id",
      "concept_name",
      "codelist"
    ) %in%
      names(codesCompared2)))

    expect_true(codesCompared2 |>
      dplyr::filter(concept_id == 3) |>
      dplyr::select(codelist) |>
      dplyr::pull() == "Only in codelist codelist1")

    expect_true(codesCompared2 |>
      dplyr::filter(concept_id == 5) |>
      dplyr::select(codelist) |>
      dplyr::pull() == "Only in codelist codelist1")

    expect_true(codesCompared2 |>
      dplyr::filter(concept_id == 4) |>
      dplyr::select(codelist) |>
      dplyr::pull() == "Both")

    expect_true(codesCompared2 |>
      dplyr::filter(concept_id == 2) |>
      dplyr::select(codelist) |>
      dplyr::pull() == "Only in codelist codelist2")

    # expected errors
    expect_error(compareCodelists(
      codelist1 = codes1,
      codelist2 = "a"
    ))
    expect_error(compareCodelists(
      codelist1 = "a",
      codelist2 = codes2
    ))

    # compare codelist
    codesCompared <- compareCodelists(
      omopgenerics::newCodelist(list("a" = c(1L, 2L), "c" = c(4L))),
      omopgenerics::newCodelist(list("b" = c(2L, 3L), "d" = c(4L)))
    )

    expect_true(inherits(codesCompared, "list"))
    expect_equal(names(codesCompared), c("a_b", "a_d", "c_b", "c_d"))
    expect_true(codesCompared$a_b |>
                  dplyr::filter(concept_id == 1) |>
                  dplyr::select("codelist") |>
                  dplyr::pull() == "Only in codelist a")

    expect_true(codesCompared$c_d |>
                  dplyr::select("codelist") |>
                  dplyr::pull() == "Both")

    expect_true(codesCompared$a_b |>
                  dplyr::filter(concept_id == 3) |>
                  dplyr::select("codelist") |>
                  dplyr::pull() == "Only in codelist b")

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

test_that("comparing two codelists- same codes found different ways", {
  backends <- c("database")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])

    # tests
    codes1 <- getCandidateCodes(
      cdm = cdm,
      keywords = "Arthritis",
      domains = "Condition",
      includeDescendants = TRUE
    )

    codes2 <- getCandidateCodes(
      cdm = cdm,
      keywords = c("arthrosis"),
      searchInSynonyms = TRUE,
      domains = "Condition",
      includeDescendants = TRUE
    )

    codesCompared <- compareCodelists(
      codelist1 = omopgenerics::newCodelistWithDetails(list("codes1" = codes1)),
      codelist2 = omopgenerics::newCodelistWithDetails(list("codes2" = codes2))
    )

    # tests
    expect_true(inherits(codesCompared, "tbl"))
    expect_true(nrow(codesCompared) == 4)

    expect_true(codesCompared |>
      dplyr::filter(concept_id == 3) |>
      dplyr::select(codelist) |>
      dplyr::pull() == "Both")

    expect_true(codesCompared |>
      dplyr::filter(concept_id == 4) |>
      dplyr::select(codelist) |>
      dplyr::pull() == "Both")

    expect_true(codesCompared |>
      dplyr::filter(concept_id == 5) |>
      dplyr::select(codelist) |>
      dplyr::pull() == "Both")

    expect_true(codesCompared |>
      dplyr::filter(concept_id == 2) |>
      dplyr::select(codelist) |>
      dplyr::pull() == "Only in codelist codes2")

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

test_that("stratifyByDoseUnit in mock", {
  cdm <- mockVocabRef()
  ing <- getDrugIngredientCodes(cdm = cdm)

  # no dose units in the mock
  expect_no_error(x <- stratifyByDoseUnit(x = ing, cdm = cdm))
  expect_identical(names(x),
                   c("1234_adalimumab_unclassified_dose_unit",
                     "1234_other_ingredient_unclassified_dose_unit"))

  # if concepts are not from the drug domain we should get only unclassified
  oa <- getCandidateCodes(cdm = cdm, "osteoarthritis")
  oa_str <- stratifyByDoseUnit(omopgenerics::newCodelist(list(oa = oa$concept_id)),
                               cdm, keepOriginal = FALSE)
  expect_true(names(oa_str)=="oa_unclassified_dose_unit")
  expect_true("codelist" %in% class(oa_str))

  oa_str <- stratifyByDoseUnit(omopgenerics::newCodelistWithDetails(list(oa = oa)),
                               cdm, keepOriginal = FALSE)
  expect_true(names(oa_str)=="oa_unclassified_dose_unit")
  expect_true(inherits(oa_str,"codelist_with_details"))

  # expected errors
  expect_error(stratifyByDoseUnit(x = ing, cdm = "a"))
  expect_error(stratifyByDoseUnit(x = "a", cdm = cdm))
  expect_error(stratifyByDoseUnit(x = list("a" = 1L), cdm = cdm))
  expect_error(stratifyByDoseUnit(x = ing, cdm = cdm, keepOriginal = "a"))

})

test_that("achilles code use", {
  skip_on_cran()
  # mock db
  cdm <- mockVocabRef("database")

  expect_message(summariseAchillesCodeUse(x = omopgenerics::emptyCodelist(), cdm))

  oa <- getCandidateCodes(cdm = cdm, keywords = "osteoarthritis")
  # two codes: "Osteoarthritis of knee" "Osteoarthritis of hip"
  result_achilles <- summariseAchillesCodeUse(omopgenerics::newCodelist(list(oa = oa$concept_id)),
                                              cdm = cdm)
  expect_true(result_achilles |>
                dplyr::filter(estimate_name == "record_count") |>
                dplyr::filter(stringr::str_detect(variable_level, "4")) |>
                dplyr::pull("estimate_value") == "400")
  expect_true(result_achilles |>
                dplyr::filter(estimate_name == "record_count") |>
                dplyr::filter(stringr::str_detect(variable_level, "5")) |>
                dplyr::pull("estimate_value") == "200")
  expect_true(nrow(result_achilles) == 4)
  expect_equal(c(rep("oa",4)),
               result_achilles |>
                 dplyr::pull("group_level"))

  # check is a summarised result
  expect_true("summarised_result" %in%  class(result_achilles))

  # applying min cell count where estimate should be obscured
  result_achilles <- summariseAchillesCodeUse(omopgenerics::newCodelist(list(oa = oa$concept_id)),
                                              cdm = cdm)
  expect_true(all(result_achilles |>
                    omopgenerics::suppress(minCellCount = 500) |>
                    dplyr::pull("estimate_value") == "-"
  ))

  expect_equal(unique(result_achilles$additional_name), "vocabulary_id")
  expect_equal(unique(result_achilles$additional_level), "SNOMED")

  # edge cases
  # concept id not in achilles
  expect_message(result_achilles <- summariseAchillesCodeUse(omopgenerics::newCodelist(list(asthma = 123L)),
                                                             cdm = cdm))
  expect_true(nrow(result_achilles) == 0)
  # expect_true("summarised_result" %in%  class(result_achilles))

  # expected errors
  expect_error(summariseAchillesCodeUse(123, #not list
                                        cdm = cdm))
  expect_error(summariseAchillesCodeUse(list(123), #not named list
                                        cdm = cdm))
  expect_error(summariseAchillesCodeUse(asthma,
                                        cdm = "cdm")) # not a cdm
  expect_error(summariseAchillesCodeUse(asthma,
                                        cdm = cdm,
                                        countBy = "not an option"))

  CDMConnector::cdmDisconnect(cdm)

  cdm <- omock::mockCdmReference()
  expect_error(summariseAchillesCodeUse(x = list("a" = 1), cdm))
})

test_that("achilles code use: multipe codelists", {

  # mock db
  cdm <- mockVocabRef("database")

  # two codelists: "Osteoarthritis of knee" "Osteoarthritis of hip"
  result_achilles <- summariseAchillesCodeUse(omopgenerics::newCodelist(
    list(knee_oa = 4L,
         hip_oa = 5L)),
    cdm = cdm)

  expect_true(result_achilles |>
                dplyr::filter(group_level == "knee_oa") |>
                dplyr::filter(estimate_name == "record_count") |>
                dplyr::pull("estimate_value") == "400")
  expect_true(result_achilles |>
                dplyr::filter(group_level == "hip_oa") |>
                dplyr::filter(estimate_name == "person_count") |>
                dplyr::pull("estimate_value") == "200")
  expect_true(result_achilles |>
                dplyr::filter(stringr::str_detect(variable_level, "4")) |>
                dplyr::filter(estimate_name == "person_count") |>
                dplyr::pull("estimate_value") == "400")
  expect_true(result_achilles |>
                dplyr::filter(stringr::str_detect(variable_level, "5")) |>
                dplyr::filter(estimate_name == "record_count") |>
                dplyr::pull("estimate_value") == "200")
  expect_true(nrow(result_achilles) == 4)
  expect_equal(rep(c("hip_oa","knee_oa"),2),
               result_achilles |>
                 dplyr::pull("group_level"))

  CDMConnector::cdmDisconnect(cdm)
})

test_that("tests with mock db", {
  backends <- c("database",
                "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])

    # check tidyWords
    expect_message(tidyWords(c("café")))
    expect_message(x <- tidyWords(c("/hip")))
    expect_identical(x, "hip")
    expect_message(x <- tidyWords(c("/hép/")))
    expect_identical(x, "hp")
    expect_message(x <- tidyWords(c("?")))
    expect_identical(x,as.character())
    expect_message(x <- tidyWords(c("?", "hello")))
    expect_identical(x,"hello")

    # tests
    # test keywords search - exact
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "Musculoskeletal disorder",
      domains = "Condition",
      includeDescendants = FALSE
    )
    expect_true((nrow(codes) == 1 &
                   codes$concept_name[1] == "Musculoskeletal disorder"))
    # variable names
    expect_true(all(c(
      "concept_id", "concept_name",
      "domain_id", "vocabulary_id"
    ) %in%
      names(codes)))

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c(
        "knee osteoarthritis",
        "hip osteoarthritis"
      ),
      domains = "Condition",
      includeDescendants = FALSE
    )
    expect_true(nrow(codes) == 2)
    expect_true("Osteoarthritis of knee" %in%
                  (codes |>
                     dplyr::pull("concept_name")))
    expect_true("Osteoarthritis of hip" %in%
                  (codes |>
                     dplyr::pull("concept_name")))

    # exact matches
    cdm$concept <- cdm$concept |>
      dplyr::union_all(
        cdm$concept |>
          dplyr::filter(concept_name == "Knee osteoarthritis") |>
          dplyr::mutate("standard_concept" = "S",
                        "concept_id" = 27L)
      )
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "knee osteo",
      domains = "Condition",
      includeDescendants = FALSE
    )
    codes1 <- getCandidateCodes(
      cdm = cdm,
      keywords = "/knee osteo/",
      domains = "Condition",
      includeDescendants = FALSE
    )
    codes2 <- getCandidateCodes(
      cdm = cdm,
      keywords = "/\bknee osteo/\b",
      domains = "Condition",
      includeDescendants = FALSE
    )
    codes3 <- getCandidateCodes(
      cdm = cdm,
      keywords = "/\bknee osteoarthritis/\b",
      domains = "Condition",
      includeDescendants = FALSE
    )
    codes4 <- getCandidateCodes(
      cdm = cdm,
      keywords = "knee osteoarthritis",
      domains = "Condition",
      includeDescendants = FALSE
    )
    expect_identical(codes$concept_id, c(4L, 27L))
    expect_identical(codes1$concept_id, c(27L))
    expect_identical(codes2$concept_id, integer(0))
    expect_identical(codes3$concept_id, c(27L))
    expect_identical(codes4$concept_id, codes$concept_id)

    # test include descendants
    cdm <- mockVocabRef()
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "Musculoskeletal disorder",
      domains = "Condition",
      includeDescendants = TRUE
    )
    expect_true((nrow(codes) == 5 &
                   all(codes$concept_id %in% c(1:5)) &
                   all(!codes$concept_id %in% c(6, 7))))

    # test include ancestor
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("Arthritis"),
      domains = "Condition",
      includeAncestor = TRUE
    )
    expect_true(any(codes$concept_name %in% "Musculoskeletal disorder"))

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("Osteoarthritis of knee"),
      domains = "Condition",
      includeAncestor = TRUE
    )
    # nb includeAncestor should only include one level above
    expect_true(!any(codes$concept_name %in% "Musculoskeletal disorder"))
    expect_true(any(codes$concept_name %in% "Arthritis"))

    # test standardConcept
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "Arthritis",
      domains = "Condition",
      standardConcept = c("Standard", "Non-standard"),
      includeDescendants = TRUE
    )
    expect_true((nrow(codes) == 5 &
                   all(codes$concept_id %in% c(3, 4, 5, 8, 17)) &
                   all(!codes$concept_id %in% c(1, 2, 7))))

    # test searchInSynonyms
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "osteoarthrosis",
      domains = "Condition",
      searchInSynonyms = TRUE
    )
    expect_true(any(codes$concept_name %in% "Arthritis"))

    # test exclusion
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = "Osteoarthritis of hip",
      domains = "Condition"
    )
    expect_true(any(!codes$concept_name %in% "Osteoarthritis of hip"))

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = c("Osteoarthritis of hip", "something else", "shoulder"),
      domains = "Condition"
    )
    expect_true(all(codes$concept_name != "Osteoarthritis of hip"))

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = c("something else", "shoulder", "Osteoarthritis of hip"),
      domains = "Condition"
    )
    expect_true(all(codes$concept_name != "Osteoarthritis of hip"))

    # test exclusion + partial match
    codes1 <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = NULL,
      domains = "Condition",
      standardConcept = c("Standard", "Non-standard")
    )

    expect_message(codes2 <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = c("hip of", "/Knee Osteoarthritis"),
      domains = "Condition",
      standardConcept = c("Standard", "Non-standard")
    ))
    expect_equal(
      {attr(codes1, "search_strategy") <- NULL; codes1 |> dplyr::filter(.data$concept_id %in% c(3L, 17L))},
      {attr(codes2, "search_strategy") <- NULL; codes2}
    )

    codes3 <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = c("hip of", "/Knee Osteoarthritis/"),
      domains = "Condition",
      standardConcept = c("Standard", "Non-standard")
    )

    codes4 <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = c("hip of", "/ee Osteoarthritis/"),
      domains = "Condition",
      standardConcept = c("Standard", "Non-standard")
    )
    expect_equal(
      {attr(codes1, "search_strategy") <- NULL; codes1 |> dplyr::filter(!.data$concept_id %in% c(5L, 8L))},
      {attr(codes3, "search_strategy") <- NULL; codes3}
    )
    expect_equal(
      {attr(codes3, "search_strategy") <- NULL; codes3},
      {attr(codes4, "search_strategy") <- NULL; codes4}
    )

    codes5 <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = c("hip of", "/\bee Osteoarthritis/\b"),
      domains = "Condition",
      standardConcept = c("Standard", "Non-standard")
    )
    expect_equal(
      {attr(codes1, "search_strategy") <- NULL; codes1 |> dplyr::filter(!.data$concept_id %in% c(5L))},
      {attr(codes5, "search_strategy") <- NULL; codes5}
    )

    codes6 <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = c("hip of", "/\bknee Osteoarthritis/\b"),
      domains = "Condition",
      standardConcept = c("Standard", "Non-standard")
    )
    expect_equal(
      {attr(codes4, "search_strategy") <- NULL; codes4},
      {attr(codes6, "search_strategy") <- NULL; codes6}
    )

    # test exclusion + symbols (to ignore)
    expect_message(codes7 <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      exclude = c("?"),
      domains = "Condition",
      standardConcept = c("Standard", "Non-standard")
    ))
    expect_equal(
      {attr(codes1, "search_strategy") <- NULL; codes1},
      {attr(codes7, "search_strategy") <- NULL; codes7}
    )

    # test non-standard
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("Musculoskeletal", "Degenerative arthropathy"),
      searchNonStandard = TRUE,
      includeDescendants = FALSE,
      domains = "Condition"
    )
    expect_true(any(codes$concept_name %in% "Osteoarthrosis"))

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("Degenerative arthropathy"),
      searchNonStandard = TRUE,
      includeDescendants = FALSE,
      domains = "Condition"
    )
    expect_true(any(codes$concept_name %in% "Osteoarthrosis"))

    # all options
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("arthritis", "knee"),
      domains = c("Condition", "observation"),
      standardConcept = "Standard",
      searchInSynonyms = TRUE,
      searchNonStandard = TRUE,
      exclude = "Childhood asthma",
      includeDescendants = TRUE,
      includeAncestor = TRUE
    )
    expect_true(nrow(codes) >= 1)

    # test search in drug
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "adalimumab",
      domains = "Drug",
      searchInSynonyms = TRUE,
      includeDescendants = FALSE,
      includeAncestor = TRUE
    )
    expect_true(codes$concept_id == "10")

    # search for drug and condition
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("arthritis", "adalimumab"),
      domains = c("Condition", "Drug"),
      searchInSynonyms = TRUE,
      includeDescendants = TRUE,
      includeAncestor = TRUE
    )
    expect_true(any(codes$concept_id %in% "10"))

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("arthritis", "adalimumab"),
      domains = NULL,
      searchInSynonyms = TRUE,
      includeDescendants = TRUE,
      includeAncestor = TRUE
    )
    expect_true(all((codes$domain_id |> sort() |> unique()) == c("Condition", "Drug", "Observation")))

    # filter on vocabulary
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("arthritis", "adalimumab"),
      vocabularyId = c("RxNorm", "LOINC"),
      domains = NULL,
      searchInSynonyms = TRUE,
      includeDescendants = TRUE,
      includeAncestor = TRUE
    )
    expect_true(all(codes |> dplyr::pull("vocabulary_id") %in% c("RxNorm","LOINC")))

   expect_warning(getCandidateCodes(
      cdm = cdm,
      keywords = c("arthritis", "adalimumab"),
      vocabularyId = c("RxNorm", "xyz"),
      domains = NULL,
      searchInSynonyms = TRUE,
      includeDescendants = TRUE,
      includeAncestor = TRUE
    ))

   expect_true(nrow(getCandidateCodes(
     cdm = cdm,
     keywords = c("arthritis"),
     vocabularyId = c("RxNorm"),
     domains = NULL,
     searchInSynonyms = TRUE,
     includeDescendants = TRUE,
     includeAncestor = TRUE
   )) == 0)

    ## Edge cases
    # check empty candidate set
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "asthmaX",
      domains = "Condition"
    )
    expect_true(nrow(codes) == 0)

    # keywords that don´t exist
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = c("Musculoskeletal disorder", "XXXXX"),
      standardConcept = c("Standard"),
      includeDescendants = FALSE
    )
    expect_true("1" %in% codes$concept_id)

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "XXXXX",
      standardConcept = c("Standard"),
      includeDescendants = FALSE
    )
    expect_true(nrow(codes) == 0)

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "a", exclude = "a",
      standardConcept = c("Standard"),
      includeDescendants = FALSE
    )
    expect_true(nrow(codes) == 0)

    ## Expected errors
    # keyword should be a character
    expect_error(getCandidateCodes(
      cdm = cdm,
      keywords = 35,
      standardConcept = c("Standard"),
      includeDescendants = FALSE
    ))

    expect_error(getCandidateCodes(
      cdm = "Not a cdm ref",
      keywords = "a",
      exclude = NULL,
      includeDescendants = TRUE,
      includeAncestor = FALSE
    ))

    # domain that doesn´t exist
    expect_warning(getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      domains = c("Condition", "Some other table")
    ))

    # standardConcept that doesn´t exist
    expect_error(getCandidateCodes(
      cdm = cdm,
      keywords = "Musculoskeletal disorder",
      standardConcept = c("Standard", "Something that doesn´t exist"),
      includeDescendants = FALSE
    ))

    expect_error(getCandidateCodes(
      cdm = cdm,
      keywords = "Musculoskeletal disorder",
      standardConcept = "Something that doesn´t exist",
      includeDescendants = FALSE
    ))

    cdm1 <- cdm
    cdm1$concept <- NULL
    expect_error(getCandidateCodes(
      cdm = cdm1,
      keywords = "Musculoskeletal disorder"
    ))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

test_that("tests with mock db - multiple domains", {
  skip_on_cran()
  backends <- c("database",
                "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])

    # tests
    # test keywords search - exact
    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "arthritis",
      domains = c("Condition", "Observation"),
      includeDescendants = FALSE
    )
    expect_true((nrow(codes) == 4 &
                   all(codes$concept_id %in% c(3:5, 9)) &
                   all(!codes$concept_id %in% c(1, 2, 6, 7, 8))))

    codes <- getCandidateCodes(
      cdm = cdm,
      keywords = "h o osteoarthritis",
      domains = c("Condition", "Observation"),
      includeDescendants = FALSE
    )
    expect_true(all(
      nrow(codes) == 3 &
        c(4,5,9) %in% codes$concept_id))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

test_that("tests with eunomia", {

  skip_on_cran()
  skip_on_ci()
  db <- DBI::dbConnect(duckdb::duckdb(),
                       dbdir = CDMConnector::eunomiaDir())
  cdm <- CDMConnector::cdmFromCon(
    con = db,
    cdmSchema = "main",
    writeSchema = "main"
  )

  codes <- getCandidateCodes(cdm=cdm,
                             keywords= "sinusitis",
                             domains = c("Condition", "observation"),
                             searchInSynonyms = TRUE,
                             searchNonStandard = TRUE,
                             includeAncestor = TRUE)

  expect_true(sum(is.na(codes$concept_name)) == 0)

  expect_equal(codes |>
                 dplyr::pull("concept_id"),
               unique(codes |>
                        dplyr::pull("concept_id")))

  codes1 <- getCandidateCodes(cdm=cdm,
                              keywords= c("Protein","disease"),
                              domains = NULL,
                              searchInSynonyms = TRUE,
                              searchNonStandard = TRUE,
                              includeAncestor = TRUE)
  codes1 <- codes1 |> dplyr::arrange(concept_id)
  expect_warning(codes2 <- getCandidateCodes(cdm=cdm,
                                             keywords= c("Protein","disease"),
                                             domains = c("Condition", "Drug", "Procedure", "Device", "Observation", "Measurement"),
                                             searchInSynonyms = TRUE,
                                             searchNonStandard = TRUE,
                                             includeAncestor = TRUE))
  codes2 <- codes2 |> dplyr::arrange(concept_id)

  expect_equal(
    {attr(codes1, "search_strategy") <- NULL; codes1},
    {attr(codes2, "search_strategy") <- NULL; codes2}
  )

})

test_that("check asCodelist", {
  a <- newCodelistWithDetails(list(asthma = dplyr::tibble(concept_id = 1L,
                                                          concept_name = "a"),
                                   copd = dplyr::tibble(concept_id = 2L,
                                                        concept_name = "b")))
  expect_no_error(asCodelist(a))
  expect_true(inherits(asCodelist(a), "codelist"))

  expect_no_error(omopgenerics::emptyCodelist() |> asCodelist())
  expect_no_error(omopgenerics::emptyConceptSetExpression() |> asCodelist())
  expect_no_error(omopgenerics::emptyCodelistWithDetails() |> asCodelist())

  a <- getCandidateCodes(mockVocabRef(), keywords = "ost")
  expect_no_error(asCodelist(a))
  expect_true(inherits(asCodelist(a), "codelist"))

})

test_that("asCodelist resolving concept set expressions", {
  cdm <- mockVocabRef()
  cse <- omopgenerics::newConceptSetExpression(
    list("a" = dplyr::tibble("concept_id" = 1L,
                             "excluded" = FALSE,
                             "descendants" = TRUE,
                             "mapped" = FALSE)))
  expect_identical(
    getDescendants(cdm, 1L) |>
    dplyr::pull("concept_id"),
  asCodelist(cse, cdm)[[1]])

  cse <- omopgenerics::newConceptSetExpression(
    list("a" = dplyr::tibble("concept_id" = c(1L, 10L),
                             "excluded" = FALSE,
                             "descendants" = TRUE,
                             "mapped" = FALSE)))
  expect_identical(
    getDescendants(cdm, c(1L, 10L)) |>
      dplyr::pull("concept_id"),
    asCodelist(cse, cdm)[[1]])

  cse <- omopgenerics::newConceptSetExpression(
    list("a" = dplyr::tibble("concept_id" = c(1L, 10L),
                             "excluded" = c(FALSE, TRUE),
                             "descendants" = c(TRUE, FALSE),
                             "mapped" = c(FALSE, FALSE))))
  expect_identical(
    getDescendants(cdm, c(1L)) |>
      dplyr::pull("concept_id"),
    asCodelist(cse, cdm)[[1]])

})

test_that("checkCodelist function", {
  x <- NULL
  expect_no_error(checkCodelist(x, allowNULL = TRUE))
  expect_error(checkCodelist(x, allowNULL = FALSE))

  x <- list("a" = c(1L, 2L))
  expect_error(checkCodelist(x))

  x <- omopgenerics::newCodelist(list("a" = c(1L, 2L)))
  expect_no_error(checkCodelist(x))

  x <- omopgenerics::newCodelistWithDetails(list("a" = dplyr::tibble("concept_id" = c(1L, 2L))))
  expect_no_error(checkCodelist(x))

  x <- omopgenerics::newConceptSetExpression(list("a" = dplyr::tibble("concept_id" = 1L, "excluded" = FALSE, "descendants" = FALSE, "mapped" = FALSE)))
  expect_no_error(checkCodelist(x, allowConceptSetExpression = TRUE))
  expect_error(checkCodelist(x, allowConceptSetExpression = FALSE))
})

test_that("table achilles code use expcted columns", {
  skip_on_cran()

  testthat::local_mocked_bindings(
    system_fonts = function(...) data.frame(family = "sans", path = ""),
    .package = "systemfonts"
  )

  # mock db
  cdm <- mockVocabRef("database")

  # two codelists: "Osteoarthritis of knee" "Osteoarthritis of hip"
  result <- summariseAchillesCodeUse(omopgenerics::newCodelist(list(knee_oa = 4L,
                                          hip_oa = 5L)),
                                     cdm = cdm)
  expect_no_error(tableAchillesCodeUse(result,
                         type = "gt",
                         header = c("cdm_name", "estimate_name"),
                         .options = list()))

  expect_no_error(tableAchillesCodeUse(result,
                           type = "gt",
                           header = c("cdm_name", "estimate_name"),
                           style = "darwin",
                           .options = list()))


  expect_no_error(tableAchillesCodeUse(result = result,
                         type = "gt",
                         header = c("codelist_name", "cdm_name", "estimate_name"),
                         hide = c("standard_concept", "standard_concept_id", "vocabulary_id"),
                         .options = list()))

  expect_no_error(tableAchillesCodeUse(result = result,
                                 type = "flextable",
                                 header = c("cdm_name", "estimate_name"),
                                 groupColumn = "codelist_name",
                                 hide = c("standard_concept", "standard_concept_id", "vocabulary_id"),
                                 .options = list()))

  expect_no_error(tableAchillesCodeUse(result = result,
                         type = "flextable",
                         header = c("estimate_name"),
                         groupColumn = "codelist_name",
                         hide = c("cdm_name"),
                         .options = list(includeHeaderName = FALSE)))

  # expected error
  expect_error(tableAchillesCodeUse(result,
                                    type = "gt",
                                    header = c("cdm_name", "estimate_name"),
                                    groupColumn = "estimate_name",
                                    .options = list()))

  # empty result
  result <- summariseAchillesCodeUse(omopgenerics::newCodelist(list(a = 99999L)),
                                     cdm = cdm)
  expect_warning(tableAchillesCodeUse(result))

  # not an achilles result
  cond <- dplyr::tibble("person_id" = 1,
                        "condition_occurrence_id" = 1,
                        "condition_start_date" = as.Date("2000-01-01"),
                        "condition_end_date" = as.Date("2000-01-01"),
                        "condition_type_concept_id" = 1L,
                        "condition_concept_id" = 4,
                        "condition_source_concept_id" = 1,
                        "condition_source_value" = "a")
  cdm <- omopgenerics::insertTable(cdm,
                                   name = "condition_occurrence",
                                   table = cond)
  result <- summariseCodeUse(omopgenerics::newCodelist(list(knee_oa = 4L, hip_oa = 5L)),
                             cdm = cdm)
  expect_warning(tableAchillesCodeUse(result))

  CDMConnector::cdmDisconnect(cdm)

  # With source data
  con <- DBI::dbConnect(duckdb::duckdb(),
                        dbdir = CDMConnector::eunomiaDir())
  cdm <- CDMConnector::cdmFromCon(con,
                                  cdmSchema = "main",
                                  writeSchema = "main")
  cdm <- buildAchillesTables(cdm)

  x <- getCandidateCodes(cdm, keywords = "o")
  x <- omopgenerics::newCodelistWithDetails(list("codes" = x))
  result <- summariseAchillesCodeUse(x, cdm)
  expect_no_error(tab <- tableAchillesCodeUse(result))
  expect_true("Standard concept ID" %in% colnames(tab$`_data`))
  expect_true("Standard concept name" %in% colnames(tab$`_data`))
  CDMConnector::cdmDisconnect(cdm)
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get the names of all available Anatomical Therapeutic Chemical (ATC) classification codes
#'
#' @inheritParams cdmDoc
#' @inheritParams levelATCDoc
#'
#' @return A vector containing the names of ATC codes for the chosen level(s)
#' found in the concept table of cdm.
#'
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get ATC 1st level classification codes
#' availableATC(cdm, level = "ATC 1st")
#'
#' # Get all ATC classification codes
#' availableATC(cdm, level = c("ATC 1st", "ATC 2nd", "ATC 3rd", "ATC 4th", "ATC 5th"))
#'}
#'
availableATC <- function(cdm, level = c("ATC 1st")){
  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  omopgenerics::assertChoice(level, choices = supportedATC())

  atc_names <- cdm$concept |>
    dplyr::filter(.data$vocabulary_id == "ATC",
                  .data$concept_class_id %in% .env$level) |>
    dplyr::pull("concept_name")

  return(atc_names)
}

supportedATC <- function(){
  c("ATC 1st", "ATC 2nd", "ATC 3rd", "ATC 4th", "ATC 5th")
}

test_that("available ATC", {
  skip_on_cran()
  backends <- c("database", "data_frame")
  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backends[[i]])

    expect_no_error(
      availableATC(cdm)
    )
    expect_no_error(
      availableATC(cdm, level = c("ATC 2nd"))
    )
    expect_error(
      availableATC(cdm, level = c("ATC 3nd"))
    )

    expect_no_error(res1 <- availableATC(cdm, level = c("ATC 1st", "ATC 2nd")))
    expect_true(res1 == "ALIMENTARY TRACT AND METABOLISM")
    expect_no_error(res2 <- availableATC(cdm, level = c("ATC 1st")))
    expect_true(setequal(intersect(res2, res1), res2)) # res 2 is a subset of res

    expect_error(availableATC("cdm"))
    expect_error(availableATC(cdm, level = "a"))
  }
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' Summarise code use among a cohort in the cdm reference
#'
#' @inheritParams cdmDoc
#' @param cohortTable A cohort table from the cdm reference.
#' @param x A codelist, codelist_with_details, or a concept_set. See `newCodelist()`,
#' `newCodelistWithDetails()`, `newConceptSetExpression()` functions for more details.
#'  If set to NULL, the codelist associated with the cohort will be used.
#' @inheritParams xDocCohort
#' @param cohortId A vector of cohort IDs to include
#' @param timing When to assess the code use relative cohort dates. This can
#' be "any"(code use any time by individuals in the cohort) or  "entry" (code
#' use on individuals' cohort start date).
#' @inheritParams countByDoc
#' @inheritParams byConceptDoc
#' @inheritParams bySexDoc
#' @inheritParams byYearDoc
#' @inheritParams ageGroupDoc
#' @param useSourceCodes Whether the codelist provided contains source codes
#' (TRUE) or standard codes (FALSE).
#' @param codelistType The reason for the codelist. Can be "index event",
#' "inclusion criteria", or "exit criteria". Only apply when x is NULL and using
#' codes associated with a cohort.
#'
#' @return A tibble with results overall and, if specified, by strata
#' @export
#'
#' @examples
#' \dontrun{
#' library(CodelistGenerator)
#' library(duckdb)
#' library(DBI)
#' library(CDMConnector)
#' con <- dbConnect(duckdb(),
#'                  dbdir = eunomiaDir())
#' cdm <- cdmFromCon(con,
#'                  cdmSchema = "main",
#'                  writeSchema = "main")
#' cdm <- generateConceptCohortSet(cdm = cdm,
#'                   conceptSet = list(a = 260139,
#'                                     b = 1127433),
#'                   name = "cohorts",
#'                   end = "observation_period_end_date",
#'                   overwrite = TRUE)
#'
#'results_cohort_mult <-
#'summariseCohortCodeUse(omopgenerics::newCodelist(list(cs = c(260139,19133873))),
#'                       cdm = cdm,
#'                       cohortTable = "cohorts",
#'                       timing = "entry")
#'
#'results_cohort_mult
#'CDMConnector::cdmDisconnect(cdm)
#'}
summariseCohortCodeUse <- function(cdm,
                                   cohortTable,
                                   x = NULL,
                                   cohortId = NULL,
                                   timing = "any",
                                   countBy = c("record", "person"),
                                   byConcept = TRUE,
                                   byYear = FALSE,
                                   bySex = FALSE,
                                   ageGroup = NULL,
                                   useSourceCodes = FALSE,
                                   codelistType = "index event"
) {

  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  omopgenerics::assertTrue(inherits(cdm[[cohortTable]], "GeneratedCohortSet"))
  omopgenerics::assertTrue(all(c("cohort_definition_id", "subject_id", "cohort_start_date",
                                 "cohort_end_date") %in% colnames(cdm[[cohortTable]])))
  omopgenerics::assertChoice(codelistType,
                             choices = c("index event", "inclusion criteria", "exit criteria"))

  if(is.null(cohortId)){
    cohortId <- sort(CDMConnector::settings(cdm[[cohortTable]]) |>
                       dplyr::pull("cohort_definition_id"))
  } else {
    cohortId <- omopgenerics::validateCohortIdArgument(cohortId = cohortId,
                                                       cohort = cdm[[cohortTable]])
  }

  checkCodelist(x, allowConceptSetExpression = FALSE)

  if(!is.null(x)){
    if(inherits(x, "codelist_with_details")){
      x <- asCodelist(x)
    }
    if(length(x) == 0){
      return(omopgenerics::emptySummarisedResult())
    }
    # if x is null we'll use cohort codelist attribute
    # otherwise will use codelist specified
    settings <- expand.grid(codelist_name = names(x),
                            cohort_definition_id = omopgenerics::settings(cdm[[cohortTable]]) |>
                              dplyr::pull("cohort_definition_id")) |>
      dplyr::filter(.data$cohort_definition_id %in% .env$cohortId) |>
      dplyr::left_join(omopgenerics::settings(cdm[[cohortTable]]) |>
                         dplyr::select("cohort_definition_id", "cohort_name"),
                       by = "cohort_definition_id") |>
      dplyr::arrange(.data$cohort_name) |>
      dplyr::mutate(codelist_name = as.character(.data$codelist_name))
  } else {
    x <- attr(cdm[[cohortTable]], "cohort_codelist") |>
      dplyr::filter(.data$codelist_type %in% !!codelistType) |>
      dplyr::select("codelist_name", "concept_id") |>
      dplyr::distinct()

    settings <- attr(cdm[[cohortTable]], "cohort_codelist") |>
      dplyr::filter(.data$codelist_type %in% !!codelistType) |>
      dplyr::select("cohort_definition_id", "codelist_name") |>
      dplyr::distinct() |>
      dplyr::collect() |>
      dplyr::filter(.data$cohort_definition_id %in% .env$cohortId) |>
      dplyr::left_join(omopgenerics::settings(cdm[[cohortTable]]) |>
                         dplyr::select("cohort_definition_id", "cohort_name"),
                       by = "cohort_definition_id") |>
      dplyr::arrange(.data$cohort_name) |>
      dplyr::mutate(codelist_name = as.character(.data$codelist_name))
  }


  cohortCodeUse <- list()
  for(i in seq_along(settings$codelist_name)){
    workingCohortName <- settings$cohort_name[[i]]
    workingCohortId <- settings$cohort_definition_id[[i]]
    workingCodelistName <- settings$codelist_name[[i]]

    if(inherits(x, "cdm_table")){
      workingCodelist <- x |>
        dplyr::filter(.data$codelist_name == !!workingCodelistName) |>
        dplyr::select("concept_id")
      workingCodelist <- list(workingCodelist) |>
        stats::setNames(workingCodelistName)
    } else {
      workingCodelist <- x[workingCodelistName]
    }

    cli::cli_inform(" Getting counts of {names(workingCodelist)} codes for cohort {workingCohortName}")
    cohortCodeUse[[i]] <- getCodeUse(workingCodelist,
                                     cdm = cdm,
                                     cohortTable = cohortTable,
                                     cohortId = workingCohortId,
                                     timing = timing,
                                     countBy = countBy,
                                     byConcept = byConcept,
                                     byYear = byYear,
                                     bySex = bySex,
                                     ageGroup = ageGroup,
                                     dateRange = as.Date(c(NA,NA)),
                                     useSourceCodes = useSourceCodes)
  }
  cohortCodeUse <- dplyr::bind_rows(cohortCodeUse) |>
    dplyr::arrange(dplyr::across(!c("variable_level", "estimate_value")))

  if (nrow(cohortCodeUse) > 0) {
    cohortCodeUse <- cohortCodeUse |>
      dplyr::mutate(
        result_id = 1L,
        cdm_name = omopgenerics::cdmName(cdm)
      ) |>
      omopgenerics::newSummarisedResult(
        settings = dplyr::tibble(

          result_id = 1L,
          result_type = "cohort_code_use",
          package_name = "CodelistGenerator",
          package_version = as.character(utils::packageVersion("CodelistGenerator")),
          timing = timing
        )
      )
  } else {
    cohortCodeUse <- omopgenerics::emptySummarisedResult()
  }

  return(cohortCodeUse)
}

test_that("summarise code use - eunomia", {
  skip_on_cran()

  cdm <- omock::mockCdmFromDataset("GiBleed", source = "duckdb")

  acetiminophen <- c(1125315L,  1127433L, 40229134L,
                     40231925L, 40162522L, 19133768L,  1127078L)
  poliovirus_vaccine <- c(40213160L)
  cs <- omopgenerics::newCodelist(list(acetiminophen = acetiminophen,
                                       poliovirus_vaccine = poliovirus_vaccine))
  startNames <- omopgenerics::listSourceTables(cdm)

  results <- summariseCodeUse(cs,
                              cdm = cdm,
                              byYear = TRUE,
                              bySex = TRUE,
                              ageGroup = list(c(0,17),
                                              c(18,65),
                                              c(66, 100)))
  endNames <- omopgenerics::listSourceTables(cdm)
  expect_true(length(setdiff(endNames, startNames)) == 0)

  expect_no_error(results_no_by_concept <- summariseCodeUse(cs,
                                                            cdm = cdm,
                                                            byYear = TRUE,
                                                            bySex = TRUE,
                                                            byConcept = FALSE))
  expect_true(all(results_no_by_concept |> dplyr::pull("group_level") |> unique() == c("acetiminophen","poliovirus_vaccine")))
  expect_true(all(results_no_by_concept |>
                    dplyr::filter(group_level == "acetiminophen", strata_level == "overall") |>
                    dplyr::pull("estimate_value") == c("14205", "2679")))

  # min cell counts:
  expect_true(
    all(
      omopgenerics::suppress(results) |>
        dplyr::filter(
          variable_name == "overall",
          strata_level == "1909",
          group_level == "acetiminophen"
        ) |>
        dplyr::pull("estimate_value") == "-"
    ))

  # check is a summarised result
  expect_true("summarised_result" %in%  class(results))
  expect_equal(omopgenerics::resultColumns(),
               colnames(results))

  # overall record count
  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "overall" &
                                strata_level == "overall" &
                                group_level == "acetiminophen" &
                                estimate_name == "record_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$drug_exposure |>
                dplyr::filter(drug_concept_id %in%  acetiminophen) |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # overall person count
  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "overall" &
                                strata_level == "overall" &
                                group_level == "acetiminophen" &
                                estimate_name == "person_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric()  ==
                cdm$drug_exposure |>
                dplyr::filter(drug_concept_id %in% acetiminophen) |>
                dplyr::select("person_id") |>
                dplyr::distinct() |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # by year
  # overall record count
  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "year" &
                                strata_level == "2008" &
                                group_level == "acetiminophen" &
                                estimate_name == "record_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric()  ==
                cdm$drug_exposure |>
                dplyr::filter(drug_concept_id %in% acetiminophen) |>
                dplyr::filter(year(drug_exposure_start_date) == 2008) |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # overall person count
  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "year" &
                                strata_level == "2008" &
                                group_level == "acetiminophen" &
                                estimate_name == "person_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$drug_exposure |>
                dplyr::filter(drug_concept_id %in% acetiminophen) |>
                dplyr::filter(year(drug_exposure_start_date) == 2008) |>
                dplyr::select("person_id") |>
                dplyr::distinct() |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # by age group and sex
  # overall record count
  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "sex" &
                                strata_level == "Male" &
                                group_level == "acetiminophen" &
                                estimate_name == "record_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$drug_exposure |>
                dplyr::filter(drug_concept_id %in% acetiminophen) |>
                PatientProfiles::addSex() |>
                dplyr::filter(sex == "Male") |>
                dplyr::tally() |>
                dplyr::pull("n"))

  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "age_group &&& sex" &
                                strata_level == "18 to 65 &&& Male" &
                                group_level == "acetiminophen" &
                                estimate_name == "record_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$drug_exposure |>
                dplyr::filter(drug_concept_id %in% acetiminophen) |>
                PatientProfiles::addAge(indexDate = "drug_exposure_start_date") |>
                PatientProfiles::addSex() |>
                dplyr::filter(sex == "Male" &
                                age >= "18" &
                                age <= "65") |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # overall person count
  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "age_group &&& sex" &
                                strata_level == "18 to 65 &&& Male" &
                                group_level == "acetiminophen" &
                                estimate_name == "person_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$drug_exposure |>
                dplyr::filter(drug_concept_id %in% acetiminophen) |>
                PatientProfiles::addAge(indexDate = "drug_exposure_start_date") |>
                PatientProfiles::addSex() |>
                dplyr::filter(sex == "Male" &
                                age >= "18" &
                                age <= "65") |>
                dplyr::select("person_id") |>
                dplyr::distinct() |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # Check date range
  results <- summariseCodeUse(cs,
                              cdm = cdm,
                              byYear = TRUE,
                              bySex = TRUE,
                              ageGroup = list(c(0,17),
                                              c(18,65),
                                              c(66, 100)),
                              dateRange = as.Date(c("2010-01-01","2015-01-01")))
  expect_equal(results |>
                 omopgenerics::settings() |>
                 dplyr::select("date_range_start", "date_range_end") |>
                 as.character(),
               c("2010-01-01","2015-01-01"))

  # overall record count
  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "overall" &
                                strata_level == "overall" &
                                group_level == "acetiminophen" &
                                estimate_name == "record_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$drug_exposure |>
                dplyr::filter(drug_exposure_start_date >= as.Date("2010-01-01"),
                              drug_exposure_start_date <= as.Date("2015-01-01")) |>
                dplyr::filter(drug_concept_id %in%  acetiminophen) |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # overall person count
  expect_true(results |>
                dplyr::filter(group_name == "codelist_name" &
                                strata_name == "overall" &
                                strata_level == "overall" &
                                group_level == "acetiminophen" &
                                estimate_name == "person_count",
                              variable_name == "overall") |>
                dplyr::pull("estimate_value") |>
                as.numeric()  ==
                cdm$drug_exposure |>
                dplyr::filter(drug_concept_id %in% acetiminophen) |>
                dplyr::filter(drug_exposure_start_date >= as.Date("2010-01-01"),
                              drug_exposure_start_date <= as.Date("2014-12-31")) |>
                dplyr::select("person_id") |>
                dplyr::distinct() |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # by year
  expect_true(
    results |>
      dplyr::filter(strata_name == "year") |>
      dplyr::pull("strata_level") |>
      unique() |>
      as.numeric() |> max() == 2014)
  expect_true(
    results |>
      dplyr::filter(strata_name == "year") |>
      dplyr::pull("strata_level") |>
      unique() |>
      as.numeric() |> min() == 2010)



  results <- summariseCodeUse(omopgenerics::newCodelist(list("acetiminophen" = acetiminophen)),
                              cdm = cdm, countBy = "person",
                              byYear = FALSE,
                              bySex = FALSE,
                              ageGroup = NULL)
  expect_true(nrow(results |>
                     dplyr::filter(estimate_name == "person_count")) > 0)
  expect_true(nrow(results |>
                     dplyr::filter(estimate_name == "record_count")) == 0)

  results <- summariseCodeUse(omopgenerics::newCodelist(list("acetiminophen" = acetiminophen)),
                              cdm = cdm, countBy = "record",
                              byYear = FALSE,
                              bySex = FALSE,
                              ageGroup = NULL)
  expect_true(nrow(results |>
                     dplyr::filter(estimate_name == "person_count")) == 0)
  expect_true(nrow(results |>
                     dplyr::filter(estimate_name == "record_count")) > 0)

  # domains covered
  # condition
  expect_true(nrow(summariseCodeUse(omopgenerics::newCodelist(list(cs= c(4112343L))),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # visit
  expect_true(nrow(summariseCodeUse(omopgenerics::newCodelist(list(cs= c(9201L))),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # drug
  expect_true(nrow(summariseCodeUse(omopgenerics::newCodelist(list(cs= c(40213160L))),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # measurement
  expect_true(nrow(summariseCodeUse(omopgenerics::newCodelist(list(cs= c(3006322L))),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # procedure and condition
  expect_true(nrow(summariseCodeUse(omopgenerics::newCodelist(list(cs= c(4107731L,4112343L))),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # no records
  expect_message(results <- summariseCodeUse(omopgenerics::newCodelist(list(cs= c(999999L))),
                                             cdm = cdm,
                                             byYear = FALSE,
                                             bySex = FALSE,
                                             ageGroup = NULL))
  expect_true(nrow(results) == 0)

  # expected errors
  expect_error(summariseCodeUse("not a concept",
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = NULL))
  expect_error(summariseCodeUse("123",
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = NULL))
  expect_error(summariseCodeUse(list("123" = 1L),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = NULL))
  expect_error(summariseCodeUse(omopgenerics::newCodelist(list(a = 123L)),
                                cdm = "not a cdm",
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = NULL))
  expect_error(summariseCodeUse(omopgenerics::newCodelist(list(a = 123L)),
                                cdm = cdm,
                                byYear = "Maybe",
                                bySex = FALSE,
                                ageGroup = NULL))
  expect_error(summariseCodeUse(omopgenerics::newCodelist(list(a = 123L)),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = "Maybe",
                                ageGroup = NULL))
  expect_error(summariseCodeUse(omopgenerics::newCodelist(list(a = 123L)),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = 25))
  expect_error(summariseCodeUse(omopgenerics::newCodelist(list(a = 123L)),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = list(c(18,17))))
  expect_error(summariseCodeUse(omopgenerics::newCodelist(list(a = 123L)),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = list(c(0,17),
                                                c(15,20))))
  expect_error(summariseCodeUse(omopgenerics::newCodelist(list(a = 123L)),
                                cdm = cdm,
                                dateRange = c("a","b")))
  CDMConnector::cdmDisconnect(cdm)
})

test_that("summarise cohort code use - eunomia", {
  skip_on_cran()

  cdm <- omock::mockCdmFromDataset("GiBleed", source = "duckdb")

  pharyngitis_codes <- omopgenerics::newCodelist(list("ph" = c(4112343L)))

  cdm <- CDMConnector::generateConceptCohortSet(cdm = cdm,
                                                conceptSet = pharyngitis_codes,
                                                name = "pharyngitis",
                                                end = "observation_period_end_date",
                                                overwrite = TRUE)

  # any
  results_all <- summariseCodeUse(pharyngitis_codes,
                                  cdm = cdm)
  results_cohort <- summariseCohortCodeUse(x = pharyngitis_codes,
                                           cdm = cdm,
                                           cohortTable = "pharyngitis",
                                           cohortId = "ph",
                                           timing = "any")
  expect_identical(results_cohort,
                   results_cohort)

  results_cohort_attr <- summariseCohortCodeUse(cdm = cdm,
                                                cohortTable = "pharyngitis",
                                                timing = "any")
  expect_identical(results_cohort, results_cohort_attr)
  expect_no_error(summariseCohortCodeUse(x = omopgenerics::newCodelist(list(cs = 4134304L)),
                                         cdm = cdm,
                                         cohortTable = "pharyngitis",
                                         timing = "any",
                                         byConcept = FALSE))
  expect_true(inherits(results_cohort, "summarised_result"))
  expect_true(all(c("result_id", "result_type", "package_name", "package_version", "timing") %in%
                    colnames(omopgenerics::settings(results_cohort))))

  expect_true(results_cohort |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "overall" &
                                strata_level == "overall" &
                                estimate_name == "person_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric() <=
                results_all |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "overall" &
                                strata_level == "overall" &
                                estimate_name == "person_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric())




  # at entry - everyone in the cohort should have the code
  results_cohort <- summariseCohortCodeUse(pharyngitis_codes,
                                           cdm = cdm,
                                           cohortTable = "pharyngitis",
                                           timing = "entry")
  results_cohort |>
    dplyr::filter(variable_name == "overall" &
                    strata_name == "overall" &
                    strata_level == "overall" &
                    estimate_name == "person_count") |>
    dplyr::pull("estimate_value") |>
    as.numeric() ==
    CDMConnector::cohortCount(cdm$pharyngitis) |>
    dplyr::pull("number_subjects")



  # 260139
  # on index
  index_260139 <- cdm$pharyngitis |>
    dplyr::left_join(cdm$condition_occurrence,
                     by=c("subject_id"="person_id")) |>
    dplyr::filter(condition_start_date == cohort_start_date) |>
    dplyr::filter(condition_concept_id == 260139) |>
    dplyr::select("subject_id") |>
    dplyr::distinct() |>
    dplyr::count() |>
    dplyr::pull()

  results_cohort_260139 <- summariseCohortCodeUse(omopgenerics::newCodelist(list(cs = 260139L)),
                                                  cdm = cdm,
                                                  cohortTable = "pharyngitis",
                                                  timing = "entry")
  expect_equal(results_cohort_260139 |>
                 dplyr::filter(variable_name == "overall" &
                                 strata_name == "overall" &
                                 strata_level == "overall" &
                                 estimate_name == "person_count") |>
                 dplyr::pull("estimate_value") |>
                 as.numeric(), index_260139)


  # 260139 or 19133873 or 1127433
  # on index
  index_260139_19133873_1127433 <- dplyr::union_all(
    cdm$pharyngitis |>
      dplyr::left_join(cdm$condition_occurrence,
                       by=c("subject_id"="person_id")) |>
      dplyr::filter(condition_start_date == cohort_start_date) |>
      dplyr::filter(condition_concept_id == 260139) |>
      dplyr::select("subject_id"),
    cdm$pharyngitis |>
      dplyr::left_join(cdm$drug_exposure,
                       by=c("subject_id"="person_id")) |>
      dplyr::filter(drug_exposure_start_date == cohort_start_date) |>
      dplyr::filter(drug_concept_id %in% c(19133873,1127433)) |>
      dplyr::select("subject_id")) |>
    dplyr::count() |>
    dplyr::pull()

  results_cohort_260139_19133873_1127433<- summariseCohortCodeUse(omopgenerics::newCodelist(list(cs = c(260139L,19133873L,1127433L))),
                                                                  cdm = cdm,
                                                                  cohortTable = "pharyngitis",
                                                                  timing = "entry")
  expect_equal(results_cohort_260139_19133873_1127433 |>
                 dplyr::filter(variable_name == "overall" &
                                 strata_name == "overall" &
                                 strata_level == "overall" &
                                 estimate_name == "record_count") |>
                 dplyr::pull("estimate_value") |>
                 as.numeric(),
               index_260139_19133873_1127433)

  expect_equal(results_cohort_260139_19133873_1127433 |>
                 dplyr::filter(stringr::str_detect(variable_name, "Acute bronchitis")) |>
                 dplyr::filter(strata_name == "overall" &
                                 strata_level == "overall" &
                                 estimate_name == "person_count") |>
                 dplyr::pull("estimate_value") |>
                 as.numeric(),
               index_260139)


  # multiple cohorts
  cdm <- CDMConnector::generateConceptCohortSet(cdm = cdm,
                                                conceptSet = omopgenerics::newCodelist(list(a = 260139L,
                                                                                            b = 1127433L )),
                                                name = "cohorts",
                                                end = "observation_period_end_date",
                                                overwrite = TRUE)

  results_cohort_mult <- summariseCohortCodeUse(omopgenerics::newCodelist(list(cs = c(260139L,19133873L,1127433L))),
                                                cdm = cdm,
                                                cohortTable = "cohorts",
                                                timing = "entry")
  expect_true(nrow(results_cohort_mult |>
                     dplyr::filter(stringr::str_detect(variable_name, "Acute bronchitis")) |>
                     dplyr::filter(strata_name == "overall" &
                                     strata_level == "overall" &
                                     estimate_name == "person_count")) == 2)

  expect_equal(c("a", "b"),  results_cohort_mult |>
                 dplyr::filter(stringr::str_detect(variable_name, "Acute bronchitis")) |>
                 dplyr::filter(strata_name == "overall" &
                                 strata_level == "overall" &
                                 estimate_name == "person_count") |>
                 visOmopResults::splitGroup() |>
                 dplyr::pull("cohort_name"))



  codes<- omopgenerics::newCodelist(list(a = 260139L,
                                         b = 1127433L))
  cdm <- CDMConnector::generateConceptCohortSet(cdm = cdm,
                                                conceptSet = codes,
                                                name = "cohorts",
                                                end = "observation_period_end_date",
                                                overwrite = TRUE)
  # if we pass codes we'll end up with 4 results - both codelists for both cohorts
  results_codes <- summariseCohortCodeUse(x = codes,
                                          cdm = cdm,
                                          cohortTable = "cohorts",
                                          timing = "entry")
  expect_true(all(c("a &&& a", "a &&& b", "b &&& a", "b &&& b") %in%
                    (results_codes |>
                       dplyr::pull("group_level") |>
                       unique())))
  # but if use cohort codelist we'll end up with 2 results - codelists for respective cohorts
  results_attr <- summariseCohortCodeUse(cdm = cdm,
                                         cohortTable = "cohorts",
                                         timing = "entry")
  expect_false(all(c("a &&& a", "a &&& b", "b &&& a", "b &&& b") %in%
                     (results_attr |>
                        dplyr::pull("group_level") |>
                        unique())))
  expect_true(all(c("a &&& a",  "b &&& b") %in%
                    (results_attr |>
                       dplyr::pull("group_level") |>
                       unique())))
  expect_identical(
    results_codes |>
      dplyr::filter(group_level == "a &&& a"),
    results_attr |>
      dplyr::filter(group_level == "a &&& a"))
  expect_identical(
    results_codes |>
      dplyr::filter(group_level == "b &&& b"),
    results_attr |>
      dplyr::filter(group_level == "b &&& b"))


  # empty cohort - no results
  cdm$pharyngitis <-  cdm$pharyngitis |>
    dplyr::filter(cohort_definition_id == 99)
  expect_true(nrow(summariseCohortCodeUse(omopgenerics::newCodelist(list(cs = 4134304L)),
                                          cdm = cdm,
                                          cohortTable = "pharyngitis",
                                          timing = "any")) == 0)
  expect_true(nrow(summariseCohortCodeUse(cdm = cdm,
                                          cohortTable = "pharyngitis",
                                          timing = "any")) == 0)

  # expected errors
  expect_error(summariseCohortCodeUse(4134304L,
                                      cdm = cdm,
                                      cohortTable = "not_a_cohort",
                                      timing = "any"))
  expect_error(summariseCohortCodeUse(list(4134304L),
                                      cdm = cdm,
                                      cohortTable = "not_a_cohort",
                                      timing = "any"))
  expect_error(summariseCohortCodeUse(list(cs = 4134304L),
                                      cdm = cdm,
                                      cohortTable = "not_a_cohort",
                                      timing = "any"))
  expect_error(summariseCohortCodeUse(list(cs = 4134304L),
                                      cdm = cdm,
                                      cohortTable = "pharyngitis",
                                      timing = "not_a_option"))
  expect_error(summariseCohortCodeUse(list(cs = 4134304L),
                                      cdm = cdm,
                                      cohortTable = "pharyngitis",
                                      timing = c("any", "entry")))

  CDMConnector::cdmDisconnect(cdm)

})

test_that("summarise cohort code use - event type", {
  skip_on_cran()

  cdm <- omock::mockCdmFromDataset("GiBleed", source = "duckdb")

  pharyngitis_codes <- omopgenerics::newCodelist(list("ph" = c(4112343L)))
  acetiminophen_codes <- list(acetiminophen = c(1125315,  1127433, 40229134,
                     40231925, 40162522, 19133768,  1127078) |>
    as.integer()) |>
    omopgenerics::newCodelist()

  cdm$pharyngitis <- CohortConstructor::conceptCohort(cdm, pharyngitis_codes,
                                                      name = "pharyngitis") |>
    CohortConstructor::requireConceptIntersect(acetiminophen_codes,
                                               window = c(-Inf, Inf))

  # all event types by default
  result_all_types <- summariseCohortCodeUse(cdm, "pharyngitis",
                                             codelistType = c("index event",
                                                              "inclusion criteria",
                                                              "exit criteria"))
  expect_true("acetiminophen" %in%
              (result_all_types |>
    omopgenerics::tidy() |>
    dplyr::pull("codelist_name") |>
    unique()))

  # only index event
  result_index <- summariseCohortCodeUse(cdm, "pharyngitis",
                                             codelistType = "index event")
  expect_false("acetiminophen" %in%
                (result_index |>
                   omopgenerics::tidy() |>
                   dplyr::pull("codelist_name") |>
                   unique()))

  # only index event
  result_inclusion <- summariseCohortCodeUse(cdm, "pharyngitis",
                                         codelistType = "inclusion criteria")
  expect_false("ph" %in%
                 (result_inclusion |>
                    omopgenerics::tidy() |>
                    dplyr::pull("codelist_name") |>
                    unique()))


  CDMConnector::cdmDisconnect(cdm)

})

test_that("summarise code use - redshift", {
  testthat::skip() # just run manually
  testthat::skip_if(Sys.getenv("CDM5_REDSHIFT_DBNAME") == "")

  db <-  DBI::dbConnect(RPostgres::Redshift(),
                        dbname   = Sys.getenv("CDM5_REDSHIFT_DBNAME"),
                        host     = Sys.getenv("CDM5_REDSHIFT_HOST"),
                        port     = Sys.getenv("CDM5_REDSHIFT_PORT"),
                        user     = Sys.getenv("CDM5_REDSHIFT_USER"),
                        password = Sys.getenv("CDM5_REDSHIFT_PASSWORD"))

  cdm <- CDMConnector::cdmFromCon(cdmName = "cdm",
                                  con = db,
                                  cdmSchema = Sys.getenv("CDM5_REDSHIFT_CDM_SCHEMA"),
                                  writeSchema = Sys.getenv("CDM5_REDSHIFT_SCRATCH_SCHEMA"),
                                  cdmVersion = "5.3")

  asthma <- list(asthma = c(317009L, 257581L)) |>
    omopgenerics::newCodelist()

  results <- summariseCodeUse(asthma,
                              cdm = cdm,
                              byYear = TRUE,
                              bySex = TRUE,
                              ageGroup = list(c(0,17),
                                              c(18,65),
                                              c(66, 100)))
  # column names
  expect_true(inherits(results, "summarised_result"))

  # overall record count
  expect_true(results |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "overall" &
                                strata_level == "overall",
                              estimate_name == "record_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$condition_occurrence |>
                dplyr::filter(condition_concept_id %in%  !!asthma[[1]]) |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # overall person count
  expect_true(results |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "overall" &
                                strata_level == "overall" &
                                estimate_name == "person_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$condition_occurrence |>
                dplyr::filter(condition_concept_id %in% !!asthma[[1]]) |>
                dplyr::select("person_id") |>
                dplyr::distinct() |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # by year
  # overall record count
  expect_true(results |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "year" &
                                strata_level == "2008",
                              estimate_name == "record_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$condition_occurrence |>
                dplyr::filter(condition_concept_id %in% !!asthma[[1]]) |>
                dplyr::filter(year(condition_start_date) == 2008) |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # overall person count
  expect_true(results |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "year" &
                                strata_level == "2008",
                              estimate_name == "person_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$condition_occurrence |>
                dplyr::filter(condition_concept_id %in% !!asthma[[1]]) |>
                dplyr::filter(year(condition_start_date) == 2008) |>
                dplyr::select("person_id") |>
                dplyr::distinct() |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # by age group and sex
  # overall record count
  expect_true(results |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "sex" &
                                strata_level == "Male",
                              estimate_name == "record_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$condition_occurrence |>
                dplyr::filter(condition_concept_id %in% !!asthma[[1]]) |>
                PatientProfiles::addSex() |>
                dplyr::filter(sex == "Male") |>
                dplyr::tally() |>
                dplyr::pull("n"))

  expect_true(results |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "age_group &&& sex" &
                                strata_level == "18 to 65 &&& Male",
                              estimate_name == "record_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$condition_occurrence |>
                dplyr::filter(condition_concept_id %in% !!asthma[[1]]) |>
                PatientProfiles::addAge(indexDate = "condition_start_date") |>
                PatientProfiles::addSex() |>
                dplyr::filter(sex == "Male" &
                                age >= "18" &
                                age <= "65") |>
                dplyr::tally() |>
                dplyr::pull("n"))

  # overall person count
  expect_true(results |>
                dplyr::filter(variable_name == "overall" &
                                strata_name == "age_group &&& sex" &
                                strata_level == "18 to 65 &&& Male",
                              estimate_name == "person_count") |>
                dplyr::pull("estimate_value") |>
                as.numeric() ==
                cdm$condition_occurrence |>
                dplyr::filter(condition_concept_id %in% !!asthma[[1]]) |>
                PatientProfiles::addAge(indexDate = "condition_start_date") |>
                PatientProfiles::addSex() |>
                dplyr::filter(sex == "Male" &
                                age >= "18" &
                                age <= "65") |>
                dplyr::select("person_id") |>
                dplyr::distinct() |>
                dplyr::tally() |>
                dplyr::pull("n"))




  results <- summariseCodeUse(asthma,
                              cdm = cdm, countBy = "person",
                              byYear = FALSE,
                              bySex = FALSE,
                              ageGroup = NULL)
  expect_true(nrow(results |>
                     dplyr::filter(estimate_name == "person_count")) > 0)
  expect_true(nrow(results |>
                     dplyr::filter(estimate_name == "record_count")) == 0)

  results <- summariseCodeUse(asthma,
                              cdm = cdm, countBy = "record",
                              byYear = FALSE,
                              bySex = FALSE,
                              ageGroup = NULL)
  expect_true(nrow(results |>
                     dplyr::filter(estimate_name == "person_count")) == 0)
  expect_true(nrow(results |>
                     dplyr::filter(estimate_name == "record_count")) > 0)


  # domains covered

  # condition
  expect_true(nrow(summariseCodeUse(list(cs = c(317009L)),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # visit
  expect_true(nrow(summariseCodeUse(list(cs = 9201L),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # drug
  expect_true(nrow(summariseCodeUse(list(cs = 19071493L),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # measurement
  expect_true(nrow(summariseCodeUse(list(cs = 2212542L),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # procedure and condition
  expect_true(nrow(summariseCodeUse(list(cs = c(4261206L,317009L)),
                                    cdm = cdm,
                                    byYear = FALSE,
                                    bySex = FALSE,
                                    ageGroup = NULL))>1)

  # no records
  expect_message(results <- summariseCodeUse(list(cs = c(999999L)),
                                             cdm = cdm,
                                             byYear = FALSE,
                                             bySex = FALSE,
                                             ageGroup = NULL))
  expect_true(nrow(results) == 0)



  # expected errors
  expect_error(summariseCodeUse(list(cs = "not a concept"),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = NULL))
  expect_error(summariseCodeUse(list(cs = 123),
                                cdm = "not a cdm",
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = NULL))
  expect_error(summariseCodeUse(list(cs = 123),
                                cdm = cdm,
                                byYear = "Maybe",
                                bySex = FALSE,
                                ageGroup = NULL))
  expect_error(summariseCodeUse(list(cs = 123),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = "Maybe",
                                ageGroup = NULL))
  expect_error(summariseCodeUse(list(cs = 123),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = 25))
  expect_error(summariseCodeUse(list(cs = 123),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = list(c(18,17))))
  expect_error(summariseCodeUse(list(cs = 123),
                                cdm = cdm,
                                byYear = FALSE,
                                bySex = FALSE,
                                ageGroup = list(c(0,17),
                                                c(15,20))))


  CDMConnector::cdmDisconnect(cdm)


})

test_that("summarise code use - eunomia source concept id NA", {
  skip_on_cran()

  cdm <- omock::mockCdmFromDataset("GiBleed", source = "duckdb")

  acetiminophen <- c(1125315,  1127433, 40229134,
                     40231925, 40162522, 19133768,  1127078) |>
    as.integer()

  cdm$drug_exposure <- cdm$drug_exposure |>
    dplyr::mutate(drug_source_concept_id = NA_character_)

  cs <- omopgenerics::newCodelist(list(acetiminophen = acetiminophen))
  results <- summariseCodeUse(cs,
                              cdm = cdm)

  expect_true(all(is.na(omopgenerics::splitAdditional(results) |>
                    dplyr::filter(variable_name != "overall") |>
                    dplyr::pull("source_concept_name"))))
  expect_true(all(is.na(omopgenerics::splitAdditional(results) |>
                    dplyr::filter(variable_name != "overall") |>
                    dplyr::pull("source_concept_id"))))

  CDMConnector::cdmDisconnect(cdm)
})

test_that("summarise cohort code use - eunomia source concept id NA", {
  skip_on_cran()

  cdm <- omock::mockCdmFromDataset("GiBleed", source = "duckdb")

  pharyngitis <- c(4112343L)

  cdm$condition_occurrence <- cdm$condition_occurrence |>
    dplyr::mutate(condition_source_concept_id = NA_character_)

  cdm <- CDMConnector::generateConceptCohortSet(cdm = cdm,
                                                conceptSet = list(pharyngitis = pharyngitis),
                                                name = "pharyngitis",
                                                end = "observation_period_end_date",
                                                overwrite = TRUE)

  results_cohort <- summariseCohortCodeUse(omopgenerics::newCodelist(list(cs = 4134304L)),
                                           cdm = cdm,
                                           cohortTable = "pharyngitis",
                                           timing = "any")

  expect_true(all(is.na(omopgenerics::splitAdditional(results_cohort) |>
                    dplyr::filter(variable_name != "overall") |>
                    dplyr::pull("source_concept_name"))))
  expect_true(all(is.na(omopgenerics::splitAdditional(results_cohort) |>
                    dplyr::filter(variable_name != "overall") |>
                    dplyr::pull("source_concept_id"))))

  CDMConnector::cdmDisconnect(cdm)

})

test_that("empty cohort", {
  skip_on_cran()

  cdm <- omock::mockCdmFromDataset("GiBleed", source = "duckdb")

  # Empty cohort
  cdm <- CDMConnector::generateConceptCohortSet(cdm = cdm,
                                                conceptSet = list(a = 260139L,
                                                                  b = 1127433L),
                                                name = "cohorts",
                                                end = "observation_period_end_date",
                                                overwrite = TRUE)
  results_cohort_mult <- summariseCohortCodeUse(x = omopgenerics::emptyCodelist(),
                                                cdm = cdm,
                                                cohortTable = "cohorts",
                                                timing = "entry")
  expect_true(inherits(results_cohort_mult, "summarised_result"))
  expect_true(nrow(results_cohort_mult) == 0)

  # Source codes
  n1 <- cdm[["condition_occurrence"]] |>
    dplyr::filter(condition_source_concept_id == 35208414) |>
    dplyr::summarise(n = dplyr::n()) |>
    dplyr::pull("n")
  n2 <- cdm[["drug_exposure"]] |>
    dplyr::filter(drug_source_concept_id == 44923712) |>
    dplyr::summarise(n = dplyr::n()) |>
    dplyr::pull("n")

  x <- newCodelist(list("codes" = c(35208414L, 44923712L)))

  expect_no_error(result <- summariseCodeUse(x, cdm, countBy = "record", useSourceCodes = TRUE))
  expect_equal(result |>
                  dplyr::pull("estimate_value") |>
                  as.integer() |>
                  sort(),
                c(n1, n2, n1+n2))

  # summarise cohort code use
  x <- newCodelist(list("codes" = c(4043071L, 40481087L, 19006318L)))
  cdm$new_cohort <- CohortConstructor::conceptCohort(cdm = cdm,
                                                     conceptSet = x,
                                                     name = "new_cohort",
                                                     useSourceFields = TRUE)
  result <- summariseCohortCodeUse(cdm,
                                   cohortTable = "new_cohort",
                                   x = x,
                                   useSourceCodes = TRUE,
                                   countBy = "person")
  n1 <- cdm$procedure_occurrence |>
    dplyr::filter(procedure_source_concept_id == 4043071L) |>
    dplyr::distinct(person_id) |>
    dplyr::tally() |>
    dplyr::pull("n")
  n2 <-  cdm$condition_occurrence |>
    dplyr::filter(condition_source_concept_id == 40481087L) |>
    dplyr::distinct(person_id) |>
    dplyr::tally() |>
    dplyr::pull("n")
  n3 <-  cdm$drug_exposure |>
    dplyr::filter(drug_source_concept_id == 19006318L) |>
    dplyr::distinct(person_id) |>
    dplyr::tally() |>
    dplyr::pull("n")
  expect_equal(result |>
                 dplyr::filter(!is.na(variable_level)) |>
                 dplyr::pull("estimate_value") |>
                 as.integer() |>
                 sort(),
               sort(c(n1, n2, n3)))
  CDMConnector::cdmDisconnect(cdm)

})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' Compare overlap between two sets of codes
#'
#' @param codelist1 Output of getCandidateCodes or a codelist
#' @param codelist2 Output of getCandidateCodes.
#'
#' @return Tibble with information on the overlap of codes in both codelists.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create a CDM object
#' downloadMockDataset(datasetName = "GiBleed",
#'                     path = NULL,
#'                     overwrite = NULL)
#' cdm <- mockCdmFromDataset(datasetName = "GiBleed")
#'
#' # Compare two candidate_codes object
#' codes1 <- getCandidateCodes(
#'   cdm = cdm,
#'   keywords = "Arthritis",
#'   domains = "Condition",
#'   includeDescendants = TRUE)
#'
#' codes2 <- getCandidateCodes(
#'   cdm = cdm,
#'   keywords = c("osteo"),
#'   domains = "Condition",
#'   includeDescendants = TRUE)
#'
#' # compareCodelists(
#' #   codelist1 = codes1,
#' #   codelist2 = codes2
#' # )
#'
#' # Compare two codelists
#' acetaminophen <- getDrugIngredientCodes(cdm,
#'                                         name = "acetaminophen",
#'                                         nameStyle = "{concept_name}",
#'                                         type = "codelist")
#'
#' hydrocodone <- getDrugIngredientCodes(cdm,
#'                                       name = "hydrocodone",
#'                                       nameStyle = "{concept_name}",
#'                                       type = "codelist")
#' compareCodelists(
#'   codelist1 = acetaminophen,
#'   codelist2 = hydrocodone)
#' # Notice that concept_name = NA as `codelist` class does not store this information
#' # for each concept.
#'
#' # Compare two codelists_with_details
#' acetaminophen <- getDrugIngredientCodes(cdm,
#'                                         name = "acetaminophen",
#'                                         nameStyle = "{concept_name}",
#'                                         type = "codelist_with_details")
#'
#' hydrocodone <- getDrugIngredientCodes(cdm,
#'                                       name = "hydrocodone",
#'                                       nameStyle = "{concept_name}",
#'                                       type = "codelist_with_details")
#' compareCodelists(
#'   codelist1 = acetaminophen,
#'   codelist2 = hydrocodone
#' )
#'
#' }
compareCodelists <- function(codelist1,
                             codelist2) {

  if(!inherits(codelist1, "candidate_codes")){
    checkCodelist(codelist1, allowConceptSetExpression = FALSE)
  }
  if(!inherits(codelist2, "candidate_codes")){
    checkCodelist(codelist2, allowConceptSetExpression = FALSE)
  }

  codelist1 <- convertCodelist(codelist1, name = "codelist1")
  codelist2 <- convertCodelist(codelist2, name = "codelist2")

  codes <- list()
  for(i in seq_along(codelist1)){
    for(j in seq_along(codelist2)){
      workingName <- paste0(names(codelist1)[[i]],"_", names(codelist2)[[j]])

      codes[[workingName]] <- dplyr::full_join(codelist1[[i]] |>
                                                 dplyr::select("concept_id", "concept_name") |>
                                                 dplyr::mutate(codelist_1 = 1) |>
                                                 dplyr::distinct(),
                                               codelist2[[j]] |>
                                                 dplyr::select("concept_id", "concept_name") |>
                                                 dplyr::mutate(codelist_2 = 1) |>
                                                 dplyr::distinct(),
                                               by = c("concept_id", "concept_name"))

      codes[[workingName]] <- codes[[workingName]] |>
        dplyr::mutate(
          "codelist" = dplyr::case_when(
            !is.na(codelist_1) & is.na(codelist_2) ~ paste0("Only in codelist ", names(codelist1)[[i]]),
            is.na(codelist_1) & !is.na(codelist_2) ~  paste0("Only in codelist ", names(codelist2)[[j]]),
            !is.na(codelist_1) & !is.na(codelist_2) ~ "Both")) |>
        dplyr::select(-c("codelist_1", "codelist_2"))
    }
  }

  if(length(codes) == 1){
    codes <- codes[[1]] |>
      dplyr::arrange(.data$concept_id)
  }
  return(codes)
}


convertCodelist <- function(codelist, name){
  if(inherits(codelist, "codelist")){
    codelist <- purrr::map(
      codelist,
      ~dplyr::tibble("concept_id" = .x,
                     "concept_name" = NA_character_)
    )
  }

  if(inherits(codelist, "codelist_with_details")){
    if("concept_name" %in% colnames(codelist[[1]])){
      codelist <- purrr::map(
        codelist,
        ~dplyr::tibble("concept_id" = dplyr::pull(.x, "concept_id"),
                       "concept_name" = dplyr::pull(.x, "concept_name"))
      )
    }else{
      codelist <- purrr::map(
        codelist,
        ~dplyr::tibble("concept_id" = dplyr::pull(.x, "concept_id"),
                       "concept_name" = NA_character_)

      )
    }
  }

  if(inherits(codelist, "candidate_codes")){
    codelist <- removeClass(codelist, "candidate_codes")
    codelist <- list("codelist" = codelist |>
                       dplyr::select("concept_id", "concept_name"))
    names(codelist) <- name
  }

  return(codelist)
}

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Subset a codelist to only those with a particular route category
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams routeCategoryDoc
#' @param negate If FALSE, only concepts with the routeCategory specified will
#' be returned. If TRUE, concepts with the routeCategory specified will be excluded.
#'
#' @return The codelist with only those concepts associated with the
#' specified route categories (if negate is FALSE) or the codelist without those
#' concepts associated with the specified route categories (if negate is TRUE).
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef()
#' codes <- subsetOnRouteCategory(
#'               x = newCodelist(list("codes" = c(20,21))),
#'               cdm = cdm,
#'               routeCategory = "topical")
#' codes
#' }
subsetOnRouteCategory <- function(x,
                                  cdm,
                                  routeCategory,
                                  negate = FALSE){

  omopgenerics::assertCharacter(routeCategory, null = FALSE, na = FALSE)
  st <- searchStrategyAttr(
    function_name = "subsetOnRouteCategory",
    cdm = "cdm",
    routeCategory = cast(routeCategory)
  )
  x <- subsetCodelistBy(x,
                        cdm,
                        by = "route_category",
                        group = routeCategory,
                        keepOriginal = FALSE,
                        st = st,
                        negate = negate)
  return(x)
}

test_that("subset on route category", {

  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backends[[i]])

    # codelist
    codelist <- newCodelist(list("codes1" = c(10L, 14L),
                                 "codes2" = c(11L, 12L)))
    expect_warning(x <- subsetOnRouteCategory(codelist, cdm, routeCategory = c("injectable"), negate = FALSE))
    expect_true(inherits(x, "codelist"))
    expect_true(x$codes1 == 10L)
    expect_no_error(x <- subsetOnRouteCategory(codelist, cdm, routeCategory = c("injectable"), negate = TRUE))
    expect_true(inherits(x, "codelist"))
    expect_true(x$codes1 == c(14L))
    expect_identical(x$codes2, c(11L, 12L))
    expect_no_error(x1 <- subsetOnRouteCategory(codelist, cdm, routeCategory = c("unclassified route category"), negate = FALSE))
    expect_true(inherits(x1, "codelist"))
    expect_identical(x1, x)

    # codelist with details
    codelist_wd <- codelist |> asCodelistWithDetails(cdm)
    expect_warning(x <- subsetOnRouteCategory(codelist_wd,  cdm, "injectable"))
    expect_true(inherits(x, "codelist_with_details"))
    expect_identical(x$codes1 |> dplyr::pull("concept_id"), 10L)

    # check empty codelist
    expect_warning(x <- subsetOnRouteCategory(codelist, cdm, "oral"))
    expect_true(length(x) == 0)
    expect_true(inherits(x, "codelist"))

    # check empty codelist_with_details
    expect_warning(x <- subsetOnRouteCategory(codelist_wd, cdm, "oral"))
    expect_true(length(x) == 0)
    expect_true(inherits(x, "codelist_with_details"))

    # expected errors
    expect_error(subsetOnRouteCategory(list("a" = 1L),  cdm, "oral"))
    expect_error(subsetOnRouteCategory(codelist,  "a", "oral"))
    expect_error(subsetOnRouteCategory(codelist,  cdm, 1234L))

    # Check lower case
    codelist <- newCodelist(list("a" = c(21L, 22L)))
    topical1 <- subsetOnRouteCategory(codelist, cdm, routeCategory = "topical")
    topical3 <- subsetOnRouteCategory(codelist, cdm, routeCategory = "Topical")
    expect_equal(topical1, topical3)

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Add concepts to a codelist
#'
#' @inheritParams cdmDoc
#' @inheritParams xDoc
#' @param concepts Concepts_ID to add
#' @inheritParams codelistNameDoc
#'
#' @return A codelist
#' @export
#'
#' @examples
#' \donttest{
#' library(omock)
#' library(CDMConnector)
#'
#' # Creating CDM object
#' cdm <- mockCdmFromDataset(datasetName = "GiBleed")
#'
#' # Creating codelist
#' codelist <- getDrugIngredientCodes(cdm,
#'                                    nameStyle = "{concept_name}")
#'
#' # Add a concept to all the codelists:
#' codelist$acetaminophen
#' codelist <- codelist |>
#'  addConcepts(cdm, concepts = c(1L))
#' codelist$acetaminophen
#'
#' # Add a concept to a specific codelist
#' codelist$amiodarone
#' codelist <- codelist |>
#'   addConcepts(cdm, concepts = c(2L), codelistName = "amiodarone")
#' codelist$amiodarone
#'
#' # See function: `excludeConcepts()` for details on how to remove specific concepts
#' # from a codelist
#' }
addConcepts <- function(x,
                        cdm,
                        concepts,
                        codelistName = NULL){

  # initial checks
  checkCodelist(x)
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  omopgenerics::assertNumeric(concepts, integerish = TRUE, min = 0)
  omopgenerics::assertCharacter(codelistName, null = TRUE)

  no_present <- setdiff(codelistName, names(x))
  if (length(no_present) > 0) {
    cli::cli_abort("{.val {no_present}} codelist{?s} {?does/do} not exist in {.obj x}.")
  }

  if(is.null(codelistName)){
    codelistName <- names(x)
  }

  # Unique concepts
  concepts <- unique(concepts)

  concepts <- list("new_codelist" = concepts) |>
    omopgenerics::newCodelist() |>
    suppressWarnings()

  if(inherits(x, "codelist_with_details")){
    concepts <- concepts |>
      CodelistGenerator::asCodelistWithDetails(cdm)
    x[codelistName] <- purrr::map(x[codelistName], ~ dplyr::bind_rows(.x, concepts$new_codelist))
    x[codelistName] <- purrr::map(x[codelistName], ~ dplyr::distinct(.x) %>% dplyr::arrange(concept_id))
  }

  if(inherits(x, "codelist")){
    x[codelistName] <- purrr::map(x[codelistName], ~ append(.x, concepts$new_codelist))
    x[codelistName] <- purrr::map(x[codelistName], ~ sort(unique(.x)))
  }

  return(x)
}

test_that("addConcepts works", {
  cdm <- mockVocabRef(backend = "data_frame")

  # Test codelist
  codelist <- omopgenerics::newCodelist(list(
      "conditions" = c(1L),
      "drug" = c(10L),
      "other" = c(13L)))

  expect_no_error(codelist1 <- addConcepts(codelist,
                                          cdm,
                                          concepts = c(10L, 13L),
                                          codelistName = "conditions"))
  expect_identical(names(codelist1), names(codelist))
  expect_identical(class(codelist1), class(codelist))
  expect_identical(codelist1$conditions, c(1L, 10L, 13L))
  expect_identical(codelist1$drug, c(10L))
  expect_identical(codelist1$other, c(13L))

  # Test codelist with details
  codelist <- codelist |> asCodelistWithDetails(cdm)

  expect_no_error(codelist1 <- addConcepts(x = codelist,
                                           cdm,
                                           concepts = c(10L, 13L),
                                           codelistName = "conditions"))
  expect_identical(names(codelist1), names(codelist))
  expect_identical(class(codelist1), class(codelist))
  expect_identical(codelist1$conditions,
                   dplyr::tibble(
                     "concept_id" = c(1L, 10L, 13L),
                     "concept_name" = c("Musculoskeletal disorder", "Adalimumab", "Descendant drug"),
                     "domain_id" = c("Condition", "Drug", "Drug"),
                     "vocabulary_id" = c("SNOMED", "RxNorm", "RxNorm"),
                     "standard_concept" = rep("standard",3)
                   ))
  expect_identical(codelist1$drug,
                   dplyr::tibble(
                     "concept_id" = 10L,
                     "concept_name" = "Adalimumab",
                     "domain_id" = "Drug",
                     "vocabulary_id" = "RxNorm",
                     "standard_concept" = "standard"
                   ))


  # Test codelist + descendants + all concepts
  expect_no_error(codelist1 <- addConcepts(codelist,
                                           cdm,
                                           concepts = c(1L),
                                           codelistName = NULL))
  expect_identical(nrow(codelist1$conditions), 1L)
  expect_identical(nrow(codelist1$drug), 2L)
  expect_identical(nrow(codelist1$other), 2L)

  # Expected errors
  expect_error(addConcepts("hola", cdm, concepts = 1L))
  expect_error(addConcepts(list("a" = 1L), cdm, concepts = 1L))
  expect_error(addConcepts(codelist, "hola", concepts = 1L))
  expect_error(addConcepts(codelist, cdm, concepts = "hola"))
  expect_error(addConcepts(codelist, cdm, concepts = 1L, codelistName = "hola"))
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#' Stratify a codelist by dose form.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @param nameStyle Naming of the new codelists, use `{codelist_name}` to
#' include the codelist name and `{dose_form}` to include the dose form name.
#' @inheritParams keepOriginalDoc
#'
#' @return The codelist with the required stratifications, as different elements
#' of the list.
#' @export
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#'
#' cdm <- mockVocabRef()
#'
#' codes <- newCodelist(list("codes" = c(10L, 20L, 21L)))
#' new_codes <- stratifyByDoseForm(x = codes,
#'                                 cdm = cdm,
#'                                 keepOriginal = TRUE)
#' new_codes
#' }
stratifyByDoseForm <- function(x,
                               cdm,
                               nameStyle = "{codelist_name}_{dose_form}",
                               keepOriginal = FALSE) {
  st <- searchStrategyAttr(
    function_name = "stratifyByDoseForm",
    cdm = "cdm"
  )
  stratifyCodelistBy(
    x = x,
    cdm = cdm,
    by = "dose_form",
    nameStyle = nameStyle,
    st = st,
    keepOriginal = keepOriginal
  )
}

addDoseFormId <- function(x) {
  cdm <- omopgenerics::cdmReference(table = x)
  x |>
    # add dose_form
    dplyr::left_join(
      cdm$concept_relationship |>
        dplyr::filter(.data$relationship_id == "RxNorm has dose form") |>
        dplyr::select(
          "concept_id" = "concept_id_1",
          "dose_form_concept_id" = "concept_id_2"
        ),
      by = "concept_id"
    ) |>
    dplyr::mutate(dose_form_concept_id = as.integer(.data$dose_form_concept_id)) |>
    dplyr::distinct()
}

addDoseForm <- function(x, keepDiscordant = FALSE) {
  cdm <- omopgenerics::cdmReference(table = x)
  x_dose_forms <- x |>
    addDoseFormId() |>
    dplyr::collect() |>
    dplyr::left_join(
      CodelistGenerator::doseFormToRoute |>
        dplyr::select(
          "dose_form_concept_id",
          "dose_form" = "dose_form_concept_name"
        ),
      by = "dose_form_concept_id"
    ) |>
    dplyr::select(!"dose_form_concept_id") |>
    dplyr::distinct()

  # can also try get dose forms by going via constitutes relationship
  # only keep if they have 1 associateddose form
  x_dose_forms_via_constitutes <-  x |>
    addDoseFormId() |>
    dplyr::select("codelist_name",
                  "concept_id_1" = "concept_id") |>
    dplyr::distinct() |>
    dplyr::inner_join(cdm$concept_relationship |>
                        dplyr::filter(.data$relationship_id == "Constitutes"),
                      by = "concept_id_1") |>
    dplyr::select("codelist_name",
                  "x_concept_id" = "concept_id_1",
                  "concept_id" = "concept_id_2") |>
    addDoseFormId() |>
    dplyr::collect()  |>
    dplyr::left_join(
      CodelistGenerator::doseFormToRoute |>
        dplyr::select(
          "dose_form_concept_id",
          "dose_form" = "dose_form_concept_name"
        ),
      by = "dose_form_concept_id"
    ) |>
    dplyr::select("codelist_name",
                  "concept_id" = "x_concept_id",
                  "dose_form") |>
    dplyr::distinct()


  # if multiple values and some are missing, keep only non-missing
  # ie so do not get considered to have unclassified dose form
  dose_forms <- x_dose_forms |>
    dplyr::group_by(.data$concept_id) |>
    dplyr::filter(!is.na(.data$dose_form) | all(is.na(.data$dose_form))) |>
    dplyr::ungroup()

  # for those without dose form, try and fill in from the constitutes relationship
  dose_forms <- dplyr::bind_rows(
    dose_forms |>
      dplyr::filter(!is.na(.data$dose_form)),
    dose_forms |>
      dplyr::filter(is.na(.data$dose_form)) |>
      dplyr::select("codelist_name",
                    "concept_id") |>
      dplyr::left_join(x_dose_forms_via_constitutes,
                       by = c("codelist_name",
                              "concept_id")))

  # if multiple discordant route, set to unknown
  if(isFALSE(keepDiscordant)){
    dose_forms <- dose_forms |>
      dplyr::group_by(.data$codelist_name, .data$concept_id) |>
      dplyr::mutate(discordant = dplyr::if_else(dplyr::n() == 1, 1L, 0L)) |>
      dplyr::mutate(dose_form = dplyr::if_else(.data$discordant == 1,
                                               .data$dose_form,
                                               NA)) |>
      dplyr::ungroup() |>
      dplyr::select(!"discordant") |>
      dplyr::distinct()
  }

  dose_forms

}

test_that("stratifyByDoseForm works", {
  cdm <- mockVocabRef()
  n1 <- names(cdm)

  # codelist
  codes <- newCodelist(list("codes1" = c(10L, 20L),
                            "codes2" = c(21L, 22L)))
  expect_no_error(new_codes <- stratifyByDoseForm(x = codes,
                                                  cdm = cdm,
                                                  keepOriginal = FALSE))
  expect_equal(names(cdm), n1)
  expect_true(inherits(new_codes, "codelist"))
  expect_equal(names(new_codes), c(
    "codes1_injection", "codes1_nasal_powder",
    "codes2_topical_liquefied_gas", "codes2_unclassified_dose_form"
  ))
  expect_true(new_codes$codes1_injection == 10L)
  expect_true(new_codes$codes1_nasal_powder == 20L)
  expect_true(new_codes$codes2_topical_liquefied_gas == 21L)
  expect_true(new_codes$codes2_unclassified_dose_form == 22L)

  # codelist with keep original
  expect_no_error(new_codes <- stratifyByDoseForm(x = codes,
                                                   cdm = cdm,
                                                   keepOriginal = TRUE))
  expect_equal(names(new_codes), c(
    "codes1","codes1_injection", "codes1_nasal_powder", "codes2",
    "codes2_topical_liquefied_gas", "codes2_unclassified_dose_form"
  ))

  # codelist with details
  codes <- codes |> asCodelistWithDetails(cdm)
  expect_no_error(new_codes <- stratifyByDoseForm(x = codes,
                                                   cdm = cdm,
                                                   keepOriginal = FALSE))
  expect_equal(names(cdm), n1)
  expect_true(inherits(new_codes, "codelist_with_details"))
  expect_equal(names(new_codes), c(
    "codes1_injection", "codes1_nasal_powder",
    "codes2_topical_liquefied_gas", "codes2_unclassified_dose_form"
  ))
  expect_equal(new_codes$codes1_injection,
               dplyr::tibble("concept_id" = 10L,
                             "concept_name" = "Adalimumab",
                             "domain_id" = "Drug",
                             "vocabulary_id" = "RxNorm",
                             "standard_concept" = "standard"))

  # codelist with details + keep originals
  expect_no_error(new_codes <- stratifyByDoseForm(x = codes,
                                                  cdm = cdm,
                                                  keepOriginal = TRUE))
  expect_equal(names(new_codes),
               c("codes1", "codes1_injection", "codes1_nasal_powder",
                 "codes2", "codes2_topical_liquefied_gas", "codes2_unclassified_dose_form"))

  # expected errors
  expect_error(stratifyByDoseForm(x = list("a" = 1L), cdm))
  expect_error(stratifyByDoseForm(x = codes, "a"))
  expect_error(stratifyByDoseForm(x = codes, cdm, keepOriginal = "a"))

})

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Subset a codelist to only those codes from a particular domain.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @param doseForm Dose form/s. See `availableDoseForms()` to explore available dose forms
#' in your codelist.
#' @param negate If FALSE, only concepts with the dose form specified will be
#' returned. If TRUE, concepts with the dose form specified will be excluded.
#'
#' @return The codelist with only those concepts associated with the dose form
#' (if negate = FALSE) or the codelist without those concepts associated with
#' the dose form (if negate = TRUE).
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef()
#'
#' codelist <- newCodelist(list("codes" = c(10L,20L,21L)))
#'
#' # Dose forms present in our codelist:
#' codelist |> associatedDoseForms(cdm)
#'
#' codes <- subsetOnDoseForm(
#'               x = codelist,
#'               cdm = cdm,
#'               doseForm = "Injection")
#' codes
#'
#' codes |> associatedDoseForms(cdm)
#' }
#'

subsetOnDoseForm <- function(x, cdm, doseForm, negate = FALSE){

  # Initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm)
  omopgenerics::assertCharacter(doseForm)
  checkCodelist(x, allowConceptSetExpression = FALSE)
  omopgenerics::assertLogical(negate, length = 1)

  x_original <- x
  if(inherits(x_original, "codelist_with_details")){
    x <- asCodelist(x)
    newX <- omopgenerics::emptyCodelistWithDetails()
  }
  if(inherits(x_original, "codelist")){
    newX <- omopgenerics::emptyCodelist()
  }

  tableCodelist <- paste0(omopgenerics::uniqueTableName(),
                          omopgenerics::uniqueId())

  for(i in seq_along(x)){
    cdm <- omopgenerics::insertTable(cdm = cdm,
                                     name = tableCodelist,
                                     table = dplyr::tibble(concept_id = x[[i]]),
                                     overwrite = TRUE,
                                     temporary = FALSE)

    cdm[[tableCodelist]] <- cdm[[tableCodelist]] |>
      dplyr::left_join(cdm[["concept_relationship"]] |>
                          dplyr::filter(.data$relationship_id == "RxNorm has dose form"),
                        by = c("concept_id" = "concept_id_1")) |>
      dplyr::select("concept_id", "dose_form_concept" = "concept_id_2") |>
      dplyr::distinct() |>
      dplyr::compute(temporary = FALSE, name = tableCodelist)

    x[[i]] <- cdm[[tableCodelist]] |>
      dplyr::left_join(
        cdm[["concept"]] |>
          dplyr::rename("dose_form_concept" = "concept_id"),
        by = c("dose_form_concept")
      ) |>
      dplyr::select("concept_id", "dose_form" = "concept_name") |>
      dplyr::distinct() |>
      dplyr::collect()

    if(isTRUE(negate)){
      x[[i]] <- x[[i]] |>
        dplyr::filter(!tolower(.data$dose_form) %in% tolower(.env$doseForm))
    }else{
      x[[i]] <- x[[i]] |>
        dplyr::filter(tolower(.data$dose_form) %in% tolower(.env$doseForm))
    }

    x[[i]] <- x[[i]] |>
      dplyr::pull("concept_id") |>
      unique() |>
      sort()

    if(inherits(x_original, "codelist_with_details")){
      newX[[names(x_original)[[i]]]] <- x_original[[i]] |>
        dplyr::inner_join(
          dplyr::tibble("concept_id" = x[[i]]),
          by = "concept_id"
        )
    }
    if(inherits(x_original, "codelist")){
      newX[[names(x_original)[[i]]]] <- x[[i]]
    }
  }

  newX <- dropEmptyCodelist(x_original, newX)

  cdm <- omopgenerics::dropSourceTable(cdm = cdm, name = tableCodelist)

  return(newX)
}

test_that("subsetOnDoseForm works", {

  cdm <- mockVocabRef()

  codelist <- newCodelist(list("codes" = c(10L,20L,21L,22L)))

  expect_no_error(codes <- subsetOnDoseForm(
    x = codelist,
    cdm = cdm,
    doseForm = "Injection"))

  expect_true(inherits(codes, "codelist"))
  expect_true(names(codes) == "codes")
  expect_identical(codes$codes, 10L)

  # codelist + negate
  expect_no_error(codes <- subsetOnDoseForm(
    x = codelist,
    cdm = cdm,
    doseForm = "Injection",
    negate = TRUE))

  expect_true(inherits(codes, "codelist"))
  expect_true(names(codes) == "codes")
  expect_identical(codes$codes, c(20L, 21L,22L))

  # codelist with details
  codelist <- asCodelistWithDetails(codelist, cdm)

  expect_no_error(codes <- subsetOnDoseForm(
    x = codelist,
    cdm = cdm,
    doseForm = c("Injection","Nasal Powder")))

  expect_true(inherits(codes, "codelist_with_details"))
  expect_true(names(codes) == "codes")
  expect_identical(codes$codes,
                   dplyr::tibble("concept_id" = c(10L, 20L),
                                 "concept_name" = c("Adalimumab", "glucagon Nasal Powder"),
                                 "domain_id" = rep("Drug",2),
                                 "vocabulary_id" = rep("RxNorm",2),
                                 "standard_concept" = rep("standard",2)))

  # codelist + negate
  expect_no_error(codes <- subsetOnDoseForm(
    x = codelist,
    cdm = cdm,
    doseForm = c("Injection"),
    negate = TRUE))

  expect_true(inherits(codes, "codelist_with_details"))
  expect_true(names(codes) == "codes")
  expect_identical(codes$codes |> dplyr::pull("concept_id"), c(20L, 21L,22L))

  # Empty codelist
  expect_warning(codes <- subsetOnDoseForm(
    x = codelist,
    cdm = cdm,
    doseForm = c("a"),
    negate = FALSE))

  # Multiple elements
  codelist <- omopgenerics::newCodelist(list("codes1" = c(10L,20L), "codes2" = c(21L,22L)))
  expect_no_error(codes <- subsetOnDoseForm(
    x = codelist,
    cdm = cdm,
    doseForm = c("Injection", "Topical Liquefied Gas"),
    negate = FALSE))
  expect_true(inherits(codes, "codelist"))
  expect_identical(names(codes), c("codes1", "codes2"))

  # lower case
  expect_no_error(codes1 <- subsetOnDoseForm(
    x = codelist,
    cdm = cdm,
    doseForm = c("injection", "topical liquefied gas"),
    negate = FALSE))
  expect_identical(codes, codes1)

  # check empty codelists
  x <- subsetOnDoseForm(omopgenerics::emptyCodelist(), cdm, doseForm = "Injection")
  expect_true(inherits(x, "codelist"))
  x <- subsetOnDoseForm(omopgenerics::emptyCodelistWithDetails(), cdm, doseForm = "Injection")
  expect_true(inherits(x, "codelist_with_details"))

  # expected errors
  expect_error(subsetOnDoseForm(list("a" = 1L), cdm, doseForm = "Injection"))
  expect_error(subsetOnDoseForm(codelist, "a", doseForm = "Injection"))
  expect_error(subsetOnDoseForm(codelist, cdm, doseForm = 1L))
  expect_error(subsetOnDoseForm(codelist, cdm, doseForm = "Injection", negate = "a"))

})

#' Find orphan codes related to a codelist using achilles counts and, if
#' available, PHOEBE concept recommendations
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams domainDoc
#'
#' @return A summarised result containing the frequency of codes related
#' to (but not in) the codelist.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#'
#' cdm <- mockVocabRef("database")
#' codes <- getCandidateCodes(cdm = cdm,
#'                           keywords = "Musculoskeletal disorder",
#'                           domains = "Condition",
#'                           includeDescendants = FALSE)
#' codelist <- omopgenerics::newCodelist(list("msk" = codes$concept_id))
#' orphan_codes <- summariseOrphanCodes(x = codelist,
#'                                       cdm = cdm)
#'
#' orphan_codes
#' CDMConnector::cdmDisconnect(cdm)
#' }
summariseOrphanCodes <- function(x,
                                 cdm,
                                 domain = c("condition",
                                            "device",
                                            "drug",
                                            "measurement",
                                            "observation",
                                            "procedure",
                                            "visit")){
  omopgenerics::validateCdmArgument(cdm,
                                    requiredTables = c("achilles_analysis",
                                                       "achilles_results",
                                                       "achilles_results_dist"))
  checkCodelist(x, allowConceptSetExpression = FALSE)
  domain <- assertDomain(domains = domain, cdm = cdm)

  if(inherits(x, "codelist_with_details")){
    x <- asCodelist(x)
  }

  # will only return codes that are used in the database
  # which we get from achilles tables
  codesUsed <- fetchAchillesCodesInUse(cdm = cdm,
                                       minimumCount = 0,
                                       collect = FALSE)
  descendantsUsed <- cdm$concept_ancestor |>
    dplyr::inner_join(codesUsed,
                      by = c("descendant_concept_id"="concept_id"))
  ancestorsUsed <- cdm$concept_ancestor |>
    dplyr::inner_join(codesUsed,
                      by = c("ancestor_concept_id"="concept_id"))
  relationshipUsed1 <- cdm$concept_relationship |>
    dplyr::inner_join(codesUsed,
                      by = c("concept_id_1"="concept_id"))
  relationshipUsed2 <- cdm$concept_relationship |>
    dplyr::inner_join(codesUsed,
                      by = c("concept_id_2"="concept_id"))

  orphanCodes <- list()
  tableCodelist <- paste0(omopgenerics::uniqueTableName(),
                          omopgenerics::uniqueId())

  for(i in seq_along(x)){
    cli::cli_inform("Getting orphan codes for {names(x)[i]}")
    cdm <- omopgenerics::insertTable(cdm = cdm,
                                     name = tableCodelist,
                                     table = dplyr::tibble("concept_id" = x[[i]]),
                                     overwrite = TRUE,
                                     temporary = FALSE)

    # get descendants used in db
    orphanDescendants <- cdm[[tableCodelist]] |>
      dplyr::inner_join(descendantsUsed,
                        by = c("concept_id" = "ancestor_concept_id")) |>
      dplyr::select("concept_id" = "descendant_concept_id") |>
      dplyr::filter(!is.na(.data$concept_id)) |>
      dplyr::distinct() |>
      dplyr::mutate("relationship" = "Descendant") |>
      dplyr::collect()

    # get direct ancestors used in db
    orphanAncestors <- cdm[[tableCodelist]] |>
      dplyr::left_join(ancestorsUsed,
                       by = c("concept_id" = "descendant_concept_id")) |>
      dplyr::filter(.data$min_levels_of_separation == 1) |>
      dplyr::select("concept_id" = "ancestor_concept_id")  |>
      dplyr::filter(!is.na(.data$concept_id)) |>
      dplyr::distinct() |>
      dplyr::mutate("relationship" = "Ancestor") |>
      dplyr::collect()

    # get relationship 1
    orphanRelationship1 <- cdm[[tableCodelist]] |>
      dplyr::left_join(relationshipUsed1,
                       by = c("concept_id" = "concept_id_2")) |>
      dplyr::select("concept_id" = "concept_id_1",
                    "relationship" = "relationship_id")  |>
      dplyr::filter(!is.na(.data$concept_id)) |>
      dplyr::distinct() |>
      dplyr::collect()

    # get relationship 2
    orphanRelationship2 <- cdm[[tableCodelist]] |>
      dplyr::left_join(relationshipUsed1,
                       by = c("concept_id" = "concept_id_1")) |>
      dplyr::select("concept_id" = "concept_id_2",
                    "relationship" = "relationship_id")  |>
      dplyr::filter(!is.na(.data$concept_id)) |>
      dplyr::distinct() |>
      dplyr::collect()

    orphanCodes[[names(x)[i]]] <- orphanDescendants |>
      dplyr::bind_rows(orphanAncestors) |>
      dplyr::bind_rows(orphanRelationship1) |>
      dplyr::bind_rows(orphanRelationship2)

    # make sure we don't have any of the original codes
    orphanCodes[[names(x)[i]]] <- orphanCodes[[names(x)[i]]] |>
      dplyr::filter(! .data$concept_id %in% x[[i]])

    # Merge rows of same concept_id but multiple relationships
    orphanCodes[[names(x)[i]]] <- orphanCodes[[names(x)[i]]] |>
      dplyr::group_by(.data$concept_id) |>
      dplyr::summarise("relationship" = paste(.data$relationship, collapse = ", "), .groups = "drop")
  }

  orphanCodes <- orphanCodes |> vctrs::list_drop_empty()

  if(length(orphanCodes) == 0){
    orphanCodesSummarisedResult <- omopgenerics::emptySummarisedResult() |>
      omopgenerics::newSummarisedResult(
        settings = dplyr::tibble(
          result_id = 1L,
          result_type = "orphan_code_use",
          package_name = "CodelistGenerator",
          package_version = as.character(utils::packageVersion(
            pkg = "CodelistGenerator"))
        )
      )
  } else {
    # Convert to a list
    orphanCodesList <- purrr::map(orphanCodes, ~. |> dplyr::pull("concept_id"))

    # Subset on domain
    orphanCodesList <- orphanCodesList |>
      omopgenerics::newCodelist() |>
      subsetOnDomain(cdm = cdm, domain = domain)

    orphanCodesSummarisedResult <- summariseAchillesCodeUse(orphanCodesList, cdm = cdm)

    # Add relationship
    if(nrow(orphanCodesSummarisedResult) > 0){
    orphanCodesSummarisedResult <- orphanCodesSummarisedResult |>
      omopgenerics::splitAdditional() |>
      dplyr::left_join(
        dplyr::bind_rows(orphanCodes) |>
          dplyr::rename("variable_level" = "concept_id") |>
          dplyr::mutate("variable_level" = as.character(.data$variable_level)),
        by = "variable_level"
      ) |>
      omopgenerics::uniteAdditional(cols = c(omopgenerics::additionalColumns(orphanCodesSummarisedResult),
                                    "relationship"))
    }

    attr(orphanCodesSummarisedResult, "settings")$result_type <- "orphan_code_use"
  }

  return(orphanCodesSummarisedResult)

}

fetchAchillesCodesInUse <- function(cdm, minimumCount = 0L, collect = TRUE){

  minimumCount <- as.integer(minimumCount)
  codes <- cdm[["achilles_results"]] |>
    dplyr::filter(.data$analysis_id %in%
                    c(
                      401L, # condition occurrence
                      701L, # drug_exposure
                      801L, # observation
                      1801L, # measurement
                      201L, # visit_occurrence
                      601L, # procedure_occurrence
                      2101L # device_exposure
                    ),
                  .data$count_value >= .env$minimumCount) |>
    dplyr::select("concept_id" = "stratum_1") |>
    dplyr::mutate(concept_id = as.integer(.data$concept_id)) |>
    dplyr::distinct()

  if(isTRUE(collect)){
    codes <- codes |>
      dplyr::pull("concept_id")
  }

  codes

}

fetchAchillesSourceCodesInUse <- function(cdm, minimumCount = 0L){

  minimumCount <- as.integer(minimumCount)

  cdm[["achilles_results"]] |>
    dplyr::filter(.data$analysis_id %in%
                    c(
                      425L, # condition occurrence
                      725L, # drug_exposure
                      825L, # observation
                      1825L, # measurement
                      225L, # visit_occurrence
                      625L, # procedure_occurrence
                      2125L # device_exposure
                    )) |>
    dplyr::filter(.data$count_value >= .env$minimumCount) |>
    dplyr::select("stratum_1") |>
    dplyr::distinct() |>
    dplyr::mutate(stratum_1 = as.integer(.data$stratum_1)) |>
    dplyr::pull("stratum_1")
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get the available version of the vocabulary used in the cdm
#'
#' @inheritParams cdmDoc
#'
#' @return The vocabulary version being used in the cdm.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' cdm <- mockVocabRef()
#' vocabularyVersion(cdm = cdm)
#' }
vocabularyVersion <- function(cdm) {
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)

  version <- as.character(cdm$vocabulary |>
                            dplyr::rename_with(tolower) |>
                            dplyr::filter(.data$vocabulary_id == "None") |>
                            dplyr::select("vocabulary_version") |>
                            dplyr::collect())
  return(version)
}

addIngredientCount <- function(cdm, concepts) {
  ingredient_ancestor <- cdm$concept_ancestor |>
    dplyr::inner_join(cdm$concept |>
                        dplyr::filter(.data$concept_class_id == "Ingredient",
                                      .data$standard_concept == "S") |>
                        dplyr::select("concept_id"),
                      by = c("ancestor_concept_id" = "concept_id"))

  ingredient_count <- concepts |>
    dplyr::distinct(.data$concept_id) |>
    dplyr::left_join(ingredient_ancestor,
                     by = c("concept_id" = "descendant_concept_id")) |>
    dplyr::count(
      .data$concept_id,
      wt = dplyr::if_else(is.na(.data$ancestor_concept_id), 0L, 1L),
      name = "ingredient_count"
    )

  concepts <- concepts |>
    dplyr::left_join(ingredient_count,
                     by = "concept_id") |>
    dplyr::collect() |>
    dplyr::distinct()
  return(concepts)
}

test_that("tests with mock db", {
  skip_on_cran()
  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])
    namesInitial <- names(cdm)
    codelist_with_details <- list("codes" = cdm[["concept"]] |>
                                    dplyr::filter(concept_id %in% c(10L, 15L, 24L)) |>
                                    dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
                                    dplyr::mutate("ancestor_concept_id" = 999L) |>
                                    dplyr::collect()) |>
      omopgenerics::newCodelistWithDetails()

    version <- vocabularyVersion(cdm = cdm)
    expect_true(length(version) == 1)
    expect_true(is.character(version))

    descendants1 <- getDescendants(
      cdm = cdm,
      conceptId = 1,
      withAncestor = FALSE
    )
    expect_true(all(sort(descendants1$concept_id) == c(1, 2, 3, 4, 5)))
    expect_true("concept_name" %in% colnames(descendants1))

    descendants2 <- getDescendants(
      cdm = cdm,
      conceptId = 1,
      withAncestor = TRUE
    )
    expect_true(all(sort(descendants2$concept_id) == c(1, 2, 3, 4, 5)))
    expect_true("ancestor_concept_id" %in% colnames(descendants2))
    expect_true(all(descendants2$ancestor_concept_id == 1))

    # expected errors
    expect_error(vocabularyVersion(cdm = "a"))
    expect_error(getDescendants(cdm = "a"))
    expect_error(availableDoseForm(cdm = "a"))
    expect_error(availableDoseForm(cdm, x = 123))
    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

buildAchillesTables <- function(cdm,
                                achillesId = NULL) {
  # initial check
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  achillesId <- validateAchillesId(achillesId = achillesId)

  # check existent tables
  cdm <- checkExistentAchillesTables(cdm = cdm)

  # remove repeated results
  cdm <- removeRepeatedIds(cdm = cdm, achillesId = achillesId)

  # create new analysis
  len <- sprintf("%i", length(achillesId))
  for (k in seq_along(achillesId)) {
    id <- achillesId[k]
    nm <- achillesAnalisisDetails |>
      dplyr::filter(.data$analysis_id == .env$id) |>
      dplyr::pull("analysis_name")
    kk <- sprintf("%i", k) |>
      stringr::str_pad(width = nchar(len), pad = " ")
    tab <- achillesAnalisisDetails$table[k]
    if(tab %in% names(cdm)){
      cli::cli_inform(c("i" = "{kk} of {len}: Get achilles result for {.pkg {nm}}."))
      cdm <- appendAchillesId(cdm, id)
    }else{
      cli::cli_inform(c("i" = "{kk} of {len}: Skipping achilles results for {.pkg {nm}} because the cdm doesn't contain table {tab}."))
    }
  }

  # append to achilles_analysis
  name <- omopgenerics::uniqueTableName()
  cdm <- omopgenerics::insertTable(
    cdm = cdm,
    name = name,
    table = achillesAnalisisDetails |>
      dplyr::filter(.data$analysis_id %in% .env$achillesId) |>
      dplyr::select(!c("table", "type", "distribution", "distributed_field"))
  )
  cdm[["achilles_analysis"]] <- cdm[["achilles_analysis"]] |>
    dplyr::union_all(cdm[[name]]) |>
    dplyr::compute(name = "achilles_analysis")
  cdm <- omopgenerics::dropSourceTable(cdm = cdm, name = name)

  return(cdm)
}

validateAchillesId <- function(achillesId, call = parent.frame()) {
  # possible ids
  possibleIds <- achillesAnalisisDetails$analysis_id

  # default analysis_id
  if (is.null(achillesId)) {
    achillesId <- possibleIds
  } else {
    achillesId <- as.integer(achillesId)
    omopgenerics::assertNumeric(achillesId, integerish = TRUE, unique = TRUE, call = call)
    ignored <- achillesId[!achillesId %in% possibleIds]
    if (length(ignored) > 0) {
      cli::cli_inform(c("i" = "{length(ignored)} analysis id{?s} {?is/are} not configured: {ignored}."))
    }
    achillesId <- achillesId[achillesId %in% possibleIds]
  }

  achillesId
}
checkExistentAchillesTables <- function(cdm) {
  notPresent <- omopgenerics::achillesTables() |>
    purrr::keep(\(x) !x %in% names(cdm))
  if (length(notPresent) > 0) {
    possibleToRead <- notPresent[notPresent %in% omopgenerics::listSourceTables(cdm = cdm)]
    if (length(possibleToRead) > 0) {
      cli::cli_inform(c(i = "Reading tables from source: {.pkg {possibleToRead}}"))
      cdm <- omopgenerics::readSourceTable(cdm = cdm, name = possibleToRead)
    }
    needToCreate <- purrr::keep(notPresent, \(x) !x %in% names(cdm))
    if (length(needToCreate) > 0) {
      for (nm in needToCreate) {
        cli::cli_inform(c(i = "Creating empty {.pkg {nm}} table."))
        cdm <- omopgenerics::emptyAchillesTable(cdm = cdm, name = nm)
      }
    }
  }
  return(cdm)
}
removeRepeatedIds <- function(cdm, achillesId) {
  repeatedIds <- list(
    achilles_analysis = cdm[["achilles_analysis"]] |>
      dplyr::distinct(.data$analysis_id) |>
      dplyr::pull(),
    achilles_results = cdm[["achilles_results"]] |>
      dplyr::distinct(.data$analysis_id) |>
      dplyr::pull(),
    achilles_results_dist = cdm[["achilles_results_dist"]] |>
      dplyr::distinct(.data$analysis_id) |>
      dplyr::pull()
  ) |>
    purrr::map(\(x) x[x %in% achillesId]) |>
    purrr::compact()
  if (length(repeatedIds) > 0) {
    for (nm in names(repeatedIds)) {
      ids <- repeatedIds[[nm]]
      "Removing {length(ids)} repeated analysis ids from {.pkg {nm}}." |>
        rlang::set_names("!") |>
        cli::cli_inform()
      cdm[[nm]] <- cdm[[nm]] |>
        dplyr::filter(!.data$analysis_id %in% .env$ids) |>
        dplyr::compute(name = nm)
    }
  }
  return(cdm)
}
appendAchillesId <- function(cdm, id) {
  # get analysis results
  analysis <- achillesAnalisisDetails |>
    dplyr::filter(.data$analysis_id == .env$id)

  tableName <- analysis$table
  by <- groupBy(analysis)
  mut <- mutateColumns(analysis)

  fun <- switch(analysis$type,
                "record_count" = "dplyr::n()",
                "person_count" = "dplyr::n_distinct(.data$person_id)")
  q <- paste0("as.integer(", fun, ")") |>
    rlang::set_names("count_value") |>
    rlang::parse_exprs()

  nm <- omopgenerics::uniqueTableName()
  res <- cdm[[tableName]] |>
    dplyr::group_by(dplyr::across(dplyr::all_of(by))) |>
    dplyr::summarise(!!!q) |>
    dplyr::mutate(!!!mut, analysis_id = !!analysis$analysis_id) |>
    dplyr::compute(name = nm) |>
    dplyr::mutate(dplyr::across(dplyr::starts_with("stratum"), as.character))

  cdm[["achilles_results"]] <- cdm[["achilles_results"]] |>
    dplyr::union_all(res) |>
    dplyr::compute(name = "achilles_results")

  omopgenerics::dropSourceTable(cdm = cdm, name = nm)

  return(cdm)

}
groupBy <- function(analysis) {
  by <- analysis |>
    dplyr::select(dplyr::starts_with("stratum_")) |>
    as.list() |>
    unlist() |>
    purrr::keep(\(x) !is.na(x))
  names(by) <- stringr::str_remove(names(by), "_name$")
  return(by)
}
mutateColumns <- function(analysis) {
  mut <- analysis |>
    dplyr::select(dplyr::starts_with("stratum_")) |>
    as.list() |>
    purrr::keep(\(x) is.na(x))
  names(mut) <- stringr::str_remove(names(mut), "_name$")
  return(mut)
}

test_that("test buildAchillesTables", {
  skip_on_cran()
  con <- duckdb::dbConnect(duckdb::duckdb(), CDMConnector::eunomiaDir())
  cdm <- CDMConnector::cdmFromCon(
    con = con, cdmSchema = "main", writeSchema = "main"
  )
  expect_false(any(c("achilles_analysis", "achilles_results", "achilles_results_dist") %in% names(cdm)))
  expect_no_error(cdm <- buildAchillesTables(cdm = cdm))
  expect_true(all(c("achilles_analysis", "achilles_results", "achilles_results_dist") %in% names(cdm)))
  expect_identical(
    cdm$achilles_analysis |> dplyr::pull("analysis_id") |> sort(),
    achillesAnalisisDetails$analysis_id |> sort()
  )
  expect_identical(
    cdm$condition_occurrence |>
      dplyr::filter(condition_concept_id == 4112343) |>
      dplyr::distinct(.data$person_id) |>
      dplyr::tally() |>
      dplyr::pull() |>
      as.integer(),
    cdm$achilles_results |>
      dplyr::filter(.data$stratum_1 == "4112343", .data$analysis_id == 400) |>
      dplyr::pull("count_value")
  )
  expect_identical(
    cdm$condition_occurrence |>
      dplyr::filter(condition_concept_id == 4112343) |>
      dplyr::tally() |>
      dplyr::pull() |>
      as.integer(),
    cdm$achilles_results |>
      dplyr::filter(.data$stratum_1 == "4112343", .data$analysis_id == 401) |>
      dplyr::pull("count_value")
  )
  expect_identical(
    cdm$condition_occurrence |>
      dplyr::filter(condition_source_concept_id == 40479768) |>
      dplyr::tally() |>
      dplyr::pull() |>
      as.integer(),
    cdm$achilles_results |>
      dplyr::filter(.data$stratum_1 == "40479768", .data$analysis_id == 425) |>
      dplyr::pull("count_value")
  )

  omopgenerics::cdmDisconnect(cdm = cdm)
})

test_that("test buildAchillesTables when not all tables required are present", {
  cdm_local <- omock::mockCdmReference() |>
    omock::mockPerson(nPerson = 100) |>
    omock::mockObservationPeriod() |>
    omock::mockConditionOccurrence()

  con = DBI::dbConnect(duckdb::duckdb())

  cdm <- CDMConnector::copyCdmTo(con = con,
                                 cdm = cdm_local,
                                 schema = "main",
                                 overwrite = TRUE)
  attr(cdm, "write_schema") <- "main"

  expect_no_error(buildAchillesTables(cdm))

  omopgenerics::cdmDisconnect(cdm = cdm)
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' Stratify a codelist by dose unit.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @param nameStyle Naming of the new codelists, use `{codelist_name}` to
#' include the codelist name and `{dose_unit}` to include the dose unit name.
#' @inheritParams keepOriginalDoc
#'
#' @return The codelist with the required stratifications, as different elements
#' of the list.
#' @export
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#'
#' cdm <- mockVocabRef()
#'
#' codes <- newCodelist(list("concepts" = c(20L, 21L)))
#' new_codes <- stratifyByDoseUnit(x = codes,
#'                                 cdm = cdm,
#'                                 keepOriginal = TRUE)
#' new_codes
#' }
stratifyByDoseUnit <- function(x,
                               cdm,
                               nameStyle = "{codelist_name}_{dose_unit}",
                               keepOriginal = FALSE){
  st <- searchStrategyAttr(
    function_name = "stratifyByDoseUnit",
    cdm = "cdm"
  )
  stratifyCodelistBy(
    x = x,
    cdm = cdm,
    by = "dose_unit",
    nameStyle = nameStyle,
    st = st,
    keepOriginal = keepOriginal
  )
}

addDoseUnit <- function(x) {
  cdm <- omopgenerics::cdmReference(table = x)
  x |>
    # add amount unit and numerator unit
    dplyr::left_join(
      cdm$drug_strength |>
        dplyr::left_join(
          cdm$concept |>
            dplyr::select(
              "amount_unit_concept_id" = "concept_id",
              "amount_unit" = "concept_name"
            ),
          by = "amount_unit_concept_id"
        ) |>
        dplyr::left_join(
          cdm$concept |>
            dplyr::select(
              "numerator_unit_concept_id" = "concept_id",
              "numerator_unit" = "concept_name"
            ),
          by = "numerator_unit_concept_id"
        ) |>
        dplyr::select("concept_id" = "drug_concept_id", "amount_unit", "numerator_unit"),
      by = "concept_id"
    ) |>
    dplyr::collect() |>
    dplyr::mutate(dose_unit = dplyr::coalesce(.data$amount_unit, .data$numerator_unit)) |>
    dplyr::select(!c("amount_unit", "numerator_unit")) |>
    dplyr::distinct()
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' Stratify a codelist by route category.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @param nameStyle Naming of the new codelists, use `{codelist_name}` to
#' include the codelist name and `{route_category}` to include the route
#' category name.
#' @inheritParams keepOriginalDoc
#'
#' @return The codelist with the required stratifications, as different elements
#' of the list.
#' @export
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef()
#' codes <- newCodelist(list("concepts" = c(20,21,22)))
#' new_codes <- stratifyByRouteCategory(x = codes,
#'                                      cdm = cdm,
#'                                      keepOriginal = TRUE)
#' new_codes
#'}
stratifyByRouteCategory <- function(x,
                                    cdm,
                                    nameStyle = "{codelist_name}_{route_category}",
                                    keepOriginal = FALSE) {
  st <- searchStrategyAttr(
    function_name = "stratifyByRouteCategory",
    cdm = "cdm"
  )
  stratifyCodelistBy(
    x = x,
    cdm = cdm,
    by = "route_category",
    nameStyle = nameStyle,
    st = st,
    keepOriginal = keepOriginal
  )
}

addRouteCategory <- function(x) {

  x_route <- x |>
    addDoseForm(keepDiscordant = TRUE) |>
    dplyr::left_join(
      CodelistGenerator::doseFormToRoute |>
        dplyr::select("dose_form_concept_name", "route_category"),
      by = c("dose_form" = "dose_form_concept_name")
    ) |>
    dplyr::select(!"dose_form") |>
    dplyr::distinct()

  # if multiple values and some are missing, keep only non-missing
  # ie so do not get considered to have unclassified route
  x_route <- x_route |>
    dplyr::group_by(.data$concept_id) |>
    dplyr::filter(!is.na(.data$route_category) | all(is.na(.data$route_category))) |>
    dplyr::ungroup()


  # if multiple discordant route, set to unknown
  x_route <- x_route |>
    dplyr::group_by(.data$codelist_name, .data$concept_id) |>
    dplyr::mutate(discordant = dplyr::if_else(dplyr::n() == 1, 1L, 0L)) |>
    dplyr::mutate(route_category = dplyr::if_else(.data$discordant == 1,
                                                  .data$route_category,
                                                  NA)) |>
    dplyr::ungroup() |>
    dplyr::select(!"discordant") |>
    dplyr::distinct()

  x_route

}

# Argument descriptions repeated > 1:

#' Helper for consistent documentation of `codelistNameDoc`
#' @param codelistName Name or names of codelist in x. If NULL, all codelist present
#' in x will be considered.
#'
#' @name codelistNameDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `.options`.
#'
#' @param .options Named list with additional formatting options.
#' visOmopResults::tableOptions() shows allowed arguments and
#' their default values.
#'
#' @name .optionsDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `ageGroup`.
#'
#' @param ageGroup If not NULL, a list of ageGroup vectors of length two.
#'
#' @name ageGroupDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `byConcept`.
#'
#' @param byConcept TRUE or FALSE. If TRUE code use will be summarised by
#' concept.
#'
#' @name byConceptDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `bySex`.
#'
#' @param bySex TRUE or FALSE. If TRUE code use will be summarised by sex.
#'
#' @name bySexDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `byYear`.
#'
#' @param byYear TRUE or FALSE. If TRUE code use will be summarised by year.
#'
#' @name byYearDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `cdm`.
#'
#' @param cdm A cdm reference to an OMOP CDM dataset. If data is held within a
#' database, the vocabulary tables should be in the same schema as the clinical
#' tables (person, observation period, and so on).
#'
#' @name cdmDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `countBy`.
#'
#' @param countBy Either "record" for record-level counts or "person" for
#' person-level counts.
#'
#' @name countByDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `domain`.
#'
#' @param domain Character vector with one or more of the OMOP CDM domains.
#' The results will be restricted to the given domains. Check the available
#' domains in the database by running 'availableDomains()', or 'associatedDomains()'
#' to explore the domains associated with a codelist. If NULL, all supported domains
#' are included: Condition, Drug, Procedure, Device, Observation, and Measurement.
#'
#' @name domainDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `doseForm`.
#'
#' @param doseForm Only codes with the specified dose form
#' will be returned. If NULL, descendant codes will be returned regardless
#' of dose form. Use 'availableDoseForms()' to see the available dose forms in the database,
#' or 'associatedDoseForms()' to see the associated dose forms in a codelist.
#'
#' @name doseFormDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `doseUnit`.
#'
#' @param doseUnit Only codes with the specified dose unit
#' will be returned. If NULL, descendant codes will be returned regardless
#' of dose unit Use 'availableDoseUnits()' to see the available dose units,
#' or 'associatedDoseUnits()' to see the associated dose forms in a codelist.
#'
#' @name doseUnitDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `groupColumn`.
#'
#' @param groupColumn Variables to use as group labels. Allowed columns are:
#' "cdm_name", "codelist_name", "domain_id", "standard_concept_name",
#' "standard_concept_id", "estimate_name", "standard_concept", "vocabulary_id".
#' These cannot be used in header.
#'
#' @name groupColumnDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `groupColumn`.
#'
#' @param groupColumn Variables to use as group labels. Allowed columns are:
#' "cdm_name", "codelist_name", "standard_concept_name", "standard_concept_id",
#' "estimate_name", "source_concept_name", "source_concept_id", "domain_id". If
#' results are stratified, "year", "sex", "age_group" can also be used.
#' These cannot be used in header.
#'
#' @name groupColumnStrataDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `header`.
#'
#' @param header A vector specifying the elements to include in the header. The
#' order of elements matters, with the first being the topmost header.
#' The header vector can contain one of the following variables: "cdm_name",
#' "codelist_name", "domain_id", "standard_concept_name", "standard_concept_id",
#' "estimate_name", "standard_concept", "vocabulary_id".
#' Alternatively, it can include other names to use as overall header labels.
#'
#' @name headerDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `header`.
#'
#' @param header A vector specifying the elements to include in the header. The
#' order of elements matters, with the first being the topmost header.
#' The header vector can contain one of the following variables: "cdm_name",
#' "codelist_name", "standard_concept_name", "standard_concept_id",
#' "estimate_name", "source_concept_name", "source_concept_id", "domain_id". If
#' results are stratified, "year", "sex", "age_group" can also be used.
#' Alternatively, it can include other names to use as overall header labels.
#'
#' @name headerStrataDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `hide`.
#'
#' @param hide Table columns to exclude, options are:  "cdm_name",
#' "codelist_name", "domain_id", "standard_concept_name", "standard_concept_id",
#' "estimate_name", "standard_concept", "vocabulary_id". These cannot be used in
#' header or groupColumn.
#'
#' @name hideDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `style`.
#'
#' @param style A character string or custom R code to define the visual formatting
#' of the table. This argument can be provided in two ways: (1) Pre-defined Styles
#' (Character String): Use a name for a built-in style (e.g., "darwin"). See
#' visOmopResults::tableStyle() for available options. (2) Custom Code (Advanced):
#' Supply a block of custom R code. This code must be specific to the table type.
#' See visOmopResults::tableStyleCode() for structural examples.
#'
#' @name tableStyleDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `hide`.
#'
#' @param hide Table columns to exclude, options are: "cdm_name",
#' "codelist_name", "year", "sex", "age_group", "standard_concept_name",
#' "standard_concept_id", "estimate_name", "source_concept_name",
#' "source_concept_id", "domain_id". If results are stratified, "year", "sex",
#' "age_group" can also be used. These cannot be used in header or groupColumn.
#'
#' @name hideStrataDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `includeDescendants`.
#'
#' @param includeDescendants Either TRUE or FALSE. If TRUE descendant concepts
#' of identified concepts will be included in the candidate codelist. If FALSE
#' only direct mappings from ICD-10 codes to standard codes will be returned.
#'
#' @name includeDescendantsDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `ingredientRange`.
#'
#' @param ingredientRange Used to restrict descendant codes to those
#' associated with a specific number of drug ingredients. Must be a vector of
#' length two with the first element the minimum number of ingredients allowed
#' and the second the maximum. A value of c(2, 2) would restrict to only
#' concepts associated with two ingredients.
#'
#' @name ingredientRangeDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `keepOriginal`.
#'
#' @param keepOriginal Whether to keep the original codelist (TRUE) or just return
#' the stratified ones (FALSE).
#'
#' @name keepOriginalDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `keepOriginal`.
#'
#' @param keepOriginal Whether to keep the original codelist (TRUE) or just return
#' the subset ones (FALSE).
#'
#' @name keepOriginalDocSubset
#' @keywords internal
NULL

#' Helper for consistent documentation of `level`.
#'
#' @param level ATC level. Can be one or more of "ATC 1st", "ATC 2nd",
#' "ATC 3rd", "ATC 4th", and "ATC 5th".
#'
#' @name levelATCDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `level`.
#'
#' @param level Can be either "ICD10 Chapter", "ICD10 SubChapter",
#' "ICD10 Hierarchy", or "ICD10 Code".
#'
#' @name levelICD10Doc
#' @keywords internal
NULL

#' Helper for consistent documentation of `minimumCount`.
#'
#' @param minimumCount Any codes with a frequency under this will be removed.
#'
#' @name minimumCountDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `nameStyle`.
#'
#' @param nameStyle Name style to apply to returned list. Can be one of
#' `"{concept_code}"`,`"{concept_id}"`, `"{concept_name}"`, or a combination (i.e.,
#' `"{concept_code}_{concept_name}"`).
#'
#' @name nameStyleDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `routeCategory`.
#'
#' @param routeCategory Only codes with the specified route will be
#' returned. If NULL, descendant codes will be returned regardless of route
#' category. Use 'availableRouteCategories()' to find the available route categories in
#' the database, and 'associatedRouteCategories()' to get drug routs associated with
#' a codelist.
#'
#' @name routeCategoryDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `standardConcept`.
#'
#' @param standardConcept  Character vector with one or more of "Standard",
#' "Classification", and "Non-standard". These correspond to the flags used
#' for the standard_concept field in the concept table of the cdm.
#'
#' @name standardConceptDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `table`.
#'
#' @param table cdm table of interest.
#'
#' @name tableDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `type`.
#'
#' @param type Can be "codelist", "codelist_with_details" or
#' "concept_set_expression".
#'
#' @name typeBroadDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `type`.
#'
#' @param type Can be "codelist" or "codelist_with_details".
#'
#' @name typeNarrowDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `type`.
#'
#' @param type Type of desired formatted table. By default (type = NULL), it will
#' create a 'gt' table. To see supported formats use visOmopResults::tableType().
#'
#' @name typeTableDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `x`.
#'
#' @param x A codelist, codelist_with_details, or a concept_set. See `newCodelist()`, `newCodelistWithDetails()`, `newConceptSetExpression()` functions for more details.
#'
#' @name xDoc
#' @keywords internal
NULL

#' Helper for consistent documentation of `x` where input can be codelist or cohort.
#'
#' @param x  A codelist, codelist_with_details, or a concept_set. See `newCodelist()`, `newCodelistWithDetails()`, `newConceptSetExpression()` functions for more details. If set to NULL, the codelist associated with the cohort will be extracted
#'
#' @name xDocCohort
#' @keywords internal
NULL

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Coerce to a concept set expression
#'
#' @param x A codelist, codelist_with_details, concept_set_expression, or a candidate_codes.
#'
#' @returns codelist
#' @export
#'
#' @examples
#' \donttest{
#' library(omock)
#' library(CDMConnector)
#'
#' # Creating CDM object
#' cdm <- mockCdmFromDataset(datasetName = "GiBleed")
#'
#' # Create concept_set_expression from a codelist
#' codelist <- getDrugIngredientCodes(cdm,
#'                                    name = "acetaminophen",
#'                                    nameStyle = "{concept_name}",
#'                                    type = "codelist")
#'
#' asConceptSetExpression(codelist)
#'
#' # Create concept_set_expression from a codelist_with_details
#' codelist <- getDrugIngredientCodes(cdm,
#'                                    name = "acetaminophen",
#'                                   nameStyle = "{concept_name}",
#'                                    type = "codelist_with_details")
#'
#' asConceptSetExpression(codelist)
#' }
asConceptSetExpression <- function(x){
  UseMethod("asConceptSetExpression")
}

#' @export
#' @rdname asConceptSetExpression
asConceptSetExpression.codelist <- function(x){
  x |>
    purrr::map(\(x) {
      dplyr::tibble(
        concept_id = as.integer(x),
        excluded = FALSE,
        descendants = FALSE,
        mapped = FALSE
      )
    }) |>
    omopgenerics::newConceptSetExpression()
}

#' @export
#' @rdname asConceptSetExpression
asConceptSetExpression.codelist_with_details  <- function(x){
  asCodelist(x) |>
    purrr::map(\(x) {
      dplyr::tibble(
        concept_id = as.integer(x),
        excluded = FALSE,
        descendants = FALSE,
        mapped = FALSE
      )
    }) |>
    omopgenerics::newConceptSetExpression()
}

test_that("codelist to concept set expression", {

 cl <- list(a = c(1L),
            b = c(2L, 3L),
            c = 3L) |>
    omopgenerics::newCodelist()

 expect_identical(
   asConceptSetExpression(cl)[["a"]],
   dplyr::tibble(concept_id = 1L,
                 excluded = FALSE,
                 descendants = FALSE,
                 mapped = FALSE)
   )
 expect_identical(
   asConceptSetExpression(cl)[["b"]],
   dplyr::tibble(concept_id = c(2L, 3L),
                 excluded = c(FALSE, FALSE),
                 descendants = c(FALSE, FALSE),
                 mapped = c(FALSE, FALSE))
 )
 expect_identical(
   asConceptSetExpression(cl)[["c"]],
   dplyr::tibble(concept_id = 3L,
                 excluded = FALSE,
                 descendants = FALSE,
                 mapped = FALSE)
 )

})

test_that("codelist to concept set expression", {

  cl <- list(a = dplyr::tibble(concept_id = c(1L),
                               concept_name = "aaa"),
             b = dplyr::tibble(concept_id = c(2L, 3L),
                               concept_name = "aaa"),
             c = dplyr::tibble(concept_id = 3L,
                               concept_name = "aaa")) |>
    omopgenerics::newCodelistWithDetails()

  expect_identical(
    asConceptSetExpression(cl)[["a"]],
    dplyr::tibble(concept_id = 1L,
                  excluded = FALSE,
                  descendants = FALSE,
                  mapped = FALSE)
  )
  expect_identical(
    asConceptSetExpression(cl)[["b"]],
    dplyr::tibble(concept_id = c(2L, 3L),
                  excluded = c(FALSE, FALSE),
                  descendants = c(FALSE, FALSE),
                  mapped = c(FALSE, FALSE))
  )
  expect_identical(
    asConceptSetExpression(cl)[["c"]],
    dplyr::tibble(concept_id = 3L,
                  excluded = FALSE,
                  descendants = FALSE,
                  mapped = FALSE)
  )

})

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get available dose units
#'
#' @description
#' Get the available dose units
#'
#'
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#'
#' @return A character vector with available routes.
#' @export
#'
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all dose units available in the CDM
#' availableDoseUnits(cdm = cdm)
#' }
availableDoseUnits <- function(cdm, standardConcept = "Standard"){

  #initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  standardConcept <- assertStandardConcept(standardConcept)

  # If no codelist is provided, return all concept class Ids in the cdm
    doseUnits <- doseUnitsInternal(cdm,
                                   newOmopTable = "drug_strength",
                                   standardConcept = standardConcept,
                                   xx = NULL)

  return(doseUnits)
}

#' Get available dose units
#'
#' @description
#' Get the dose units associated with a codelist
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#'
#' @return A character vector with available routes.
#' @export
#'
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' codelist <- newCodelist(list("codes1" = c(194152L, 1830279L, 40558872L),
#'                              "codes2" = c(44022939L, 1830282L)))
#' associatedDoseUnits(cdm = cdm,
#'                    x = codelist)
#' }
associatedDoseUnits <- function(x,
                                cdm,
                                standardConcept = "Standard"){

  #initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  standardConcept <- assertStandardConcept(standardConcept)
  checkCodelist(x, allowConceptSetExpression = FALSE)

  if(inherits(x, "codelist_with_details")){
    x <- asCodelist(x)
  }

  if(inherits(x, "codelist")){
    newOmopTable <- paste0(omopgenerics::uniqueTableName(),
                           omopgenerics::uniqueId())
    codesTable <- paste0(omopgenerics::uniqueTableName(),
                         omopgenerics::uniqueId())
    cdm <- subsetOmopTable(cdm, newOmopTable, "drug_strength", x)

    doseUnits <- list()
    for( i in seq_along(x) ){
      doseUnits[[names(x)[[i]]]] <- doseUnitsInternal(cdm,
                                                      newOmopTable = newOmopTable,
                                                      codesTable = codesTable,
                                                      standardConcept = standardConcept,
                                                      xx = x[[i]])
    }
    omopgenerics::dropSourceTable(cdm, name = c(newOmopTable, codesTable))
  }

  return(doseUnits)
}


doseUnitsInternal <- function(cdm,
                              newOmopTable,
                              codesTable,
                              standardConcept,
                              xx){

  doseUnits <- cdm[[newOmopTable]]

  if(!is.null(xx)){
    cdm <- omopgenerics::insertTable(
      cdm, name = codesTable, dplyr::tibble(drug_concept_id = xx))
    doseUnits <- doseUnits |>
      dplyr::rename("drug_concept_id" = "concept_id") |>
      dplyr::inner_join(cdm[[codesTable]], by = "drug_concept_id")
  }

  doseUnits <- doseUnits |>
    dplyr::select("amount_unit_concept_id", "numerator_unit_concept_id") |>
    tidyr::pivot_longer(cols = c("amount_unit_concept_id", "numerator_unit_concept_id"),
                        values_to = "concept_id") |>
    dplyr::select(-"name") |>
    dplyr::filter(!is.na(.data$concept_id)) |>
    dplyr::distinct() |>
    dplyr::left_join(
      cdm[["concept"]] |>
        dplyr::select("concept_id", "concept_name", "standard_concept"),
      by = c("concept_id")) |>
    dplyr::mutate(standard_concept = dplyr::case_when(
      is.na(.data$standard_concept) ~ "non-standard",
      .data$standard_concept == "C" ~ "classification",
      .data$standard_concept == "S" ~ "standard"
    )) |>
    dplyr::filter(.data$standard_concept %in% .env$standardConcept) |>
    dplyr::pull("concept_name") |>
    unique() |>
    sort()

  return(doseUnits)
}

test_that("available dose unit", {
  skip_on_cran()
  # none in our mock
  cdm <- mockVocabRef()
  namesInitial <- names(cdm)

  expect_no_error(x <- availableDoseUnits(cdm))
  expect_equal(c("milligram", "percent"), x)
  expect_true(all(names(cdm) == namesInitial))
  expect_no_error(x <- availableDoseUnits(cdm, standardConcept = "Non-standard"))
  expect_true(length(x) == 0)
  expect_no_error(r <- associatedDoseUnits(cdm, x = omopgenerics::newCodelist(list("codes1" = c(21L),
                                                                         "codes2" = c(22L)))))
  expect_equal(names(r), c("codes1", "codes2"))
  expect_true(r$codes1 == "milligram")
  expect_true(length(r$codes2) == 0)
  expect_true(all(names(cdm) == namesInitial))

  x <- list("codes" = dplyr::tibble("concept_id" = 21L,
                                    "concept_name" = "nitrogen Topical Liquefied Gas",
                                    "domain_id" = "Drug",
                                    "vocabulary_id" = "RxNorm",
                                    "ancestor_concept_id" = 999L)) |>
    omopgenerics::newCodelistWithDetails()

  expect_no_error(x <- associatedDoseUnits(cdm, x = x))
  expect_true(x$codes == "milligram")

  # expected errors
  expect_error(availableDoseUnits("a"))
  expect_error(availableDoseUnits(cdm, x = 123))

  expect_error(associatedDoseUnits("a"))
  expect_error(associatedDoseUnits(cdm, x = 123))
  expect_error(associatedDoseUnits(cdm, list("codes" = c(21L))))
  expect_error(associatedDoseUnits(cdm, x, standardConcept = "maybe"))

})

#' @importFrom omopgenerics cohortCodelist
#' @export
omopgenerics::cohortCodelist

#' @importFrom omopgenerics importCodelist
#' @export
omopgenerics::importCodelist

#' @importFrom omopgenerics importConceptSetExpression
#' @export
omopgenerics::importConceptSetExpression

#' @importFrom omopgenerics exportCodelist
#' @export
omopgenerics::exportCodelist

#' @importFrom omopgenerics exportConceptSetExpression
#' @export
omopgenerics::exportConceptSetExpression

#' @importFrom omopgenerics newCodelist
#' @export
omopgenerics::newCodelist

#' @importFrom omopgenerics newCodelistWithDetails
#' @export
omopgenerics::newCodelistWithDetails

#' @importFrom omopgenerics newConceptSetExpression
#' @export
omopgenerics::newConceptSetExpression

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom lifecycle deprecated
#' @importFrom rlang .data
#' @importFrom rlang .env
#' @importFrom rlang :=
## usethis namespace: end
NULL

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' Get concept ids from JSON files containing concept sets
#' `r lifecycle::badge("deprecated")`
#'
#' @param path Path to a file or folder containing JSONs of concept sets.
#' @inheritParams cdmDoc
#' @inheritParams typeBroadDoc
#'
#' @return Named list with concept_ids for each concept set.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create a CDM object
#' cdm <- mockCdmReference()
#'
#' # Load JSON files
#' x <- codesFromConceptSet(cdm = cdm,
#'                          path =  system.file(package = "CodelistGenerator",
#'                                              "concepts_for_mock"))
#' x
#'
#' # Load JSON files as codelist_with_details
#' x <- codesFromConceptSet(cdm = cdm,
#'                          path =  system.file(package = "CodelistGenerator",
#'                                              "concepts_for_mock"),
#'                          type = "codelist_with_details")
#' x
#'
#' # Load JSON files as concept_set_expression
#' x <- codesFromConceptSet(cdm = cdm,
#'                          path =  system.file(package = "CodelistGenerator",
#'                                              "concepts_for_mock"),
#'                          type = "concept_set_expression")
#' x
#'
#' }
codesFromConceptSet <- function(path,
                                cdm,
                                type = c("codelist")) {

  lifecycle::deprecate_warn(what = "codesFromConceptSet()",
                            when = "4.0.0",
                            with = I("omopgenerics::importConceptSetExpression() |> asCodelist()"))


  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  type <- validateType(type = type)
  checkInputs(path = path, cdm = cdm)

  if (dir.exists(path)) {
    conceptSets <- dplyr::tibble(concept_set_path = list.files(
      path = path,
      full.names = TRUE
    ))
  } else {
    conceptSets <- dplyr::tibble(concept_set_path = .env$path)
  }
  conceptSets <- conceptSets |>
    dplyr::filter(tools::file_ext(.data$concept_set_path) == "json") |>
    dplyr::mutate(
      concept_set_name =
        tools::file_path_sans_ext(basename(.data$concept_set_path))
    ) |>
    dplyr::mutate(cohort_definition_id = dplyr::row_number())
  if (conceptSets |> nrow() == 0) {
    cli::cli_abort(glue::glue("No 'json' file found in {path}"))
  }

  # first part: read jsons
  tryCatch(
    expr = conceptList <- readConceptSet(conceptSets),
    error = function(e) {
      cli::cli_abort(
        "The json file is not a properly formated OMOP concept set."
      )
    }
  )

  if(type == "concept_set_expression"){
    conceptList <- conceptList |>
      dplyr::select("concept_id",
                    "excluded" = "is_excluded",
                    "descendants" = "include_descendants",
                    "mapped" = "include_mapped",
                    "cohort_name")

    conceptList <- split(
      conceptList,
      conceptList[, c("cohort_name")]
    )

    for(j in seq_along(conceptList)){
      conceptList[[j]] <- conceptList[[j]] |>
        dplyr::select(!"cohort_name")
    }
    conceptList <- omopgenerics::newConceptSetExpression(conceptList)
    return(conceptList)
  }


  if(any(conceptList$include_mapped == TRUE)){
    exc <- paste(conceptList |>
                   dplyr::filter(.data$include_mapped == TRUE) |>
                   dplyr::pull("cohort_name"), collapse = "; ")
    cli::cli_abort(
      glue::glue("Mapped as TRUE not supported (found in {exc})"))
  }

  # second part: produce output list
  if(nrow(conceptList) == 0){
    cli::cli_abort("No concepts found in concept set")
  }

  conceptListTable <- omopgenerics::uniqueTableName()
  cdm <- omopgenerics::insertTable(cdm = cdm,
                                   name = conceptListTable,
                                   table = conceptList,
                                   overwrite = TRUE,
                                   temporary = FALSE)

  conceptFinalList <- formatConceptList(cdm = cdm,
                                        conceptListTable = conceptListTable)

  if(type == "codelist"){
    conceptFinalList <- omopgenerics::newCodelist(conceptFinalList)
  }

  if(type == "codelist_with_details"){
    conceptFinalList <- addDetails(conceptList = conceptFinalList,
                                   cdm = cdm)

    x <- purrr::list_rbind(conceptFinalList) |>
      dplyr::filter(is.na(.data$standard_concept)) |>
      dplyr::pull("concept_id")
    if(length(x) > 0){
      cli::cli_warn("Concept_id{?s} {.val {as.character(x)}} {?is/are} not present in the CDM.")
    }
    conceptFinalList <- omopgenerics::newCodelistWithDetails(conceptFinalList)
  }

  # return list
  return(conceptFinalList)
}

#' Get concept ids from JSON files containing cohort definitions
#'
#' @param path Path to a file or folder containing JSONs of cohort definitions.
#' @inheritParams cdmDoc
#' @inheritParams typeBroadDoc
#'
#' @return Named list with concept_ids for each concept set.
#' @export
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' cdm <- mockVocabRef("database")
#' x <- codesFromCohort(cdm = cdm,
#'                      path =  system.file(package = "CodelistGenerator",
#'                      "cohorts_for_mock"))
#' x
#' CDMConnector::cdmDisconnect(cdm)
#' }
#'
codesFromCohort <- function(path,
                            cdm,
                            type = c("codelist")) {

  rlang::check_installed("jsonlite")

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  type <- validateType(type = type)
  checkInputs(path = path, cdm = cdm)

  # list jsons
  files <- listJsonFromPath(path)

  # obtain codelistTibble
  codelistTibble <- NULL
  unknown <- 1
  for (k in seq_along(files)) {
    codelistTibble <- codelistTibble |>
      dplyr::union_all(extractCodes(files[k], unknown))
  }

  if(is.null(codelistTibble)){
    cli::cli_abort("No codes found")
  }

  if(type == "concept_set_expression"){
    codelistTibble <- codelistTibble |>
      dplyr::mutate(include_mapped = FALSE) |>
      dplyr::select("concept_id",
                    "excluded" = "is_excluded",
                    "descendants" = "include_descendants",
                    "mapped" = "include_mapped",
                    "codelist_name")

    codelistTibble <- split(
      codelistTibble,
      codelistTibble[, c("codelist_name")]
    )

    for(j in seq_along(codelistTibble)){
      codelistTibble[[j]] <- codelistTibble[[j]] |>
        dplyr::select(!"codelist_name")
    }
    codelistTibble <- omopgenerics::newConceptSetExpression(codelistTibble)
    return(codelistTibble)
  }


  codelistTable <- omopgenerics::uniqueTableName()
  cdm <- omopgenerics::insertTable(cdm = cdm,
                                   name = codelistTable,
                                   table = codelistTibble,
                                   overwrite = TRUE,
                                   temporary = FALSE)

  # obtain descendants
  codelistTibble <- appendDescendants(cdm, codelistTable) |>
    dplyr::collect(name = codelistTable,
                   overwrite = TRUE,
                   temporary = FALSE)

  omopgenerics::dropSourceTable(cdm = cdm, name = codelistTable)
  cdm[[codelistTable]] <- NULL

  # exclude
  codelistTibble <- excludeCodes(codelistTibble)

  # split into list
  codelist <- tibbleToList(codelistTibble)

  if(type == "codelist"){
    codelist <- omopgenerics::newCodelist(codelist)
  }

  if(type == "codelist_with_details"){
    codelist <- addDetails(conceptList = codelist,
                           cdm = cdm)
    codelist <- omopgenerics::newCodelistWithDetails(codelist)
  }


  # return
  return(codelist)
}

listJsonFromPath <- function(path) {
  if (file.info(path)[["isdir"]]) {
    files <- list.files(path, full.names = TRUE)
  } else {
    files <- path
  }
  files <- files[tools::file_ext(files) == "json"]
  if (length(files) == 0) {
    cli::cli_abort(paste0("No json files find in ", path))
  }
  return(files)
}

extractCodes <- function(file, unknown) {

  json <- jsonlite::read_json(file)[["ConceptSets"]]
  codelistTibble <- NULL
  for (k in seq_along(json)) {
    name <- json[[k]][["name"]]
    if (is.null(name)) {
      name <- paste0("unkown concept set ", unknown)
      unknown <- unknown + 1
    }
    #else if (name %in% names(codes)) {
    #  basefile <- basename(file)
    #  name <- paste0(name, " (", substr(basefile, 1, nchar(basefile) - 5), ")")
    #}
    concepts <- json[[k]][["expression"]][["items"]]
    conceptId <- NULL
    includeDescendants <- NULL
    isExcluded <- NULL

    for (j in seq_along(concepts)) {
      if(!is.null(concepts[[j]][["includeMapped"]]) &&
         concepts[[j]][["includeMapped"]] == TRUE ){
        cli::cli_abort(
          glue::glue("Mapped as TRUE not supported (found in {name})"))
      }
      conceptId <- c(conceptId, concepts[[j]][["concept"]][["CONCEPT_ID"]])
      exc <- concepts[[j]][["isExcluded"]]
      isExcluded <- c(
        isExcluded, ifelse(is.null(exc), FALSE, exc)
      )
      incD <- concepts[[j]][["includeDescendants"]]
      includeDescendants <- c(
        includeDescendants, ifelse(is.null(incD), FALSE, incD)
      )
    }
    codelistTibble <- codelistTibble |>
      dplyr::union_all(dplyr::tibble(
        codelist_name = name, concept_id = conceptId,
        include_descendants = includeDescendants, is_excluded = isExcluded
      ) |>
        dplyr::mutate(filename = file))
  }
  return(codelistTibble)
}

appendDescendants <- function(cdm, codelistTable) {

  cdm[[codelistTable]] |>
    dplyr::mutate(concept_id = as.integer(.data$concept_id)) |>
    dplyr::filter(.data$include_descendants == TRUE) |>
    dplyr::rename("ancestor_concept_id" = "concept_id") |>
    dplyr::inner_join(
      cdm[["concept_ancestor"]] |>
        dplyr::select("ancestor_concept_id", "descendant_concept_id") |>
        dplyr::mutate(ancestor_concept_id = as.integer(.data$ancestor_concept_id)),
      by = "ancestor_concept_id"
    ) |>
    dplyr::select(-"ancestor_concept_id") |>
    dplyr::rename("concept_id" = "descendant_concept_id") |>
    dplyr::union_all(
      cdm[[codelistTable]]  |>
        dplyr::filter(.data$include_descendants == "FALSE")
    ) |>
    dplyr::select(-"include_descendants")

}

excludeCodes <- function(codelistTibble) {
  codelistTibble |>
    dplyr::filter(.data$is_excluded == FALSE) |>
    dplyr::select(-"is_excluded") |>
    dplyr::anti_join(
      codelistTibble |>
        dplyr::filter(.data$is_excluded == TRUE),
      by = c("codelist_name", "concept_id")
    )
}

tibbleToList <- function(codelistTibble) {

  codelistTibble <- codelistTibble |>
    dplyr::mutate(nam = paste0(.data$codelist_name, "; ",
                               .data$filename))

  nam <- unique(codelistTibble$nam)
  codelist <- lapply(nam, function(x) {
    codelistTibble |>
      dplyr::filter(.data$nam == .env$x) |>
      dplyr::pull("concept_id") |>
      unique()
  })
  names(codelist) <- nam


  # check if we have any concept sets with the same name but different definitions
  # keep first for each name
  cs_names <- stringr::str_extract(names(codelist), "^[^;]*")
  cs_names_unique <- unique(cs_names)

  codelist_dedup <- list()

  for(i in seq_along(cs_names_unique)){
    same_name_cs <- codelist[which(cs_names_unique[i] == cs_names)]
    check_consistent <- all(sapply(same_name_cs, identical, same_name_cs[[1]]))

    if(isFALSE(check_consistent)){
      cli::cli_abort(message = "Different definitions for concept set {cs_names_unique[i]} found")
    }
    # keep first
    codelist_dedup[[cs_names_unique[i]]] <- same_name_cs[[1]]
  }

  return(codelist_dedup)
}

addDetails <- function(conceptList,
                       cdm,
                       cols = c("concept_name", "domain_id", "vocabulary_id", "standard_concept")){

  if(length(conceptList) == 0){
    return(conceptList)
  }

  if(isFALSE(inherits(conceptList[[1]], "tbl_df"))){
    for(i in seq_along(conceptList)){
      conceptList[[i]] <- dplyr::tibble(concept_id = conceptList[[i]],
                                        concept_set = names(conceptList)[i])
    }
    extraCols <- as.character()
  }else{
    cols <- setdiff(cols,
                    colnames(conceptList[[1]]))
    conceptList <- purrr::imap(conceptList, ~ dplyr::mutate(.x, "concept_set" = .y))
    extraCols <- names(conceptList[[1]])[!names(conceptList[[1]]) %in% c("concept_id", "concept_name", "domain_id", "standard_concept", "vocabulary_id", "concept_set")]
  }

  conceptList <- dplyr::bind_rows(unclass(conceptList))

  tableConceptList <- omopgenerics::uniqueTableName()
  cdm <- omopgenerics::insertTable(cdm = cdm,
                                   table = conceptList,
                                   name = tableConceptList,
                                   overwrite = TRUE,
                                   temporary = FALSE)

  if("standard_concept" %in% cols){
    conceptList <- cdm[[tableConceptList]] |>
      dplyr::left_join(cdm[["concept"]] |>
                         dplyr::select("concept_id", dplyr::all_of(cols)) |>
                         dplyr::mutate("standard_concept" = dplyr::if_else(
                           is.na(.data$standard_concept),
                           "non-standard",
                           .data$standard_concept)
                         ),
                       by = "concept_id")

    conceptList <- conceptList |>
      dplyr::mutate(
        standard_concept = ifelse(.data$standard_concept == "C",
                                  "classification",
                                  .data$standard_concept)) |>
      dplyr::mutate(
        standard_concept = ifelse(.data$standard_concept == "S",
                                  "standard",
                                  .data$standard_concept))  |>
      dplyr::collect()
  }else{
    conceptList <- cdm[[tableConceptList]] |>
      dplyr::left_join(cdm[["concept"]] |>
                         dplyr::select("concept_id", dplyr::all_of(cols)),
                       by = "concept_id")

    conceptList <- conceptList |>
      dplyr::collect()
  }

  omopgenerics::dropSourceTable(cdm = cdm, name = tableConceptList)

  # Arrange tables
  conceptList <- conceptList |>
    dplyr::select("concept_id", "concept_set",
                  dplyr::any_of(c("concept_name", "domain_id", "vocabulary_id", "standard_concept",
                           extraCols)))

  conceptList <- split(
    x = conceptList |> dplyr::select(!"concept_set"),
    f = as.factor(conceptList$concept_set)
  )

  for(i in seq_along(conceptList)){
    conceptList[[i]] <- conceptList[[i]] |>
      dplyr::arrange(.data$concept_id)
  }

  return(conceptList)
}

#' Put concept ids from all cohorts of interest in the required list format
#'
#' @param conceptList table with all the concept ids read, with their respective
#' cohort, exclusion and descendant information.
#' @inheritParams cdmDoc
#'
#' @return list of concept_ids and respective cohort_definition_ids of interest
#' @noRd
formatConceptList <- function(cdm, conceptListTable) {
  conceptList <- cdm[[conceptListTable]] |>
    dplyr::filter(.data$include_descendants == FALSE) |>
    dplyr::union_all(
      cdm[["concept_ancestor"]] |>
        dplyr::select(
          "concept_id" = "ancestor_concept_id",
          "descendant_concept_id"
        ) |>
        dplyr::mutate(concept_id = as.integer(.data$concept_id)) |>
        dplyr::right_join(
          cdm[[conceptListTable]] |>
            dplyr::mutate(concept_id = as.integer(.data$concept_id)) |>
            dplyr::filter(.data$include_descendants == TRUE),
          by = "concept_id"
        ) |>
        dplyr::mutate(descendant_concept_id = dplyr::if_else(
          # in case concept is not in ancestor table
          # (even though it should be)
          is.na(.data$descendant_concept_id),
          .data$concept_id, .data$descendant_concept_id)) |>
        dplyr::select(-"concept_id") |>
        dplyr::rename("concept_id" = "descendant_concept_id")
    ) |>
    dplyr::select(-"include_descendants") |>
    dplyr::collect()
  # eliminate the ones that is_excluded = TRUE
  conceptList <- conceptList |>
    dplyr::filter(.data$is_excluded == FALSE) |>
    dplyr::select("cohort_name", "concept_id") |>
    dplyr::anti_join(
      conceptList |>
        dplyr::filter(.data$is_excluded == TRUE),
      by = c("cohort_name","concept_id")
    )

  conceptFinalList <- list()
  for(n in conceptList[["cohort_name"]] |> unique()) {
    conceptFinalList[[n]] <- conceptList |>
      dplyr::filter(.data$cohort_name == n) |>
      dplyr::select("concept_id") |>
      dplyr::pull()
  }
  return(conceptFinalList)
}

#' Get concept ids and information from a list of json files provided
#'
#' @param conceptSets paths and names of all jsons to be read.
#'
#' @return raw table with all the concept_ids read, and their information
#' @noRd
readConceptSet <- function(conceptSets) {
  names <- c("CONCEPT_CLASS_ID",
             "CONCEPT_CODE",
             "CONCEPT_ID",
             "CONCEPT_NAME",
             "DOMAIN_ID",
             "INVALID_REASON",
             "INVALID_REASON_CAPTION",
             "STANDARD_CONCEPT",
             "STANDARD_CONCEPT_CAPTION",
             "VOCABULARY_ID",
             "isExcluded",
             "includeMapped",
             "includeDescendants")

  for (k in seq_len(nrow(conceptSets))) {
    conceptSetName <- conceptSets$concept_set_name[k]
    conceptSet <- jsonlite::read_json(conceptSets$concept_set_path[k])
    conceptSet <- lapply(conceptSet$items, function(x) {
      x <- append(x, x[["concept"]])
      x[["concept"]] <- NULL
      return(x)
    }) |>
      dplyr::bind_rows() |>
      dplyr::mutate(
        cohort_name = conceptSetName
      )
    # Add columns missing from the read file with default values
    conceptSet[setdiff(names, names(conceptSet))] <- NA_character_
    conceptSet <- conceptSet |>
      dplyr::mutate(
        isExcluded = ifelse(is.na(.data$isExcluded), FALSE, .data$isExcluded),
        includeMapped = ifelse(
          is.na(.data$includeMapped), FALSE, .data$includeMapped
        ),
        includeDescendants = ifelse(
          is.na(.data$includeDescendants), FALSE, .data$includeDescendants
        )
      )

    if (k == 1) {
      conceptList <- conceptSet
    } else {
      conceptList <- dplyr::bind_rows(conceptList, conceptSet)
    }
  }
  conceptList <- conceptList |>
    dplyr::select(
      "cohort_name",
      "concept_id" = "CONCEPT_ID",
      "is_excluded" = "isExcluded",
      "include_descendants" = "includeDescendants",
      "include_mapped" = "includeMapped"
    )

  return(conceptList)
}

test_that("test inputs - mock", {
  skip_on_cran()
  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])

    # expected errors
    expect_error(codesFromCohort())
    expect_error(codesFromCohort(cdm = cdm))
    expect_error(codesFromCohort(cdm = cdm, path = 1))
    expect_error(codesFromCohort(cdm = cdm, path = "not/a/path"))


    # codesFromCohort won´t work with concept sets
    expect_error( codesFromCohort(
      cdm = cdm, path =  system.file(package = "CodelistGenerator",
                                     "concepts_for_mock")
    ))

    # we currently don´t support the use of mapped in a concept set
    expect_error(
    omopgenerics::importConceptSetExpression(
      system.file(package = "CodelistGenerator",
                  "concepts_for_mock_with_mapped")
    ) |>
      asCodelist(cdm)
    )

    # working concept set example with mock
    x <- omopgenerics::importConceptSetExpression(
     path =  system.file(package = "CodelistGenerator",
                                     "concepts_for_mock")
    ) |>
      asCodelist(cdm)
    expect_true(x$arthritis_no_desc == "3")
    expect_true(all(c("3", "4", "5") %in% x$arthritis_desc))

    expect_no_error(
    omopgenerics::importConceptSetExpression(
      system.file(package = "CodelistGenerator",
                  "concepts_for_mock")) |>
      asCodelist(cdm) |>
      asCodelistWithDetails(cdm)
    )


    x <- omopgenerics::importConceptSetExpression(
       path =  system.file(package = "CodelistGenerator",
                                     "concepts_for_mock_with_exclude")
    ) |>
      asCodelist(cdm)
    expect_true(all(c("3", "5") %in% x$oa_with_excluded))
    expect_true(!c("4") %in% x$oa_with_excluded)

    # withDetails
    x <- omopgenerics::importConceptSetExpression(
      path =  system.file(package = "CodelistGenerator",
                          "concepts_for_mock")
    ) |>
      asCodelist(cdm) |>
      asCodelistWithDetails(cdm)
    expect_true("Arthritis" %in% x$arthritis_desc$concept_name)
    expect_true(3 %in% x$arthritis_no_desc$concept_id)

    # distinction between non-standard and a concept that is not in the cdm
    expect_equal(x$codelist_with_details |>
                   dplyr::pull("standard_concept"),
                 c("standard","non-standard",NA))


    # working cohort set example with mock
    x <- codesFromCohort(
      cdm = cdm, path =  system.file(package = "CodelistGenerator",
                                     "cohorts_for_mock")
    )
    expect_true(all(c("3", "4", "5") %in% x[["arthritis"]]))
    expect_true(x[["OA no descendants"]] == 3)

    x <- codesFromCohort(
      cdm = cdm, path =  system.file(package = "CodelistGenerator",
                                     "cohorts_for_mock_with_exclude")
    )
    expect_true(all(c("3", "5") %in% x[["OA"]]))
    expect_true(!c("4") %in% x[["OA"]])

    x <- codesFromCohort(
      cdm = cdm, path =  system.file(package = "CodelistGenerator",
                                     "cohorts_for_mock_with_exclude"),
      type = "codelist_with_details"
    )
    expect_true(inherits(x, "codelist_with_details"))

    x <- codesFromCohort(
      cdm = cdm, path =  system.file(package = "CodelistGenerator",
                                     "cohorts_for_mock_with_exclude"),
      type = "concept_set_expression"
    )
    expect_true(inherits(x, "concept_set_expression") |
                  inherits(x, "conceptSetExpression"))

    # we´ll get an error if we have the same name concept set in multiple cohorts,
    # but with different definitions
    expect_error(codesFromCohort(
      cdm = cdm, path =  system.file(package = "CodelistGenerator",
                                     "cohorts_for_mock_dups")
    ))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }

})

test_that("test inputs - redshift", {
  skip_on_cran()
  testthat::skip_if(Sys.getenv("CDM5_REDSHIFT_DBNAME") == "")
  testthat::skip_if_offline()

  db <-  DBI::dbConnect(RPostgres::Redshift(),
                        dbname   = Sys.getenv("CDM5_REDSHIFT_DBNAME"),
                        host     = Sys.getenv("CDM5_REDSHIFT_HOST"),
                        port     = Sys.getenv("CDM5_REDSHIFT_PORT"),
                        user     = Sys.getenv("CDM5_REDSHIFT_USER"),
                        password = Sys.getenv("CDM5_REDSHIFT_PASSWORD"))


  cdm <- CDMConnector::cdmFromCon(con = db,
                                  cdmSchema = "cdmv531",
                                  writeSchema = "public",
                                  cdmVersion = "5.3")

  # working example
  x <- codesFromConceptSet(
    cdm = cdm, path =  system.file(package = "CodelistGenerator", "concepts")
  )
  expect_true(typeof(x) == "list")
  expect_true(all(names(x) %in% c("influenza", "acetaminophen")))
  expect_true(x$influenza == 4266367)
  expect_true(all(1125315 %in% x$acetaminophen))
  expect_true(length(x$acetaminophen) > 1)

  x <- codesFromConceptSet(
    cdm = cdm,
    path =  system.file(package = "CodelistGenerator",
                        "concepts"),
    type = "codelist_with_details"
  )
  expect_true("Influenza" %in% x$influenza$concept_name)

  x <- codesFromCohort(
    cdm = cdm,
    path =  system.file(package = "CodelistGenerator",
                        "cohorts"),
    type = "codelist_with_details"
  )
  expect_true("Influenza" %in% x$influenza$concept_name)

  # check is sorted alphabetically
  x <- codesFromCohort(
    cdm = cdm,
    path =  system.file(package = "CodelistGenerator",
                        "cohorts2"),
    type = "codelist_with_details"
  )

  expect_equal(sort(x[[1]]$concept_name), x[[1]]$concept_name)
  expect_equal(sort(x[[2]]$concept_name), x[[2]]$concept_name)
  expect_equal(sort(x[[3]]$concept_name), x[[3]]$concept_name)

  CDMConnector::cdmDisconnect(cdm)
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' Stratify a codelist by domain category.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @param nameStyle Naming of the new codelists, use `{codelist_name}` to
#' include the codelist name and `{domain}` to include the domain name.
#' @inheritParams keepOriginalDoc
#'
#' @return The codelist with the required stratifications, as different elements
#' of the list.
#' @export
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef()
#' codes <- newCodelist(list("concepts_1" = c(20L,21L,22L),
#'                          "concepts_2" = c(10L,13L,21L)))
#' new_codes <- stratifyByDomain(x = codes,
#'                              cdm = cdm,
#'                              keepOriginal = TRUE)
#' new_codes
#'}
#'
stratifyByDomain <- function(x,
                             cdm,
                             nameStyle = "{codelist_name}_{domain}",
                             keepOriginal = FALSE) {
  st <- searchStrategyAttr(
    function_name = "stratifyByDomain",
    cdm = "cdm"
  )
  stratifyCodelistBy(
    x = x,
    cdm = cdm,
    by = "domain",
    nameStyle = nameStyle,
    st = st,
    keepOriginal = keepOriginal
  )
}

addDomain <- function(x) {
  cdm <- omopgenerics::cdmReference(table = x)
  x |>
    # add domain
    dplyr::left_join(
      cdm$concept |>
        dplyr::select("concept_id", "domain" = "domain_id"),
      by = "concept_id"
    ) |>
    dplyr::select("codelist_name", "concept_id", "domain") |>
    dplyr::collect() |>
    dplyr::distinct()
}

test_that("stratify by domain works", {

  cdm <- mockVocabRef()

  codes <- omopgenerics::newCodelist(list("concepts_1" = c(20L,21L), "concepts_2" = c(13L,21L,22L)))
  expect_no_error(new_codes <- stratifyByDomain(x = codes, cdm = cdm, keepOriginal = FALSE))
  expect_true(inherits(new_codes,"codelist"))
  expect_true(all(names(new_codes) == c("concepts_1_drug", "concepts_2_drug", "concepts_2_unit")))
  expect_equal(new_codes$concepts_1_drug, c(20L, 21L))
  expect_equal(new_codes$concepts_2_drug, c(13L, 21L))
  expect_equal(new_codes$concepts_2_unit, c(22L))

  newCodelist <- list("concepts" = cdm[["concept"]] |>
                        dplyr::filter(.data$concept_id %in% c(13L, 20L, 21L, 24L)) |>
                        dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id", "standard_concept") |>
                        dplyr::collect()) |>
    omopgenerics::newCodelistWithDetails()

  expect_no_error(new_codes <- stratifyByDomain(x = newCodelist,
                                                cdm = cdm,
                                                keepOriginal = TRUE))

  expect_true(inherits(new_codes,"codelist_with_details"))
  expect_identical(c("concepts", "concepts_condition", "concepts_drug"), names(new_codes))
  expect_identical(new_codes$concepts, newCodelist$concepts)
  expect_identical(new_codes$concepts_condition, newCodelist$concepts |> dplyr::filter(.data$domain_id == "Condition"))
  expect_identical(new_codes$concepts_drug, newCodelist$concepts |> dplyr::filter(.data$domain_id == "Drug"))

  # expected errors
  expect_error(stratifyByDomain(list("a" = c(1L)), cdm, keepOriginal = TRUE))
  expect_error(stratifyByDomain(newCodelist, "a", keepOriginal = TRUE))
  expect_error(stratifyByDomain(newCodelist, cdm, keepOriginal = "a"))

})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

runSearch <- function(keywords,
                      cdm,
                      exclude,
                      domains,
                      vocabularyId,
                      standardConcept,
                      searchInSynonyms,
                      searchNonStandard,
                      includeDescendants,
                      includeAncestor) {

  if(!is.null(attr(cdm, "dbcon"))){
    prefix <- paste0(sample(letters, 5, TRUE),
                     collapse = "")
  }

  # connect to relevant vocabulary tables
  # will return informative error if not found
  conceptDb <- cdm$concept
  conceptAncestorDb <- cdm$concept_ancestor
  conceptSynonymDb <- cdm$concept_synonym
  conceptRelationshipDb <- cdm$concept_relationship
  drugStrengthDb <- cdm$drug_strength

  ## domains, standardConcept vocab to lower
  domains <- tolower(domains)
  standardConcept <- tolower(standardConcept)
  # new name for clarity
  standardConceptFlags <- standardConcept

  # formatting of conceptDb variables
  conceptDb <- conceptDb |>
    dplyr::mutate(
      domain_id = tolower(.data$domain_id),
      vocabulary_id = tolower(.data$vocabulary_id),
      standard_concept = dplyr::case_when(
        is.na(.data$standard_concept) ~ "non-standard",
        .data$standard_concept == "C" ~ "classification",
        .data$standard_concept == "S" ~ "standard",
        .default = as.character(.data$standard_concept)
      )
    )

  cli::cli_inform("Limiting to concept type, domains, and vocabularies of interest")
  concept <- conceptDb |>
    dplyr::filter(.data$standard_concept %in% .env$standardConceptFlags,
                  .data$domain_id %in% .env$domains,
                  .data$vocabulary_id %in% .env$vocabularyId) |>
    dplyr::compute()

  # will only collect conceptSynonym later if needed
  if (searchInSynonyms) {
    conceptSynonym <- conceptSynonymDb |>
      dplyr::left_join(
        conceptDb |>
          dplyr::select("concept_id", "domain_id", "vocabulary_id", "standard_concept"),
        by = "concept_id"
      ) |>
      dplyr::filter(.data$domain_id %in% .env$domains &
                      .data$vocabulary_id %in% .env$vocabularyId &
                      .data$standard_concept %in% .env$standardConceptFlags) |>
      dplyr::rename_with(tolower)
  } else {
    conceptSynonym <- NULL
  }

  # collect the drug_strength table if drug
  if ("drug" %in% domains) {
    drugStrength <- drugStrengthDb |>
      dplyr::left_join(
        conceptDb |>
          dplyr::rename("drug_concept_id" = "concept_id") |>
          dplyr::select("drug_concept_id", "domain_id", "vocabulary_id", "standard_concept"),
        by = "drug_concept_id"
      ) |>
      dplyr::filter(.data$domain_id %in% .env$domains &
                    .data$vocabulary_id %in% .env$vocabularyId &
                    .data$standard_concept %in% .env$standardConceptFlags) |>
      dplyr::rename_with(tolower)
  }


  candidateCodesList <- list()

  workingConcept <- concept |>
    dplyr::filter(.data$domain_id %in% .env$domains,
                  .data$vocabulary_id %in% .env$vocabularyId)


  # Start finding candidate codes
  # 1) first, get codes to exclude
  # and anti_join throughout to make sure these don't appear
  # exact matches

  if (length(exclude) > 0) {
    # Get standard, condition concepts which include one of the exclusion words
    # always use exact matching
    excludeCodes <- getMatches(
      words = exclude,
      conceptDf = workingConcept
    )
    if(length(excludeCodes) == 0){
      exclude <- NULL
    }
  }

  # 2) Get standard, condition concepts which
  # include one of the keywords
  cli::cli_inform("Getting concepts to include")
  candidateCodes <- getMatches(
    words = keywords,
    conceptDf = workingConcept
  )

  candidateCodes <- candidateCodes |>
    dplyr::mutate(found_from = "From initial search",
                  found_id = 1L) |>
    dplyr::compute()

  # run exclusion
  if (length(exclude) > 0) {
    if (excludeCodes |>
        utils::head(10) |>
        dplyr::tally() |>
        dplyr::pull("n") > 0) {
      candidateCodes <- candidateCodes |>
        dplyr::anti_join(
          excludeCodes |>
            dplyr::select("concept_id"),
          by = "concept_id"
        )
    }
  }

  # 3) also search in synonyms if option set to true
  # left join back to concept from workingConcept table
  if (searchInSynonyms) {
    cli::cli_inform("Adding concepts using synonymns")
    candidateCodesInSynonyms <- getMatches(
      words = tidyWords(keywords),
      conceptDf = conceptSynonym |>
        dplyr::rename("concept_name" = "concept_synonym_name")
    ) |>
      dplyr::select("concept_id") |>
      dplyr::distinct() |>
      dplyr::left_join(workingConcept,
                       by = "concept_id"
      )

    candidateCodes <- dplyr::union_all(
      candidateCodes,
      candidateCodesInSynonyms |>
        dplyr::mutate(found_from = "In synonyms",
                      found_id = 2L)
    ) |>
      dplyr::distinct()
  }

  # run exclusion
  if (length(exclude) > 0) {
    if (excludeCodes |>
        utils::head(10) |>
        dplyr::tally() |>
        dplyr::pull("n") > 0) {
      candidateCodes <- candidateCodes |>
        dplyr::anti_join(
          excludeCodes |>
            dplyr::select("concept_id"),
          by = "concept_id"
        ) |>
        dplyr::compute()
    }
  }

  candidateCodesList[[paste0(domains, collapse = ",")]] <- candidateCodes


  # 5) add any codes lower in the hierarchy
  if (includeDescendants) {
    if (candidateCodes |>
        utils::head(10) |>
        dplyr::tally() |>
        dplyr::pull("n") > 0) {
      cli::cli_inform("Adding descendants")
      candidateCodeDescendants <- addDescendants(
        workingCandidateCodes = candidateCodes,
        conceptAncestorDf = conceptAncestorDb,
        conceptDf = concept
      )

      candidateCodes <- dplyr::union_all(
        candidateCodes,
        candidateCodeDescendants |>
          dplyr::mutate(found_from = "From descendants",
                        found_id = 3L) |>
          dplyr::anti_join(candidateCodes |>
                             dplyr::select("concept_id"),
                           by = "concept_id")) |>
        dplyr::compute()

      # run exclusion
      if (length(exclude) > 0) {
        if (excludeCodes |>
            utils::head(10) |>
            dplyr::tally() |>
            dplyr::pull("n") > 0) {
          candidateCodes <- candidateCodes |>
            dplyr::anti_join(excludeCodes |>
                               dplyr::select("concept_id"),
                             by = "concept_id"
            )|>
            dplyr::compute()
        }
      }
    }
  }

  # 6) add any codes one level above in the hierarchy
  if (includeAncestor) {
    if (candidateCodes |>
        utils::head(10) |>
        dplyr::tally() |>
        dplyr::pull("n") > 0) {
      cli::cli_inform("Adding ancestor")

      candidateCodeAncestor <- addAncestor(
        workingCandidateCodes = candidateCodes,
        conceptAncestorDf = conceptAncestorDb,
        conceptDf = conceptDb
      )

      candidateCodes <- dplyr::union_all(
        candidateCodes,
        candidateCodeAncestor |>
          dplyr::mutate(found_from = "From ancestor",
                        found_id = 4L)
      ) |>
        dplyr::distinct() |>
        dplyr::compute()

      # run exclusion
      if (length(exclude) > 0) {
        if (excludeCodes |>
            utils::head(10) |>
            dplyr::tally() |>
            dplyr::pull("n") > 0) {
          candidateCodes <- candidateCodes |>
            dplyr::anti_join(excludeCodes |> dplyr::select("concept_id"),
                             by = "concept_id"
            )|>
            dplyr::compute()
        }
      }
    }
  }

  # 7) add codes from non-standard
  # nb we do this last so as to not include descendants
  # which can blow up candiate codelist when there
  # are multiple mappings
  if (searchNonStandard) {
    cli::cli_inform("Adding codes from non-standard")
    conceptNs <- conceptDb |>
      dplyr::filter(.data$standard_concept == "non-standard") |>
      dplyr::rename_with(tolower)

    if (conceptNs |>
        utils::head(10) |>
        dplyr::tally() |>
        dplyr::pull("n") > 0) {
      candidateCodesNs <- getMatches(
        words = tidyWords(keywords),
        conceptDf = conceptNs
      )
    }

    if (conceptNs |>
        utils::head(10) |>
        dplyr::tally() |>
        dplyr::pull("n") > 0) {
      candidateCodesNs <- candidateCodesNs |>
        dplyr::select("concept_id") |>
        dplyr::left_join(
          conceptRelationshipDb |>
            dplyr::filter(.data$relationship_id == "Mapped from") |>
            dplyr::rename_with(tolower),
          by = c("concept_id" = "concept_id_2")
        ) |>
        dplyr::select("concept_id"= "concept_id_1") |>
        dplyr::distinct() |>
        dplyr::left_join(concept,
                         by = "concept_id"
        ) |>
        dplyr::compute()

      candidateCodes <- dplyr::union_all(
        candidateCodes,
        candidateCodesNs |>
          dplyr::mutate(found_from = "From non-standard",
                        found_id = 5L)
      )  |>
        dplyr::compute()
    }

    # run exclusion
    if (length(exclude) > 0) {
      if (excludeCodes |>
          utils::head(10) |>
          dplyr::tally() |>
          dplyr::pull("n") > 0) {
        candidateCodes <- candidateCodes |>
          dplyr::anti_join(excludeCodes |> dplyr::select("concept_id"),
                           by = "concept_id"
          )|>
          dplyr::compute()
      }
    }
  }

  candidateCodes <- candidateCodes |>
    dplyr::select(c("concept_id", "found_from", "found_id")) |>
    dplyr::inner_join(cdm$concept, by = "concept_id") |>
    dplyr::distinct() |>
    dplyr::collect()

  if (nrow(candidateCodes) > 0) {
    # 8) Finish up
    cli::cli_inform("Search completed. Finishing up.")

    # remove duplicates (found in different ways)
    # keep first time it was found
    # for drug,  same concept_id with different ingredient_concept_id
    # will be removed as well (with only the first kept).
    candidateCodes <- candidateCodes |>
      dplyr::arrange(.data$found_id) |>
      dplyr::group_by(.data$concept_id)  |>
      dplyr::filter(dplyr::row_number(.data$concept_id) == 1) |>
      dplyr::ungroup() |>
      dplyr::select(!"found_id")

    # make sure we only have codes from the domain of interest
    candidateCodes <- candidateCodes |>
      dplyr::mutate(
        standard_concept1 = dplyr::case_when(
          is.na(.data$standard_concept) ~ "non-standard",
          .data$standard_concept == "C" ~ "classification",
          .data$standard_concept == "S" ~ "standard",
          .default = as.character(.data$standard_concept)
        )
      ) |>
      dplyr::filter(tolower(.data$domain_id) %in% tolower(.env$domains),
                    tolower(.data$vocabulary_id) %in% tolower(.env$vocabularyId),
                    tolower(.data$standard_concept1) %in% .env$standardConceptFlags) |>
      dplyr::select(-"standard_concept1")
  }


  if(!is.null(attr(cdm, "dbcon"))){
    omopgenerics::dropSourceTable(cdm = cdm,
                            name = dplyr::starts_with(paste0("cg_",prefix)))
  }

  return(candidateCodes)

}


# helper functions for runSearch
tidyWords <- function(words, message = TRUE) {
  omopgenerics::assertCharacter(words)

  # to avoid invalid UTF-8 error
  words <- iconv(words, from = "", to = "UTF-8",sub="")

  # Remove / at the beginning and at the end for exact matching
  words <- stringr::str_replace(words, "^/(.*)/$", "\\1")
  words <- stringr::str_replace(words, "^/\b(.*)/\b$", "\\1")

  # throw a warning if there is one of these symbols
  if(isTRUE(message)){
  locs <- stringr::str_locate_all(words, "[^\\x20-\\x7E]|[[:punct:]]|[^\\da-zA-Z ]|[^\x01-\x7F]+")
  symbols <- purrr::map2(words, locs, ~{
    if (nrow(.y) == 0) return(character(0))
    stringr::str_sub(.x, .y[,1], .y[,2])
  })

  symbols <- unlist(symbols)
  dash <- stringr::str_detect(words, "-")

  msg <- as.character()
  if(length(symbols) > 0){
    symbols <- unique(symbols)
    msg <- "Symbols {.val {symbols}} will be ignored. "
  }
  if(any(dash)){
    msg <- append(msg, "`-` will be replaced by an empty space.")
  }
  if(length(msg) > 0){
    cli::cli_inform(msg, call = parent.frame())
  }
  }

  # some generic formatting
  workingWords <- stringr::str_remove_all(words, "[^\\x20-\\x7E]")
  workingWords <- stringi::stri_trans_nfkc(workingWords)
  workingWords <- trimws(workingWords)
  workingWords <- stringr::str_replace_all(workingWords, "-", " ")
  workingWords <- stringr::str_replace_all(workingWords, "[[:punct:]]", "")
  workingWords <- stringr::str_remove_all(workingWords, "[^[\\da-zA-Z ]]")
  workingWords <- stringr::str_remove_all(workingWords, "[^\x01-\x7F]+")
  workingWords <- stringr::str_to_lower(workingWords)
  workingWords <- trimws(workingWords)

  workingWords <- workingWords[workingWords != ""]

  return(workingWords)
}

getMatches <- function(words,
                       conceptDf) {

  conceptDf <- conceptDf |> # start with all
    dplyr::mutate(concept_name = tolower(.data$concept_name))

  # because there may be a lot of synonyms, get these from a loop
  # (stringr::str_detect slows considerably
  # as more options are added in a single call using "|")

  # note, where one term is multiple words (e.g "knee osteoarthritis"),
  # split up and search
  # so that they don´t need to be next to each other
  # (e.g. to find "osteoarthritis of knee")).
  # This only will be applied if the word is not between / (e.g., (1) "/knee osteoarthritis/
  # will only exclude those concepts that contain "knee osteoarthritis", so
  # "osteoarthritis of knee" would not be excluded, but "history of knee osteoarthritis"
  # would be, (2) /ee osteoarthritis/ will also include those concepts with "knee osteoarthritis".
  # If we want to apply exact matches accounting for words boundaries, we need to use "/\bknee osteoarthritis/\b".

  # Notice that this will only be applied for exclude argument, as it is the only one
  # that does not apply tidyWords before using getMatches() function.
  conceptsFound <- list()

  # Exact matches only (including word boundaries)
  exactMatchesBoundaries <- tidyWords(words[stringr::str_detect(words,"^/\b.*/\b$")])
  for(i in seq_along(exactMatchesBoundaries)){
    workingExclude <- exactMatchesBoundaries[i]
    workingConcepts <- conceptDf # start with all

    if (nchar(workingExclude) >= 1) {
      cToSearch <-  workingExclude
      workingConcepts <- workingConcepts |>
        dplyr::filter(stringr::str_like(.data$concept_name, .env$cToSearch))
    }

    conceptsFound[[i]] <- workingConcepts |>
      dplyr::compute()
  }

  exactMatches <- tidyWords(words[stringr::str_detect(words,"^/.*/$")])
  for (i in seq_along(exactMatches)) {
    workingExclude <- exactMatches[i]
    workingConcepts <- conceptDf # start with all

    if (nchar(workingExclude) >= 1) {
      cToSearch <-  paste0("%", workingExclude, "%")
      workingConcepts <- workingConcepts |>
        dplyr::filter(stringr::str_like(.data$concept_name, .env$cToSearch))
    }

    conceptsFound[[i+length(exactMatchesBoundaries)]] <- workingConcepts |>
      dplyr::compute()
  }

  # Flexible search only
  words <- tidyWords(words[!c(stringr::str_detect(words, "^/.*/$")|stringr::str_detect(words,"^/\b.*/\b$")) ])
  for (i in seq_along(words)) {
    workingExclude <- unlist(strsplit(words[i], " "))
    workingConcepts <- conceptDf # start with all

    for (j in seq_along(workingExclude)) {
      if (nchar(workingExclude[j]) >= 1) {

        cToSearch <-  paste0("%", workingExclude[j], "%")
        workingConcepts <- workingConcepts |>
          dplyr::filter(stringr::str_like(.data$concept_name, .env$cToSearch))

      }
    }
    conceptsFound[[i+length(exactMatchesBoundaries)+length(exactMatches)]] <- workingConcepts |>
      dplyr::compute()
  }

  if(length(conceptsFound) == 0){
    return()
  }

  if(length(conceptsFound)==1){
    conceptsFound <- conceptsFound[[1]] |> dplyr::distinct()
  } else {
    conceptsFoundList <- list()
    conceptsFoundList[[1]] <- conceptsFound[[1]]
    for(i in 1:(length(conceptsFound)-1)){
      conceptsFoundList[[1]] <- dplyr::union_all(conceptsFoundList[[1]],
                                                 conceptsFound[[i+1]])

    }
    conceptsFound <- conceptsFoundList[[1]] |> dplyr::distinct()
  }

  return(conceptsFound)
}

addDescendants <- function(workingCandidateCodes,
                           conceptAncestorDf,
                           conceptDf) {

  candidateCodeDescendants <- workingCandidateCodes |>
    dplyr::select("concept_id") |>
    dplyr::rename("ancestor_concept_id" = "concept_id") |>
    dplyr::left_join(
      conceptAncestorDf,
      by = "ancestor_concept_id"
    ) |>
    dplyr::select("concept_id" = "descendant_concept_id") |>
    dplyr::distinct() |>
    dplyr::compute()

  candidateCodeDescendants <- candidateCodeDescendants |>
    dplyr::left_join(conceptDf, by = "concept_id") |>
    dplyr::compute()

  return(candidateCodeDescendants)
}

addAncestor <- function(workingCandidateCodes,
                        conceptAncestorDf,
                        conceptDf) {
  candidateCodeAncestor <- workingCandidateCodes |>
    dplyr::select("concept_id") |>
    dplyr::rename("descendant_concept_id" = "concept_id") |>
    dplyr::left_join(conceptAncestorDf,
                     by = "descendant_concept_id"
    ) |>
    dplyr::filter(.data$min_levels_of_separation == "1") |>
    dplyr::select("ancestor_concept_id") |>
    dplyr::rename("concept_id" = "ancestor_concept_id") |>
    dplyr::left_join(conceptDf,
                     by = "concept_id"
    )

  # keep if not already in candidateCodes
  candidateCodeAncestor <- candidateCodeAncestor |>
    dplyr::anti_join(
      workingCandidateCodes |>
        dplyr::select("concept_id"),
      by = "concept_id"
    )

  return(candidateCodeAncestor)
}

test_that("trailing spaces", {
  expect_equal(tidyWords("abc"), tidyWords(" abc "))
})
test_that("hyphens", {
  expect_equal(tidyWords("ab c"), tidyWords("ab-c"))
})
test_that("capitalisation", {
  expect_equal(tidyWords("AbC"), tidyWords("abc"))
})

test_that("non utf", {
  expect_equal(tidyWords("[ ¹⁸ F]AlF-NOTA-FAPI-04"),
               "falf nota fapi 04")
})

# This code is experimental and will be moved to omopgenerics when stable

#' Create a new `code_search` object
#'
#' `r lifecycle::badge('experimental')`
#' This function will be moved to omopgenerics (and re-exported by
#' CodelistGenerator) once stable.
#'
#' @param codes A tibble with concept ids. It must contain "search_id",
#' "concept_id", "found_from", "vocabulary_name", "concept_name", "domain_id",
#' "vocabulary_id", "concept_class_id", "standard_concept", "concept_code",
#' "valid_start_date", "valid_end_date" and "invalid_reason" as columns.
#' @param searchStrategy A tibble with the search strategy used to derive the
#' codes. It must contain "search_id", "strategy_name" and "strategy_value" as
#' columns.
#'
#' @returns A `code_search` object.
#'
newCodeSearch <- function(codes, searchStrategy) {
  colsCodes <- codeSerachColumns |>
    dplyr::filter(.data$table == "codes") |>
    dplyr::pull("column")
  omopgenerics::assertTable(codes, columns = colsCodes)

  colsStrategy <- codeSerachColumns |>
    dplyr::filter(.data$table == "search_strategy") |>
    dplyr::pull("column")
  omopgenerics::assertTable(searchStrategy, columns = colsStrategy)

  # make them tibbles
  codes <- dplyr::as_tibble(codes) |>
    dplyr::relocate(dplyr::all_of(colsCodes))
  searchStrategy <- dplyr::as_tibble(searchStrategy) |>
    dplyr::relocate(dplyr::all_of(colsStrategy))

  # cast columns
  codes <- castColumns(codes, "codes")
  searchStrategy <- castColumns(searchStrategy, "search_strategy")

  structure(
    .Data = codes,
    search_strategy = searchStrategy,
    class = c("code_search", class(codes))
  )
}

validateCodeSearch <- function(codeSearch) {
  omopgenerics::assertClass(codeSearch, "code_search")
}

#' Empty code search object
#'
#' @param searchStrategy The search strategy used to generate the candidate
#' codes.
#'
#' @returns An empty `code_search` object.
#' @export
#'
emptyCodeSearch <- function(searchStrategy = NULL) {
  if (is.null(searchStrategy)) {
    searchStrategy <- dplyr::tibble(
      strategy_id = integer(),
      strategy_name = character(),
      strategy_value = character()
    )
  }
  dplyr::tibble(
    concept_id = integer(),
    found_from = character(),
    concept_name = character(),
    vocabulary_version = character(),
    domain_id = character(),
    vocabulary_id = character(),
    concept_class_id = character(),
    standard_concept = character(),
    concept_code = character(),
    valid_start_date = as.Date(character()),
    valid_end_date = as.Date(character()),
    invalid_reason = character()
  ) |>
    newCodeSearch(searchStrategy = searchStrategy)
}

castColumns <- function(x, table) {
  cols <- codeSerachColumns |>
    dplyr::filter(.data$table == .env$table)
  q <- paste0(cols$convert, "(.data$", cols$column, ")") |>
    rlang::parse_exprs() |>
    rlang::set_names(nm = cols$column)
  x |>
    dplyr::mutate(!!!q)
}

#' @export
print.code_search <- function(x, ...) {
  # info
  cat("\033[1;34mi\033[0m This is a candidate code searh, see: <link to vignette>\n")
  # search strategy
  st <- searchStrategy(x) |>
    dplyr::filter(.data$strategy_id == 1L) |>
    tidyr::pivot_wider(names_from = "strategy_name", values_from = "strategy_value")
  if (nrow(st) > 0) {
    pkg <- paste0("\033[32m", st$package_name %||% "unknown package", "\033[0m (\033[1m",  st$package_version %||% "unknown version", "\033[0m)")
    cat(paste0("\033[3mCandidate codes\033[0m generated using ", pkg, ":\n"))
    fun <- st$function_name %||% ""
    parameters <- st |>
      dplyr::select(!dplyr::any_of(c("package_name", "package_version", "strategy_id", "function_name"))) |>
      purrr::imap_chr(\(x, nm) paste0(nm, " = ", x)) |>
      paste0(collapse = ",\n  ")
    cat(paste0(fun, "(\n  ", parameters, "\n)\n"))
  } else {
    cat("\033[3mNo search strategy found.\033[0m\n")
  }
  NextMethod()
}

#' Import a `code_search` object from an Excel spreadsheet
#'
#' `r lifecycle::badge('experimental')`
#' This function will be moved to omopgenerics (and re-exported by
#' CodelistGenerator) once stable.
#'
#' @param path Path to the Excel spreadsheet.
#'
#' @returns A `code_search` object.
#' @export
#'
importCodeSearch <- function(path) {
  omopgenerics::assertCharacter(path, length = 1)
  rlang::check_installed("openxlsx")

  if (file.exists(path)) {
    file <- path
  } else if (file.exists(paste0(path, ".xlsx"))) {
    file <- paste0(path, ".xlsx")
  } else if (file.exists(paste0(path, ".xls"))) {
    file <- paste0(path, ".xls")
  } else {
    if (dir.exists(path)) {
      file <- list.files(path, pattern = ".xls$|.xlsx$", full.names = TRUE)
      if (length(file) > 1) {
        cli::cli_abort(c(x = "Multiple Excel spreadsheets found in {.path {path}} please specify just one."))
      }
    } else {
      cli::cli_abort(c(x = "Provided file/path does not exist."))
    }
  }

  sheets <- openxlsx::getSheetNames(file = file)
  notPresent <- c("SearchStrategy", "CandidateCodes") |>
    purrr::keep(\(x) !x %in% sheets)
  if (length(notPresent) > 0) {
    cli::cli_abort(c(x = "{.var {notPresent}} sheet{?s} not present in excel file."))
  }

  newCodeSearch(
    codes = openxlsx::read.xlsx(xlsxFile = file, sheet = "CandidateCodes", detectDates = TRUE),
    searchStrategy = openxlsx::read.xlsx(xlsxFile = file, sheet = "SearchStrategy")
  )
}

#' Export a `code_search` object into an Excel spreadsheet
#'
#' `r lifecycle::badge('experimental')`
#' This function will be moved to omopgenerics (and re-exported by
#' CodelistGenerator) once stable.
#'
#' @param codeSearch A `code_search` object.
#' @param file Name of the Excel spreadsheet to be created.
#' @param path Path where to create the Excel file.
#'
#' @returns The `code_search` object is exported to a file.
#' @export
#'
exportCodeSearch <- function(codeSearch, file, path = getwd()) {
  # initial checks
  codeSearch <- validateCodeSearch(codeSearch)
  omopgenerics::assertCharacter(file, length = 1)

  rlang::check_installed("openxlsx")

  if (!endsWith(file, ".xlsx") & !endsWith(file, ".xls")) {
    file <- paste0(file, ".xlsx")
  }

  # add instructions
  wb <- openxlsx::loadWorkbook(
    file = system.file("Instructions.xlsx", package = "CodelistGenerator")
  )

  # add search strategy
  openxlsx::addWorksheet(wb = wb, sheetName = "SearchStrategy")
  openxlsx::writeData(wb = wb, sheet = "SearchStrategy", x = searchStrategy(codeSearch))

  # add candidate codes
  openxlsx::addWorksheet(wb = wb, sheetName = "CandidateCodes")
  openxlsx::writeData(wb = wb, sheet = "CandidateCodes", x = codeSearch)

  openxlsx::saveWorkbook(wb = wb, file = file.path(path, file), overwrite = TRUE)
}


#' Get the search strategy used to create a certain code_search
#'
#' `r lifecycle::badge('experimental')`
#' This function will be moved to omopgenerics (and re-exported by
#' CodelistGenerator) once stable.
#'
#' @param codeSearch A `code_search` object.
#'
#' @returns A search strategy tibble.
#' @export
#'
searchStrategy <- function(codeSearch) {
  validateCodeSearch(codeSearch = codeSearch)
  attr(codeSearch, "search_strategy")
}

cast <- function(x) {
  if (length(x) == 0) {
    x <- cast0(x)
  } else if (length(x) == 1) {
    x <- cast1(x)
  } else {
    x <- paste0("c(", paste0(purrr::map_chr(x, cast1), collapse = ", "), ")")
  }
  return(x)
}
cast0 <- function(x) {
  if (is.null(x)) {
    x <- "NULL"
  } else if (is.logical(x)) {
    x <- "logical()"
  } else if (is.integer(x)) {
    x <- "integer()"
  } else if (is.numeric(x)) {
    x <- "numeric()"
  } else if (is.character(x)) {
    x <- "character()"
  } else {
    cli::cli_abort(c(x = "Not indeitified type: {.cls {class(x)}}"))
  }
  return(x)
}
cast1 <- function(x) {
  if (is.logical(x)) {
    x <- as.character(x)
  } else if (is.integer(x)) {
    x <- sprintf("%iL", x)
  } else if (is.numeric(x)) {
    x <- as.character(x)
  } else if (is.character(x)) {
    x <- paste0('"', x, '"')
  } else {
    cli::cli_abort(c(x = "not indeitified type: {.cls {class(x)}}"))
  }
  return(x)
}

prepareAsCodelist <- function(x) {
  comp <- codeSerachColumns |>
    dplyr::filter(.data$table == "codes") |>
    dplyr::pull("column")
  cols <- colnames(x)
  cols <- cols[!cols %in% comp]

  if (length(cols) == 0) {
    cli::cli_inform(c("!" = "No column specifying T/F for the different codes is detected"))
    return(list())
  }

  cols <- rlang::set_names(cols, omopgenerics::toSnakeCase(x = cols))

  cols |>
    purrr::map(\(col) {
      x |>
        dplyr::filter(as.logical(.data[[col]])) |>
        dplyr::select(dplyr::any_of(comp))
    })
}

#' @export
asCodelist.code_search <- function(x, ...) {
  x |>
    prepareAsCodelist() |>
    purrr::map(\(x) as.integer(x$concept_id)) |>
    newCodelist()
}

#' @export
asConceptSetExpression.code_search <- function(x, ...) {
  x |>
    prepareAsCodelist() |>
    # purrr::map(\(x) {
    #   x |>
    #     dplyr::mutate(descendants = FALSE, exclude = FALSE, mapped = FALSE)
    # }) |>
    newConceptSetExpression()
}

#' @export
asCodelistWithDetails.code_search <- function(x, ...) {
  x |>
    prepareAsCodelist() |>
    newCodelistWithDetails()
}

test_that("emptyCodeSearch returns a code_search object", {
  skip_on_cran()

  # empty object
  expect_no_error(cs <- emptyCodeSearch())
  expect_s3_class(cs, "code_search")
  expect_equal(nrow(cs), 0L)
  cols <- codeSerachColumns$column[codeSerachColumns$table == "codes"]
  expect_true(all(cols %in% colnames(cs)))
  st <- searchStrategy(cs)
  expect_equal(nrow(st), 0L)
  cols <- codeSerachColumns$column[codeSerachColumns$table == "search_strategy"]
  expect_true(all(cols %in% colnames(st)))
  cs1 <- emptyCodeSearch(searchStrategy = NULL)
  expect_identical(cs, cs1)

  # empty object with code search
  st <- dplyr::tibble(
    strategy_id = 1L,
    strategy_name = c("package_name", "function_name", "package_version"),
    strategy_value = c("CodelistGenerator", "search", "1.0")
  )
  expect_no_error(cs <- emptyCodeSearch(searchStrategy = st))
  expect_equal(nrow(searchStrategy(cs)), 3)
  expect_identical(searchStrategy(cs), st)

  # helper
  codesTibble <- function(n = 10) {
    dplyr::tibble(
      concept_id = as.integer(seq_len(n)),
      found_from = "Descendants",
      concept_name = paste0("Concept ", seq_len(n)),
      vocabulary_version = "v1.0",
      domain_id = "Condition",
      vocabulary_id = "SNOMED",
      concept_class_id = "Clinical Finding",
      standard_concept = "S",
      concept_code = paste0("C00", seq_len(n)),
      valid_start_date = as.Date("1970-01-01"),
      valid_end_date = as.Date("2099-12-31"),
      invalid_reason = NA_character_
    )
  }

  expect_no_error(cs <- newCodeSearch(codes = codesTibble(), searchStrategy = st))
  expect_true(nrow(cs) == 10)
  expect_true(nrow(searchStrategy(cs)) == 3)
  expect_s3_class(cs, "code_search")
  expect_s3_class(cs, "tbl_df")
  expect_s3_class(searchStrategy(cs), "tbl_df")
  expect_identical(searchStrategy(cs), st)

  # print works
  expect_output(print(cs))
  expect_output(print(emptyCodeSearch()))

  # errors
  bad_codes <- dplyr::tibble(concept_id = 1L)  # missing many required columns
  expect_error(newCodeSearch(codes = bad_codes, searchStrategy = st))
  expect_error(searchStrategy(list()))
  expect_error(searchStrategy(dplyr::tibble()))
  expect_error(cs <- newCodeSearch(codes = codesTibble(), searchStrategy = NULL))
  bad_strategy <- dplyr::tibble(strategy_name = "keyword")  # missing strategy_id, strategy_value
  expect_error(newCodeSearch(codes = makeCodesTibble(), searchStrategy = bad_strategy))

  # cast
  codes <- codesTibble()
  codes$concept_id <- as.double(codes$concept_id)
  codes$valid_start_date <- as.double(codes$valid_start_date)
  cs <- newCodeSearch(codes = codes, searchStrategy = st)
  expect_type(cs$concept_id, "integer")
  expect_s3_class(cs$valid_start_date, "Date")

  # import and export
  tmp <- tempdir()
  exportCodeSearch(cs, file = "test_export", path = tmp)
  expect_true(file.exists(file.path(tmp, "test_export.xlsx")))
  cs0 <- importCodeSearch(path = file.path(tmp, "test_export.xlsx"))
  expect_identical(cs, cs0)

  exportCodeSearch(cs, file = "test_export2.xlsx", path = tmp)
  expect_true(file.exists(file.path(tmp, "test_export2.xlsx")))
  cs1 <- importCodeSearch(path = file.path(tmp, "test_export2.xlsx"))
  expect_identical(cs, cs1)

  expect_error(importCodeSearch(path = "/nonexistent/path/file.xlsx"))

  bad_file <- file.path(tmp, "bad.xlsx")
  wb <- openxlsx::createWorkbook()
  openxlsx::addWorksheet(wb, "OnlyThisSheet")
  openxlsx::saveWorkbook(wb, bad_file, overwrite = TRUE)
  expect_error(importCodeSearch(path = bad_file))

  # extra sheets not a problem
  file <- file.path(tmp, "test_export2.xlsx")
  wb <- openxlsx::loadWorkbook(file = file)
  openxlsx::addWorksheet(wb = wb, sheetName = "ExtraSheet")
  openxlsx::writeData(wb = wb, sheet = "ExtraSheet", x = cars) |>
  openxlsx::saveWorkbook(wb = wb, file = file, overwrite = TRUE)
  expect_true(file.exists(file))
  expect_true("ExtraSheet" %in% openxlsx::getSheetNames(file))
  expect_no_error(cs2 <- importCodeSearch(path = file))
  expect_identical(cs, cs2)

  # asCodelist
  codes <- codesTibble(n = 4)
  codes$new_codelist <- c(TRUE, FALSE, TRUE, FALSE)
  cs <- newCodeSearch(codes = codes, searchStrategy = st)
  expect_no_error(cd <- asCodelist(cs))
  expect_s3_class(cd, "codelist")
  expect_true("new_codelist" %in% names(cd))
  expect_identical(cd[["new_codelist"]], c(1L, 3L))
  cs <- cs |>
    dplyr::mutate(second_codelist = concept_id %% 2 == 0)
  expect_no_error(cd <- asCodelist(cs))
  expect_true("new_codelist" %in% names(cd))
  expect_true("second_codelist" %in% names(cd))
  expect_true(length(cd) == 2)
  expect_identical(cd[["second_codelist"]], c(2L, 4L))

  # asConceptSetExpression
  codes <- codesTibble(n = 4)
  codes$new_codelist <- c(TRUE, FALSE, TRUE, FALSE)
  cs <- newCodeSearch(codes = codes, searchStrategy = st)
  expect_no_error(cd <- asConceptSetExpression(cs))
  expect_s3_class(cd, "concept_set_expression")
  expect_true("new_codelist" %in% names(cd))
  expect_true(all(c("concept_id", "excluded", "descendants", "mapped") %in% colnames(cd$new_codelist)))
  cs <- cs |>
    dplyr::mutate(second_codelist = concept_id %% 2 == 0)
  expect_no_error(cd <- asConceptSetExpression(cs))
  expect_true("new_codelist" %in% names(cd))
  expect_true("second_codelist" %in% names(cd))
  expect_true(length(cd) == 2)

  # asCodelistWithDetails
  codes <- codesTibble(n = 4)
  codes$new_codelist <- c(TRUE, FALSE, TRUE, FALSE)
  cs <- newCodeSearch(codes = codes, searchStrategy = st)
  expect_no_error(cd <- asCodelistWithDetails(cs))
  expect_s3_class(cd, "codelist_with_details")
  expect_true("new_codelist" %in% names(cd))
  cols <- codeSerachColumns$column[codeSerachColumns$table == "codes"]
  expect_true(all(cols %in% colnames(cd$new_codelist)))
  cs <- cs |>
    dplyr::mutate(second_codelist = concept_id %% 2 == 0)
  expect_no_error(cd <- asCodelistWithDetails(cs))
  expect_true("new_codelist" %in% names(cd))
  expect_true("second_codelist" %in% names(cd))
  expect_true(length(cd) == 2)

  # cast helper
  expect_equal(cast(NULL), "NULL")
  expect_equal(cast(character()), "character()")
  expect_equal(cast(integer()), "integer()")
  expect_equal(cast(1L), "1L")
  expect_equal(cast("hello"), '"hello"')
  expect_equal(cast(c(1L, 2L, 3L)), "c(1L, 2L, 3L)")
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Run benchmark of codelistGenerator analyses
#'
#' @param cdm a CDM reference object
#'
#' @return a tibble with time taken for different analysis
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#'
#' cdm <- mockVocabRef()
#'
#' timings <- benchmarkCodelistGenerator(cdm)
#' }
#'

benchmarkCodelistGenerator <- function(cdm){

  # Initial checks ----
  omopgenerics::validateCdmArgument(cdm)

  timings <- list()

  # Create vocabulary based codelists ----
  ## getDrugIngredientCodes ----
  tictoc::tic()
  x <- getDrugIngredientCodes(cdm = cdm,
                              name = c("acetaminophen", "codeine", "adalimumab"),
                              nameStyle = "{concept_code}_{concept_name}",
                              doseForm  = NULL,
                              doseUnit  = NULL,
                              routeCategory = NULL,
                              ingredientRange = c(1,Inf),
                              type = "codelist")
  t <- tictoc::toc(quiet = TRUE)

  timings[["getDrugIngredientCodes"]] <- dplyr::tibble(
    task = "getting drug ingredient codes (acetaminophen, codeine, adalimumab)",
    time_taken_secs = as.numeric(t$toc - t$tic)
  )

  ## getATCCodes ----
  tictoc::tic()
  x <- getATCCodes(cdm = cdm,
                   level = c("ATC 1st"),
                   name = c("Alimentary tract and metabolism"),
                   nameStyle = "{concept_code}_{concept_name}",
                   doseForm = NULL,
                   doseUnit = NULL,
                   routeCategory = NULL,
                   type = "codelist")
  t <- tictoc::toc(quiet = TRUE)

  timings[["getATCCodes"]] <- dplyr::tibble(
    task = "getting ATC codes (ATC 1st level, 1 name of interest)",
    time_taken_secs = as.numeric(t$toc - t$tic)
  )

  ## getCandidateCodes ----
  tictoc::tic()
  x <- getCandidateCodes(
    cdm,
    keywords = "dementia",
    exclude = c("child"),
    domains = NULL,
    standardConcept = "Standard",
    searchInSynonyms = TRUE,
    searchNonStandard = TRUE,
    includeDescendants = TRUE,
    includeAncestor = TRUE
  )
  t <- tictoc::toc(quiet = TRUE)

  timings[["getCandidateCodes"]] <- dplyr::tibble(
    task = "getting candidate codes for dementia, excluding `child`, within all domains, and all options set to TRUE",
    time_taken_secs = as.numeric(t$toc - t$tic)
  )

  # Prepare summarised result -----
  timings <- dplyr::bind_rows(timings) |>
    dplyr::mutate(time_taken_mins = round(.data$time_taken_secs / 60, 2)) |>
    dplyr::mutate(dbms = attr(attr(cdm, "cdm_source"), "source_type")) |>
    dplyr::mutate(n_person = cdm$person |>
                    dplyr::count() |>
                    dplyr::pull()) |>
    dplyr::mutate(n_concepts = cdm$concept |>
                    dplyr::select("concept_id") |>
                    dplyr::distinct() |>
                    dplyr::count() |>
                    dplyr::pull())

  # As a summarised result
  timings <- timings |>
    dplyr::mutate(
      "result_id" = 1L,
      "cdm_name"  = omopgenerics::cdmName(cdm),
      "group_name"  = "task",
      "group_level" = stringr::str_to_sentence(gsub("_"," ",.data$task)),
      "strata_name" = "overall",
      "strata_level"   = "overall",
      "variable_name"  = "overall",
      "variable_level" = "overall",
      "estimate_name"  = "time_taken_minutes",
      "estimate_type"  = "numeric",
      "estimate_value" = as.character(.data$time_taken_mins),
      "additional_name" = paste0(
        "dbms &&& n_person &&& ",
        "n_concepts"
      ),
      additional_level = paste0(
        .data$dbms, " &&& ",
        formatC(as.numeric(.data$n_person), format = "f", digits = 0, big.mark = ","),
        " &&& ",
        formatC(as.numeric(.data$n_concepts), format = "f", digits = 0, big.mark = ",")
      )
    ) |>
    dplyr::select(dplyr::all_of(
      colnames(omopgenerics::emptySummarisedResult())
    )) |>
    omopgenerics::newSummarisedResult(settings = dplyr::tibble(
      result_id = 1L,
      result_type = "CodelistGenerator benchmark",
      package_name = "CodelistGenerator",
      package_version = as.character(utils::packageVersion("CodelistGenerator"))
    ))

  return(timings)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get the dose forms for drug concepts
#'
#' @inheritParams cdmDoc
#'
#' @return The dose forms available for drug concepts.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all domains available in the CDM
#' availableDoseForms(cdm = cdm)
#' }
availableDoseForms <- function(cdm) {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)

  # all dose forms in the cdm
    doseForms <- cdm[["concept_relationship"]] |>
      dplyr::filter(.data$relationship_id == "RxNorm has dose form") |>
      dplyr::select("dose_form_concept_id" = "concept_id_2") |>
      dplyr::distinct() |>
      dplyr::collect()

    doseForms <- doseForms |>
      dplyr::left_join(
        CodelistGenerator::doseFormToRoute,
        by = "dose_form_concept_id"
      ) |>
      dplyr::pull("dose_form_concept_name") |>
      unique() |>
      sort()

    return(doseForms)

}

#' Get the dose forms associated with drug concepts in a codelist
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#'
#' @return The dose forms available for drug concepts.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all dose forms available in a codelist
#' codelist <- newCodelist(list("codes1" = c(194152L, 1830279L, 40558872L),
#'                              "codes2" = c(44022939L)))
#' associatedDoseForms(x = codelist, cdm = cdm)
#' }
associatedDoseForms <- function(x,
                                cdm) {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  checkCodelist(x, allowConceptSetExpression = FALSE)

  original <- x

  # codelist table
  nm <- omopgenerics::uniqueTableName()
  x <- dplyr::as_tibble(x) |>
    dplyr::rename(codelist_name = dplyr::any_of("codelist_with_details_name")) |>
    dplyr::select("codelist_name", "concept_id")

  cdm <- omopgenerics::insertTable(cdm = cdm,
                                   name = nm,
                                   table = x)
  on.exit(omopgenerics::dropSourceTable(cdm = cdm, name = nm))

  x <- addDoseForm(x = cdm[[nm]])

  # correct missing
  x <- correctMissingValue(x, "dose_form")

  # stratify codelist
  x <- stratifyCodelist(
    x = x,
    by = c("codelist_name"),
    nameStyle = "{codelist_name}"
  )

  x <- purrr::map(x,
                  \(x) dplyr::pull(x, "dose_form") |> unique() |> sort())

  return(x)
}

test_that("tests with mock db", {
  skip_on_cran()
  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])
    codelist_with_details <- list("codes" = cdm[["concept"]] |>
                                    dplyr::filter(concept_id %in% c(10L, 15L, 24L)) |>
                                    dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
                                    dplyr::mutate("ancestor_concept_id" = 999L) |>
                                    dplyr::collect()) |>
      omopgenerics::newCodelistWithDetails()

    codelist <- list("first_codelist" = c(1L, 20L), "second_codelist" = c(10L, 15L)) |>
      omopgenerics::newCodelist()

    # Whole db
    expect_no_error(doseForms <- availableDoseForms(cdm = cdm))
    expect_identical(c("Injectable Foam", "Injection", "Nasal Powder", "Topical Liquefied Gas"),
                     doseForms)
    expect_no_error(doseForms <- associatedDoseForms(cdm = cdm, x = codelist))
    expect_true(all(names(doseForms) == c("first_codelist", "second_codelist")))
    expect_identical(doseForms$first_codelist, c("nasal_powder", "unclassified_dose_form"))
    expect_identical(doseForms$second_codelist, c("injection", "unclassified_dose_form"))


    # expected errors
    expect_error(availableDoseForms(cdm = "a"))

    expect_error(associatedDoseForms(cdm = "a"))
    expect_error(associatedDoseForms(cdm, x = NULL))
    expect_error(associatedDoseForms(cdm, x = 123))
    expect_error(associatedDoseForms(cdm, x = list(123)))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get the names of all available drug ingredients
#'
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#'
#' @return A vector containing the concept names for all ingredient level codes
#' found in the concept table of cdm.
#'
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all drug ingredients available in the CDM for standard concepts
#' availableDrugIngredients(cdm = cdm)
#'}
availableDrugIngredients <- function(cdm, standardConcept = "Standard") {

  #initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  standardConcept <- assertStandardConcept(standardConcept)

  drugIngredients <- drugIngredientsInternal(cdm,
                                             newOmopTable = "concept",
                                             standardConcept = standardConcept,
                                             xx = NULL)

  return(drugIngredients)
}

#' Get the names of drug ingredients associated with codelist
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#'
#' @return A vector containing the concept names for all ingredient level codes
#' found in the concept table of cdm.
#'
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all drug ingredients associated with a codelist
#' codelist <- newCodelist(list("codes1" = c(37498042L),
#'                              "codes2" = c( 42899580L, 35741956L)))
#' associatedDrugIngredients(x = codelist, cdm = cdm,
#'                          standardConcept = c("Standard", "Non-standard"))
#'}
associatedDrugIngredients <- function(x, cdm, standardConcept = "Standard") {

  #initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  standardConcept <- assertStandardConcept(standardConcept)
  checkCodelist(x, allowConceptSetExpression = FALSE)

  if(inherits(x, "codelist_with_details")){
    x <- asCodelist(x)
  }

  if(inherits(x, "codelist")){
    newOmopTable <- paste0(omopgenerics::uniqueTableName(),
                           omopgenerics::uniqueId())
    codesTable <- paste0(omopgenerics::uniqueTableName(),
                         omopgenerics::uniqueId())
    cdm <- subsetOmopTable(cdm, newOmopTable, "concept", x)

    drugIngredients <- list()
    for( i in seq_along(x) ){
      drugIngredients[[names(x)[[i]]]] <- drugIngredientsInternal(cdm,
                                                                  newOmopTable = newOmopTable,
                                                                  codesTable = codesTable,
                                                                  standardConcept = standardConcept,
                                                                  xx = x[[i]])
    }
    omopgenerics::dropSourceTable(cdm, name = c(newOmopTable, codesTable))
  }

  return(drugIngredients)
}


drugIngredientsInternal <- function(cdm,
                                    newOmopTable,
                                    codesTable,
                                    standardConcept,
                                    xx){

  ingredientConcepts <- cdm[[newOmopTable]]

  if(!is.null(xx)){
    cdm <- omopgenerics::insertTable(
      cdm, name = codesTable, dplyr::tibble(descendant_concept_id = xx))

    ingredientConcepts <- cdm$concept_ancestor |>
      dplyr::select("ancestor_concept_id", "descendant_concept_id") |>
      dplyr::inner_join(cdm[[codesTable]], by = "descendant_concept_id") |>
      dplyr::select("concept_id" = "ancestor_concept_id") |>
      dplyr::left_join(cdm$concept |>
                         dplyr::select("concept_id",
                                       "concept_name",
                                       "concept_class_id",
                                       "standard_concept"),
                       by = "concept_id") |>
      dplyr::mutate(standard_concept = dplyr::case_when(
        is.na(.data$standard_concept) ~ "non-standard",
        .data$standard_concept == "C" ~ "classification",
        .data$standard_concept == "S" ~ "standard"
      )) |>
      dplyr::filter(.data$concept_class_id == "Ingredient",
                    .data$standard_concept %in% .env$standardConcept) |>
      dplyr::select("concept_name") |>
      dplyr::distinct() |>
      dplyr::pull("concept_name") |>
      sort()
  } else {
    ingredientConcepts <- ingredientConcepts |>
      dplyr::filter(.data$concept_class_id == "Ingredient") |>
      dplyr::mutate(standard_concept = dplyr::case_when(
        is.na(.data$standard_concept) ~ "non-standard",
        .data$standard_concept == "C" ~ "classification",
        .data$standard_concept == "S" ~ "standard"
      )) |>
      dplyr::filter(.data$standard_concept %in% .env$standardConcept) |>
      dplyr::pull("concept_name")|>
      sort()
  }


}

test_that("available ingredients", {
  skip_on_cran()
  backends <- c("database", "data_frame")
  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backends[[i]])
    namesInitial <- names(cdm)
    expect_no_error(res <- availableDrugIngredients(cdm))

    # Whole CDM
    manual_res <- cdm$concept |>
      dplyr::filter(standard_concept == "S" & concept_class_id == "Ingredient") |>
      dplyr::pull("concept_name")
    expect_true(setequal(res, manual_res))

    # codelist
    codelist <- list("first_codelist" = c(10L), "second_codelist" = c(7L, 9L)) |>
      omopgenerics::newCodelist()

    expect_no_error(res1 <- associatedDrugIngredients(cdm, x = codelist))
    expect_equal(names(res1), c("first_codelist", "second_codelist"))
    expect_true(res1$first_codelist == "Adalimumab")
    expect_true(length(res1$second_codelist) == 0)
    expect_equal(names(cdm), namesInitial)

    c <- list("descendant_drug" = c(13L), "ingredient_concept" = c(10L)) |>
      omopgenerics::newCodelist() |>
      associatedDrugIngredients(cdm)
    expect_identical(c[[1]], c("Adalimumab", "Other ingredient"))
    expect_identical(c[[2]], c("Adalimumab"))

    codelist_with_details <- list("codes" = cdm[["concept"]] |>
                                    dplyr::filter(concept_id %in% c(10L)) |>
                                    dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
                                    dplyr::mutate("ancestor_concept_id" = 999L) |>
                                    dplyr::collect()) |>
      omopgenerics::newCodelistWithDetails()
    expect_no_error(res3 <- associatedDrugIngredients(cdm, x = codelist_with_details))
    expect_true(res3$codes == "Adalimumab")
    expect_no_error(res3 <- associatedDrugIngredients(cdm, x = codelist_with_details, standardConcept = "Non-standard"))
    expect_true(length(res3$codes) == 0)

    # Expected errors
    expect_error(availableDrugIngredients(cdm, x = list("codes" = 10L)))
    expect_error(availableDrugIngredients(cdm = 1))
    expect_error(availableDrugIngredients(cdm, standardConcept = "S"))

    expect_error(associatedDrugIngredients(cdm, x = list("codes" = 10L)))
    expect_error(associatedDrugIngredients(cdm))
    expect_error(associatedDrugIngredients(cdm, x, standardConcept = "S"))

  }
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get the available concept classes used in a given set of domains
#'
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#' @inheritParams domainDoc
#'
#' @return The concept classes
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmFromDataset(datasetName = "GiBleed")
#'
#' # Get all available concept_class_ids in the CDM
#' availableConceptClassIds(cdm,
#'                          standardConcept = "Standard")
#'
#' # Get all available concept_class_ids in the CDM for a specific domain
#' availableConceptClassIds(cdm,
#'                          standardConcept = "Standard",
#'                          domain = "Condition")
#'
#' # Notice that this corresponds to the information provided by `concept_class_id`
#' # column in the `concept` table
#' }
availableConceptClassIds <- function(cdm,
                            standardConcept = "Standard",
                            domain = NULL) {
  #initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  domain <- assertDomain(domain, cdm)
  standardConcept <- assertStandardConcept(standardConcept)

    conceptClassIds <- conceptClassIdsInternal(cdm,
                                               newOmopTable = "concept",
                                               domain = domain,
                                               standardConcept = standardConcept,
                                               xx = NULL)

  return(conceptClassIds)
}

#' Get the concept classes associated with a codelist
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#' @inheritParams domainDoc
#'
#' @return The concept classes
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmFromDataset(datasetName = "GiBleed")
#'
#' # Get concept_class_ids in a codelist
#' x <- newCodelist(list("codes1" = c(1118088L, 40213201L, 35208414L),
#'                       "codes2" = c(1557272L, 4336464L, 4295880L)))
#' associatedConceptClassIds(x,
#'                           cdm,
#'                           standardConcept = "Standard")
#'
#' # Notice that this corresponds to the information provided by `concept_class_id`
#' # column in the `concept` table
#' }
associatedConceptClassIds <- function(x,
                                      cdm,
                                      standardConcept = "Standard",
                                      domain = NULL) {
  #initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  domain <- assertDomain(domain, cdm)
  standardConcept <- assertStandardConcept(standardConcept)
  checkCodelist(x, allowConceptSetExpression = FALSE)

  if(inherits(x, "codelist_with_details")){
    x <- asCodelist(x)
  }

  if(inherits(x, "codelist")){
    newOmopTable <- paste0(omopgenerics::uniqueTableName(),
                           omopgenerics::uniqueId())
    codesTable <- paste0(omopgenerics::uniqueTableName(),
                         omopgenerics::uniqueId())
    cdm <- subsetOmopTable(cdm, newOmopTable, "concept", x)

    conceptClassIds <- list()
    for( i in seq_along(x) ){
      conceptClassIds[[names(x)[[i]]]] <- conceptClassIdsInternal(cdm,
                                                                  newOmopTable = newOmopTable,
                                                                  codesTable = codesTable,
                                                                  domain = domain,
                                                                  standardConcept = standardConcept,
                                                                  xx = x[[i]])
    }
    omopgenerics::dropSourceTable(cdm, name = c(newOmopTable, codesTable))
  }

  return(conceptClassIds)
}

conceptClassIdsInternal <- function(cdm, newOmopTable, codesTable, domain, standardConcept, xx){

  conceptDb <- cdm[[newOmopTable]]

  if(!is.null(xx)){
    cdm <- omopgenerics::insertTable(
      cdm, name = codesTable, dplyr::tibble(concept_id = xx))
    conceptDb <- conceptDb |>
      dplyr::inner_join(cdm[[codesTable]], by = "concept_id")
  }

  conceptDb <- conceptDb |>
    dplyr::filter(tolower(.data$domain_id) %in% .env$domain) |>
    dplyr::mutate(standard_concept = dplyr::case_when(
      is.na(.data$standard_concept) ~ "non-standard",
      .data$standard_concept == "C" ~ "classification",
      .data$standard_concept == "S" ~ "standard"
    )) |>
    dplyr::filter(.data$standard_concept %in% .env$standardConcept)

  conceptClassId <- conceptDb |>
    dplyr::select("concept_class_id") |>
    dplyr::distinct() |>
    dplyr::collect() |>
    dplyr::pull()

  conceptClassId <- sort(conceptClassId)

  return(conceptClassId)
}

test_that("tests with mock db", {
  skip_on_cran()
  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])
    codelist_with_details <- list("codes" = cdm[["concept"]] |>
                                    dplyr::filter(concept_id %in% c(10L, 15L, 24L)) |>
                                    dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
                                    dplyr::mutate("ancestor_concept_id" = 999L) |>
                                    dplyr::collect()) |>
      omopgenerics::newCodelistWithDetails()

    codelist <- omopgenerics::newCodelist(list("codes1" = c(10L, 15L, 24L), "codes2" = c(1L, 2L, 3L, 9L)))

    # Available concept class id ----
    expect_no_error(conceptClasses <- availableConceptClassIds(cdm = cdm, standardConcept = c("Standard", "Non-standard")))
    expect_true(all(conceptClasses == c("ATC 1st", "Brand Name", "Clinical Drug Form",
                                        "Clinical Finding", "Diagnosis", "Dose Form",
                                        "Drug", "ICD Code", "ICD10 Chapter",
                                        "ICD10 SubChapter", "Ingredient", "Observation",
                                        "Unit" )))

    expect_no_error(conceptClasses <- availableConceptClassIds(cdm = cdm, domain = "Condition"))
    expect_true(conceptClasses == "Clinical Finding")

    expect_no_error(conceptClasses <- associatedConceptClassIds(cdm = cdm, x = codelist))
    expect_true(inherits(conceptClasses, "list"))
    expect_true(all(conceptClasses$codes1 == c("Ingredient")))
    expect_true(all(conceptClasses$codes2 == c("Clinical Finding", "Observation")))

    expect_no_error(conceptClasses <- associatedConceptClassIds(cdm = cdm, x = codelist_with_details))
    expect_true(all(conceptClasses$codes == c("Ingredient")))

    # expected errors
    expect_error(availableConceptClassIds(cdm = "a"))
    expect_error(availableConceptClassIds(cdm, standardConcept = FALSE))
    expect_error(availableConceptClassIds(cdm, x = 123))
    expect_error(availableConceptClassIds(cdm, x = list("c" = 12)))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get the descendant codes of Anatomical Therapeutic Chemical (ATC) classification codes
#'
#' @inheritParams cdmDoc
#' @inheritParams levelATCDoc
#' @param name ATC name of interest. For example, c("Dermatologicals",
#' "Nervous System"), would result in a list of length two with the descendant
#' concepts for these two particular ATC groups.
#' @inheritParams nameStyleDoc
#' @inheritParams doseFormDoc
#' @inheritParams doseUnitDoc
#' @inheritParams routeCategoryDoc
#' @inheritParams typeNarrowDoc
#'
#' @return Concepts with their format based on the type argument
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Create a codelist with 1st level ATC codes available in the CDM
#' codelist <- getATCCodes(cdm = cdm,
#'                         level = "ATC 1st")
#' codelist
#'
#' # Tune the name of the generated codelists
#' codelist <- getATCCodes(cdm = cdm,
#'                         level = "ATC 1st",
#'                         nameStyle = "{concept_name}_{concept_code}")
#' codelist
#'
#' # Search for a specific ATC name of interest
#' codelist <- getATCCodes(cdm = cdm,
#'                         level = "ATC 2nd",
#'                         name = "immunostimulants")
#' codelist
#'
#' # Restrict concepts to specific dose forms, dose units, or route categories.
#' # Remember that you can use `availableDoseForm()`, `availableDoseUnit()` and
#' # `availableRouteCategory()` to explore your codelist.
#' codelist <- getATCCodes(cdm = cdm,
#'                         level = "ATC 2nd",
#'                         doseForm = NULL,
#'                         doseUnit = NULL,
#'                         routeCategory = NULL,)
#' codelist
#'
#' # You can also create directly a codelist_with_details using the argument `type`
#' codelist <- getATCCodes(cdm = cdm,
#'                         level = "ATC 1st",
#'                         type = "codelist_with_details")
#' codelist
#' }
getATCCodes <- function(cdm,
                        level = c("ATC 1st"),
                        name = NULL,
                        nameStyle = "{concept_code}_{concept_name}",
                        doseForm = NULL,
                        doseUnit = NULL,
                        routeCategory = NULL,
                        type = "codelist") {

  # initial checks
  if(type == "concept_set_expression"){
    cli::cli_abort("concept_set_expression not yet supported")
  }
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  omopgenerics::assertChoice(level, c("ATC 1st", "ATC 2nd", "ATC 3rd", "ATC 4th", "ATC 5th"))
  omopgenerics::assertCharacter(name, null = TRUE)
  nameStyle <- checkNameStyle(nameStyle)
  omopgenerics::assertCharacter(doseForm, null = TRUE)
  omopgenerics::assertCharacter(doseUnit, null = TRUE)
  omopgenerics::assertCharacter(routeCategory, null = TRUE)
  type <- validateType(type = type)

  searchStrategy <- searchStrategyAttr(
    function_name = "getATCCodes",
    cdm = "cdm",
    level = cast(level),
    name = cast(name),
    doseForm = cast(doseForm)
  )

  atc_groups <- cdm$concept |>
    dplyr::filter(.data$vocabulary_id == "ATC") |>
    dplyr::filter(.data$concept_class_id %in% .env$level) |>
    dplyr::select("concept_id", "concept_name", "concept_code") |>
    dplyr::collect()

  if (!is.null(name)) {
    atc_groups <- atc_groups |>
      dplyr::filter(tidyWords(.data$concept_name) %in% tidyWords(.env$name))
  }

  if (nrow(atc_groups) > 0) {
    if (nrow(atc_groups) > 1 & type == "code_search") {
      cli::cli_abort(c(x = "Only one ATC type is supported when type = {.cls code_search}"))
    }
    # to avoid potential memory problems will batch
    atc_descendants <- fetchBatchedDescendants(cdm = cdm,
                                               codes = atc_groups$concept_id,
                                               batchSize = 500,
                                               doseForm = doseForm)
  } else {
    cli::cli_inform(c("!" = "- No matching ATC codes found"))
    return(emptyObject(type = type, searchStrategy = searchStrategy))
  }

  atc_descendants <- atc_descendants |>
    dplyr::select(!dplyr::any_of(c("min_levels_of_separation", "max_levels_of_separation"))) |>
    # split different ancestors into multiple cols
    tidyr::separate_wider_delim(
      cols = "ancestor_concept_id",
      delim = ";",
      names_sep = "",
      too_few = "align_start"
    )

  atc_descendants <- atc_descendants |>
    # one row per concept + ancestor
    tidyr::pivot_longer(cols = dplyr::starts_with("ancestor_concept_id"),
                        names_to = NULL,
                        values_to = "ancestor_concept_id",
                        values_drop_na = TRUE
    )

  atc_descendants <- atc_descendants |>
    dplyr::left_join(
      atc_groups |>
        dplyr::mutate("concept_name" = omopgenerics::toSnakeCase(.data$concept_name),
                      "concept_id" = as.character(.data$concept_id)) |>
        dplyr::mutate("name" = glue::glue(.env$nameStyle)) |>
        dplyr::select("concept_id", "name"),
      by = c("ancestor_concept_id" = "concept_id")
    )

  if(type == "codelist"){
    atc_descendants <- split(
      x = atc_descendants$concept_id,
      f = as.factor(atc_descendants$name),
      drop = TRUE
    ) |>
      omopgenerics::newCodelist()
  }else if(type == "codelist_with_details"){
    atc_descendants <- split(
      x = atc_descendants,
      f = as.factor(atc_descendants$name),
      drop = TRUE
    ) |>
      purrr::map(~dplyr::select(., -"name")) |>
      omopgenerics::newCodelistWithDetails()
  } else if (type == "code_search") {
    nm <- unique(atc_descendants$name)
    atc_descendants <- atc_descendants |>
      dplyr::select(!c("name", "ancestor_concept_id")) |>
      dplyr::mutate(
        found_from = "Descendants",
        vocabulary_version = vocabularyVersion(cdm = cdm),
      ) |>
      newCodeSearch(searchStrategy = searchStrategy)
  }


  if(!is.null(routeCategory)){
    atc_descendants <- subsetOnRouteCategory(atc_descendants,
                                             cdm = cdm,
                                             routeCategory = routeCategory)
  }

  if(!is.null(doseUnit)){
    atc_descendants <- subsetOnDoseUnit(atc_descendants,
                                        cdm = cdm,
                                        doseUnit = doseUnit)
  }

  return(atc_descendants)
}

test_that("getATCCodes working", {
  skip_on_cran()
  backends <- c("database", "data_frame")
  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backend = backends[i])

    atcCodes <- getATCCodes(cdm, level = "ATC 1st")
    expect_true("codelist" %in% class(atcCodes))
    expect_true(all(atcCodes[[1]] == c(12L, 13L)))
    expect_true(c("1234_alimentary_tract_and_metabolism") %in%
                  names(atcCodes))
    expect_true(inherits(atcCodes, "codelist"))

    atcCodes2 <- getATCCodes(cdm, level = "ATC 1st",
                             name = "ALIMENTARY TRACT AND METABOLISM")
    expect_true(all(atcCodes2[[1]] == c(12L, 13L)))

    atcCodes3 <- getATCCodes(cdm, level = "ATC 1st",
                             name = "ALIMENTARY TRACT AND METABOLISM",
                             type = "codelist_with_details")
    expect_true(!is.null(atcCodes3[[1]]$concept_name))
    expect_true(inherits(atcCodes3, "codelist_with_details"))
    expect_identical(names(atcCodes3), "1234_alimentary_tract_and_metabolism")

    atcCodes3 <- getATCCodes(cdm, level = "ATC 1st",
                             name = "ALIMENTARY TRACT AND METABOLISM",
                             nameStyle = "{concept_id}_{concept_name}")
    expect_identical(names(atcCodes3), "12_alimentary_tract_and_metabolism")
    expect_true(all(atcCodes3[[1]] == c(12L, 13L)))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

test_that("getATCCodes expected errors/warning", {
  skip_on_cran()
  backends <- c("database", "data_frame")
  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backend = backends[i])
    expect_error(getATCCodes(cdm, level = "Not an ATC level"))
    expect_message(x <- getATCCodes(cdm, level = "ATC 1st",
                             name = "Not an ATC name"))
    expect_true(length(x) == 0)
    expect_error(getATCCodes(cdm, nameStyle = "hello"))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }

})

test_that("getDrugIngredientCodes working", {
  skip_on_cran()
  backends <- c("database", "data_frame")
  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backend = backends[i])

    ing_codes0 <- getDrugIngredientCodes(cdm, nameStyle = "{concept_code}_{concept_name}")
    expect_true(all(ing_codes0[[1]] == c(10L, 13L)))
    expect_true(inherits(ing_codes0, "codelist"))
    expect_identical(names(ing_codes0),  c("1234_adalimumab", "1234_other_ingredient"))

    ing_codes1 <- getDrugIngredientCodes(cdm, nameStyle = "{concept_id}_{concept_name}")
    expect_true(all(ing_codes1[[1]] == c(10L, 13L)))
    expect_identical(names(ing_codes1),  c("10_adalimumab", "19_other_ingredient"))

    expect_warning(x <- getDrugIngredientCodes(cdm, name = c(1L,10L)))
    expect_true(names(x) == "1234_adalimumab")

    ing_codes2 <- getDrugIngredientCodes(cdm, name = "Adalimumab")
    expect_true(all(ing_codes2[[1]] == c(10L, 13L)))
    expect_true(names(ing_codes2) == "1234_adalimumab")

    ing_codes2 <- getDrugIngredientCodes(cdm, name = 10)
    expect_true(all(ing_codes2[[1]] == c(10L, 13L)))
    expect_true(names(ing_codes2) == "1234_adalimumab")

    ing_codes3 <- getDrugIngredientCodes(cdm,
                                         name = "Adalimumab",
                                         doseForm = "injectable foam")
    expect_true(all(ing_codes3[[1]] == c(13L)))

    ing_codes4 <- getDrugIngredientCodes(cdm,
                                         name = "Adalimumab",
                                         doseForm = "injection")
    expect_true(all(ing_codes4[[1]] == c(10L)))

    ing_codes5 <- getDrugIngredientCodes(cdm,
                                         name = "Adalimumab",
                                         doseForm = "injection",
                                         type = "codelist_with_details")
    expect_true(!is.null(ing_codes5[[1]]$concept_name))
    expect_true(inherits(ing_codes5, "codelist_with_details"))

    # limiting on ingredients
    ing_codes_all <- getDrugIngredientCodes(cdm,
                                            ingredientRange = c(1, Inf))
    ing_codes_mono <- getDrugIngredientCodes(cdm,
                                             ingredientRange = c(1, 1))
    ing_codes_comb <- getDrugIngredientCodes(cdm,
                                             ingredientRange = c(2, Inf))

    expect_equal(ing_codes_all, ing_codes0)
    expect_true(all(c(10L) %in% ing_codes_mono$`1234_adalimumab`))
    expect_null(ing_codes_mono$`1234_other_ingredient`)
    expect_true(all(c(13L) %in% ing_codes_comb$`1234_adalimumab`))
    expect_true(all(c(13L) %in% ing_codes_comb$`1234_other_ingredient`))

    expect_error(getDrugIngredientCodes(cdm,
                                        ingredientRange = c(3, 2)))
    expect_error(getDrugIngredientCodes(cdm,
                                        ingredientRange = c(3)))
    expect_error(getDrugIngredientCodes(cdm,
                                        ingredientRange = c("a", "b")))
    expect_error(getDrugIngredientCodes(cdm,
                                        ingredientRange = c(-1, 25)))


    expect_warning(empty_res_1 <- getDrugIngredientCodes(cdm,
                                                         name = "not_a_drug_name",
                                                         type = "codelist"))
    expect_no_error(omopgenerics::newCodelist(empty_res_1))
    expect_warning(empty_res_2 <- getDrugIngredientCodes(cdm,
                                                         name = "not_a_drug_name",
                                                         type = "codelist_with_details"))
    expect_no_error(omopgenerics::newCodelistWithDetails(empty_res_2))

    expect_warning(getDrugIngredientCodes(cdm,
                                          name = c("Adalimumab", "not_a_drug_name")))
    expect_warning(getDrugIngredientCodes(cdm,
                                          name = c("Adalimumab",
                                                   "xxxxx",
                                                   "zzzzz")))

    # simple concept expression
    expect_true(nrow(getDrugIngredientCodes(cdm,
                                            name = "adalimumab",
                                            type = "concept_set_expression")[[1]]) == 1)

    # by adding criteria, we will now have specified codes
    expect_true(nrow(getDrugIngredientCodes(cdm,
                                            name = "adalimumab",
                                            type = "concept_set_expression",
                                            ingredientRange = c(1,4))[[1]]) == 2)
    expect_true(getDrugIngredientCodes(cdm,
                                       name = "adalimumab",
                                       type = "concept_set_expression",
                                       ingredientRange = c(1, 1))[[1]] |>
                  dplyr::pull("concept_id") == 10L)
    expect_true(getDrugIngredientCodes(cdm,
                                       name = "adalimumab",
                                       type = "concept_set_expression",
                                       ingredientRange = c(2, 2))[[1]] |>
                  dplyr::pull("concept_id") == 13L)
    expect_warning(
      expect_warning(
      # no concepts with 3 ingredients
      getDrugIngredientCodes(cdm,
                             name = "adalimumab",
                             type = "concept_set_expression",
                             ingredientRange = c(3, 3))
      )
    )

    # options not yet supported for concept set expression
    expect_error(getDrugIngredientCodes(cdm,
                                        name = "adalimumab",
                                        type = "concept_set_expression",
                                        doseUnit = c("mm")))
    expect_error(getDrugIngredientCodes(cdm,
                                        name = "adalimumab",
                                        type = "concept_set_expression",
                                        doseForm = "zzz"))
    expect_error(getDrugIngredientCodes(cdm,
                                        name = "adalimumab",
                                        type = "concept_set_expression",
                                        routeCategory = "xxx"))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

test_that("no duplicate names example 1",{
  skip_on_cran()
  person <- dplyr::tibble(
    person_id = 1L,
    gender_concept_id = 0L,
    year_of_birth = 1990L,
    race_concept_id = 0L,
    ethnicity_concept_id = 0L
  )
  observationPeriod <- dplyr::tibble(
    observation_period_id = 1L,
    person_id = 1L,
    observation_period_start_date = as.Date("2000-01-01"),
    observation_period_end_date = as.Date("2024-12-31"),
    period_type_concept_id = 0L
  )
  concept <- data.frame(
    concept_id = 1:19L,
    concept_name = c(
      "Musculoskeletal disorder",
      "Osteoarthrosis",
      "Arthritis",
      "Osteoarthritis of knee",
      "Osteoarthritis of hip",
      "Osteonecrosis",
      "Degenerative arthropathy",
      "Knee osteoarthritis",
      "H/O osteoarthritis",
      "Adalimumab",
      "Injection",
      "ALIMENTARY TRACT AND METABOLISM",
      "Descendant drug",
      "Injectable",
      "Diseases of the musculoskeletal system and connective tissue",
      "Arthropathies",
      "Arthritis",
      "OA",
      "Adalimumab"
    ),
    domain_id = c(rep("Condition", 8), "Observation", rep("Drug", 5),
                  rep("Condition", 4), "Drug"),
    vocabulary_id = c(
      rep("SNOMED", 6),
      rep("Read", 2),
      "LOINC", "RxNorm", "OMOP",
      "ATC",
      "RxNorm", "OMOP",
      "ICD10", "ICD10", "ICD10", "ICD10", "RxNorm"
    ),
    standard_concept = c(
      rep("S", 6),
      rep(NA, 2),
      "S", "S", NA,
      NA, "S", NA, NA, NA, NA, NA, "S"
    ),
    concept_class_id = c(
      rep("Clinical Finding", 6),
      rep("Diagnosis", 2),
      "Observation", "Ingredient", "Dose Form",
      "ATC 1st", "Drug", "Dose Form",
      "ICD10 Chapter", "ICD10 SubChapter",
      "ICD Code","ICD Code", "Ingredient"
    ),
    concept_code = as.character(c(2:20)),
    valid_start_date = as.Date(NA),
    valid_end_date = as.Date(NA),
    invalid_reason = NA_character_
  )
  conceptAncestor <- dplyr::bind_rows(
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 1L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 3L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 2L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 3L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 4L,
      min_levels_of_separation = 2L,
      max_levels_of_separation = 2L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 5L,
      min_levels_of_separation = 2L,
      max_levels_of_separation = 2L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 4L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 5L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 10L,
      descendant_concept_id = 10L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 10L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 12L,
      descendant_concept_id = 12L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 12L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 19L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    )
  )
  conceptSynonym <- dplyr::bind_rows(
    data.frame(
      concept_id = 2L,
      concept_synonym_name = "Arthritis"
    ),
    data.frame(
      concept_id = 3L,
      concept_synonym_name = "Osteoarthrosis"
    )
  )|>
    dplyr::mutate(language_concept_id  = NA_integer_)

  vocabulary <- dplyr::bind_rows(
    data.frame(
      vocabulary_id = "SNOMED",
      vocabulary_name = "SNOMED",
      vocabulary_reference = "1",
      vocabulary_version = "1",
      vocabulary_concept_id = 1L
    ),
    data.frame(
      vocabulary_id = "None",
      vocabulary_name = "OMOP Standardized Vocabularies",
      vocabulary_reference = "Omop generated",
      vocabulary_version = "v5.0 22-JUN-22",
      vocabulary_concept_id = 44819096L
    )
  )

  drugStrength <- dplyr::bind_rows(
    data.frame(
      drug_concept_id = 10L,
      ingredient_concept_id = 10L,
      amount_value = NA_real_,
      amount_unit_concept_id = 8576L,
      numerator_value = 0.010,
      numerator_unit_concept_id = 8576L,
      denominator_value = 0.5,
      denominator_unit_concept_id = 8587L,
      box_size = NA_integer_,
      valid_start_date = as.Date(NA),
      valid_end_date = as.Date(NA)
    )
  )

  cdmSource <- dplyr::as_tibble(
    data.frame(
      cdm_source_name  = "mock",
      cdm_source_abbreviation = NA_character_,
      cdm_holder = NA_character_,
      source_description = NA_character_,
      source_documentation_reference = NA_character_,
      cdm_etl_reference = NA_character_,
      source_release_date = as.Date(NA),
      cdm_release_date = as.Date(NA),
      cdm_version = "5.3",
      vocabulary_version  = NA_character_
    )
  )

  conceptRelationship <- dplyr::bind_rows(
    data.frame(
      concept_id_1 = 2L,
      concept_id_2 = 7L,
      relationship_id = "Mapped from"
    ),
    data.frame(
      concept_id_1 = 4L,
      concept_id_2 = 8L,
      relationship_id = "Mapped from"
    ),
    data.frame(
      concept_id_1 = 10L,
      concept_id_2 = 11L,
      relationship_id = "RxNorm has dose form"
    ),
    data.frame(
      concept_id_1 = 3L,
      concept_id_2 = 6L,
      relationship_id = "Due to of"
    ),
    data.frame(
      concept_id_1 = 13L,
      concept_id_2 = 14L,
      relationship_id = "RxNorm has dose form"
    ),
    data.frame(
      concept_id_1 = 15L,
      concept_id_2 = 16L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 16L,
      concept_id_2 = 17L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 17L,
      concept_id_2 = 18L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 18L,
      concept_id_2 = 3L,
      relationship_id = "Maps to"
    )
  ) |>
    dplyr::mutate(valid_start_date = as.Date(NA),
                  valid_end_date = as.Date(NA),
                  invalid_reason = NA_character_)

  cdm_df <- omopgenerics::cdmFromTables(tables = list(person = person,
                                                      concept = concept,
                                                      concept_ancestor = conceptAncestor,
                                                      concept_synonym = conceptSynonym,
                                                      concept_relationship = conceptRelationship,
                                                      vocabulary = vocabulary,
                                                      drug_strength = drugStrength,
                                                      observation_period = observationPeriod,
                                                      cdm_source = cdmSource),
                                        cdmName = "mock")

  # into db
  db <- DBI::dbConnect(duckdb::duckdb(), ":memory:")

  cdm <- CDMConnector::copyCdmTo(con = db,
                                 cdm = cdm_df,
                                 schema = "main",
                                 overwrite = TRUE)
  attr(cdm, "write_schema") <- "main"

  expect_no_error(getDrugIngredientCodes(
    cdm = cdm, name = "Adalimumab", nameStyle = "{concept_name}"
  ))

  expect_no_error(getDrugIngredientCodes(
    cdm = cdm, name = "Adalimumab", nameStyle = "{concept_code}"
  ))

  ingredient_list <- getDrugIngredientCodes(
    cdm = cdm, name = "Adalimumab"
  )

  expect_equal(names(ingredient_list) |>
                 length() |>
                 as.numeric(),
               2
  )

  expect_equal(names(ingredient_list) |>
                 unique()|>
                 length() |>
                 as.numeric(),
               2
  )

  expect_false(all(startsWith(names(ingredient_list), "a")))

  expect_true(all(names(ingredient_list)
                  %in%
                    c("11_adalimumab", "20_adalimumab")))

  cdm <- mockVocabRef()

  ingredient_list <- getDrugIngredientCodes(
    cdm = cdm, name = "Adalimumab"
  )

  expect_false(all(startsWith(names(ingredient_list), "a")))

  expect_true(all(names(ingredient_list)
                  %in%
                    c("1234_adalimumab")))
})

test_that("no duplicate names example 2",{
  skip_on_cran()
  person <- dplyr::tibble(
    person_id = 1L, gender_concept_id = 0L, year_of_birth = 1990L,
    race_concept_id = 0L, ethnicity_concept_id = 0L
  )
  observationPeriod <- dplyr::tibble(
    observation_period_id = 1L, person_id = 1L,
    observation_period_start_date = as.Date("2000-01-01"),
    observation_period_end_date = as.Date("2024-12-31"),
    period_type_concept_id = 0L
  )
  concept <- data.frame(
    concept_id = 1:19L,
    concept_name = c(
      "Musculoskeletal disorder",
      "Osteoarthrosis",
      "Arthritis",
      "Osteoarthritis of knee",
      "Osteoarthritis of hip",
      "Osteonecrosis",
      "Degenerative arthropathy",
      "Knee osteoarthritis",
      "H/O osteoarthritis",
      "Adalimumab",
      "Injection",
      "ALIMENTARY TRACT AND METABOLISM",
      "Descendant drug",
      "Injectable",
      "Diseases of the musculoskeletal system and connective tissue",
      "Arthropathies",
      "Arthritis",
      "OA",
      "ALIMENTARY TRACT AND METABOLISM"
    ),
    domain_id = c(rep("Condition", 8), "Observation", rep("Drug", 5),
                  rep("Condition", 4), "Drug"),
    vocabulary_id = c(
      rep("SNOMED", 6),
      rep("Read", 2),
      "LOINC", "RxNorm", "OMOP",
      "ATC",
      "RxNorm", "OMOP",
      "ICD10", "ICD10", "ICD10", "ICD10", "ATC"
    ),
    standard_concept = c(
      rep("S", 6),
      rep(NA, 2),
      "S", "S", NA,
      NA, "S", NA, NA, NA, NA, NA, "S"
    ),
    concept_class_id = c(
      rep("Clinical Finding", 6),
      rep("Diagnosis", 2),
      "Observation", "Ingredient", "Dose Form",
      "ATC 1st", "Drug", "Dose Form",
      "ICD10 Chapter", "ICD10 SubChapter",
      "ICD Code","ICD Code", "ATC 1st"
    ),
    concept_code = as.character(c(2:20)),
    valid_start_date = as.Date(NA),
    valid_end_date = as.Date(NA),
    invalid_reason = NA_character_
  )
  conceptAncestor <- dplyr::bind_rows(
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 1L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 3L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 2L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 3L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 4L,
      min_levels_of_separation = 2L,
      max_levels_of_separation = 2L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 5L,
      min_levels_of_separation = 2L,
      max_levels_of_separation = 2L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 4L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 5L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 10L,
      descendant_concept_id = 10L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 10L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 12L,
      descendant_concept_id = 12L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 12L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 19L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    )
  )
  conceptSynonym <- dplyr::bind_rows(
    data.frame(
      concept_id = 2L,
      concept_synonym_name = "Arthritis"
    ),
    data.frame(
      concept_id = 3L,
      concept_synonym_name = "Osteoarthrosis"
    )
  )|>
    dplyr::mutate(language_concept_id  = NA_integer_)

  conceptRelationship <- dplyr::bind_rows(
    data.frame(
      concept_id_1 = 2L,
      concept_id_2 = 7L,
      relationship_id = "Mapped from"
    ),
    data.frame(
      concept_id_1 = 4L,
      concept_id_2 = 8L,
      relationship_id = "Mapped from"
    ),
    data.frame(
      concept_id_1 = 10L,
      concept_id_2 = 11L,
      relationship_id = "RxNorm has dose form"
    ),
    data.frame(
      concept_id_1 = 3L,
      concept_id_2 = 6L,
      relationship_id = "Due to of"
    ),
    data.frame(
      concept_id_1 = 13L,
      concept_id_2 = 14L,
      relationship_id = "RxNorm has dose form"
    ),
    data.frame(
      concept_id_1 = 15L,
      concept_id_2 = 16L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 16L,
      concept_id_2 = 17L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 17L,
      concept_id_2 = 18L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 18L,
      concept_id_2 = 3L,
      relationship_id = "Maps to"
    )
  ) |>
    dplyr::mutate(valid_start_date = as.Date(NA),
                  valid_end_date = as.Date(NA),
                  invalid_reason = NA_character_)

  vocabulary <- dplyr::bind_rows(
    data.frame(
      vocabulary_id = "SNOMED",
      vocabulary_name = "SNOMED",
      vocabulary_reference = "1",
      vocabulary_version = "1",
      vocabulary_concept_id = 1L
    ),
    data.frame(
      vocabulary_id = "None",
      vocabulary_name = "OMOP Standardized Vocabularies",
      vocabulary_reference = "Omop generated",
      vocabulary_version = "v5.0 22-JUN-22",
      vocabulary_concept_id = 44819096L
    )
  )

  drugStrength <- dplyr::bind_rows(
    data.frame(
      drug_concept_id = 10L,
      ingredient_concept_id = 10L,
      amount_value = NA_real_,
      amount_unit_concept_id = 8576L,
      numerator_value = 0.010,
      numerator_unit_concept_id = 8576L,
      denominator_value = 0.5,
      denominator_unit_concept_id = 8587L,
      box_size = NA_integer_,
      valid_start_date = as.Date(NA),
      valid_end_date = as.Date(NA)
    )
  )

  cdmSource <- dplyr::as_tibble(
    data.frame(
      cdm_source_name  = "mock",
      cdm_source_abbreviation = NA_character_,
      cdm_holder = NA_character_,
      source_description = NA_character_,
      source_documentation_reference = NA_character_,
      cdm_etl_reference = NA_character_,
      source_release_date = as.Date(NA),
      cdm_release_date = as.Date(NA),
      cdm_version = "5.3",
      vocabulary_version  = NA_character_
    )
  )

  cdm_df <- omopgenerics::cdmFromTables(tables = list(person = person,
                                                      concept = concept,
                                                      concept_ancestor = conceptAncestor,
                                                      concept_synonym = conceptSynonym,
                                                      concept_relationship = conceptRelationship,
                                                      vocabulary = vocabulary,
                                                      drug_strength = drugStrength,
                                                      observation_period = observationPeriod,
                                                      cdm_source = cdmSource),
                                        cdmName = "mock")

  # into db
  db <- DBI::dbConnect(duckdb::duckdb(), ":memory:")

  cdm <- CDMConnector::copyCdmTo(con = db,
                                 cdm = cdm_df,
                                 schema = "main",
                                 overwrite = TRUE)
  attr(cdm, "write_schema") <- "main"

  atc_list <- getATCCodes(
    cdm = cdm, name = "ALIMENTARY TRACT AND METABOLISM"
  )

  expect_equal(names(atc_list) |>
                 length() |>
                 as.numeric(),
               2
  )

  expect_equal(names(atc_list) |>
                 unique()|>
                 length() |>
                 as.numeric(),
               2
  )

  expect_false(all(startsWith(names(atc_list), "a")))
  expect_true(startsWith(names(atc_list)[1], "1"))
  expect_true(startsWith(names(atc_list)[2], "2"))

  expect_true(all(names(atc_list)
                  %in%
                    c("13_alimentary_tract_and_metabolism", "20_alimentary_tract_and_metabolism")))

  cdm <- mockVocabRef()

  atc_list <- getATCCodes(
    cdm = cdm, name = "ALIMENTARY TRACT AND METABOLISM"
  )

  expect_false(all(startsWith(names(atc_list), "a")))
  expect_true(all(startsWith(names(atc_list), "1")))
})

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Coerce to a codelist
#'
#' @param x A codelist, codelist_with_details, concept_set_expression, or a candidate_codes.
#' @inheritParams cdmDoc
#' @param ... For extensibility
#'
#' @returns codelist
#' @export
#'
#' @examples
#' \donttest{
#' library(omock)
#' library(CDMConnector)
#'
#' # Creating CDM object
#' cdm <- mockCdmFromDataset(datasetName = "GiBleed")
#'
#' # Create codelist from a codelist_with_details
#' codelist <- getDrugIngredientCodes(cdm,
#'                                    name = "acetaminophen",
#'                                    nameStyle = "{concept_name}",
#'                                    type = "codelist_with_details")
#'
#' asCodelist(codelist)
#'
#' # Create codelist from a candidate_codes
#' codelist <- getCandidateCodes(cdm,
#'                               keywords = "arthritis")
#'
#' asCodelist(codelist)
#'
#' }
asCodelist <- function(x, ...){
  UseMethod("asCodelist")
}

#' @export
#' @rdname asCodelist
asCodelist.codelist <- function(x, ...){
  x |>
    newCodelist()
}

#' @export
#' @rdname asCodelist
asCodelist.codelist_with_details  <- function(x, ...){
  x |>
    purrr::map(~dplyr::pull(., "concept_id")) |>
    newCodelist()
}

#' @export
#' @rdname asCodelist
asCodelist.concept_set_expression  <- function(x, cdm, ...){

 # don't support mapped
 hasMapped <- x |>
    purrr::keep(~ any(.x$mapped, na.rm = TRUE)) |>
    names()
 if (length(hasMapped) > 0) {
  cli::cli_abort(
    glue::glue("Mapped as TRUE not supported (found in {hasMapped})"))
 }

 omopgenerics::validateConceptSetArgument(x, cdm = cdm)
}

#' @export
#' @rdname asCodelist
asCodelist.candidate_codes  <- function(x, ...){
  list("candidate_codes" = x |>
         dplyr::pull("concept_id")) |>
    newCodelist()
}

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Subset a codelist to only those codes from a particular domain.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams domainDoc
#' @param negate If FALSE, only concepts with the domain specified will be
#' returned. If TRUE, concepts with the domain specified will be excluded.
#'
#' @return The codelist with only those concepts associated with the domain
#' (if negate = FALSE) or the codelist without those concepts associated with
#' the domain (if negate = TRUE).
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef()
#' codes <- subsetOnDomain(
#'               x = newCodelist(list("codes" = c(10,13,15))),
#'               cdm = cdm,
#'               domain = "Drug")
#' codes
#' }
subsetOnDomain <- function(x,
                           cdm,
                           domain,
                           negate = FALSE){

  omopgenerics::assertCharacter(domain, null = FALSE, na = FALSE)
  st <- searchStrategyAttr(
    function_name = "subsetOnDomain",
    cdm = "cdm",
    domain = cast(domain)
  )
  x <- subsetCodelistBy(x,
                        cdm,
                        by = "domain",
                        group = domain,
                        keepOriginal = FALSE,
                        st = st,
                        negate = negate)
  return(x)
}

test_that("subset on domain", {

  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    cdm <- mockVocabRef(backends[[i]])
    ing_codes <- getDrugIngredientCodes(cdm)

    # all will have been empty and dropped
    expect_warning(ing_codes_sub <- subsetOnDomain(ing_codes, cdm, "condition"))
    expect_true(length(ing_codes_sub) == 0)
    expect_true(inherits(ing_codes_sub, "codelist"))

    ing_codes_sub2 <- subsetOnDomain(ing_codes,  cdm, "drug")
    expect_identical(ing_codes, ing_codes_sub2)
    expect_true(inherits(ing_codes_sub2, "codelist"))

    mixed_codelist <- omopgenerics::newCodelist(list("mixed_codes" = c(1L, # condition
                           9L,  # observation
                           10L))) # drug
    mixed_codelist1 <- subsetOnDomain(mixed_codelist,  cdm, "condition")
    expect_true(mixed_codelist1[[1]] == 1L)
    expect_true(length(mixed_codelist1[[1]]) == 1)
    expect_true(inherits(mixed_codelist1, "codelist"))

    mixed_codelist2 <- subsetOnDomain(mixed_codelist,  cdm,
                                      c("condition", "drug"))
    expect_true(all(sort(mixed_codelist2[[1]]) == c(1L, 10L)))
    expect_true(length(mixed_codelist2[[1]]) == 2)

    mixed_codelist3 <- subsetOnDomain(mixed_codelist,  cdm,
                                      c("condition", "drug", "observation"))
    expect_true(all(sort(mixed_codelist3[[1]]) == c(1L, 9L, 10L)))
    expect_true(length(mixed_codelist3[[1]]) == 3)

    mixed_codelist4 <- subsetOnDomain(omopgenerics::newCodelist(list("codes" = c(3L,4L))),  cdm,
                                      c("condition", "drug", "observation"))
    expect_true(inherits(mixed_codelist4, "codelist"))
    mixed_codelist5 <- subsetOnDomain(getDrugIngredientCodes(cdm, type = "codelist_with_details"),
                                      cdm, "drug")
    expect_true(inherits(mixed_codelist5,"codelist_with_details"))
    expect_true(!"codelist" %in% class(mixed_codelist5))

    # Check negate argument
    all_concepts <- omopgenerics::newCodelist(list("a" = cdm$concept |> dplyr::pull("concept_id")))
    codes1 <- subsetOnDomain(all_concepts, cdm, domain = c("Drug","Observation", "Unit"))
    codes2 <- subsetOnDomain(all_concepts, cdm, domain = "Condition", negate = TRUE)

    expect_identical(codes1, codes2)
    expect_identical(cdm$concept |>
      dplyr::filter(concept_id %in% codes2$a) |>
      dplyr::pull("domain_id") |>
      unique() |>
      sort(na.last = TRUE), c("Drug", "Observation", "Unit"))

    # Check if domain = NA
    cdm$concept <- cdm$concept |>
      dplyr::mutate(domain_id = dplyr::if_else(concept_id == 1, NA, domain_id)) |>
      dplyr::compute(name = "concept")
    expect_warning(
    codes1 <- subsetOnDomain(omopgenerics::newCodelist(list("a" = 1L)), cdm,
                             domain = c("Condition"))
    )
    expect_true(length(codes1) == 0)

    # check empty codelists
    expect_warning(
    x <- subsetOnDomain(omopgenerics::emptyCodelist(), cdm, domain = "Condition")
    )
    expect_true(inherits(x, "codelist"))

    expect_warning(
      x <- subsetOnDomain(omopgenerics::emptyCodelistWithDetails(), cdm,
                          domain = "Condition"))
    expect_true(inherits(x, "codelist_with_details"))
    expect_true(!"codelist" %in% class(x))

    # expected errors
    expect_error(subsetOnDomain(list("a" = 1L),  cdm, "condition"))
    expect_error(subsetOnDomain(ing_codes,  "a", "condition"))
    expect_error(subsetOnDomain(ing_codes,  cdm, 1234))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }

})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' Generate example vocabulary database
#'
#' @param backend 'database' (duckdb) or 'data_frame'.
#' @return cdm reference with mock vocabulary.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' cdm <- mockVocabRef()
#' cdm
#' }
mockVocabRef <- function(backend = "data_frame") {

  # initial checks
  omopgenerics::assertChoice(backend,
                             choices = c("database", "data_frame"),
                             length = 1)

  if(backend == "database"){
    rlang::check_installed("duckdb")
  }

  # compulsory tables
  person <- dplyr::tibble(
    person_id = c(1L:400L), gender_concept_id = 0L, year_of_birth = 1990L,
    race_concept_id = 0L, ethnicity_concept_id = 0L
  )
  observationPeriod <- dplyr::tibble(
    observation_period_id = 1L, person_id = 1L,
    observation_period_start_date = as.Date("2000-01-01"),
    observation_period_end_date = as.Date("2024-12-31"),
    period_type_concept_id = 0L
  )

  # condition accurrence table
  conditionOccurrence <- rbind(
    dplyr::tibble(
      condition_occurrence_id = c(1L:400L),
      condition_concept_id = 4L,
      person_id = c(1L:400L),
      condition_start_date = as.Date("2010-01-01"),
      condition_end_date = as.Date("2010-01-01"),
      condition_type_concept_id = NA_integer_,
      condition_source_concept_id = NA_integer_,
      condition_source_value = NA_character_
    ),
    dplyr::tibble(
      condition_occurrence_id = c(201L:400L),
      condition_concept_id = 5L,
      person_id = c(1L:200L),
      condition_start_date = as.Date("2010-01-01"),
      condition_end_date = as.Date("2010-01-01"),
      condition_type_concept_id = NA_integer_,
      condition_source_concept_id = NA_integer_,
      condition_source_value = NA_character_
    ),
    dplyr::tibble(
      condition_occurrence_id = c(401L:500L),
      condition_concept_id = 9L,
      person_id = c(1L:100L),
      condition_start_date = as.Date("2010-01-01"),
      condition_end_date = as.Date("2010-01-01"),
      condition_type_concept_id = NA_integer_,
      condition_source_concept_id = NA_integer_,
      condition_source_value = NA_character_
    )
  )

  # vocab tables
  concept <- data.frame(
    concept_id = c(1:10, 46234469, 12:13, 44784844, 15:21 , 35604877L, 35604394L, 22:24, 25:26) |>
      as.integer(),
    concept_name = c(
      "Musculoskeletal disorder",
      "Osteoarthrosis",
      "Arthritis",
      "Osteoarthritis of knee",
      "Osteoarthritis of hip",
      "Osteonecrosis",
      "Degenerative arthropathy",
      "Knee osteoarthritis",
      "H/O osteoarthritis",
      "Adalimumab",
      "Injection",
      "ALIMENTARY TRACT AND METABOLISM",
      "Descendant drug",
      "Injectable Foam",
      "Diseases of the musculoskeletal system and connective tissue",
      "Arthropathies",
      "Arthritis",
      "OA",
      "Other ingredient",
      "glucagon Nasal Powder",
      "nitrogen Topical Liquefied Gas",
      "Nasal Powder",
      "Topical Liquefied Gas",
      "percent",
      "milligram",
      "Other musculoskeletal disorder",
      "Brand 1",
      "Brand 2"
    ),
    domain_id = c(rep("Condition", 8), "Observation", rep("Drug", 5),
                  rep("Condition", 4), rep("Drug",5), rep("Unit",2),
                  "Condition",
                  rep("Drug", 2)),
    vocabulary_id = c(
      rep("SNOMED", 6),
      rep("Read", 2),
      "LOINC", "RxNorm", "OMOP",
      "ATC",
      "RxNorm", "OMOP",
      rep("ICD10",4),
      rep("RxNorm",5),
      rep("UCUM",2),
      "SNOMED",
      rep("RxNorm Extension",2)
    ),
    standard_concept = c(
      rep("S", 6),
      rep(NA, 2),
      "S", "S", NA,
      NA, "S",
      rep(NA,5),
      rep("S",3),
      rep(NA,2),
      rep("S",2),
      rep(NA,3)
    ),
    concept_class_id = c(
      rep("Clinical Finding", 6),
      rep("Diagnosis", 2),
      "Observation", "Ingredient", "Dose Form",
      "ATC 1st", "Drug", "Dose Form",
      "ICD10 Chapter", "ICD10 SubChapter",
      "ICD Code","ICD Code", "Ingredient",
      rep("Clinical Drug Form",2),
      rep("Dose Form",2),
      rep("Unit",2),
      "Clinical Finding",
      rep("Brand Name",2)
    ),
    concept_code = "1234",
    valid_start_date = as.Date(NA),
    valid_end_date = as.Date(NA),
    invalid_reason = NA_character_
  )

  conceptAncestor <- dplyr::bind_rows(
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 1L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 3L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 2L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 3L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 4L,
      min_levels_of_separation = 2L,
      max_levels_of_separation = 2L
    ),
    data.frame(
      ancestor_concept_id = 1L,
      descendant_concept_id = 5L,
      min_levels_of_separation = 2L,
      max_levels_of_separation = 2L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 4L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 3L,
      descendant_concept_id = 5L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 10L,
      descendant_concept_id = 10L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 10L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 12L,
      descendant_concept_id = 12L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 12L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    ),
    data.frame(
      ancestor_concept_id = 19L,
      descendant_concept_id = 13L,
      min_levels_of_separation = 1L,
      max_levels_of_separation = 1L
    )
  )

  conceptSynonym <- dplyr::bind_rows(
    data.frame(
      concept_id = 2L,
      concept_synonym_name = "Arthritis"
    ),
    data.frame(
      concept_id = 3L,
      concept_synonym_name = "Osteoarthrosis"
    )
  )|>
    dplyr::mutate(language_concept_id  = NA_integer_)

  conceptRelationship <- dplyr::bind_rows(
    data.frame(
      concept_id_1 = 2L,
      concept_id_2 = 7L,
      relationship_id = "Mapped from"
    ),
    data.frame(
      concept_id_1 = 4L,
      concept_id_2 = 8L,
      relationship_id = "Mapped from"
    ),
    data.frame(
      concept_id_1 = 10L,
      concept_id_2 = 46234469L,
      relationship_id = "RxNorm has dose form"
    ),
    data.frame(
      concept_id_1 = 3L,
      concept_id_2 = 6L,
      relationship_id = "Due to of"
    ),
    data.frame(
      concept_id_1 = 13L,
      concept_id_2 = 44784844L,
      relationship_id = "RxNorm has dose form"
    ),
    data.frame(
      concept_id_1 = 15L,
      concept_id_2 = 16L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 16L,
      concept_id_2 = 17L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 17L,
      concept_id_2 = 18L,
      relationship_id = "Subsumes"
    ),
    data.frame(
      concept_id_1 = 18L,
      concept_id_2 = 3L,
      relationship_id = "Maps to"
    ),
    data.frame(
      concept_id_1 = 20L,
      concept_id_2 = 35604877L,
      relationship_id = "RxNorm has dose form"
    ),
    data.frame(
      concept_id_1 = 21L,
      concept_id_2 = 35604394L,
      relationship_id = "RxNorm has dose form"
    ),
    data.frame(
      concept_id_1 = 1L,
      concept_id_2 = 24L,
      relationship_id = "Mapped from"
    ),
    data.frame(
      concept_id_1 = 10L,
      concept_id_2 = 25L,
      relationship_id = "Has brand name"
    ),
    data.frame(
      concept_id_1 = 20L,
      concept_id_2 = 25L,
      relationship_id = "Has brand name"
    ),
    data.frame(
      concept_id_1 = 20L,
      concept_id_2 = 26L,
      relationship_id = "Has brand name"
    ),
    data.frame(
      concept_id_1 = 13L,
      concept_id_2 = 26L,
      relationship_id = "Has brand name"
    ),
    data.frame(
      concept_id_1 = 21L,
      concept_id_2 = 26L,
      relationship_id = "Has brand name"
    )
  ) |>
    dplyr::mutate(valid_start_date = as.Date(NA),
                  valid_end_date = as.Date(NA),
                  invalid_reason = NA_character_)

  vocabulary <- dplyr::bind_rows(
    data.frame(
      vocabulary_id = "SNOMED",
      vocabulary_name = "SNOMED",
      vocabulary_reference = "1",
      vocabulary_version = "1",
      vocabulary_concept_id = 1L
    ),
    data.frame(
      vocabulary_id = "None",
      vocabulary_name = "OMOP Standardized Vocabularies",
      vocabulary_reference = "Omop generated",
      vocabulary_version = "v5.0 22-JUN-22",
      vocabulary_concept_id = 44819096L
    )
  )

  drugStrength <- dplyr::bind_rows(
    data.frame(
      drug_concept_id = 10L,
      ingredient_concept_id = 10L,
      amount_value = NA_real_,
      amount_unit_concept_id = 8576L,
      numerator_value = 0.010,
      numerator_unit_concept_id = 8576L,
      denominator_value = 0.5,
      denominator_unit_concept_id = 8587L,
      box_size = NA_integer_,
      valid_start_date = as.Date(NA),
      valid_end_date = as.Date(NA)
    ),
    data.frame(
      drug_concept_id = 20L,
      ingredient_concept_id = NA_integer_,
      amount_value = NA_real_,
      amount_unit_concept_id = 22L,
      numerator_value = NA_real_,
      numerator_unit_concept_id = NA_integer_,
      denominator_value = 0.5,
      denominator_unit_concept_id = NA_integer_,
      box_size = NA_integer_,
      valid_start_date = as.Date(NA),
      valid_end_date = as.Date(NA)
    ),
    data.frame(
      drug_concept_id = 21L,
      ingredient_concept_id = NA_integer_,
      amount_value = NA_real_,
      amount_unit_concept_id = 23L,
      numerator_value = NA_real_,
      numerator_unit_concept_id = NA_integer_,
      denominator_value = 0.5,
      denominator_unit_concept_id = NA_integer_,
      box_size = NA_integer_,
      valid_start_date = as.Date(NA),
      valid_end_date = as.Date(NA)
    )
  )

  cdmSource <- dplyr::as_tibble(
    data.frame(
      cdm_source_name  = "mock",
      cdm_source_abbreviation = NA_character_,
      cdm_holder = NA_character_,
      source_description = NA_character_,
      source_documentation_reference = NA_character_,
      cdm_etl_reference = NA_character_,
      source_release_date = as.Date(NA),
      cdm_release_date = as.Date(NA),
      cdm_version = "5.3",
      vocabulary_version  = NA_character_
    )
  )

  cdm_df <- omopgenerics::cdmFromTables(tables = list(person = person,
                                                      concept = concept,
                                                      condition_occurrence = conditionOccurrence,
                                                      concept_ancestor = conceptAncestor,
                                                      concept_synonym = conceptSynonym,
                                                      concept_relationship = conceptRelationship,
                                                      vocabulary = vocabulary,
                                                      drug_strength = drugStrength,
                                                      observation_period = observationPeriod,
                                                      cdm_source = cdmSource),
                                        cdmName = "mock")

  cdm_df <- buildAchillesTables(cdm_df) |>
    suppressMessages()

  if (backend == "data_frame") {
    return(cdm_df)
  }

  # into db
  db <- DBI::dbConnect(duckdb::duckdb(), ":memory:")

  cdm <- CDMConnector::copyCdmTo(con = db,
                                 cdm = cdm_df,
                                 schema = "main",
                                 overwrite = TRUE)
  attr(cdm, "write_schema") <- "main"
  cdm
}

test_that("mock vocab db", {
  cdmDb <- mockVocabRef("database")
  cdmDF <- mockVocabRef("data_frame")

  conceptFromDb <- cdmDb$concept |> dplyr::collect()
  conceptFromDf <- cdmDF$concept |> dplyr::collect()

  expect_equal(conceptFromDb,
                    conceptFromDf,
                    ignore_attr = TRUE)

  CDMConnector::cdmDisconnect(cdm = cdmDb)
  })

test_that("check intersectCodelists works", {
  skip_on_cran()
  cdm <- mockVocabRef()

  # Codelist
  x <- getDrugIngredientCodes(cdm,
                              nameStyle = "{concept_name}")
  intersection <- intersectCodelists(x, keepOriginal = FALSE)
  expect_true(inherits(intersection, "codelist"))
  expect_true(intersection[[1]] == 13)
  expect_true(names(intersection) == "intersection_adalimumab_other_ingredient")

  intersection <- intersectCodelists(x, keepOriginal = TRUE)
  expect_true(all(intersection[[1]] == c(10,13)))
  expect_true(intersection[[2]] == c(13))
  expect_true(intersection[[3]] == c(13))
  expect_true(all(names(intersection) == c("adalimumab",
                                           "intersection_adalimumab_other_ingredient",
                                           "other_ingredient")))


  # Codelist with details
  x <- asCodelistWithDetails(x, cdm)
  expect_no_error(inter <- intersectCodelists(x, keepOriginal = TRUE))
  expect_identical(names(inter), c("adalimumab", "intersection_adalimumab_other_ingredient","other_ingredient"))
  expect_no_error(inter <- intersectCodelists(x, keepOriginal = FALSE))
  expect_identical(names(inter), "intersection_adalimumab_other_ingredient")
  expect_identical(inter[[1]], x[[2]])

  # no intersection codelist with details
  x[[1]] <- x[[1]] |> dplyr::mutate("ancestor" = c(1))
  x[[2]] <- x[[2]] |> dplyr::mutate("ancestor" = c(2))
  expect_no_error(inter <- intersectCodelists(x, keepOriginal = FALSE))
  expect_true(names(inter) == "intersection_adalimumab_other_ingredient")
  expect_identical(nrow(inter$intersection_adalimumab_other_ingredient), 0L)

  # No intersection
  x <- newCodelist(list("a" = c(1L,2L), "b" = c(3L,4L)))
  expect_no_error(intersectCodelists(x))

  # Expected errors
  expect_error(intersectCodelists(x, keepOriginal = "a"))
  expect_error(intersectCodelists(x, keepOriginal = c(TRUE, FALSE)))

})

test_that("intersect codelists - eunomia", {
  skip_on_cran()
  CDMConnector::requireEunomia("synpuf-1k", "5.3")
  con <- DBI::dbConnect(duckdb::duckdb(),
                        CDMConnector::eunomiaDir("synpuf-1k", "5.3"))
  cdm <- CDMConnector::cdmFromCon(con = con,
                                  cdmName = "Eunomia Synpuf",
                                  cdmSchema = "main",
                                  writeSchema = "main")

  # three codelists, one for each
  meds <- getDrugIngredientCodes(cdm,
                                 c("acetaminophen",
                                   "aspirin",
                                   "codeine"),
                                 nameStyle = "{concept_name}")
  meds_intersect <- intersectCodelists(meds)
  expect_true(length(meds_intersect) == 1)
  expect_true(names(meds_intersect) == "intersection_acetaminophen_aspirin_codeine")

  meds_2_intersect_keep <- intersectCodelists(meds, keepOriginal = TRUE)
  expect_true(length(meds_2_intersect_keep) == 4)
  expect_true(all(c("acetaminophen",
                    "aspirin",
                    "codeine",
                    "intersection_acetaminophen_aspirin_codeine") %in%
                    names(meds_2_intersect_keep)))

  omopgenerics::cdmDisconnect(cdm)
})

test_that("stratify by vocabulary works", {
  cdm <- mockVocabRef()

  # codelist
  codelist <- newCodelist(list("codes1" = c(1L, 5L, 10L),
                               "codes2" = c(2L, 15L, 22L)))
  expect_no_error(x <- stratifyByVocabulary(codelist, cdm, keepOriginal = TRUE))
  expect_true(inherits(x, "codelist"))
  expect_identical(names(x), c("codes1", "codes1_rx_norm", "codes1_snomed",
                               "codes2", "codes2_icd10", "codes2_snomed",
                               "codes2_ucum"))
  expect_identical(x$codes1, c(1L, 5L, 10L))
  expect_identical(x$codes1_rx_norm, c(10L))
  expect_identical(x$codes1_snomed, c(1L, 5L))
  expect_identical(x$codes2, c(2L, 15L, 22L))
  expect_identical(x$codes2_icd10, 15L)
  expect_identical(x$codes2_snomed, 2L)
  expect_identical(x$codes2_ucum, 22L)

  # codelist + without keep original
  expect_no_error(x <- stratifyByVocabulary(codelist, cdm))
  expect_true(inherits(x, "codelist"))
  expect_identical(sort(names(x)), sort(c("codes1_rx_norm", "codes1_snomed",
                               "codes2_icd10", "codes2_snomed",
                               "codes2_ucum")))

  # codelist with details
  codelist_with_details <- asCodelistWithDetails(codelist, cdm)
  expect_no_error(x <- stratifyByVocabulary(codelist_with_details, cdm, keepOriginal = TRUE))
  expect_true(inherits(x, "codelist_with_details"))
  expect_identical(sort(names(x)), sort(c("codes1_rx_norm", "codes1_snomed",
                               "codes2_icd10", "codes2_snomed",
                               "codes2_ucum", "codes1", "codes2")))
  expect_identical(x$codes2, codelist_with_details$codes2)
  expect_identical(x$codes1_rx_norm, codelist_with_details$codes1 |> dplyr::filter(.data$concept_id == 10L))

  # codelist with details without vocabulary_id column and another extra column and negate
  codelist_with_details <- purrr::map(codelist_with_details,
                                      ~dplyr::select(.x, -"vocabulary_id") |>
                                        dplyr::mutate("extra" = "hello")) |>
    newCodelistWithDetails()
  expect_no_error(x <- stratifyByVocabulary(codelist_with_details, cdm))
  expect_true(inherits(x, "codelist_with_details"))
  expect_identical(names(x), c("codes1_rx_norm", "codes1_snomed", "codes2_icd10", "codes2_snomed", "codes2_ucum"))
  expect_identical(x$codes2_icd10, codelist_with_details$codes2 |> dplyr::filter(.data$concept_id %in% c(15L)))

  # unclassified vocabulary
  cdm[["concept"]] <- cdm[["concept"]] |>
    dplyr::mutate("vocabulary_id" = dplyr::if_else(.data$concept_id == 1, NA, .data$vocabulary_id))
  expect_no_error(x <- stratifyByVocabulary(codelist_with_details, cdm))
  expect_true(inherits(x, "codelist_with_details"))
  expect_identical(sort(names(x)), sort(c("codes1_rx_norm", "codes1_snomed", "codes1_unclassified_vocabulary",
                               "codes2_icd10", "codes2_snomed","codes2_ucum")))
  expect_identical(x$codes1_unclassified_vocabulary$concept_id, 1L)

  # empty codelist
  expect_warning(x <- stratifyByVocabulary(omopgenerics::emptyCodelist(), cdm))
  expect_identical(omopgenerics::emptyCodelist(), x)
  expect_warning(x <- stratifyByVocabulary(omopgenerics::emptyCodelistWithDetails(), cdm))
  expect_identical(omopgenerics::emptyCodelistWithDetails(), x)

  # expected errors
  expect_error(stratifyByVocabulary(list("a" = 1L),  cdm))
  expect_error(stratifyByVocabulary(codelist,  "a"))
  expect_error(stratifyByVocabulary(codelist,  cdm, keepOriginal = "maybe"))
})

test_that("subsetOnDoseUnit in mock", {
  skip_on_cran()
  cdm <- mockVocabRef()
  ing <- getDrugIngredientCodes(cdm = cdm)

  # no dose units - empty codelist
  expect_warning(x <- subsetOnDoseUnit(x = ing, cdm = cdm,
                                        doseUnit = "hello"))
  expect_true(inherits(x, "codelist"))
  expect_true(length(x) == 0)

  # argument NEGATE
  codes <- omopgenerics::newCodelist(list("codes" = c(20L,21L)))
  codes1 <- subsetOnDoseUnit(x = codes, cdm = cdm, doseUnit = c("milligram"))
  codes2 <- subsetOnDoseUnit(x = codes, cdm = cdm, doseUnit = c("percent"), negate = TRUE)
  expect_identical(codes1, codes2)

  # lower case
  codes3 <- subsetOnDoseUnit(x = codes, cdm = cdm, doseUnit = c("Milligram"))
  expect_identical(codes1, codes3)

  # Check if dose = NA are included
  codes1 <- subsetOnDoseUnit(x = omopgenerics::newCodelist(list("codes" = c(10L,21L))), cdm = cdm, doseUnit = c("milligram"))
  expect_identical(codes1$codes, 21L)

  # Check codelist with details
  codes_wd <- codes |> asCodelistWithDetails(cdm)
  codes1 <- subsetOnDoseUnit(x = codes_wd, cdm, doseUnit = "milligram")
  expect_true(inherits(codes1, "codelist_with_details"))
  expect_true(!"codelist" %in% class(codes1))
  expect_identical(codes1$codes,
                   dplyr::tibble(
                     "concept_id" = 21L,
                     "concept_name" = "nitrogen Topical Liquefied Gas",
                     "domain_id" = "Drug",
                     "vocabulary_id" = "RxNorm",
                     "standard_concept" = "standard"
                   ))


  # check empty codelists
  expect_warning(x <- subsetOnDoseUnit(omopgenerics::emptyCodelist(), cdm,
                                       doseUnit = "milligram"))
  expect_true(inherits(x, "codelist"))
  expect_warning(x <- subsetOnDoseUnit(omopgenerics::emptyCodelistWithDetails(),
                                       cdm, doseUnit = "milligram"))
  expect_true(inherits(x, "codelist_with_details"))

  # expected errors
  expect_error(subsetOnDoseUnit(x = ing, cdm = cdm,
                                doseUnit = 1))
  expect_error(subsetOnDoseUnit(x = ing, cdm = "a",
                                doseUnit = "milligram"))
  expect_error(subsetOnDoseUnit(x = list("a" = 1L), cdm = cdm,
                                doseUnit = "milligram"))

})

test_that("stratify by brand works", {
  skip_on_cran()
  cdm <- mockVocabRef()

  codes <- omopgenerics::newCodelist(list("concepts_1" = c(20L,21L), "concepts_2" = c(13L,21L,22L)))
  expect_warning(new_codes <- stratifyByBrand(x = codes,cdm = cdm, keepOriginal = FALSE))
  expect_true(inherits(new_codes,"codelist"))
  expect_true(all(names(new_codes) == c("concepts_1_brand_1", "concepts_1_brand_2", "concepts_2_brand_2", "concepts_2_unclassified_brand")))
  expect_true(new_codes$concepts_1_brand_1 == c(20L))
  expect_equal(new_codes$concepts_1_brand_2, c(20L, 21L))
  expect_true(all(new_codes$concepts_2_brand_2 == c(13L, 21L)))
  expect_true(new_codes$concepts_2_unclassified_brand == c(22L))

  # Only unclassified
  expect_no_error(x <- stratifyByBrand(x = omopgenerics::newCodelist(list("concepts_1" = c(1L,2L,3L))), cdm))
  expect_true(inherits(x, "codelist"))
  expect_true(names(x) == "concepts_1_unclassified_brand")

  # Mixed and empty result
  c <- list("concepts_1" = c(1L,2L,3L), "concepts_2" = c(10L, 20L)) |> omopgenerics::newCodelist()
  expect_warning(x <- stratifyByBrand(x = c, cdm))
  expect_true(all(names(x) == c("concepts_1_unclassified_brand", "concepts_2_brand_1",  "concepts_2_brand_2", "concepts_2_unclassified_brand")))
  expect_true(inherits(x, "codelist"))
  expect_equal(x$concepts_2_unclassified_brand, 10L)

  newCodelist <- list("concepts" = cdm[["concept"]] |>
                        dplyr::filter(.data$concept_id %in% c(13L, 20L, 21L, 24L)) |>
                        dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id", "standard_concept") |>
                        dplyr::mutate("ancestor_concept_id" = 999L) |>
                        dplyr::collect()) |>
    omopgenerics::newCodelistWithDetails()

  expect_warning(new_codes <- stratifyByBrand(x = newCodelist,
                                              cdm = cdm,
                                              keepOriginal = TRUE))

  expect_true(inherits(new_codes,"codelist_with_details"))
  expect_true(all(names(new_codes) == c("concepts", "concepts_brand_1", "concepts_brand_2", "concepts_unclassified_brand")))
  expect_true(new_codes$concepts_brand_1$concept_id == c(20L))
  expect_equal(new_codes$concepts_brand_2$concept_id,  c(13L, 20L, 21L))
  expect_equal(colnames(new_codes$concepts_brand_1), c("concept_id","concept_name", "domain_id", "vocabulary_id", "standard_concept", "ancestor_concept_id"))
  expect_equal(colnames(new_codes$concepts_brand_2), c("concept_id","concept_name", "domain_id", "vocabulary_id", "standard_concept", "ancestor_concept_id"))
  expect_equal(colnames(new_codes$concepts), c("concept_id","concept_name", "domain_id", "vocabulary_id", "standard_concept", "ancestor_concept_id"))

  # Expected errors
  expect_error(stratifyByBrand(list("a" = c(1L)), cdm))
  expect_error(stratifyByBrand(newCodelist, "a"))
  expect_error(stratifyByBrand(newCodelist, cdm, keepOriginal = "a"))
})

test_that("availableRouteCategories works with categories", {
  skip_on_cran()
  # none in our mock cdm
  cdm <- mockVocabRef()

  codelist_with_details <- list("codes" = cdm[["concept"]] |>
                                  dplyr::filter(concept_id %in% c(10L, 15L, 24L)) |>
                                  dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
                                  dplyr::mutate("ancestor_concept_id" = 999L) |>
                                  dplyr::collect()) |>
    omopgenerics::newCodelistWithDetails()

  codelist <- list("first_codelist" = c(1L, 20L), "second_codelist" = c(7L, 9L)) |>
    omopgenerics::newCodelist()

  # Whole db
  expect_no_error(x <- availableRouteCategories(cdm))
  expect_true(inherits(x, "character"))
  expect_true(all(c("injectable", "topical", "transmucosal_nasal") == x))

  # Codelist with details
  expect_no_error(x <- associatedRouteCategories(cdm, x = codelist_with_details))
  expect_true(names(x) == "codes")
  expect_identical(x$codes,c("injectable", "unclassified_route_category"))

  # Codelist
  expect_no_error(x <- associatedRouteCategories(cdm, x = codelist))
  expect_identical(x$first_codelist, c("transmucosal_nasal", "unclassified_route_category"))
  expect_identical(x$second_codelist, c("unclassified_route_category"))

  # Expected errors
  expect_error(availableRouteCategories("a"))
  expect_error(availableRouteCategories(cdm, x = 1243))
  expect_error(availableRouteCategories(cdm, x = list("codes" = 10L)))

  expect_error(associatedRouteCategories("a"))
  expect_error(associatedRouteCategories(cdm))
  expect_error(associatedRouteCategories(cdm, x = list("codes" = 10L)))
  expect_error(associatedRouteCategories(cdm, x, "maybe"))
})

test_that("union codelists - mock", {
  skip_on_cran()
  cdm <- mockVocabRef(backend = "data_frame")

  ing_codes <- getDrugIngredientCodes(cdm,
                                      nameStyle = "{concept_name}")

  # codelist
  ing_codes_union <- unionCodelists(ing_codes)
  expect_true(inherits(ing_codes_union, "codelist"))
  expect_true(length(ing_codes_union) == 1)
  expect_identical(ing_codes_union[[1]],
                   unique(c(ing_codes[[1]], ing_codes[[2]])))
  expect_identical(names(ing_codes_union), "adalimumab_other_ingredient")
  ing_codes_union <- unionCodelists(ing_codes, newCodelistName = "my_name")
  expect_identical(names(ing_codes_union), "my_name")

  # codelist with details
  x1 <- asCodelistWithDetails(ing_codes, cdm)
  x1 <- unionCodelists(x1)
  expect_true(inherits(x1, "codelist_with_details"))
  expect_true(length(x1) == 1)
  expect_identical(x1[[1]],
                   asCodelistWithDetails(ing_codes, cdm)[[1]])

  # note, when codelists will come out in alphabetical order
  # so the unioned codelist will not necessarily be at the end
  ing_codes_union_2 <- unionCodelists(ing_codes,
                                      keepOriginal = TRUE)
  expect_true(length(ing_codes_union_2) == 3)
  expect_identical(ing_codes_union_2[["adalimumab_other_ingredient"]],
                   unique(c(ing_codes_union_2[["adalimumab"]],
                            ing_codes_union_2[["other_ingredient"]])))
  expect_identical(ing_codes_union_2[["adalimumab"]],
                   ing_codes[["adalimumab"]])
  expect_identical(ing_codes_union_2[["other_ingredient"]],
                   ing_codes[["other_ingredient"]])


  # expected errors
  expect_error(unionCodelists(ing_codes, keepOriginal = "a"))
  expect_error(unionCodelists(ing_codes, keepOriginal = c(TRUE, FALSE)))

  })

test_that("union codelists - eunomia", {
  skip_on_cran()
  CDMConnector::requireEunomia("synpuf-1k", "5.3")
  con <- DBI::dbConnect(duckdb::duckdb(),
                        CDMConnector::eunomiaDir("synpuf-1k", "5.3"))
  cdm <- CDMConnector::cdmFromCon(con = con,
                                  cdmName = "Eunomia Synpuf",
                                  cdmSchema = "main",
                                  writeSchema = "main")

  # three codelists, one for each
  meds <- getDrugIngredientCodes(cdm,
                                 c("acetaminophen",
                                   "aspirin",
                                   "codeine"),
                                 nameStyle = "{concept_name}")
  meds_union <- unionCodelists(meds)
  expect_true(length(meds_union) == 1)
  expect_true(names(meds_union) == "acetaminophen_aspirin_codeine")

  meds_2_union_keep <- unionCodelists(meds, keepOriginal = TRUE)
  expect_true(length(meds_2_union_keep) == 4)
  expect_true(all(c("acetaminophen",
                "aspirin",
                "codeine",
                "acetaminophen_aspirin_codeine") %in%
                names(meds_2_union_keep)))

  expect_no_error(
    xx <- unionCodelists(
      x = meds,
      keepOriginal = TRUE,
      newCodelistName = "new_codelist",
      codelistsToJoin = c("acetaminophen", "aspirin")
    )
  )
  expect_true(length(xx) == 4)
  expect_true(all(
    c("acetaminophen", "aspirin", "codeine", "new_codelist") %in%
      names(xx)
  ))
  expect_true(all(xx$new_codelist %in% c(xx$acetaminophen, xx$aspirin)))
  expect_true(!all(xx$new_codelist %in% xx$codeine))
  expect_true(length(xx$new_codelist) ==
                length(unique(unlist(xx[c("acetaminophen", "aspirin")]))))

  omopgenerics::cdmDisconnect(cdm)
})

test_that("multiplication works", {
  skip_on_cran()
  cdm <- mockVocabRef()
  cl <- omopgenerics::newCodelist(list(a = c(1L,2L,3L),
                                       b = c(3L,4L,5L)))

  cl_s1 <- stratifyByConcept(cl, cdm, keepOriginal = FALSE)
  expect_true(length(cl_s1) == 6)


  cl_s2 <- stratifyByConcept(cl, cdm, keepOriginal = TRUE)
  expect_true(length(cl_s2) == 8)
  expect_true(all(sort(names(cl)) == sort(setdiff(names(cl_s2), names(cl_s1)))))



  cl <- omopgenerics::newCodelistWithDetails(list(a = data.frame(concept_id = c(1L,2L,3L),
                                                                 concept_name = c("a", "b", "c")),
                                                  b =  data.frame(concept_id = c(1L,2L,3L),
                                                                  concept_name = c("c", "d", "e"))))

  cl_s1 <- stratifyByConcept(cl, cdm, keepOriginal = FALSE)
  expect_true(length(cl_s1) == 6)


  cl_s2 <- stratifyByConcept(cl, cdm, keepOriginal = TRUE)
  expect_true(length(cl_s2) == 8)
  expect_true(all(sort(names(cl)) == sort(setdiff(names(cl_s2), names(cl_s1)))))

  # if concepts are not in the cdm
  cdm <- mockVocabRef()
  cl <- omopgenerics::newCodelist(list(a = c(1L, 2L, 3L), b = c(3L, 4L, 5L, 99999L)))
  #expect_warning(cl_s1 <- stratifyByConcept(cl, cdm, keepOriginal = FALSE))
  #expect_true(inherits(cl_s1, "codelist"))
  #expect_true(length(cl_s1) == 6) # concept 99999 will have been dropped

  # if concepts are a codelist with details
  x <- omopgenerics::newCodelist(list("a" = c(1L,2L,3L)))
  x <- asCodelistWithDetails(x, cdm)
  expect_no_error(cl_s2 <- stratifyByConcept(x, cdm, keepOriginal = FALSE))
  expect_true(inherits(cl_s2, "codelist_with_details"))
  expect_identical(cl_s2$a_arthritis,
                   dplyr::tibble(
                     "concept_id" = 3L,
                     "concept_name" = "Arthritis",
                     "domain_id" = "Condition",
                     "vocabulary_id" = "SNOMED",
                     "standard_concept" = "standard"
                   ))

  # if concepts
  # expected errors
  expect_error(stratifyByConcept(list("a" = c(1L,2L,3L)), cdm))
  expect_error(stratifyByConcept(omopgenerics::newCodelistlist("a" = c(1L,2L,3L)),
                                 cdm = "a",
                                 keepOriginal = FALSE))
  expect_error(stratifyByConcept(omopgenerics::newCodelistlist("b" = c(1L,2L,3L)),
                                 cdm = cdm,
                                 keepOriginal = "a"))

})

test_that("excludeConcepts works", {
  cdm <- mockVocabRef(backend = "data_frame")

  # Test codelist
  codelist <- omopgenerics::newCodelist(list(
    "conditions" = c(1L),
    "drug" = c(10L, 1L, 13L, 2L, 3L),
    "other" = c(13L, 10L)))

  expect_no_error(codelist1 <- excludeConcepts(codelist,
                                               cdm,
                                               concepts = c(10L, 13L),
                                               codelistName = "drug"))
  expect_identical(names(codelist1), names(codelist))
  expect_identical(class(codelist1), class(codelist))
  expect_identical(codelist1$conditions, c(1L))
  expect_identical(codelist1$drug, c(1L, 2L, 3L))
  expect_identical(sort(codelist1$other), sort(c(13L, 10L)))

  # Test codelist with details
  codelist <- codelist |> asCodelistWithDetails(cdm)

  expect_no_error(codelist1 <- excludeConcepts(x = codelist,
                                               cdm,
                                               concepts = c(10L, 13L),
                                               codelistName = "drug"))
  expect_identical(names(codelist1), names(codelist))
  expect_identical(class(codelist1), class(codelist))
  expect_identical(codelist1$drug,
                   dplyr::tibble(
                     "concept_id" = c(1L, 2L, 3L),
                     "concept_name" = c("Musculoskeletal disorder", "Osteoarthrosis", "Arthritis"),
                     "domain_id" = rep("Condition",3),
                     "vocabulary_id" = rep("SNOMED", 3),
                     "standard_concept" = rep("standard",3)
                   ))
  expect_identical(codelist1$other,
                   dplyr::tibble(
                     "concept_id" = c(10L, 13L),
                     "concept_name" = c("Adalimumab","Descendant drug"),
                     "domain_id" = c("Drug", "Drug"),
                     "vocabulary_id" = c("RxNorm", "RxNorm"),
                     "standard_concept" = rep("standard",2)
                   ))


  # Test codelist + descendants + all concepts
  expect_warning(codelist1 <- excludeConcepts(codelist,
                                               cdm,
                                               concepts = c(1L),
                                               codelistName = NULL))
  expect_true(!"conditions" %in% names(codelist1))
  expect_identical(nrow(codelist1$drug), 4L)
  expect_identical(nrow(codelist1$other), 2L)

  # Expected errors
  expect_error(excludeConcepts("hola", cdm, concepts = 1L))
  expect_error(excludeConcepts(list("a" = 1L), cdm, concepts = 1L))
  expect_error(excludeConcepts(codelist, "hola", concepts = 1L))
  expect_error(excludeConcepts(codelist, cdm, concepts = "hola"))
  expect_error(excludeConcepts(codelist, cdm, concepts = 1L, codelistName = "hola"))
})

test_that("subset on vocabulary works", {
  skip_on_cran()
  cdm <- mockVocabRef()

  # codelist
  codelist <- newCodelist(list("codes1" = c(1L, 5L, 10L),
                               "codes2" = c(2L, 15L, 22L)))
  expect_no_error(x <- subsetOnVocabulary(codelist, cdm, vocabulary = "SNOMED"))
  expect_true(inherits(x, "codelist"))
  expect_identical(names(x), c("codes1", "codes2"))
  expect_identical(x$codes1, c(1L, 5L))
  expect_identical(x$codes2, c(2L))

  # codelist + negate
  expect_no_error(x <- subsetOnVocabulary(codelist,
                                          cdm,
                                          vocabulary = "SNOMED",
                                          negate = TRUE))
  expect_true(inherits(x, "codelist"))
  expect_identical(names(x), c("codes1", "codes2"))
  expect_identical(x$codes1, c(10L))
  expect_identical(x$codes2, c(15L, 22L))

  # codelist with details
  codelist_with_details <- asCodelistWithDetails(codelist, cdm)
  expect_no_error(x <- subsetOnVocabulary(codelist_with_details, cdm, vocabulary = "SNOMED"))
  expect_true(inherits(x, "codelist_with_details"))
  expect_identical(names(x), c("codes1", "codes2"))
  expect_identical(x$codes1, codelist_with_details[[1]] |> dplyr::filter(.data$concept_id != 10L))
  expect_identical(x$codes2, codelist_with_details[[2]] |> dplyr::filter(!.data$concept_id %in% c(15L, 22L)))

  # codelist with details without vocabulary_id column and another extra column and negate
  codelist_with_details <- purrr::map(codelist_with_details,
                                      ~dplyr::select(.x, -"vocabulary_id") |>
                                        dplyr::mutate("extra" = "hello")) |>
    newCodelistWithDetails()
  expect_no_error(x <- subsetOnVocabulary(codelist_with_details, cdm, vocabulary = "SNOMED", negate = TRUE))
  expect_true(inherits(x, "codelist_with_details"))
  expect_identical(names(x), c("codes1", "codes2"))
  expect_identical(x$codes1, codelist_with_details[[1]] |> dplyr::filter(.data$concept_id == 10L))
  expect_identical(x$codes2, codelist_with_details[[2]] |> dplyr::filter(.data$concept_id %in% c(15L, 22L)))

  # lower case + more than one vocab
  expect_no_error(x <- subsetOnVocabulary(codelist_with_details, cdm, vocabulary = c("snomed", "UCUM"), negate = FALSE))
  expect_identical(x$codes1$concept_id, c(1L, 5L))
  expect_identical(x$codes2$concept_id, c(2L, 22L))
  expect_no_error(x <- subsetOnVocabulary(codelist_with_details, cdm, vocabulary = c("snomed", "UCUM"), negate = TRUE))
  expect_identical(x$codes1$concept_id, c(10L))
  expect_identical(x$codes2$concept_id, c(15L))

  # 1 codelist ends up empty
  expect_warning(x <- subsetOnVocabulary(codelist_with_details, cdm, vocabulary = "UCUM"))
  expect_true(inherits(x, "codelist_with_details"))
  expect_identical(names(x), c("codes2"))
  expect_warning(x <- subsetOnVocabulary(codelist, cdm, vocabulary = "UCUM"))
  expect_true(inherits(x, "codelist"))
  expect_identical(names(x), c("codes2"))

  # all codelists ends up empty
  expect_warning(x <- subsetOnVocabulary(
    omopgenerics::newCodelistWithDetails(list("codes1" = codelist_with_details$codes1)),
    cdm,
    vocabulary = "UCUM"))
  expect_equal(omopgenerics::emptyCodelistWithDetails(), x)

  # empty codelist
  expect_warning(x <- subsetOnVocabulary(omopgenerics::emptyCodelist(), cdm, vocabulary = "UCUM"))
  expect_identical(omopgenerics::emptyCodelist(), x)
  expect_warning(x <- subsetOnVocabulary(omopgenerics::emptyCodelistWithDetails(), cdm, vocabulary = "UCUM"))
  expect_identical(omopgenerics::emptyCodelistWithDetails(), x)

  # expected errors
  expect_error(subsetOnVocabulary(list("a" = 1L),  cdm, "snomed"))
  expect_error(subsetOnVocabulary(codelist,  "a", "snomed"))
  expect_error(subsetOnVocabulary(codelist,  cdm, vocabulary =  1234))
  expect_error(subsetOnVocabulary(codelist,  cdm, vocabulary = NULL))
  expect_error(subsetOnVocabulary(codelist,  cdm, "snomed", negate = 123))
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get descendant codes of drug ingredients
#'
#' @inheritParams cdmDoc
#' @param name Names of ingredients of interest. For example, c("acetaminophen",
#' "codeine"), would result in a list of length two with the descendant
#' concepts for these two particular drug ingredients. Users can also specify the
#' concept ID instead of the name (e.g., c(1125315, 42948451)) using a numeric vector.
#' @inheritParams nameStyleDoc
#' @inheritParams doseFormDoc
#' @inheritParams doseUnitDoc
#' @inheritParams routeCategoryDoc
#' @inheritParams ingredientRangeDoc
#' @inheritParams typeNarrowDoc
#'
#' @return Concepts with their format based on the type argument.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' cdm <- mockVocabRef()
#' getDrugIngredientCodes(cdm = cdm,
#'                        name = "Adalimumab",
#'                        nameStyle = "{concept_name}")
#'}
getDrugIngredientCodes <- function(cdm,
                                   name = NULL,
                                   nameStyle = "{concept_code}_{concept_name}",
                                   doseForm = NULL,
                                   doseUnit = NULL,
                                   routeCategory = NULL,
                                   ingredientRange = c(1, Inf),
                                   type = "codelist") {

  # Initial checks
  omopgenerics::validateCdmArgument(cdm)
  nameStyle <- checkNameStyle(nameStyle)
  type <- validateType(type = type)
  omopgenerics::assertNumeric(ingredientRange, length = 2, min = 0)
  omopgenerics::assertTrue(ingredientRange[1] <= ingredientRange[2])
  omopgenerics::assertCharacter(doseForm, null = TRUE)
  omopgenerics::assertCharacter(doseUnit, null = TRUE)
  omopgenerics::assertCharacter(routeCategory, null = TRUE)

  if(type == "concept_set_expression") {
    if(!is.null(doseForm)){
      cli::cli_abort("Dose forms not yet supported for concept set expressions")
    }
    if(!is.null(doseUnit)){
      cli::cli_abort("Dose units not yet supported for concept set expressions")
    }
    if(!is.null(routeCategory)){
      cli::cli_abort("Route categorises not yet supported for concept set expressions")
    }
  }

  searchStrategy <- searchStrategyAttr(
    functionName = "getDrugIngredientCodes",
    cdm = "cdm",
    name = cast(name),
    doseForm = cast(doseForm),
    ingredientRange = cast(ingredientRange)
  )

  if(!is.null(name)){
    if(is.character(name)){
      omopgenerics::assertCharacter(name)
    }else if(is.numeric(name)){
      omopgenerics::assertNumeric(name)
    }else{
      cli::cli_abort("Argument `name` must be either a character vector with the ingredients' names, a
                                numerical vector with the ingredients' concepts ID or NULL")
    }
  }

  ingredientConcepts <- cdm$concept |>
    dplyr::filter(.data$standard_concept == "S",
                  .data$concept_class_id == "Ingredient") |>
    dplyr::select("concept_id", "concept_name", "concept_code") |>
    dplyr::collect()

  if (!is.null(name)) {
    ingredientConcepts <- ingredientConcepts |>
      filterIngredientConcepts(name)

    if (nrow(ingredientConcepts) == 0) {
      return(emptyObject(type = type, searchStrategy = searchStrategy))
    }

    if (nrow(ingredientConcepts) != 1 & type == "code_search") {
      cli::cli_abort(c(x = "`code_search` type only supports one ingredient at a time."))
    }
  }

  # simple case can just return concept set expression with descendants
  if (all(type == "concept_set_expression",
         is.null(doseForm),
         is.null(doseUnit),
         is.null(routeCategory),
         ingredientRange == c(1, Inf))) {
    cse <- ingredientConcepts |>
      dplyr::mutate(name = glue::glue(.env$nameStyle)) |>
      dplyr::select("concept_id",
                    "name") |>
      dplyr::mutate("excluded" = FALSE,
                    "descendants" = TRUE,
                    "mapped" = FALSE)
    cse <- split(
      x = cse |>
        dplyr::select(!"name"),
      f = as.factor(cse$name),
      drop = TRUE
    ) |>
      omopgenerics::newConceptSetExpression()
    return(cse)
  }

  # to avoid potential memory problems will batch
  if (nrow(ingredientConcepts) > 0) {
    ingredientCodes <- fetchBatchedDescendants(cdm,
                                               codes = ingredientConcepts$concept_id,
                                               ingredientRange = ingredientRange,
                                               batchSize = 500,
                                               doseForm = doseForm
    ) |>
      dplyr::filter(.data$standard_concept == "S")
  }

  if (nrow(ingredientCodes) == 0) {
    cli::cli_warn("No descendant codes found")
    return(emptyObject(type = type, searchStrategy = searchStrategy))
  }

  ingredientCodes <- ingredientCodes  |>
    dplyr::select(
      "concept_id", "concept_name", "domain_id", "vocabulary_id",
      "standard_concept", "ancestor_concept_id", "concept_class_id",
      "concept_code", "valid_start_date", "valid_end_date", "invalid_reason"
    ) |>
    # split different ancestors into multiple cols
    tidyr::separate_wider_delim(
      cols = "ancestor_concept_id",
      delim = ";",
      names_sep = "",
      too_few = "align_start"
    )

  ingredientCodes <- ingredientCodes |>
    # one row per concept + ancestor
    tidyr::pivot_longer(cols = dplyr::starts_with("ancestor_concept_id"),
                        names_to = NULL,
                        values_to = "ancestor_concept_id",
                        values_drop_na = TRUE)

  ingredientCodes <- ingredientCodes |>
    dplyr::left_join(
      ingredientConcepts |>
        dplyr::mutate("concept_name" = omopgenerics::toSnakeCase(.data$concept_name)) |>
        dplyr::mutate("concept_id" = as.character(.data$concept_id)) |>
        dplyr::mutate("name" = glue::glue(.env$nameStyle)) |>
        dplyr::select("concept_id", "name"),
      by = c("ancestor_concept_id" = "concept_id")
    )

  if (type == "codelist") {
    ingredientCodes <- split(
      x = ingredientCodes$concept_id,
      f = as.factor(ingredientCodes$name),
      drop = TRUE
    ) |>
      omopgenerics::newCodelist()
  } else if (type == "codelist_with_details") {
    ingredientCodes <- split(
      x = ingredientCodes,
      f = as.factor(ingredientCodes$name),
      drop = TRUE
    ) |>
      purrr::map(~dplyr::select(., -"name")) |>
      omopgenerics::newCodelistWithDetails()
  } else if (type == "concept_set_expression") {
    ingredientCodes <- split(
      x = ingredientCodes|>
        dplyr::select("concept_id") |>
        dplyr::mutate("excluded" = FALSE,
                      "descendants" = FALSE,
                      "mapped" = FALSE),
      f = as.factor(ingredientCodes$name),
      drop = TRUE
    )|>
      omopgenerics::newConceptSetExpression()
  } else if (type == "code_search") {
    nm <- unique(ingredientCodes$name)
    ingredientCodes <- ingredientCodes |>
      dplyr::select(!c("name", "ancestor_concept_id")) |>
      dplyr::mutate(
        !!nm := TRUE,
        found_from = "Descendants",
        vocabulary_version = vocabularyVersion(cdm = cdm)
      ) |>
      newCodeSearch(searchStrategy = searchStrategy)
  }

  if (!is.null(routeCategory)) {
    ingredientCodes <- subsetOnRouteCategory(ingredientCodes,
                                             cdm = cdm,
                                             routeCategory = routeCategory)
  }

  if (!is.null(doseUnit)) {
    ingredientCodes <- subsetOnDoseUnit(ingredientCodes,
                                        cdm = cdm,
                                        doseUnit = doseUnit)
  }

  return(ingredientCodes)
}

filterIngredientConcepts <- function(ingredientConcepts, name){
  if(is.character(name)){
    ingredientConcepts <- ingredientConcepts |>
      dplyr::filter(tidyWords(.data$concept_name, message = FALSE) %in%
                      tidyWords(.env$name, message = FALSE))
  }else if(is.numeric(name)){
    ingredientConcepts <- ingredientConcepts |>
      dplyr::filter(.data$concept_id %in% .env$name)
  }
  if(nrow(ingredientConcepts) == 0){
    cli::cli_warn("- No matching Ingredient codes found")
  } else if(nrow(ingredientConcepts) < length(name)){

    if(is.character(name)) {
      missingIngredientConcepts <- setdiff(tidyWords(name), tidyWords(ingredientConcepts$concept_name))
      cli::cli_warn("- No matching Ingredient codes found for {missingIngredientConcepts}")
    }else if(is.numeric(name)){
      missingIngredientConcepts <- setdiff(name, ingredientConcepts$concept_id)
      cli::cli_warn("- No matching Ingredient codes found for Concept ID {missingIngredientConcepts}")
    }
  }
  return(ingredientConcepts)
}

fetchBatchedDescendants <- function(cdm,
                                    codes,
                                    batchSize,
                                    ingredientRange = c(0, Inf),
                                    doseForm) {
  codeBatches <- split(
    codes,
    ceiling(seq_along(codes) / batchSize)
  )

  descendants <- vector("list", length(codeBatches))

  cli::cli_progress_bar(
    total = length(descendants),
    format = " -- getting descendants {cli::pb_bar} {cli::pb_current} of {cli::pb_total} batched groups"
  )

  for (i in seq_along(descendants)) {
    cli::cli_progress_update()
    x <- getDescendants(
      cdm = cdm,
      conceptId = codeBatches[[i]],
      withAncestor = TRUE
    )

    x <- list("codes" = x) |> newCodelistWithDetails()

    if(ingredientRange[1] != 1 | ingredientRange[2]!= Inf){
    x <- subsetOnIngredientRange(x,
                                 cdm = cdm,
                                 ingredientRange = ingredientRange,
                                 negate = FALSE)
    }

    if(!is.null(doseForm)){
      x <- subsetOnDoseForm(x,
                            cdm = cdm,
                            doseForm = doseForm,
                            negate = FALSE)
    }

    if(length(x$codes) == 0){
      x$codes <- dplyr::tibble(
        "ancestor_concept_id" = character(),
        "concept_id" = integer(),
        "concept_name" = character(),
        "domain_id" = character(),
        "vocabulary_id" = character(),
        "standard_concept" = character()
      )
    }
    descendants[[i]] <- x$codes
  }

  cli::cli_progress_done()
  descendants <- dplyr::bind_rows(descendants)

  return(descendants)
}

checkNameStyle <- function(nameStyle){
  x <- nameStyle %in% c("{concept_code}", "{concept_id}", "{concept_name}",
                        "{concept_code}_{concept_id}", "{concept_code}_{concept_name}", "{concept_id}_{concept_code}",
                        "{concept_id}_{concept_name}", "{concept_name}_{concept_code}", "{concept_name}_{concept_id}",
                        "{concept_code}_{concept_id}_{concept_name}", "{concept_code}_{concept_name}_{concept_id}",
                        "{concept_id}_{concept_code}_{concept_name}", "{concept_id}_{concept_name}_{concept_code}",
                        "{concept_name}_{concept_code}_{concept_id}", "{concept_name}_{concept_id}_{concept_code}")
  if(isFALSE(x)){
    cli::cli_abort("nameStyle {nameStyle} is not supported. nameStyle argument must be a choice between {{concept_name}}, {{concept_id}}, or {{concept_code}},
                   or any combination of the three separated by `_` (i.e., {{concept_id}}_{{concept_name}}.")}

  return(nameStyle)
}

emptyObject <- function(type, searchStrategy) {
  if (type == "codelist") {
    omopgenerics::emptyCodelist()
  } else if (type == "codelist_with_details") {
    omopgenerics::emptyCodelistWithDetails()
  } else if (type == "concept_set_expression") {
    # note, currently no empty concept set expression function
    omopgenerics::newConceptSetExpression(list())
  } else if (type == "code_search") {
    emptyCodeSearch(searchStrategy = searchStrategy)
  }
}
searchStrategyAttr <- function(...) {
  dplyr::tibble(
    strategy_id = 1L,
    package_name = "CodelistGenerator",
    package_version = as.character(utils::packageVersion("CodelistGenerator")),
    ...
  ) |>
    tidyr::pivot_longer(
      !"strategy_id",
      names_to = "strategy_name",
      values_to = "strategy_value"
    )
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Generate a codelist from the intersection of different codelists. The generated
#' codelist will come out in alphabetical order.
#'
#' @inheritParams xDoc
#' @inheritParams keepOriginalDoc
#'
#' @return A codelist
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create a CDM object
#' cdm <- mockCdmReference()
#'
#' # Intersect two codelists
#' codelist <- newCodelist(list("mood" = c(37110496L, 4226696L, 4304866L),
#'                              "manic" = c(37110496L, 4226696L)))
#'
#' intersectCodelists(codelist, keepOriginal = TRUE)
#'
#'
#' # Intersect two codelists_with_details
#' codelist <- asCodelistWithDetails(codelist, cdm)
#'
#' intersectCodelists(codelist, keepOriginal = FALSE)
#' }
intersectCodelists <- function(x,
                               keepOriginal = FALSE) {

  checkCodelist(x)
  omopgenerics::assertLogical(keepOriginal, length = 1)

  newName <- paste0("intersection_",paste0(names(x), collapse = "_"))

  if(inherits(x, "codelist")){
    newX <- list(Reduce(intersect, x))
    names(newX) <- newName

    if(isTRUE(keepOriginal)){
      newX <- purrr::list_flatten(list(x, newX)) |>
        omopgenerics::newCodelist()
    } else {
      newX <- newX |> omopgenerics::newCodelist()
    }
  }

  if(inherits(x, "codelist_with_details")){
    newX <- purrr::imap(
      x,
      ~ dplyr::mutate(.x, "codelist" = .y)) |>
      dplyr::bind_rows(.id = "codelist")

    newX <- newX |>
      dplyr::group_by(dplyr::across(-dplyr::all_of("codelist"))) |>
      dplyr::mutate("codelist" = dplyr::n()) |>
      dplyr::ungroup() |>
      dplyr::filter(.data$codelist == length(names(x))) |>
      dplyr::select(-"codelist") |>
      dplyr::distinct()

    newX <- list("x" = newX)
    names(newX) <- newName

    if(isTRUE(keepOriginal)){
      newX <- purrr::list_flatten(list(x, newX)) |>
        omopgenerics::newCodelistWithDetails()
    }else{
      newX <- newX |> omopgenerics::newCodelistWithDetails()
    }
  }

  return(newX)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Format the result of summariseOrphanCodes into a table
#'
#' @param result A `<summarised_result>` with results of the type
#' "orphan_codes".
#' @inheritParams typeTableDoc
#' @inheritParams tableStyleDoc
#' @inheritParams headerDoc
#' @inheritParams groupColumnDoc
#' @inheritParams hideDoc
#' @inheritParams .optionsDoc
#'
#' @return A table with a formatted version of the summariseOrphanCodes
#' result.
#'
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef("database")
#' codes <- getCandidateCodes(cdm = cdm,
#' keywords = "Musculoskeletal disorder",
#' domains = "Condition",
#' includeDescendants = FALSE)
#'
#' orphan_codes <- summariseOrphanCodes(x = newCodelist(list("msk" = codes$concept_id)),
#' cdm = cdm)
#'
#' tableOrphanCodes(orphan_codes)
#'
#' CDMConnector::cdmDisconnect(cdm)
#'}
#'
tableOrphanCodes <- function(result,
                             type = NULL,
                             header = c("cdm_name", "estimate_name"),
                             groupColumn = character(),
                             hide = character(),
                             style = NULL,
                             .options = list()) {

  rlang::check_installed("visOmopResults", version = "1.4.0")

  if(nrow(result) == 0){
    cli::cli_warn("`result` object is empty")
    return(visOmopResults::emptyTable(type = type))
  }

  result <- result |>
    visOmopResults::filterSettings(.data$result_type == "orphan_code_use")

  if(nrow(result) == 0){
    cli::cli_warn("No orphan code results found in result object")
    return(visOmopResults::emptyTable(type = type))
  }

  x <- internalTableAchillesResult(
    result = result,
    resultType = "orphan_code_use",
    type = type,
    header = header,
    groupColumn = groupColumn,
    hide = hide,
    style = style,
    .options = .options
  )

  return(x)
}

internalTableAchillesResult <- function(result,
                                        type,
                                        resultType,
                                        header,
                                        groupColumn,
                                        hide,
                                        style,
                                        .options) {
  omopgenerics::assertCharacter(header, null = TRUE)
  omopgenerics::assertCharacter(hide, null = TRUE)
  if (!is.list(groupColumn) & !is.null(groupColumn)) groupColumn <- list(groupColumn)
  omopgenerics::assertCharacter(groupColumn[[1]], null = TRUE)
  omopgenerics::assertCharacter(style, null = TRUE, length = 1)
  omopgenerics::assertCharacter(type, null = TRUE, length = 1)

  # filter result + nice estimate name
  x <- result |>
    visOmopResults::filterSettings(.data$result_type == .env$resultType) |>
    dplyr::mutate(estimate_name = stringr::str_to_sentence(gsub("_", " ", .data$estimate_name)))

  header <- reformulateTableAchilles(header)
  groupColumn[[1]] <- reformulateTableAchilles(groupColumn[[1]])
  hide <- reformulateTableAchilles(hide)

  # visOmopTable
  x <- visOmopResults::visOmopTable(
    result = x,
    estimateName = character(),
    header = header,
    groupColumn = groupColumn,
    type = type,
    rename = c(
      "Domain ID" = "domain_id", "Vocabulary ID" = "vocabulary_id",
      "Database name" = "cdm_name", "Standard concept ID" = "variable_level",
      "Standard concept name" = "variable_name", "Source concept ID" = "source_concept_id"
    ),
    hide = hide,
    style = style,
    .options = .options
  ) |>
    suppressWarnings() # to remove after next release of visOmopResults

  return(x)
}

reformulateTableAchilles <- function(vect) {
  vect <- as.character(vect)
  vect[vect == "standard_concept_id"] <- "variable_level"
  vect[vect == "standard_concept_name"] <- "variable_name"
  vect
}

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Subset a codelist to only those codes with a range of number of ingredients
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams ingredientRangeDoc
#' @param negate If FALSE, only concepts with the ingredient range specified will be
#' returned (both limits included). If TRUE, concepts with number of ingredients outside
#' the range will be returned.
#'
#' @return The codelist with only those concepts associated with the domain
#' (if negate = FALSE) or the codelist without those concepts associated with
#' the domain (if negate = TRUE).
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef()
#' codes <- subsetOnIngredientRange(
#'               x = newCodelist(list("codes" = c(10L, 13L))),
#'               cdm = cdm,
#'               ingredientRange = c(2, 10))
#' codes
#' }
subsetOnIngredientRange <- function(x,
                                    cdm,
                                    ingredientRange,
                                    negate = FALSE){

  # Initial checks
  omopgenerics::assertNumeric(ingredientRange, length = 2, min = 0)
  omopgenerics::assertTrue(ingredientRange[1] <= ingredientRange[2])

  if(ingredientRange[2] == Inf){
    ingredientRange[2] <- 9999999
  }

  st <- searchStrategyAttr(
    function_name = "subsetOnIngredientRange",
    cdm = "cdm",
    ingredientRange = cast(ingredientRange)
  )
  x <- subsetCodelistBy(x,
                   cdm,
                   by = "ingredient_range",
                   group = ingredientRange,
                   st = st,
                   negate = negate)

  return(x)
}

test_that("subset on ingredient range works", {
  skip_on_cran()
  con <- DBI::dbConnect(duckdb::duckdb(),
                        CDMConnector::eunomiaDir("synpuf-1k", "5.3"))
  cdm <- CDMConnector::cdmFromCon(con = con,
                                  cdmName = "Eunomia Synpuf",
                                  cdmSchema   = "main",
                                  writeSchema = "main",
                                  achillesSchema = "main")

  n1 <- names(cdm)
  x <- omopgenerics::newCodelist(list("codes1" = c(43260756L, 44054657L,44084854L),
                                      "codes2" = c(41025222L, 41448307L,42721793L)))
  # codelist
  expect_no_error(res <- subsetOnIngredientRange(x,
                                                 cdm,
                                                 ingredientRange = c(5, 10),
                                                 negate = FALSE))
  expect_equal(n1, names(cdm))
  expect_true(inherits(res, "codelist"))
  expect_equal(res$codes1, c(43260756L, 44054657L))
  expect_equal(res$codes2, c(41448307L))

  # same ingredient range
  expect_no_error(res <- subsetOnIngredientRange(x,
                                                 cdm,
                                                 ingredientRange = c(8, 8),
                                                 negate = FALSE))

  # codelist + negate argument
  expect_no_error(res <- subsetOnIngredientRange(x,
                                                 cdm,
                                                 ingredientRange = c(5, 10),
                                                 negate = TRUE))
  expect_equal(n1, names(cdm))
  expect_true(inherits(res, "codelist"))
  expect_equal(res$codes1, c(44084854L))
  expect_equal(res$codes2, c(41025222L, 42721793L))

  # empty 1 codelist
  expect_warning(res <- subsetOnIngredientRange(x,
                                                 cdm,
                                                 ingredientRange = c(13, 23),
                                                 negate = FALSE))
  expect_equal(n1, names(cdm))
  expect_true(inherits(res, "codelist"))
  expect_equal(names(res), "codes2")

  expect_no_error(res <- subsetOnIngredientRange(omopgenerics::emptyCodelist(),
                                                 cdm,
                                                 ingredientRange = c(13, 23),
                                                 negate = FALSE))
  expect_true(inherits(res, "codelist"))

  # empty codelist with details
  expect_no_error(res <- subsetOnIngredientRange(omopgenerics::emptyCodelistWithDetails(),
                                                 cdm,
                                                 ingredientRange = c(13, 23),
                                                 negate = FALSE))
  expect_true(inherits(res, "codelist_with_details"))

  # codelist with details
  x <- x |> asCodelistWithDetails(cdm)
  expect_no_error(res <- subsetOnIngredientRange(x,
                                                 cdm,
                                                 ingredientRange = c(5, 10),
                                                 negate = FALSE))
  expect_equal(n1, names(cdm))
  expect_true(inherits(res, "codelist_with_details"))
  expect_equal(res$codes1,
               x$codes1 |>
                 dplyr::filter(concept_id %in% c(43260756L, 44054657L)))
  expect_equal(res$codes2,
               x$codes2 |>
                 dplyr::filter(concept_id %in% c(41448307L)))

  # all ingredeint range
  expect_no_error(res <- subsetOnIngredientRange(x,
                                                 cdm,
                                                 ingredientRange = c(1, Inf),
                                                 negate = FALSE))
  expect_equal(n1, names(cdm))
  expect_true(inherits(res, "codelist_with_details"))
  expect_equal(length(res$codes1$concept_id),3L)
  expect_equal(length(res$codes2$concept_id),3L)

  # no drug concept
  x <- omopgenerics::newCodelist(list("a" = c(44517160L, 44084854L)))
  expect_no_error(res <- subsetOnIngredientRange(x,
                                                 cdm,
                                                 ingredientRange = c(1, Inf),
                                                 negate = FALSE))
  expect_equal(n1, names(cdm))
  expect_true(inherits(res, "codelist"))
  expect_equal(res$a,44084854L)

  # check empty codelists
  x <- subsetOnIngredientRange(omopgenerics::emptyCodelist(), cdm, ingredientRange = c(1, Inf))
  expect_true(inherits(x, "codelist"))
  x <- subsetOnIngredientRange(omopgenerics::emptyCodelistWithDetails(), cdm, ingredientRange = c(1, Inf))
  expect_true(inherits(x, "codelist_with_details"))

  # expected errors:
  expect_error(subsetOnIngredientRange(list("a" = 1L), cdm, ingredientRange = c(5,10)))
  expect_error(subsetOnIngredientRange(x, "a", ingredientRange = c(5,10)))
  expect_error(subsetOnIngredientRange(x, cdm, ingredientRange = c("hola")))
  expect_error(subsetOnIngredientRange(x, cdm, ingredientRange = c(10,5)))

})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


#' Show mappings from non-standard vocabularies to standard.
#'
#' @param candidateCodelist Dataframe.
#' @inheritParams cdmDoc
#' @param nonStandardVocabularies Character vector.
#'
#' @return Tibble with the information of potential standard to non-standard
#' mappings for the codelist of interest.
#' @export
#'
#' @examples
#' \donttest{
#' cdm <- CodelistGenerator::mockVocabRef()
#' codes <- CodelistGenerator::getCandidateCodes(
#'   cdm = cdm,
#'   keywords = "osteoarthritis"
#' )
#' CodelistGenerator::getMappings(
#'   cdm = cdm,
#'   candidateCodelist = codes,
#'   nonStandardVocabularies = "READ"
#')
#'}
getMappings <- function(candidateCodelist,
                        cdm = NULL,
                        nonStandardVocabularies = c(
                          "ATC", "ICD10CM", "ICD10PCS",
                          "ICD9CM", "ICD9Proc",
                          "LOINC", "OPCS4", "Read",
                          "RxNorm", "RxNorm Extension",
                          "SNOMED"
                        )) {

  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  omopgenerics::assertCharacter(nonStandardVocabularies)
  omopgenerics::assertTable(candidateCodelist)
  omopgenerics::assertTrue("concept" %in% names(cdm))
  omopgenerics::assertTrue("concept_relationship" %in% names(cdm))
  omopgenerics::assertTrue("concept_ancestor" %in% names(cdm))
  omopgenerics::assertTrue("concept_synonym" %in% names(cdm))
  omopgenerics::assertTrue("vocabulary" %in% names(cdm))

  conceptDb <- cdm$concept
  conceptRelationshipDb <- cdm$concept_relationship

  # lowercase names
  conceptDb <- dplyr::rename_with(conceptDb, tolower)
  conceptRelationshipDb <- dplyr::rename_with(
    conceptRelationshipDb,
    tolower
  )

  # vocabs to upper case
  nonStandardVocabularies <- toupper(nonStandardVocabularies)
  conceptDb <- conceptDb |>
    dplyr::mutate(vocabulary_id = toupper(.data$vocabulary_id))

  # check nonStandardVocabularies exist
  nonStandardVocabulariesInDb <- conceptDb |>
    dplyr::select("vocabulary_id") |>
    dplyr::distinct() |>
    dplyr::collect() |>
    dplyr::pull()
  for (i in seq_along(nonStandardVocabularies)) {
    nonStandardVocabulariesCheck <- nonStandardVocabularies[i] %in%
      nonStandardVocabulariesInDb
    omopgenerics::assertTrue(nonStandardVocabulariesCheck)
    if (!isTRUE(nonStandardVocabulariesCheck)) {
      cli::cli_abort("- Vocabulary {nonStandardVocabularies[i]} not found")
    }
  }

  mappedCodes <- conceptDb |>
    dplyr::inner_join(conceptRelationshipDb |>
                        dplyr::filter(.data$relationship_id == "Mapped from") |>
                        dplyr::filter(.data$concept_id_1 %in% !!candidateCodelist$concept_id) |>
                        dplyr::select("concept_id_1", "concept_id_2") |>
                        dplyr::rename("concept_id" = "concept_id_2"),
                      by = c("concept_id")
    ) |>
    dplyr::filter(.data$vocabulary_id %in% .env$nonStandardVocabularies) |>
    dplyr::distinct() |>
    dplyr::collect()

  mappedCodes <- mappedCodes |>
    dplyr::select(
      "concept_id_1", "concept_id",
      "concept_name", "concept_code",
      "vocabulary_id"
    )

  mappedCodes <- mappedCodes |>
    dplyr::select("concept_id_1") |>
    dplyr::rename("concept_id" = "concept_id_1") |>
    dplyr::left_join(conceptDb |>
                       dplyr::filter(.data$concept_id %in% !!mappedCodes$concept_id_1) |>
                       dplyr::collect(),
                     by = c("concept_id")
    ) |>
    dplyr::select("concept_id", "concept_name", "vocabulary_id") |>
    dplyr::rename("standard_vocabulary_id" = "vocabulary_id") |>
    dplyr::rename("concept_id_1" = "concept_id") |>
    dplyr::rename("standard_concept_name" = "concept_name") |>
    dplyr::full_join(mappedCodes,
                     by = "concept_id_1"
    ) |>
    dplyr::rename("standard_concept_id" = "concept_id_1") |>
    dplyr::rename("non_standard_concept_id" = "concept_id") |>
    dplyr::rename("non_standard_concept_code" = "concept_code") |>
    dplyr::rename("non_standard_concept_name" = "concept_name") |>
    dplyr::rename("non_standard_vocabulary_id" = "vocabulary_id")

  mappedCodes <- mappedCodes |>
    dplyr::distinct() |>
    dplyr::arrange(.data$standard_concept_id)

  return(mappedCodes)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Generate a codelist from the union of different codelists. The generated codelist
#' will come out in alphabetical order.
#'
#' @inheritParams xDoc
#' @param newCodelistName Character vector with the name of the new codelist. If
#' NULL all codelists names will be combined.
#' @inheritParams keepOriginalDoc
#' @param codelistsToJoin Character vector with the names of the codelists to be
#' unioned.
#'
#' @return A codelist
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' cdm <- mockVocabRef()
#' getDrugIngredientCodes(cdm,
#'                         nameStyle = "{concept_name}") |>
#' unionCodelists()
#' }
unionCodelists <- function(x,
                           newCodelistName = NULL,
                           keepOriginal = FALSE,
                           codelistsToJoin = names(x)) {

  checkCodelist(x)
  omopgenerics::assertLogical(keepOriginal, length = 1)
  omopgenerics::assertCharacter(newCodelistName, length = 1, null = TRUE)
  omopgenerics::assertChoice(codelistsToJoin, names(x), unique = TRUE)

  xNames <- codelistsToJoin

  allCodes <- purrr::list_c(x[codelistsToJoin]) |>
    unique()
  if (is.null(newCodelistName)) {
    newCodelistName <- paste0(xNames, collapse = "_")
  }

  newX <- list()
  newX[[newCodelistName]] <- allCodes
  if (isTRUE(keepOriginal)) {
    newX <- purrr::list_flatten(list(x, newX))
  }

  if (inherits(x, "codelist")) {
    newX <- newX |> omopgenerics::newCodelist()
  }
  if (inherits(x, "codelist_with_details")) {
    newX <- newX |> omopgenerics::newCodelistWithDetails()
  }
  if (inherits(x, "concept_set_expression")) {
    newX <- newX |> omopgenerics::newConceptSetExpression()
  }

  return(newX)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get descendant codes for a given concept
#'
#' @inheritParams cdmDoc
#' @param conceptId concept_id to search
#' @param withAncestor If TRUE, return column with ancestor. In case of multiple
#' ancestors, concepts will be separated by ";".
#'
#' @return The descendants of a given concept id.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' cdm <- mockVocabRef()
#' getDescendants(cdm = cdm, conceptId = 1)
#' }

getDescendants <- function(cdm,
                           conceptId,
                           withAncestor = FALSE) {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  omopgenerics::assertNumeric(conceptId, integerish = TRUE)
  omopgenerics::assertLogical(withAncestor)

  if(isFALSE(withAncestor)){
    descendants <- getDescendantsOnly(cdm, conceptId)}

  if(isTRUE(withAncestor)){
    descendants <- getDescendantsAndAncestor(cdm, conceptId)}

  return(descendants)
}

getDescendantsOnly <- function(cdm, conceptId) {
  descendants <- cdm$concept_ancestor |>
    dplyr::filter(.data$ancestor_concept_id %in% .env$conceptId) |>
    dplyr::select("descendant_concept_id") |>
    dplyr::distinct() |>
    dplyr::rename("concept_id" = "descendant_concept_id") |>
    dplyr::left_join(cdm$concept,
                     by = "concept_id")

  descendants <- descendants |>
    dplyr::collect()

  return(descendants)
}

getDescendantsAndAncestor <- function(cdm, conceptId) {
  conceptIdDbTable <- omopgenerics::uniqueTableName()
  cdm <- omopgenerics::insertTable(cdm = cdm,
                                   name = conceptIdDbTable,
                                   table = dplyr::tibble(ancestor_concept_id = as.integer(conceptId)),
                                   overwrite = TRUE)

  descendants <- cdm$concept_ancestor |>
    dplyr::inner_join(cdm[[conceptIdDbTable]],
                      by = "ancestor_concept_id") |>
    dplyr::rename("concept_id" = "descendant_concept_id") |>
    dplyr::left_join(cdm$concept,
                     by = "concept_id") |>
    dplyr::compute()

  descendants <- descendants |>
    dplyr::collect() |>
    dplyr::mutate(name = paste0("concept_", .data$ancestor_concept_id))

  if(nrow(descendants)>0){
    descendants <- descendants |>
      tidyr::pivot_wider(names_from = "name",
                         values_from = "ancestor_concept_id")

    # one row per concept, with ancestor (of which there may be multiple)
    working_cols <- stringr::str_subset(string = colnames(descendants),
                                        pattern = paste(c(colnames(cdm$concept),
                                                          colnames(cdm$concept_ancestor)),
                                                        collapse = "|"),
                                        negate = TRUE)

    descendants <- descendants |>
      tidyr::unite(col="ancestor_concept_id",
                   dplyr::all_of(working_cols), sep=";")
    # quicker to replace NAs afterwards rather than inside unite
    # (especially when there are many columns)
    descendants$ancestor_concept_id <- stringr::str_replace_all(
      string = descendants$ancestor_concept_id,
      pattern = ";NA|NA;",
      replacement = ""
    )
  }

  omopgenerics::dropSourceTable(cdm, conceptIdDbTable)

  # nb conceptId will also be a descendant of itself
  return(descendants)

}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Table showing the route category associated with each dose form.
#' @format A data frame
#' \describe{
#'  \item{dose_form_concept_id}{Concept ID of each dose form}
#'  \item{dose_form_concept_name}{Concept name of each dose form}
#'  \item{route_category}{Route category associated to the dose form}
#' }
#' @keywords data
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' doseFormToRoute
#' }
"doseFormToRoute"

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Subset a codelist to only those codes from a particular vocabulary.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @param vocabulary Vocabulary to subset with (i.e., SNOMED)
#' @param negate If FALSE, only concepts with the vocabulary specified will be
#' returned. If TRUE, concepts with the vocabulary specified will be excluded.
#'
#' @return The codelist with only those concepts associated with the vocabulary
#' (if negate = FALSE) or the codelist without those concepts associated with
#' the vocabulary (if negate = TRUE).
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef()
#' codes <- subsetOnVocabulary(
#'               x = newCodelist(list("codes" = c(1L,13L,15L))),
#'               cdm = cdm,
#'               vocabulary = "SNOMED")
#' codes
#' }
subsetOnVocabulary <- function(x,
                               cdm,
                               vocabulary,
                               negate = FALSE){

  # Initial checks
  checkCodelist(x, allowConceptSetExpression = FALSE)
  omopgenerics::validateCdmArgument(cdm)
  omopgenerics::assertCharacter(vocabulary)
  omopgenerics::assertLogical(negate, length = 1)
  x_original <- x

  if(length(x) == 0){
    cli::cli_warn("{.val x} is empty. Returning an empty codelist.")
    return(x)
  }

  # Add vocabulary_id table if it's not already there
  if(inherits(x_original, "codelist")){
    x <- x |> addDetails(cdm, cols = "vocabulary_id")
  }

  if(inherits(x_original, "codelist_with_details")){
    if(!c("vocabulary_id" %in% colnames(x[[1]]))){
      x <- x |> addDetails(cdm, cols = "vocabulary_id")
      noVocab <- TRUE
    }else{
      noVocab <- FALSE
    }
  }

  # Filter vocabulary
  if(isTRUE(negate)){
    x <- purrr::map(x, ~ dplyr::filter(.x, !tolower(.data$vocabulary_id) %in% tolower(.env$vocabulary)))
  }else{
    x <- purrr::map(x, ~ dplyr::filter(.x, tolower(.data$vocabulary_id) %in% tolower(.env$vocabulary)))
  }

  if(inherits(x_original, "codelist")){
    x <- purrr::map(x, ~dplyr::pull(.x, "concept_id")) |>
      omopgenerics::newCodelist()
  }

  if(inherits(x_original, "codelist_with_details")){
    if(isTRUE(noVocab)){
      x <- purrr::map(x, ~dplyr::select(.x, -"vocabulary_id"))
    }
    x <- x |>
      omopgenerics::newCodelistWithDetails()
  }

  x <- dropEmptyCodelist(x_original, x)

  return(x)
}

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Subset a codelist to only those codes from a particular domain.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @param nameStyle Naming of the new codelists, use `{codelist_name}` to
#' include the codelist name and `{vocabulary}` to include the vocabulary name.
#' @inheritParams keepOriginalDoc
#'
#' @return The codelist with the required stratifications, as different elements
#' of the list.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#'
#' cdm <- mockVocabRef()
#'
#' codes <- stratifyByVocabulary(
#'   x = newCodelist(list("codes" = c(10L, 13L, 15L))),
#'   cdm = cdm,
#'   keepOriginal = TRUE
#' )
#'
#' codes
#' }
#'
stratifyByVocabulary <- function(x,
                                 cdm,
                                 nameStyle = "{codelist_name}_{vocabulary}",
                                 keepOriginal = FALSE) {
  st <- searchStrategyAttr(
    function_name = "stratifyByVocabulary",
    cdm = "cdm"
  )
  stratifyCodelistBy(
    x = x,
    cdm = cdm,
    by = "vocabulary",
    nameStyle = nameStyle,
    st = st,
    keepOriginal = keepOriginal
  )
}

addVocabulary <- function(x) {
  cdm <- omopgenerics::cdmReference(table = x)
  x |>
    # add vocabulary
    dplyr::left_join(
      cdm$concept |>
        dplyr::select("concept_id", "vocabulary" = "vocabulary_id"),
      by = "concept_id"
    ) |>
    dplyr::collect() |>
    dplyr::distinct()
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Summarise code use in patient-level data.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams countByDoc
#' @inheritParams byConceptDoc
#' @inheritParams bySexDoc
#' @inheritParams byYearDoc
#' @inheritParams ageGroupDoc
#' @param dateRange Two dates. The first indicating the earliest cohort start
#' date and the second indicating the latest possible cohort end date. If NULL
#' or the first date is set as missing, the earliest observation_start_date in
#' the observation_period table will be used for the former. If NULL or the
#' second date is set as missing, the latest observation_end_date in the
#' observation_period table will be used for the latter.
#' @param useSourceCodes Whether the codelist provided contains source codes (TRUE) or standard codes (FALSE).
#'
#' @return A tibble with count results overall and, if specified, by strata.
#' @export
#'
#' @examples
#' \dontrun{
#' library(omopgenerics)
#' library(CodelistGenerator)
#' con <- DBI::dbConnect(duckdb::duckdb(),
#'                       dbdir = CDMConnector::eunomiaDir())
#' cdm <- CDMConnector::cdmFromCon(con,
#'                                 cdmSchema = "main",
#'                                 writeSchema = "main")
#' acetiminophen <- c(1125315,  1127433, 40229134,
#'                   40231925, 40162522, 19133768,  1127078)
#' poliovirus_vaccine <- c(40213160)
#' cs <- newCodelist(list(acetiminophen = acetiminophen,
#'                        poliovirus_vaccine = poliovirus_vaccine))
#' results <- summariseCodeUse(cs,cdm = cdm)
#' results
#' CDMConnector::cdmDisconnect(cdm)
#'}
#'
summariseCodeUse <- function(x,
                             cdm,
                             countBy = c("record", "person"),
                             byConcept = TRUE,
                             byYear = FALSE,
                             bySex = FALSE,
                             ageGroup = NULL,
                             dateRange = as.Date(c(NA, NA)),
                             useSourceCodes = FALSE){

  checkCodelist(x, allowConceptSetExpression = FALSE)


  if(inherits(x, "codelist_with_details")){
    x <- asCodelist(x)
  }

  if(length(x) == 0){
    return(omopgenerics::emptySummarisedResult())
  }

  codeUse <- list()
  for(i in seq_along(x)){
    cli::cli_inform("Getting use of codes from {names(x)[i]} ({i} of {length(x)})")
    codeUse[[i]] <- getCodeUse(x[i],
                               cdm = cdm,
                               cohortTable = NULL,
                               cohortId = NULL,
                               timing = "any",
                               countBy = countBy,
                               byConcept = byConcept,
                               byYear = byYear,
                               bySex = bySex,
                               ageGroup = ageGroup,
                               dateRange = dateRange,
                               useSourceCodes = useSourceCodes)
  }
  codeUse <- dplyr::bind_rows(codeUse)

  if(nrow(codeUse) > 0) {

    if(is.na(dateRange[1])){
      dateRangeStart <- "none"
    } else {
      dateRangeStart <- as.character(dateRange[1])
    }
    if(is.na(dateRange[2])){
      dateRangeEnd <- "none"
    } else {
      dateRangeEnd <- as.character(dateRange[2])
    }

    codeUse <- codeUse |>
      dplyr::mutate(
        result_id = 1L,
        cdm_name = omopgenerics::cdmName(cdm)
      ) |>
      omopgenerics::newSummarisedResult(
        settings = dplyr::tibble(
          result_id = 1L,
          result_type = "code_use",
          package_name = "CodelistGenerator",
          package_version = as.character(utils::packageVersion("CodelistGenerator")),
          date_range_start = .env$dateRangeStart,
          date_range_end   = .env$dateRangeEnd
        )
      )
  } else {
    codeUse <- omopgenerics::emptySummarisedResult()
  }

  return(codeUse)

}

getCodeUse <- function(x,
                       cdm,
                       cohortTable,
                       cohortId,
                       timing,
                       countBy,
                       byConcept,
                       byYear,
                       bySex,
                       ageGroup,
                       dateRange,
                       useSourceCodes,
                       call = parent.frame()) {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  omopgenerics::assertCharacter(timing, length = 1)
  omopgenerics::assertChoice(timing, choices = c("any","entry"))
  omopgenerics::assertChoice(countBy, choices = c("record", "person"))
  if(!inherits(x[[1]], "cdm_table")){
    omopgenerics::assertList(x)
    omopgenerics::assertNumeric(x[[1]], integerish = TRUE)
  }
  omopgenerics::assertLogical(byConcept)
  omopgenerics::assertLogical(byYear)
  omopgenerics::assertLogical(bySex)
  checkAgeGroup(ageGroup = ageGroup)
  omopgenerics::assertDate(dateRange, length = 2, na = TRUE)

  if(is.null(attr(cdm, "write_schema"))){
    cli::cli_abort("cdm must have a write_schema specified",
                   call = call)
  }

  tableCodelist <- paste0(omopgenerics::uniqueTableName(),
                          omopgenerics::uniqueId())
  if(inherits(x[[1]], "cdm_table")){
    cdm[[tableCodelist]] <- x[[1]] |>
      dplyr::left_join(
        cdm[["concept"]] |> dplyr::select("concept_id", "domain_id"),
        by = "concept_id") |>
      dplyr::compute(name = tableCodelist,
                     overwrite = TRUE,
                     temporary = FALSE,
                     logPrefix = "CodelistGenerator.getCodeUse_joinConcept")
  } else {
    cdm <- omopgenerics::insertTable(cdm = cdm,
                                     name = tableCodelist,
                                     table = dplyr::tibble(concept_id = x[[1]]),
                                     overwrite = TRUE,
                                     temporary = FALSE)
    cdm[[tableCodelist]] <- cdm[[tableCodelist]] |>
      dplyr::left_join(
        cdm[["concept"]] |> dplyr::select("concept_id", "domain_id"),
        by = "concept_id")
  }
  tableDomainsData <- paste0(omopgenerics::uniqueTableName(),
                             omopgenerics::uniqueId())
  cdm <- omopgenerics::insertTable(cdm = cdm,
                                   name = tableDomainsData,
                                   table = conceptDomainsData,
                                   overwrite = TRUE,
                                   temporary = FALSE)

  cdm[[tableCodelist]] <- cdm[[tableCodelist]] |>
    dplyr::mutate(domain_id = tolower(.data$domain_id)) |>
    dplyr::left_join(cdm[[tableDomainsData]],
                     by = "domain_id") |>
    dplyr::compute(name = tableCodelist,
                   temporary = FALSE,
                   overwrite = TRUE,
                   logPrefix = "CodelistGenerator.getCodeUse_conceptsDomains")
  omopgenerics::dropSourceTable(cdm = cdm, name = tableDomainsData)
  cdm[[tableDomainsData]] <- NULL

  intermediateTable <- paste0(omopgenerics::uniqueTableName(),
                              omopgenerics::uniqueId())
  records <- getRelevantRecords(cdm = cdm,
                                tableCodelist = tableCodelist,
                                cohortTable = cohortTable,
                                cohortId = cohortId,
                                timing = timing,
                                intermediateTable = intermediateTable,
                                useSourceCodes = useSourceCodes)

  if(!is.null(records) &&
     (records |> utils::head(1) |> dplyr::tally() |> dplyr::pull("n") > 0)) {

    if((!is.na(dateRange[1])) | (!is.na(dateRange[2]))){
      records <- filterDateRange(records, cdm, dateRange)
    }

    if(bySex == TRUE | !is.null(ageGroup)){
      records <- records |>
        PatientProfiles::addDemographics(age = !is.null(ageGroup),
                                         ageGroup = ageGroup,
                                         sex = bySex,
                                         priorObservation = FALSE,
                                         futureObservation =  FALSE,
                                         indexDate = "start_date",
                                         name =  omopgenerics::tableName(records))
    }

    byAgeGroup <- !is.null(ageGroup)
    codeCounts <- getSummaryCounts(records = records,
                                   cdm = cdm,
                                   countBy = countBy,
                                   byConcept = byConcept,
                                   byYear = byYear,
                                   bySex = bySex,
                                   byAgeGroup = byAgeGroup)

    if (is.null(cohortTable)) {
      cohortName <- NA
    } else {
      cohortName <- omopgenerics::settings(cdm[[cohortTable]]) |>
        dplyr::filter(.data$cohort_definition_id == cohortId) |>
        dplyr::pull("cohort_name")
    }
    codeCounts <-  codeCounts |>
      dplyr::mutate(
        "codelist_name" := !!names(x),
        "cohort_name" = .env$cohortName,
        "estimate_type" = "integer",
        "variable_name" = dplyr::if_else(is.na(.data$standard_concept_name), "overall", .data$standard_concept_name),
        "variable_level" = as.character(.data$standard_concept_id)
      ) |>
      omopgenerics::uniteGroup(cols = c("cohort_name", "codelist_name")) |>
      omopgenerics::uniteAdditional(
        cols = c("source_concept_name", "source_concept_id",
                 "source_concept_value", "type_concept_id", "type_concept_name",
                 "domain_id", "table"),
        ignore = "overall"
      ) |>
      dplyr::select(
        "group_name", "group_level", "strata_name", "strata_level",
        "variable_name", "variable_level", "estimate_name", "estimate_type",
        "estimate_value", "additional_name", "additional_level"
      )

  } else {
    codeCounts <- omopgenerics::emptySummarisedResult()
    cli::cli_inform(c(
      "i" = "No records found in the cdm for the concepts provided."
    ))
  }

  omopgenerics::dropSourceTable(cdm = cdm,
                          name = tableCodelist)
  cdm[[tableCodelist]] <- NULL
  omopgenerics::dropSourceTable(
    cdm = cdm,
    name = dplyr::starts_with(intermediateTable)
  )

  return(codeCounts)
}


getRelevantRecords <- function(cdm,
                               tableCodelist,
                               cohortTable,
                               cohortId,
                               timing,
                               intermediateTable,
                               useSourceCodes){

  codes <- cdm[[tableCodelist]] |> dplyr::collect()

  tableName <- purrr::discard(unique(codes$table), is.na)
  standardConceptIdName <- purrr::discard(unique(codes$standard_concept), is.na)
  sourceConceptIdName <- purrr::discard(unique(codes$source_concept), is.na)
  sourceConceptValueName <- purrr::discard(unique(codes$source_concept_value), is.na)
  typeConceptIdName <- purrr::discard(unique(codes$type_concept), is.na)
  startDateName <- purrr::discard(unique(codes$start_date_name), is.na)
  endDateName <- purrr::discard(unique(codes$end_date_name), is.na)

  tmpTblName1 <- omopgenerics::uniqueTableName()
  tmpTblName2 <- omopgenerics::uniqueTableName()

  on.exit({
    omopgenerics::dropSourceTable(cdm = cdm,
                                        name = tmpTblName1);
    omopgenerics::dropSourceTable(cdm = cdm,
                                        name = tmpTblName2)
    })

  if(!is.null(cohortTable)){
    if(!is.null(cohortId) &
       needsIdFilter(cdm[[cohortTable]], cohortId)){
      cohortSubjects <- cdm[[cohortTable]] |>
        dplyr::filter(.data$cohort_definition_id %in% cohortId)
    } else {
      cohortSubjects <- cdm[[cohortTable]]
    }

    if(timing == "entry"){
      # will need person id and cohort start dates
      cohortSubjects <- cohortSubjects |>
        dplyr::select("subject_id", "cohort_start_date") |>
        dplyr::rename("person_id" = "subject_id") |>
        dplyr::distinct()
    } else {
      # otherwise any and we only need person ids
      cohortSubjects <- cohortSubjects |>
        dplyr::select("subject_id") |>
        dplyr::rename("person_id" = "subject_id") |>
        dplyr::distinct()
    }
  }

  if(length(tableName)>0){
    codeRecords <- cdm[[tableName[[1]]]] |>
      dplyr::select("person_id",
                    startDateName[[1]],
                    endDateName[[1]],
                    .env$standardConceptIdName[[1]],
                    .env$sourceConceptIdName[[1]],
                    .env$sourceConceptValueName[[1]],
                    .env$typeConceptIdName[[1]])
    if(!is.null(cohortTable)){
      # keep only records of those in the cohorts of interest
      if(timing == "entry"){
        workingDateName <- startDateName[[1]]
        codeRecords <- codeRecords |>
          dplyr::inner_join(cohortSubjects,
                            dplyr::join_by("person_id",
                               !!dplyr::sym(workingDateName) == "cohort_start_date"))
      } else {
        codeRecords <- codeRecords |>
          dplyr::inner_join(cohortSubjects,
                            by = "person_id")
      }
      codeRecords <- codeRecords |>
        dplyr::compute(name = tmpTblName1)
    }

    if(is.null(codeRecords)){
      return(NULL)
    }

    tableCodes <- paste0(omopgenerics::uniqueTableName(),
                            omopgenerics::uniqueId())
    cdm <- omopgenerics::insertTable(cdm = cdm,
                                     name = tableCodes,
                                     table = codes |>
                                       dplyr::filter(.data$table == !!tableName[[1]]) |>
                                       dplyr::select("concept_id", "domain_id"),
                                     overwrite = TRUE,
                                     temporary = FALSE)
    codeRecords <- codeRecords |>
      dplyr::mutate(start_date = !!dplyr::sym(startDateName[[1]]),
                    table = !!omopgenerics::tableName(cdm[[tableName[[1]]]])
                    ) |>
      dplyr::select(dplyr::all_of(c("person_id",
                                    standardConceptIdName[[1]],
                                    sourceConceptIdName[[1]],
                                    sourceConceptValueName[[1]],
                                    typeConceptIdName[[1]],
                                    "table",
                                    "start_date"))) |>
      dplyr::rename("standard_concept_id" = .env$standardConceptIdName[[1]],
                    "source_concept_id" = .env$sourceConceptIdName[[1]],
                    "source_concept_value" = .env$sourceConceptValueName[[1]],
                    "type_concept_id" = .env$typeConceptIdName[[1]])

    if(isTRUE(useSourceCodes)){
      codeRecords <- codeRecords |>
        dplyr::inner_join(cdm[[tableCodes]],
                          by = c("source_concept_id"="concept_id")) |>
        dplyr::compute(
          name = paste0(intermediateTable,"_grr"),
          temporary = FALSE,
          schema = attr(cdm, "write_schema"),
          overwrite = TRUE,
          logPrefix = "CodelistGenerator.getRelevantRecords_filter"
        )
    }else{
      codeRecords <- codeRecords |>
        dplyr::inner_join(cdm[[tableCodes]],
                          by = c("standard_concept_id"="concept_id")) |>
        dplyr::compute(
          name = paste0(intermediateTable,"_grr"),
          temporary = FALSE,
          schema = attr(cdm, "write_schema"),
          overwrite = TRUE,
          logPrefix = "CodelistGenerator.getRelevantRecords_filter"
        )
    }
    omopgenerics::dropSourceTable(cdm = cdm, name = tableCodes)
    cdm[[tableCodes]] <- NULL

  } else {
    return(NULL)
  }

  # get for any additional domains and union
  if(length(tableName) > 1) {
    for(i in 1:(length(tableName)-1)) {
      workingRecords <-  cdm[[tableName[[i+1]]]]
      if(!is.null(cohortTable)){
        # keep only records of those in the cohorts of interest
        if(timing == "entry"){
          workingDateName <- startDateName[[i+1]]
          workingRecords <- workingRecords |>
            dplyr::inner_join(cohortSubjects,
                              dplyr::join_by("person_id",
                                             !!dplyr::sym(workingDateName) == "cohort_start_date"))
        } else {
          workingRecords <- workingRecords |>
            dplyr::inner_join(cohortSubjects,
                              by = "person_id")
        }
      }

      cdm <- omopgenerics::insertTable(cdm = cdm,
                                       name = tmpTblName2,
                                       table = codes |>
                                         dplyr::filter(.data$table == tableName[[i+1]]) |>
                                         dplyr::select("concept_id", "domain_id"),
                                       overwrite = TRUE,
                                       temporary = FALSE)

      workingRecords <-  workingRecords |>
        dplyr::mutate(start_date = !!dplyr::sym(startDateName[[i+1]]),
                      table = !!omopgenerics::tableName(cdm[[tableName[[i+1]]]])
        ) |>
        dplyr::select(dplyr::all_of(c("person_id",
                                      standardConceptIdName[[i+1]],
                                      sourceConceptIdName[[i+1]],
                                      sourceConceptValueName[[i+1]],
                                      typeConceptIdName[[i+1]],
                                      "table", "start_date"))) |>
        dplyr::rename("standard_concept_id" = .env$standardConceptIdName[[i+1]],
                      "source_concept_id" = .env$sourceConceptIdName[[i+1]],
                      "source_concept_value" = .env$sourceConceptValueName[[i+1]],
                      "type_concept_id" = .env$typeConceptIdName[[i+1]])

      if(isTRUE(useSourceCodes)){
        workingRecords <- workingRecords |>
          dplyr::inner_join(cdm[[tmpTblName2]],
                            by = c("source_concept_id"="concept_id")) |>
          dplyr::compute(
            name = paste0(intermediateTable,"_grr1"),
            temporary = FALSE,
            schema = attr(cdm, "write_schema"),
            overwrite = TRUE,
            logPrefix = "CodelistGenerator.getRelevantRecords_filter1"
          )
      }else{
        workingRecords <- workingRecords |>
          dplyr::inner_join(cdm[[tmpTblName2]],
                            by = c("standard_concept_id"="concept_id")) |>
          dplyr::compute(
            name = paste0(intermediateTable,"_grr1"),
            temporary = FALSE,
            schema = attr(cdm, "write_schema"),
            overwrite = TRUE,
            logPrefix = "CodelistGenerator.getRelevantRecords_filter1"
          )
      }

      if(workingRecords |> utils::head(1) |> dplyr::tally() |> dplyr::pull("n") >0){
        codeRecords <- codeRecords |>
          dplyr::union_all(workingRecords)  |>
          dplyr::compute(
            name = paste0(intermediateTable,"_grr_i"),
            temporary = FALSE,
            schema = attr(cdm, "write_schema"),
            overwrite = TRUE,
            logPrefix = "CodelistGenerator.getRelevantRecords_unionAll"
          )
      }
   }
  }

  if(codeRecords |> utils::head(1) |> dplyr::tally() |> dplyr::pull("n") >0){
    codeRecords <- codeRecords |>
      PatientProfiles::addConceptName(column = "standard_concept_id",
                                      nameStyle = "standard_concept_name") |>
      PatientProfiles::addConceptName(column = "source_concept_id",
                                      nameStyle = "source_concept_name")|>
      PatientProfiles::addConceptName(column = "type_concept_id",
                                      nameStyle = "type_concept_name") |>
      dplyr::mutate(standard_concept_name = dplyr::if_else(is.na(.data$standard_concept_name),
                                                         "NA", .data$standard_concept_name),
                    source_concept_name = dplyr::if_else(is.na(.data$source_concept_name),
                                                         "NA", .data$source_concept_name),
                    type_concept_name = dplyr::if_else(is.na(.data$type_concept_name),
                                                         "NA", .data$type_concept_name)) |>
      dplyr::compute(
        name = paste0(intermediateTable,"_grr_cr"),
        temporary = FALSE,
        schema = attr(cdm, "write_schema"),
        overwrite = TRUE,
        logPrefix = "CodelistGenerator.getRelevantRecords_codeRecords"
      )
  } else {
    codeRecords <- codeRecords |>
      dplyr::mutate(standard_concept_name = NA_character_,
                    source_concept_name = NA_character_,
                    type_concept_name = NA_character_)
  }

  return(codeRecords)

}

filterDateRange <- function(records, cdm, dateRange){
  dateRange <- getDateRange(cdm, dateRange)
  records |>
    dplyr::filter(.data$start_date >= !!dateRange[1],
                  .data$start_date <= !!dateRange[2])
}

getSummaryCounts <- function(records,
                             cdm,
                             countBy,
                             byConcept,
                             byYear,
                             bySex,
                             byAgeGroup) {
  if ("record" %in% countBy) {
    recordSummary <- records |>
      dplyr::tally(name = "estimate_value") |>
      dplyr::collect() |>
      dplyr::arrange(dplyr::desc(.data$estimate_value)) |>
      dplyr::mutate(estimate_value = as.character(.data$estimate_value))

    if(isTRUE(byConcept)) {
      recordSummary <- dplyr::bind_rows(
        recordSummary,
        records |>
          dplyr::group_by(
            .data$standard_concept_id, .data$standard_concept_name,
            .data$source_concept_id, .data$source_concept_name,
            .data$source_concept_value,
            .data$type_concept_id,  .data$type_concept_name,
            .data$domain_id, .data$table
          ) |>
          dplyr::tally(name = "estimate_value") |>
          dplyr::collect() |>
          dplyr::arrange(dplyr::desc(.data$estimate_value)) |>
          dplyr::mutate(estimate_value = as.character(.data$estimate_value))
      )
    }else{
      recordSummary <- recordSummary |>
        dplyr::mutate("standard_concept_id" = NA_integer_,
                      "standard_concept_name" = NA_character_,
                      "source_concept_id" = NA_integer_,
                      "source_concept_name" = NA_character_,
                      "source_concept_value" = NA_character_,
                      "type_concept_id" = NA_integer_,
                      "type_concept_name" =  NA_character_,
                      "domain_id" = NA_character_,
                      "table" = NA_character_)
    }
    recordSummary <- recordSummary |>
      dplyr::mutate(
        strata_name = "overall",
        strata_level = "overall",
        estimate_name = "record_count"
      )
  } else {
    recordSummary <- dplyr::tibble()
  }

  if ("person" %in% countBy) {
    personSummary <- records |>
      dplyr::select("person_id") |>
      dplyr::distinct() |>
      dplyr::tally(name = "estimate_value") |>
      dplyr::collect() |>
      dplyr::arrange(dplyr::desc(.data$estimate_value)) |>
      dplyr::mutate(estimate_value = as.character(.data$estimate_value))

    if (isTRUE(byConcept)) {
      personSummary <- dplyr::bind_rows(
        personSummary,
        records |>
          dplyr::select(
            "person_id", "standard_concept_id", "standard_concept_name",
            "source_concept_id", "source_concept_name",
            "source_concept_value", "type_concept_id", "type_concept_name",
            "domain_id", "table"
          ) |>
          dplyr::distinct() |>
          dplyr::group_by(
            .data$standard_concept_id, .data$standard_concept_name,
            .data$source_concept_id, .data$source_concept_name,
            .data$type_concept_name,
            .data$source_concept_value, .data$type_concept_id,
            .data$domain_id, .data$table
          ) |>
          dplyr::tally(name = "estimate_value") |>
          dplyr::collect() |>
          dplyr::arrange(dplyr::desc(.data$estimate_value)) |>
          dplyr::mutate(estimate_value = as.character(.data$estimate_value))
      )
    }
    personSummary <- personSummary |>
      dplyr::mutate(
        strata_name = "overall",
        strata_level = "overall",
        estimate_name = "person_count")
  } else {
    personSummary <- dplyr::tibble()
  }

 if(byYear == TRUE) {
   records <- records |>
     dplyr::mutate(year = clock::get_year(.data$start_date))
 }

  if ("record" %in% countBy & byYear == TRUE) {
    recordSummary <- dplyr::bind_rows(
      recordSummary,
      getGroupedRecordCount(records = records, cdm = cdm, groupBy = "year")
    )
  }
  if ("person" %in% countBy & byYear == TRUE) {
    personSummary <- dplyr::bind_rows(
      personSummary,
      getGroupedPersonCount(records = records, cdm = cdm, groupBy = "year")
    )
  }
  if ("record" %in% countBy & bySex == TRUE) {
    recordSummary <- dplyr::bind_rows(
      recordSummary,
      getGroupedRecordCount(records = records, cdm = cdm, groupBy = "sex")
    )
  }
  if ("person" %in% countBy & bySex == TRUE) {
    personSummary <- dplyr::bind_rows(
      personSummary,
      getGroupedPersonCount(records = records, cdm = cdm, groupBy = "sex")
    )
  }
  if ("record" %in% countBy & byAgeGroup == TRUE) {
    recordSummary <- dplyr::bind_rows(
      recordSummary,
      getGroupedRecordCount(records = records, cdm = cdm, groupBy = "age_group")
    )
  }
  if ("person" %in% countBy & byAgeGroup == TRUE) {
    personSummary <- dplyr::bind_rows(
      personSummary,
      getGroupedPersonCount(records = records, cdm = cdm, groupBy = "age_group")
    )
  }
  if ("record" %in% countBy && byAgeGroup == TRUE && bySex == TRUE) {
    recordSummary <- dplyr::bind_rows(
      recordSummary,
      getGroupedRecordCount(records = records, cdm = cdm, groupBy = c("age_group", "sex"))
    )
  }
  if ("person" %in% countBy && byAgeGroup == TRUE && bySex == TRUE) {
    personSummary <- dplyr::bind_rows(
      personSummary,
      getGroupedPersonCount(records = records, cdm = cdm, groupBy = c("age_group", "sex"))
    )
  }
  summary <- dplyr::bind_rows(recordSummary, personSummary)
  return(summary)
}

getGroupedRecordCount <- function(records,
                                  cdm,
                                  groupBy){

  groupedCounts <- dplyr::bind_rows(
    records |>
      dplyr::group_by(dplyr::pick(.env$groupBy)) |>
      dplyr::tally(name = "estimate_value") |>
      dplyr::collect() |>
      dplyr::arrange(dplyr::desc(.data$estimate_value)) |>
      dplyr::mutate(estimate_value = as.character(.data$estimate_value)),
    records |>
      dplyr::group_by(dplyr::pick(.env$groupBy,
                                  "standard_concept_id", "standard_concept_name",
                                  "source_concept_id", "source_concept_name",
                                  "source_concept_value", "type_concept_id",
                                  "type_concept_name",
                                  "domain_id", "table")) |>
      dplyr::tally(name = "estimate_value") |>
      dplyr::collect() |>
      dplyr::arrange(dplyr::desc(.data$estimate_value)) |>
      dplyr::mutate(estimate_value = as.character(.data$estimate_value))
    )  |>
    omopgenerics::uniteStrata(cols = groupBy) |>
    dplyr::mutate(estimate_name = "record_count")

  return(groupedCounts)

}

getGroupedPersonCount <- function(records,
                                  cdm,
                                  groupBy){

  groupedCounts <- dplyr::bind_rows(
    records |>
      dplyr::select(dplyr::all_of(c("person_id", .env$groupBy))) |>
      dplyr::distinct() |>
      dplyr::group_by(dplyr::pick(.env$groupBy)) |>
      dplyr::tally(name = "estimate_value") |>
      dplyr::collect() |>
      dplyr::arrange(dplyr::desc(.data$estimate_value)) |>
      dplyr::mutate(estimate_value = as.character(.data$estimate_value)),
    records |>
      dplyr::select(dplyr::all_of(c(
        "person_id", "standard_concept_id", "standard_concept_name",
        "source_concept_id", "source_concept_name",
        "source_concept_value", "type_concept_id", "type_concept_name",
        "domain_id", "table", .env$groupBy
      ))) |>
      dplyr::distinct() |>
      dplyr::group_by(dplyr::pick(
        .env$groupBy, "standard_concept_id", "standard_concept_name",
        "source_concept_id", "source_concept_name",
        "source_concept_value", "type_concept_id", "type_concept_name",
        "domain_id", "table"
      )) |>
      dplyr::tally(name = "estimate_value")  |>
      dplyr::collect() |>
      dplyr::arrange(dplyr::desc(.data$estimate_value)) |>
      dplyr::mutate(estimate_value = as.character(.data$estimate_value))) |>
    omopgenerics::uniteStrata(cols = groupBy) |>
    dplyr::mutate(estimate_name = "person_count")

  return(groupedCounts)

}

checkCategory <- function(category, overlap = FALSE) {
  omopgenerics::assertList(category, unique = TRUE)

  if (is.null(names(category))) {
    names(category) <- rep("", length(category))
  }

  # check length
  category <- lapply(category, function(x) {
    if (length(x) == 1) {
      x <- c(x, x)
    } else if (length(x) > 2) {
      cli::cli_abort(
        paste0(
          "Categories should be formed by a lower bound and an upper bound, ",
          "no more than two elements should be provided."
        ),
        call. = FALSE
      )
    }
    return(x)
  })

  # check lower bound is smaller than upper bound
  checkLower <- unlist(lapply(category, function(x) {
    x[1] <= x[2]
  }))
  if (!(all(checkLower))) {
    cli::cli_abort("Lower bound should be equal or smaller than upper bound")
  }

  # built tibble
  result <- lapply(category, function(x) {
    dplyr::tibble(lower_bound = x[1], upper_bound = x[2])
  }) |>
    dplyr::bind_rows() |>
    dplyr::mutate(category_label = names(.env$category)) |>
    dplyr::mutate(category_label = dplyr::if_else(
      .data$category_label == "",
      paste0(.data$lower_bound, " to ", .data$upper_bound),
      .data$category_label
    )) |>
    dplyr::arrange(.data$lower_bound)

  # check overlap
  if(!overlap) {
    if (nrow(result) > 1) {
      lower <- result$lower_bound[2:nrow(result)]
      upper <- result$upper_bound[1:(nrow(result) - 1)]
      if (!all(lower > upper)) {
        cli::cli_abort("There can not be overlap between categories")
      }
    }
  }

  return(result)
}

checkAgeGroup <- function(ageGroup, overlap = FALSE) {
  omopgenerics::assertList(ageGroup, null = TRUE)
  if (!is.null(ageGroup)) {
    if (is.numeric(ageGroup[[1]])) {
      ageGroup <- list("age_group" = ageGroup)
    }
    for (k in seq_along(ageGroup)) {
      invisible(checkCategory(ageGroup[[k]], overlap))
    }
    if (is.null(names(ageGroup))) {
      names(ageGroup) <- paste0("age_group_", seq_along(ageGroup))
    }
    if ("" %in% names(ageGroup)) {
      id <- which(names(ageGroup) == "")
      names(ageGroup)[id] <- paste0("age_group_", id)
    }
  }
  return(ageGroup)
}

getDateRange <- function(cdm, dateRange) {

  if(is.na(dateRange[1])){
    obs_date_range <- cdm[["observation_period"]] |>
      dplyr::summarise(
        min_start = as.Date(min(.data$observation_period_start_date, na.rm = TRUE))
      ) |>
      dplyr::collect()
    dateRange[1] <- obs_date_range$min_start
  }

  if(is.na(dateRange[2])){
    obs_date_range <- cdm[["observation_period"]] |>
      dplyr::summarise(
        max_end = as.Date(max(.data$observation_period_end_date, na.rm = TRUE))
      ) |>
      dplyr::collect()
    dateRange[2] <- obs_date_range$max_end
  }

  return(dateRange)
}

#' Get the available vocabularies available in the cdm
#'
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#' @inheritParams domainDoc
#'
#' @return Names of available vocabularies.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all vocabularies available in the CDM
#' availableVocabularies(cdm)
#'
#' # Get all vocabularies available in the CDM for `Standard` and `Condition` concepts
#' availableVocabularies(cdm,
#'                       standardConcept = "Standard",
#'                       domain = "Condition")
#' }
availableVocabularies <- function(cdm,
                         standardConcept = "Standard",
                         domain = NULL) {

  # Initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  domain <- assertDomain(domain, cdm)
  standardConcept <- assertStandardConcept(standardConcept)

    vocabs <- cdm[["concept"]] |>
      dplyr::mutate(standard_concept = dplyr::case_when(
        is.na(.data$standard_concept) ~ "non-standard",
        .data$standard_concept == "C" ~ "classification",
        .data$standard_concept == "S" ~ "standard"
      )) |>
      dplyr::filter(.data$standard_concept %in% .env$standardConcept)

    if(!is.null(domain)){
      vocabs <- vocabs |>
        dplyr::filter(tolower(.data$domain_id) %in% .env$domain)
    }

    vocabs <- vocabs |>
      dplyr::select("vocabulary_id") |>
      dplyr::distinct() |>
      dplyr::pull("vocabulary_id") |>
      sort()

  return(vocabs)
}

#' Get the vocabularies associated with a codelist
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#' @inheritParams domainDoc
#'
#' @return Names of available vocabularies.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all vocabularies from a codelist
#' codelist <- newCodelist(list("codes1" = c(35604877L, 35604394L),
#'                              "codes2" = c(4214687L)))
#' associatedVocabularies(cdm = cdm,
#'                       x = codelist)
#' }
associatedVocabularies <- function(x,
                                   cdm,
                                   standardConcept = "Standard",
                                   domain = NULL) {

  # Initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  domain <- assertDomain(domain, cdm)
  standardConcept <- assertStandardConcept(standardConcept)
  checkCodelist(x, allowConceptSetExpression = FALSE)

  # If codelist/codelist_with_details is provided
  vocabs <- addDetails(cdm = cdm,
                       conceptList = x)

  if(!is.null(domain)){
    vocabs <- purrr::map(vocabs, ~ dplyr::filter(.x, tolower(.data$domain_id) %in% .env$domain))
  }

  vocabs <- purrr::map(vocabs, ~ dplyr::filter(.x, .data$standard_concept %in% .env$standardConcept))

  vocabs <- purrr::map(vocabs, ~dplyr::pull(.x, "vocabulary_id") |> unique() |> sort())

  return(vocabs)
}

assertStandardConcept <- function(standardConcept, call = parent.frame()){
  omopgenerics::assertChoice(standardConcept,
                             c("Standard", "Non-standard", "Classification"),
                             call = call)
  return(invisible(tolower(standardConcept)))
}

assertDomain <- function(domains, cdm, call = parent.frame()){

  if(is.null(domains)){
    domains <- availableDomains(cdm)
  }else{
    omopgenerics::assertCharacter(domains, null = TRUE)
    domains <- stringr::str_to_sentence(domains)
    dif <- setdiff(domains, availableDomains(cdm))
    if(length(dif) > 0){
      cli::cli_warn("The domain{?s} {.val {dif}} {?is/are} not present in the cdm.", call = call)
    }
  }

  return(invisible(tolower(domains)))
}

assertVocab <- function(vocabularyId, domain, standardConcept, cdm, call = parent.frame()){
  if(is.null(vocabularyId)){
    vocabs <- availableVocabularies(cdm, domain = domain, standardConcept = standardConcept)
  }else{
    vocabs <- omopgenerics::assertCharacter(vocabularyId, null = TRUE)
    dif <- setdiff(vocabs, availableVocabularies(cdm))
    if(length(dif) > 0){
      cli::cli_warn("The vocab{?s} {.val {dif}} {?is/are} not present in the cdm.", call = call)
    }
  }

  return(invisible(tolower(vocabs)))
}

test_that("tests with mock db", {
  skip_on_cran()
  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])
    namesInitial <- names(cdm)
    codelist_with_details <- list("codes" = cdm[["concept"]] |>
                                    dplyr::filter(concept_id %in% c(10L, 15L, 24L)) |>
                                    dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
                                    dplyr::mutate("ancestor_concept_id" = 999L) |>
                                    dplyr::collect()) |>
      omopgenerics::newCodelistWithDetails()

    # Whole cdm
    vocabs <- availableVocabularies(cdm = cdm, domain = "Condition", standardConcept = "Standard")
    expect_true(inherits(vocabs, "character"))
    expect_true(length(vocabs) >= 1)
    expect_true(vocabs == "SNOMED")
    vocabs <- availableVocabularies(cdm = cdm, domain = "Drug", standardConcept = "Non-standard")
    expect_true(all(vocabs == c("ATC", "OMOP", "RxNorm", "RxNorm Extension")))

    # List
    expect_error(x <- associatedVocabularies(cdm, list("codes" = c(1L,20L))))

    # Codelist
    codelist <- list("first_codelist" = c(1L, 20L), "second_codelist" = c(7L, 9L)) |>
      omopgenerics::newCodelist()
    expect_no_error(x <- associatedVocabularies(codelist, cdm, standardConcept = c("Standard", "Non-standard", "Classification")))
    expect_true(all(names(x) == c("first_codelist", "second_codelist")))
    expect_true(all(x[[1]] == c("RxNorm", "SNOMED")))
    expect_true(all(x[[2]] == c("LOINC", "Read")))

    expect_no_error(x <- associatedVocabularies(codelist, cdm, standardConcept = c("Standard")))
    expect_true(all(names(x) == c("first_codelist", "second_codelist")))
    expect_true(all(x[[2]] == c("LOINC")))

    # Codelist with details
    expect_no_error(x <- associatedVocabularies(codelist_with_details, cdm, standardConcept = c("Standard", "Non-standard", "Classification")))
    expect_true(names(x) == "codes")
    expect_true(all(x[[1]] == c("ICD10", "RxNorm", "SNOMED")))

    # expected errors
    expect_error(availableVocabularies(cdm, x = 123))
    expect_error(availableVocabularies(cdm, x = list("a" = 123)))
    expect_error(availableVocabularies(cdm = "a"))

    expect_error(associatedVocabularies(cdm))
    expect_error(associatedVocabularies(cdm, x = list("a" = 123)))
    expect_error(associatedVocabularies(cdm = "a"))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Format the result of summariseCodeUse into a table.
#'
#' @param result A `<summarised_result>` with results of the type "code_use".
#' @inheritParams typeTableDoc
#' @inheritParams tableStyleDoc
#' @inheritParams headerStrataDoc
#' @inheritParams groupColumnStrataDoc
#' @inheritParams hideStrataDoc
#' @inheritParams .optionsDoc
#'
#' @return A table with a formatted version of the summariseCodeUse result.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' library(omopgenerics)
#' library(CodelistGenerator)
#' con <- DBI::dbConnect(duckdb::duckdb(),
#'                       dbdir = CDMConnector::eunomiaDir())
#' cdm <- CDMConnector::cdmFromCon(con,
#'                                 cdmSchema = "main",
#'                                 writeSchema = "main")
#'acetiminophen <- c(1125315,  1127433, 40229134,
#'40231925, 40162522, 19133768,  1127078)
#'poliovirus_vaccine <- c(40213160)
#'cs <- list(acetiminophen = acetiminophen,
#'           poliovirus_vaccine = poliovirus_vaccine)
#'results <- summariseCodeUse(newCodelist(cs),cdm = cdm)
#'tableCodeUse(results)
#'CDMConnector::cdmDisconnect(cdm)
#'}
#'

tableCodeUse <- function(result,
                         type = NULL,
                         header = c("cdm_name", "estimate_name"),
                         groupColumn = character(),
                         hide = c("date_range_start", "date_range_end"),
                         style = NULL,
                         .options = list()) {

  rlang::check_installed("visOmopResults", version = "1.4.0")

  # checks
  if(nrow(result) == 0){
    cli::cli_warn("`result` object is empty")
    return(visOmopResults::emptyTable(type = type))
  }

  result <- result |>
    visOmopResults::filterSettings(.data$result_type == "code_use")

  if(nrow(result) == 0){
    cli::cli_warn("No code use results found in result object")
    return(visOmopResults::emptyTable(type = type))
  }

  x <- internalTableCodeUse(
    result = result,
    resultType = "code_use",
    type = type,
    header = header,
    groupColumn = groupColumn,
    hide = hide,
    style = style,
    .options = .options
  )

  return(x)
}


internalTableCodeUse <- function(result,
                                 resultType,
                                 type,
                                 header,
                                 groupColumn,
                                 hide,
                                 style,
                                 .options) {
  omopgenerics::assertCharacter(header, null = TRUE)
  omopgenerics::assertCharacter(hide, null = TRUE)
  if (!is.list(groupColumn) & !is.null(groupColumn)) groupColumn <- list(groupColumn)
  omopgenerics::assertCharacter(groupColumn[[1]], null = TRUE)
  omopgenerics::assertCharacter(style, null = TRUE, length = 1)
  omopgenerics::assertCharacter(type, null = TRUE, length = 1)

   # .options
  .options <- optionsCodeUse(.options)

  x <- result |>
    visOmopResults::filterSettings(.data$result_type == .env$resultType) |>
    dplyr::mutate(estimate_name = stringr::str_to_sentence(gsub("_", " ", .data$estimate_name)))

  levels <- c("overall", unique(x$variable_name)[unique(x$variable_name) != "overall"])

  x <- x |>
    dplyr::mutate("variable_name" = factor(.data$variable_name, levels = levels)) |>
    dplyr::arrange(dplyr::across(dplyr::everything()))

  header <- reformulateTableAchilles(header)
  groupColumn[[1]] <- reformulateTableAchilles(groupColumn[[1]])
  hide <- reformulateTableAchilles(hide)

  # visTable
  x <- visOmopResults::visOmopTable(
    result = x,
    estimateName = character(),
    header = header,
    groupColumn = groupColumn,
    type = type,
    settingsColumn = omopgenerics::settingsColumns(x),
    rename = c(
      "Domain ID" = "domain_id", "Database name" = "cdm_name",
      "Standard concept ID" = "standard_concept_id",
      "Source concept ID" = "source_concept_id",
      "Standard concept ID" = "variable_level",
      "Standard concept name" = "variable_name"
    ),
    hide = hide,
    style = style,
    .options = .options
  ) |>
    suppressWarnings() # to remove in next visOmopResults release

  return(x)
}

optionsCodeUse <- function(userOptions) {

  defaultOpts <- visOmopResults::tableOptions()

  idsIgnored <- ! names(userOptions) %in% names(defaultOpts)
  if (sum(idsIgnored) > 0) {
    cli::cli_warn("The following elements in `.options` do not refer to allowed arguments and will be ignored: {paste0(names(userOptions)[idsIgnored], collapse = ', ')}.")
  }

  for (opt in names(userOptions)) {
    defaultOpts[[opt]] <- userOptions[[opt]]
  }

  return(defaultOpts)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

checkInputs <- function(...) {
  inputs <- list(...)
  lapply(names(inputs), function(x) {
    funName <- paste0(
      "check", toupper(substr(x, 1, 1)), substr(x, 2, nchar(x))
    )
    varName <- eval(parse(text = paste0("names(formals(", funName, "))")))
    eval(parse(text = paste0(
      funName, "(",
      paste0(paste0("inputs[[\"", varName, "\"]]"), collapse = ", "), ")"
    )))
  })
  invisible(NULL)
}

checkPath <- function(path) {
  if(typeof(path) != "character" || length(path) != 1) {
    cli::cli_abort("path is not a character of length 1")
  }

  if (!file.exists(path)) {
    cli::cli_abort(glue::glue("Invalid path: {path}"))
  }
}

checkCdm <- function(cdm) {
  if (!(inherits(cdm, "cdm_reference"))) {
    cli::cli_abort(
      "Argument cdm is not a valid cdm reference, please use
      CDMConnector::cdmFromCon() to create a valid cdm reference"
    )
  }
}

checkCodelist <- function(x, allowNULL = TRUE, allowConceptSetExpression = TRUE, call = parent.frame()){
  type <- class(x)

  # If X is NULL
  if (is.null(x)) {
    if (allowNULL) {
      return(x)
    } else {
      cli::cli_abort(c(x = "`x` cannot be NULL."), call = call)
    }
  }

  opts <- c("codelist", "codelist_with_details")
  if (allowConceptSetExpression) {
    funs <- "newCodelist()/newCodelistWithDetails()/newConceptSetExpression()/newCodeSearch()"
    opts <- c(opts, "concept_set_expression")
  } else {
    funs <- "newCodelist()/newCodelistWithDetails()/newCodeSearch()"
  }
  opts <- c(opts, "code_search")

  # if x is a list ask to convert
  if ("list" %in% type & !any(opts %in% type)) {
    cli::cli_abort(
      message = c(x = "Please convert your list `x` into a codelist using {funs} functions from omopgenerics package.
                       If you are unfamiliar with this classes, please visit: https://darwin-eu.github.io/CodelistGenerator/articles/a03_TypesOfCodelist.html"),
      call = call
    )
  }

  # if x is concept_set_expression but not allowed
  if ("concept_set_expression" %in% type & !allowConceptSetExpression) {
    cli::cli_abort(c(x = "concept_set_expression is not supported for this function yet. Please convert your codelist to a codelist or a codelist with details using asCodelist() or asCodelistWithDetails().
                     If you are unfamiliar with this classes, please visit: https://darwin-eu.github.io/CodelistGenerator/articles/a03_TypesOfCodelist.html"), call = call)
  }

  # If x is not any of the supported classes
  if (!any(opts %in% type)) {
    cli::cli_abort(c(x = "Please convert your list `x` into a codelist using {funs} functions from omopgenerics package.
                     If you are unfamiliar with this classes, please visit: https://darwin-eu.github.io/CodelistGenerator/articles/a03_TypesOfCodelist.html"), call = call)
  }

  return(x)
}

dropEmptyCodelist <- function(x_original, newX, call = parent.frame()){

  newX <- newX |> vctrs::list_drop_empty()

  n1 <- names(x_original)
  n2 <- names(newX)

  n1 <- setdiff(n1, n2)
  if(length(n1) > 0){
    cli::cli_warn("{.val {n1}} codelist{?s} will be removed from the final codelist, as there are no elements left after subsetting.",
                  call = call)
  }

  if(length(newX) == 0){
    newX <- createEmptyCodelist(newX)
  }
  return(newX)
}

createEmptyCodelist <- function(x){
  if(inherits(x, "codelist")){
    x <- omopgenerics::emptyCodelist()
  }
  if(inherits(x, "codelist_with_details")){
    x <- omopgenerics::emptyCodelistWithDetails()
  }
  return(x)
}

validateType <- function(type, call = parent.frame()) {
  if (is.null(type)) {
    type <- getOption("CodelistGenerator.type", NULL)
  }
  omopgenerics::assertChoice(type, choices = c(
    "codelist", "codelist_with_details", "concept_set_expression", "code_search"
  ), call = call)
  return(type)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Get the domains available in the cdm
#'
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#'
#' @return A vector with the domains of the cdm.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all domains available in the CDM for standard concepts
#' availableDomains(cdm = cdm, standardConcept = "Standard")
#' }
availableDomains <- function(cdm,
                    standardConcept = "Standard") {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  standardConcept <- assertStandardConcept(standardConcept)

    newOmopTable <- "concept"
    domains <- domainsInternal(cdm,
                               newOmopTable = newOmopTable,
                               standardConcept = standardConcept,
                               xx = NULL)

  return(domains)
}

#' Get the domains associated with a codelist
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams standardConceptDoc
#'
#' @return A vector with the domains of the cdm.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omock)
#'
#' # Create CDM object
#' cdm <- mockCdmReference()
#'
#' # Get all domains available in a codelist
#' codelist <- newCodelist(list("codes1" = c(194152L, 1830279L, 40558872L),
#'                              "codes2" = c(44022939L)))
#' associatedDomains(x = codelist, cdm = cdm,
#'                  standardConcept = c("Non-standard", "Standard"))
#' }
associatedDomains <- function(x,
                              cdm,
                              standardConcept = "Standard") {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm)
  standardConcept <- assertStandardConcept(standardConcept)
  checkCodelist(x, allowConceptSetExpression = FALSE)

  # If no codelist is provided, return all concept class Ids in the cdm
  if(is.null(x)){
    newOmopTable <- "concept"
    domains <- domainsInternal(cdm,
                               newOmopTable = newOmopTable,
                               standardConcept = standardConcept,
                               xx = NULL)
  }

  if(inherits(x, "codelist_with_details")){
    x <- asCodelist(x)
  }

  if(inherits(x, "codelist")){
    newOmopTable <- paste0(omopgenerics::uniqueTableName(),
                           omopgenerics::uniqueId())
    codesTable <- paste0(omopgenerics::uniqueTableName(),
                         omopgenerics::uniqueId())
    cdm <- subsetOmopTable(cdm, newOmopTable, "concept", x)

    domains <- list()
    for( i in seq_along(x) ){
      domains[[names(x)[[i]]]] <- domainsInternal(cdm,
                                                  newOmopTable = newOmopTable,
                                                  codesTable = codesTable,
                                                  standardConcept = standardConcept,
                                                  xx = x[[i]])
    }
    omopgenerics::dropSourceTable(cdm, name = c(newOmopTable, codesTable))
  }

  return(domains)
}

domainsInternal <- function(cdm, newOmopTable, codesTable, standardConcept, xx){
  conceptDb <- cdm[[newOmopTable]]

  if(!is.null(xx)){
    cdm <- omopgenerics::insertTable(
      cdm, name = codesTable, dplyr::tibble(concept_id = xx))
    conceptDb <- conceptDb |>
      dplyr::inner_join(cdm[[codesTable]], by = "concept_id")
  }

  conceptDb <- conceptDb |>
    dplyr::mutate(standard_concept = dplyr::case_when(
      is.na(.data$standard_concept) ~ "non-standard",
      .data$standard_concept == "C" ~ "classification",
      .data$standard_concept == "S" ~ "standard"
    )) |>
    dplyr::filter(.data$standard_concept %in% .env$standardConcept)

  domains <- conceptDb |>
    dplyr::select("domain_id") |>
    dplyr::distinct() |>
    dplyr::collect() |>
    dplyr::pull() |>
    sort()

  return(domains)
}

test_that("tests with mock db", {
  skip_on_cran()
  backends <- c("database", "data_frame")

  for (i in seq_along(backends)) {
    # mock db
    cdm <- mockVocabRef(backends[[i]])
    codelist_with_details <- list("codes" = cdm[["concept"]] |>
                                    dplyr::filter(concept_id %in% c(10L, 15L, 24L)) |>
                                    dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
                                    dplyr::mutate("ancestor_concept_id" = 999L) |>
                                    dplyr::collect()) |>
      omopgenerics::newCodelistWithDetails()

    codelist <- omopgenerics::newCodelist(list("codes1" = c(1L, 20L), "codes2" = c(1L, 2L, 3L, 9L)))

    # Available domains ----
    expect_no_error(domains <- availableDomains(cdm = cdm))
    expect_true(all(c("Condition", "Drug", "Observation", "Unit") == domains))
    expect_no_error(domains <- associatedDomains(cdm = cdm, x = codelist))
    expect_true(inherits(domains, "list"))
    expect_true(all(c("Condition", "Drug") == domains$codes1))
    expect_true(all(c("Condition", "Observation") == domains$codes2))
    expect_no_error(domains <- associatedDomains(cdm = cdm, x = codelist_with_details))
    expect_true(inherits(domains, "list"))
    expect_true(all(c("Drug") == domains$codes))


    # expected errors
    expect_error(availableDomains(cdm = "a"))
    expect_error(availableDomains(cdm, standardConcept = FALSE))

    expect_error(associatedDomains(cdm = "a"))
    expect_error(associatedDomains(cdm))
    expect_error(associatedDomains(cdm, x = list("a" = 123)))
    expect_error(associatedDomains(cdm, standardConcept = FALSE))

    if (backends[[i]] == "database") {
      CDMConnector::cdmDisconnect(cdm)
    }
  }
})

stratifyCodelistBy <- function(x,
                               cdm,
                               by,
                               nameStyle,
                               keepOriginal,
                               st,
                               call = parent.frame()) {
  # initial checks
  checkCodelist(x, allowConceptSetExpression = FALSE, call = call)
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm, call = call)
  omopgenerics::assertLogical(keepOriginal, length = 1, call = call)
  nameStyle <- prepareNameStyle(nameStyle = nameStyle, call = call)

  if (length(x) == 0) {
    cli::cli_warn(c("!" = "Empty codelist provided. Returning an empty codelist."))
    return(x)
  }

  original <- x

  # codelist table
  nm <- omopgenerics::uniqueTableName()

  if (inherits(x, "code_search")) {
    x <- asCodelist(x)
  }
  x <- dplyr::as_tibble(x) |>
    dplyr::rename(codelist_name = dplyr::any_of("codelist_with_details_name")) |>
    dplyr::select("codelist_name", "concept_id")
  cdm <- omopgenerics::insertTable(cdm = cdm, name = nm, table = x)
  on.exit(omopgenerics::dropSourceTable(cdm = cdm, name = nm))

  if (by == "domain") {
    x <- addDomain(x = cdm[[nm]])
  } else if (by == "brand") {
    x <- addBrand(x = cdm[[nm]])
  } else if (by == "dose_form") {
    x <- addDoseForm(x = cdm[[nm]])
  } else if (by == "dose_unit") {
    x <- addDoseUnit(x = cdm[[nm]])
  } else if (by == "vocabulary") {
    x <- addVocabulary(x = cdm[[nm]])
  } else if (by == "route_category") {
    x <- addRouteCategory(x = cdm[[nm]])
  } else if (by == "concept") {
    x <- addConcept(x = cdm[[nm]])
  }

  # correct missing
  x <- correctMissingValue(x, by)

  # check nameStyle
  checkNameStyleStratify(nameStyle = nameStyle, x = x, by = by, call = call)

  # warn concepts in multiple domains
  warnMultiple(x = x, by = by)

  # stratify codelist
  x <- stratifyCodelist(
    x = x,
    by = c("codelist_name", by),
    nameStyle = nameStyle
  )

  # add class (and details)
  x <- prepareCodelist(x = x, original = original, searchStrategy = st)

  # add original codes
  if (isTRUE(keepOriginal)) {
    x <- keepOriginalCodelists(x = x, original = original)
  }

  return(x)
}

correctMissingValue <- function(x, by){
  missingValue <- paste("Unclassified", by)
  x <- x |>
    dplyr::mutate(dplyr::across(
      .cols = dplyr::all_of(by),
      .fns = \(x) omopgenerics::toSnakeCase(dplyr::coalesce(x, missingValue))
    ))

  return(x)
}
prepareNameStyle <- function(nameStyle, call) {
  omopgenerics::assertCharacter(nameStyle, length = 1, call = call)
  nameStyle |>
    stringr::str_replace_all(
      pattern = stringr::fixed("{codelist_with_details_name}"),
      replacement = "{codelist_name}"
    )
}
checkNameStyleStratify <- function(nameStyle, x, by, call) {
  args <- list(
    nameStyle = nameStyle,
    codelist_name = unique(x$codelist_name),
    call = call
  )
  args[[by]] <- unique(x[[by]])
  do.call(omopgenerics::validateNameStyle, args)
}
warnMultiple <- function(x, by) {
  concepts <- x |>
    dplyr::distinct(dplyr::across(dplyr::all_of(c("concept_id", by)))) |>
    dplyr::group_by(.data$concept_id) |>
    dplyr::filter(dplyr::n() > 1) |>
    dplyr::distinct(.data$concept_id) |>
    dplyr::pull()
  if (length(concepts) > 0) {
    cli::cli_warn("{length(concepts)} concept ID{?s} have duplicated `{by}` and will be assigned to multiple codelists ({concepts}).")
  }
  invisible()
}
stratifyCodelist <- function(x, by, nameStyle) {
  # split codelists
  x <- x |>
    dplyr::group_by(dplyr::across(dplyr::all_of(by))) |>
    dplyr::group_split() |>
    as.list()

  # create names
  nms <- x |>
    purrr::map_chr(\(x) {
      x |>
        dplyr::distinct(dplyr::across(dplyr::all_of(by))) |>
        dplyr::mutate(name = glue::glue(nameStyle)) |>
        dplyr::pull("name")
    })
  names(x) <- nms

  return(x)
}
prepareCodelist <- function(x, original, searchStrategy) {
  if (inherits(original, "codelist")) {
    x <- x |>
      purrr::map(\(x) x$concept_id) |>
      omopgenerics::newCodelist()
  } else if (inherits(original, "code_search")) {
    st <- searchStrategy(original)
    id <- max(st$strategy_id)
    st <- st |>
      dplyr::union_all(
        searchStrategy |>
          dplyr::mutate(strategy_id = .env$id + 1L)
      )
    codes <- x |>
      dplyr::as_tibble() |>
      dplyr::select("codelist_name", "concept_id") |>
      dplyr::mutate(value = TRUE) |>
      tidyr::pivot_wider(
        names_from = "codelist_name",
        values_from = "value",
        values_fill = FALSE
      )
    for (nm in names(x)) {
      if (!nm %in% colnames(codes)) {
        codes[[nm]] <- FALSE
      }
    }
    defCols <- codeSerachColumns |>
      dplyr::filter(.data$table == "codes") |>
      dplyr::pull("table")
    x <- original |>
      dplyr::select(dplyr::any_of(defCols)) |>
      dplyr::left_join(codes, by = "concept_id") |>
      dplyr::mutate(dplyr::across(
        dplyr::all_of(names(x)),
        \(x) dplyr::coalesce(x, FALSE)
      )) |>
      newCodeSearch(searchStrategy = st)
  } else {
    x <- x |>
      purrr::map(\(x) {
        nm <- unique(x$codelist_name)
        x |>
          dplyr::select("concept_id") |>
          dplyr::mutate(concept_id = as.integer(.data$concept_id)) |>
          dplyr::inner_join(
            original[[nm]] |>
              dplyr::mutate(concept_id = as.integer(.data$concept_id)),
            by = "concept_id"
          )
      }) |>
      omopgenerics::newCodelistWithDetails()
  }
  return(x)
}

#' Summarise code use from achilles counts.
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams countByDoc
#'
#' @return A tibble with summarised counts.
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' cdm <- mockVocabRef("database")
#' oa <- getCandidateCodes(cdm = cdm, keywords = "osteoarthritis")
#' codelist <- omopgenerics::newCodelist(list(oa = oa$concept_id))
#' result_achilles <- summariseAchillesCodeUse(codelist, cdm = cdm)
#' result_achilles
#' CDMConnector::cdmDisconnect(cdm)
#' }

summariseAchillesCodeUse <- function(x,
                                     cdm,
                                     countBy = c("record", "person")) {

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm,
                                           requiredTables = c("achilles_analysis",
                                                              "achilles_results",
                                                              "achilles_results_dist"))
  checkCodelist(x, allowConceptSetExpression = FALSE)
  omopgenerics::assertChoice(countBy, choices = c("record", "person"))

  if(length(x) == 0){
    cli::cli_inform(
      c(
        "i" = "Empty codelist - no achilles counts to return"
      ))
    return(omopgenerics::emptySummarisedResult())
  }

  if(inherits(x, "codelist_with_details")){
    x <- asCodelist(x)
  }

  version <- achillesVersionDate(cdm)
  cli::cli_inform(version)

  allCodes <-  purrr::list_c(x)

  codesWithDetails <- addDetails(x, cdm)
  codesWithDetails <- purrr::list_rbind(codesWithDetails) |>
    dplyr::distinct()

  codeUse <- list()

  if("record" %in% countBy){
    allRecordCount <- getAchillesRecordCounts(cdm = cdm, conceptId = allCodes)
    if(nrow(allRecordCount)>=1){
      allRecordCount <- allRecordCount |>
        dplyr::mutate(concept_id = as.character(.data$concept_id)) |>
        dplyr::left_join(codesWithDetails |>
                           dplyr::mutate(concept_id = as.character(.data$concept_id)),
                         by = "concept_id")
      for(i in seq_along(x)){
        codeUse[[paste0(i, "_record")]] <- allRecordCount |>
          dplyr::filter(.data$concept_id %in% x[[i]]) |>
          dplyr::rename(
            "variable_name" = "concept_name", # standard concept name
            "variable_level" = "concept_id"   # standard concept id
          ) |>
          dplyr::mutate(estimate_name = "record_count") |>
          dplyr::mutate(group_level = names(x)[i])
      }
    }
  }

  if("person" %in% countBy){
    allPersonCount <- getAchillesPersonCounts(cdm = cdm, conceptId = allCodes)
    if(nrow(allPersonCount)>=1){
      allPersonCount <- allPersonCount |>
        dplyr::mutate(concept_id = as.character(.data$concept_id)) |>
        dplyr::left_join(codesWithDetails |>
                           dplyr::mutate(concept_id = as.character(.data$concept_id)),
                         by = "concept_id")
      for(i in seq_along(x)){
        codeUse[[paste0(i, "_person")]] <- allPersonCount |>
          dplyr::filter(.data$concept_id %in% x[[i]]) |>
          dplyr::rename(
            "variable_name" = "concept_name", # standard concept name
            "variable_level" = "concept_id"   # standard concept id
          ) |>
          dplyr::mutate(estimate_name = "person_count") |>
          dplyr::mutate(group_level = names(x)[i])
      }
    }
  }

  if(length(codeUse) == 0){
    cli::cli_inform(
      c(
        "i" = "No achilles counts found for the concepts provided."
      ))
    return(omopgenerics::emptySummarisedResult())
  } else {
    codeUse <- dplyr::bind_rows(codeUse)

    codeUse <- codeUse |>
      dplyr::mutate(
        result_id = 1L,
        cdm_name = CDMConnector::cdmName(cdm),
        group_name = "codelist_name",
        domain_id = tolower(.data$domain_id),
        estimate_type = "integer",
        estimate_value = as.character(.data$n)
      ) |>
      omopgenerics::uniteAdditional(cols = "vocabulary_id") |>
      omopgenerics::uniteStrata(cols = c("domain_id")) |>
      dplyr::select(dplyr::any_of(omopgenerics::resultColumns("summarised_result")))

    codeUse <- codeUse |>
      omopgenerics::newSummarisedResult(
        settings = dplyr::tibble(
          result_id = 1L,
          result_type = "achilles_code_use",
          package_name = "CodelistGenerator",
          package_version = as.character(utils::packageVersion(
            pkg = "CodelistGenerator"))
        )
      )
  }

  return(codeUse)

}

getSourceCodes <- function(cdm, codes){
  # Add table domains data in the cdm
  tableDomainsData <- paste0(omopgenerics::uniqueTableName(),
                             omopgenerics::uniqueId())
  cdm <- omopgenerics::insertTable(cdm = cdm,
                                   name = tableDomainsData,
                                   table = conceptDomainsData,
                                   overwrite = TRUE,
                                   temporary = FALSE)

  # Add codelist in the cdm
  codes <- codes |>
    dplyr::select("concept_id", "domain_id") |>
    dplyr::mutate("domain_id" = tolower(.data$domain_id)) |>
    dplyr::distinct() |>
    dplyr::mutate("concept_id" = as.integer(.data$concept_id))

  tableCodelist <- paste0(omopgenerics::uniqueTableName(),
                          omopgenerics::uniqueId())
  cdm <- omopgenerics::insertTable(cdm = cdm,
                                   name = tableCodelist,
                                   table = codes,
                                   overwrite = TRUE,
                                   temporary = FALSE)

  # Intersect
  cdm[[tableCodelist]] <- cdm[[tableCodelist]] |>
    dplyr::left_join(
      cdm[[tableDomainsData]],
      by = "domain_id"
    ) |>
    dplyr::compute(name = tableCodelist, temporary = FALSE)

  # Get source codes
  intermediateTable <- paste0(omopgenerics::uniqueTableName(),
                              omopgenerics::uniqueId())
  records <- getRelevantRecords(cdm = cdm,
                                tableCodelist = tableCodelist,
                                cohortTable = NULL,
                                cohortId = NULL,
                                timing = "any",
                                intermediateTable = intermediateTable,
                                useSourceCodes = FALSE)

  cdm <- omopgenerics::dropSourceTable(cdm = cdm, name = c(tableDomainsData, intermediateTable, tableCodelist))

  records <- records |>
    dplyr::select("standard_concept_id", "source_concept_id", "source_concept_value", "standard_concept_name", "source_concept_name") |>
    dplyr::distinct() |>
    dplyr::collect()

  return(records)
}

achillesVersionDate <- function(cdm){

  omopgenerics::validateCdmArgument(cdm,
                                    requiredTables = c("achilles_analysis",
                                                       "achilles_results",
                                                       "achilles_results_dist"))

  cdm[["achilles_results"]] |>
    dplyr::filter(.data$analysis_id == 0) |>
    dplyr::collect() |>
    dplyr::mutate(achilles_version = paste0("Using achilles results from version ",
                                            .data$stratum_2,
                                            " which was run on ",
                                            .data$stratum_3)) |>
    dplyr::pull("achilles_version")

}

getAchillesPersonCounts <- function(cdm, conceptId = NULL){
  fetchAchillesCounts(cdm = cdm,
                      analysisId = c(400, # condition occurrence
                                     700, # drug_exposure
                                     800, # observation
                                     1800, # measurement
                                     200, # visit_occurrence
                                     600, # procedure_occurrence
                                     2100  # device_exposure
                      ),
                      conceptId = conceptId)
}

getAchillesRecordCounts <- function(cdm, conceptId = NULL){
  fetchAchillesCounts(cdm = cdm,
                      analysisId = c(401, # condition occurrence
                                     701, # drug_exposure
                                     801, # observation
                                     1801, # measurement
                                     201, # visit_occurrence
                                     601, # procedure_occurrence
                                     2101  # device_exposure
                      ),
                      conceptId = conceptId)
}

fetchAchillesCounts <- function(cdm, analysisId, conceptId = NULL){
  analyses <- cdm[["achilles_results"]] |>
    dplyr::filter(.data$analysis_id %in%  .env$analysisId) |>
    dplyr::select("stratum_1", "count_value") |>
    dplyr::rename("concept_id" = "stratum_1",
                  "n"="count_value") |>
    dplyr::collect()

  if(!is.null(conceptId)){
    analyses <- analyses |>
      dplyr::filter(.data$concept_id %in%  .env$conceptId)
  }

  # the same code might appear in multiple tables so we will sum them
  analyses |>
    dplyr::group_by(.data$concept_id) |>
    dplyr::summarise(n = sum(.data$n, na.rm = TRUE)) |>
    dplyr::mutate(n = as.integer(.data$n))
}

subsetCodelistBy <- function(x,
                             cdm,
                             by,
                             group,
                             negate,
                             st,
                             keepOriginal = FALSE,
                             call = parent.frame()){
  # initial checks
  checkCodelist(x, allowConceptSetExpression = FALSE, call = call)
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm, call = call)
  omopgenerics::assertLogical(negate, length = 1, call = call)
  omopgenerics::assertLogical(keepOriginal, length = 1, call = call)

  if (length(x) == 0) {
    cli::cli_warn(c("!" = "Empty codelist provided. Returning an empty codelist."), call = call)
    return(x)
  }

  original <- x

  # codelist table
  nm <- omopgenerics::uniqueTableName()
  if (inherits(x, "code_search")) {
    x <- asCodelist(x)
  }
  x <- dplyr::as_tibble(x) |>
    dplyr::rename(codelist_name = dplyr::any_of("codelist_with_details_name")) |>
    dplyr::select("codelist_name", "concept_id")
  cdm <- omopgenerics::insertTable(cdm = cdm, name = nm, table = x)
  on.exit(omopgenerics::dropSourceTable(cdm = cdm, name = nm))

  if (by == "domain") {
    x <- addDomain(x = cdm[[nm]])
  } else if (by == "dose_form") {
    x <- addDoseForm(x = cdm[[nm]])
  } else if (by == "dose_unit") {
    x <- addDoseUnit(x = cdm[[nm]])
  } else if (by == "vocabulary") {
    x <- addVocabulary(x = cdm[[nm]])
  } else if (by == "route_category") {
    x <- addRouteCategory(x = cdm[[nm]])
  } else if (by == "ingredient_range") {
    x <- addIngredientCount(cdm, x <- cdm[[nm]])
  }

  if (by == "ingredient_range") {
    if (isTRUE(negate)) {
      x <- x |>
        dplyr::filter(.data$ingredient_count < !!group[1] |
                      .data$ingredient_count > !!group[2])
    } else {
      x <- x |>
        dplyr::filter(.data$ingredient_count >= !!group[1],
                      .data$ingredient_count <= !!group[2])
    }
  } else {

    # correct missing
    x <- correctMissingValue(x, by)

    if(isTRUE(negate)){
      x <- x |>
        dplyr::filter(!.data[[by]] %in% omopgenerics::toSnakeCase(.env$group))
    }else{
      x <- x |>
        dplyr::filter(.data[[by]] %in% omopgenerics::toSnakeCase(.env$group))
    }
  }

  x <- stratifyCodelist(x, by = "codelist_name", nameStyle = "{codelist_name}")

  # add class (and details)
  x <- prepareCodelist(x = x, original = original, searchStrategy = st)

  if (!inherits(x, "code_search")) {
    x <- dropEmptyCodelist(original, x, call = call)
  }

  # add original codes
  if (isTRUE(keepOriginal)) {
    x <- keepOriginalCodelists(x = x, original = original)
  }

  return(x)
}

keepOriginalCodelists <- function(x, original) {
  if (inherits(x, "code_search")) {
    cols <- codeSerachColumns |>
      dplyr::filter(.data$table == "codes") |>
      dplyr::pull("table")
    codelists <- setdiff(colnames(original), cols)
    x <- x |>
      dplyr::full_join(
        original |>
          dplyr::select("concept_id", dplyr::all_of(codelists)),
        by = "concept_id",
        suffix = c(".new", ".original")
      ) |>
      dplyr::mutate(dplyr::across(
        !dplyr::all_of(cols),
        \(x) dplyr::coalesce(x, FALSE)
      ))
  } else {
    x <- c(x, original)
  }
  return(x)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Format the result of summariseAchillesCodeUse into a table
#'
#' @param result A `<summarised_result>` with results of the type
#' "achilles_code_use".
#' @inheritParams typeTableDoc
#' @inheritParams tableStyleDoc
#' @inheritParams headerDoc
#' @inheritParams groupColumnDoc
#' @inheritParams hideDoc
#' @inheritParams tableStyleDoc
#' @inheritParams .optionsDoc
#'
#' @return A table with a formatted version of the summariseCohortCodeUse
#' result.
#'
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#'
#' cdm <- mockVocabRef("database")
#' oa <- getCandidateCodes(cdm = cdm, keywords = "osteoarthritis")
#' result_achilles <- summariseAchillesCodeUse(newCodelist(list(oa = oa$concept_id)),
#'                                             cdm = cdm)
#' tableAchillesCodeUse(result_achilles)
#' CDMConnector::cdmDisconnect(cdm)
#'}
#'
tableAchillesCodeUse <- function(result,
                                 type = NULL,
                                 header = c("cdm_name", "estimate_name"),
                                 groupColumn = character(),
                                 hide = character(),
                                 style = NULL,
                                 .options = list()) {

  rlang::check_installed("visOmopResults", version = "1.4.0")

  # checks
  if(nrow(result) == 0){
    cli::cli_warn("`result` object is empty")
    return(visOmopResults::emptyTable(type = type))
  }

  result <- result |>
    visOmopResults::filterSettings(.data$result_type == "achilles_code_use")

  if(nrow(result) == 0){
    cli::cli_warn("No achilles code use results found in result object")
    return(visOmopResults::emptyTable(type = type))
  }

  x <- internalTableAchillesResult(
    result = result,
    resultType = "achilles_code_use",
    type = type,
    header = header,
    groupColumn = groupColumn,
    hide = hide,
    style = style,
    .options = .options
  )

  return(x)
}

# Copyright 2025 DARWIN EU®
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Filter a codelist to keep only the codes being used in patient records
#'
#' @inheritParams xDoc
#' @inheritParams cdmDoc
#' @inheritParams minimumCountDoc
#' @inheritParams tableDoc
#'
#' @return The filtered codelist with only the codes used in the database
#' @export
#'
#' @examples
#' \donttest{
#' library(CodelistGenerator)
#' library(omopgenerics)
#' cdm <- mockVocabRef("database")
#' codes <- getCandidateCodes(cdm = cdm,
#'                            keywords = "arthritis",
#'                            domains = "Condition",
#'                            includeDescendants = FALSE)
#' x <- subsetToCodesInUse(newCodelist(list("cs1" = codes$concept_id,
#'                                "cs2" = 999)),
#'                                 cdm = cdm)
#'
#' x
#' CDMConnector::cdmDisconnect(cdm)
#' }
subsetToCodesInUse <- function(x,
                               cdm,
                               minimumCount = 0L,
                               table = c("condition_occurrence",
                                         "device_exposure",
                                         "drug_exposure",
                                         "measurement",
                                         "observation",
                                         "procedure_occurrence",
                                         "visit_occurrence")){

  # Initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm,
                                           requiredTables = c("achilles_analysis",
                                                              "achilles_results",
                                                              "achilles_results_dist"))
  omopgenerics::assertNumeric(minimumCount, integerish = TRUE, min = 0, length = 1)
  omopgenerics::assertChoice(table, choices = c("condition_occurrence",
                                                "device_exposure",
                                                "drug_exposure",
                                                "measurement",
                                                "observation",
                                                "procedure_occurrence",
                                                "visit_occurrence"))
  checkCodelist(x, allowConceptSetExpression = FALSE)

  x_original <- x
  if(inherits(x_original, "codelist_with_details")){
    x <- asCodelist(x)
    newX <- omopgenerics::emptyCodelistWithDetails()
  }
  if(inherits(x_original, "codelist")){
    newX <- omopgenerics::emptyCodelist()
  }

  dbCodes <- codesInUse(cdm = cdm,
                        minimumCount = minimumCount,
                        table = table)

  if(is.null(dbCodes)){
    for(i in seq_along(x)){
      cli::cli_inform("No codes from any codelist found in the database")
      return(invisible(omopgenerics::emptyCodelist()))
    }
  } else {
    for(i in seq_along(x)){

      x[[i]] <- intersect(x[[i]], dbCodes)

      if(!length(x[[i]]) >= 1){
        cli::cli_inform("No codes from codelist {names(x)[i]} found in the database")
        x_original[[i]] <- x[[i]]
      }

      if(inherits(x_original, "codelist_with_details")){
        newX[[names(x_original)[[i]]]] <- x_original[[i]] |>
          dplyr::inner_join(
            dplyr::tibble("concept_id" = x[[i]]),
            by = "concept_id"
          )
      }
      if(inherits(x_original, "codelist")){
        newX[[names(x_original)[[i]]]] <- x[[i]]
      }
    }
  }

  newX <- dropEmptyCodelist(x_original, newX)

  return(newX)
}


codesInUse <- function(cdm,
                       minimumCount = 0L,
                       table = c("condition_occurrence",
                                 "device_exposure",
                                 "drug_exposure",
                                 "measurement",
                                 "observation",
                                 "procedure_occurrence",
                                 "visit_occurrence")){

  # initial checks
  cdm <- omopgenerics::validateCdmArgument(cdm = cdm,
                                           requiredTables = c("achilles_analysis",
                                                              "achilles_results",
                                                              "achilles_results_dist"))
  omopgenerics::assertNumeric(minimumCount, integerish = TRUE, min = 0, length = 1)
  omopgenerics::assertCharacter(table)

  codes <- fetchAchillesCodesInUse(cdm, minimumCount = minimumCount)

  codes
}

test_that("tests with mock db", {
  skip_on_cran()
  # mock db
  cdm <- mockVocabRef("database")

  codes <- getCandidateCodes(
    cdm = cdm,
    keywords = "arthritis",
    domains = "Condition",
    includeDescendants = FALSE
  )
  expect_true(all(c("4", "5") %in%
                    subsetToCodesInUse(omopgenerics::newCodelist(list("cs" = codes$concept_id)),
                                       cdm = cdm)[[1]]))

  expect_warning(expect_true(length(subsetToCodesInUse(
    omopgenerics::newCodelist(list("cs1" = codes$concept_id,
                                   "cs2" = 999L)),
    cdm = cdm)) == 1)) # will just have cs1

  x <- subsetToCodesInUse(omopgenerics::newCodelist(list("cs" = codes$concept_id)),
                          cdm = cdm)
  expect_true(inherits(x, "codelist"))
  x <- subsetToCodesInUse(omopgenerics::newCodelist(list("cs" = codes$concept_id)),
                          cdm = cdm)
  expect_true(inherits(x, "codelist"))
  x <- list("concepts" = cdm$concept |>
              dplyr::filter(concept_id %in% c(4L,5L,6L)) |>
              dplyr::select("concept_id", "concept_name", "domain_id", "vocabulary_id") |>
              dplyr::mutate("ancestor_concept_id" = 999L) |>
              dplyr::collect()) |>
    omopgenerics::newCodelistWithDetails()
  x <- subsetToCodesInUse(x,
                          cdm = cdm)
  expect_true(inherits(x, "codelist_with_details"))
  expect_true(all(x$concepts$concept_id == c(4L, 5L)))

  # no codes in db
  codes <- getCandidateCodes(
    cdm = cdm,
    keywords = "Musculoskeletal",
    domains = "Condition",
    includeDescendants = FALSE
  )
  expect_warning(subsetToCodesInUse(omopgenerics::newCodelist(list("cs" = codes$concept_id)),
                                    cdm = cdm))

  CDMConnector::cdmDisconnect(cdm)

  cdm <- omock::mockCdmReference()
  expect_error(codesInUse(cdm))
  expect_error(subsetToCodesInUse(x = list("a" = 1), cdm = cdm))
})

test_that("sql server with achilles", {

  testthat::skip_if(Sys.getenv("CDM5_SQL_SERVER_SERVER") == "")
  testthat::skip_if(Sys.getenv("SQL_SERVER_DRIVER") == "")
  testthat::skip_if(packageVersion("CDMConnector") <= "1.2.0")

  db <- DBI::dbConnect(odbc::odbc(),
                       Driver   = Sys.getenv("SQL_SERVER_DRIVER"),
                       Server   = Sys.getenv("CDM5_SQL_SERVER_SERVER"),
                       Database = Sys.getenv("CDM5_SQL_SERVER_CDM_DATABASE"),
                       UID      = Sys.getenv("CDM5_SQL_SERVER_USER"),
                       PWD      = Sys.getenv("CDM5_SQL_SERVER_PASSWORD"),
                       TrustServerCertificate="yes",
                       Port     = Sys.getenv("CDM5_SQL_SERVER_PORT"))
  cdm <- CDMConnector::cdmFromCon(db,
                                  cdmSchema = c("CDMV54", "dbo"),
                                  achillesSchema = c("CDMV54", "dbo"),
                                  writeSchema = c("ohdsi", "dbo"))

  asthma_codes <- getCandidateCodes(
    cdm = cdm,
    keywords = "asthma",
    domains = c("Condition"),
    includeDescendants = TRUE
  )
  asthma_cl <- omopgenerics::newCodelist(list("cs" = asthma_codes$concept_id))

  asthma_codes_present <- subsetToCodesInUse(x = asthma_cl,
                                             cdm = cdm)

  expect_equal(sort(asthma_codes_present[[1]]),
               sort(cdm$condition_occurrence |>
                      dplyr::filter(.data$condition_concept_id %in%
                                      !!asthma_codes$concept_id) |>
                      dplyr::select("condition_concept_id") |>
                      dplyr::distinct() |>
                      dplyr::pull()))


  CDMConnector::cdmDisconnect(cdm)
})

# Copyright 2024 DARWIN EU (C)
#
# This file is part of CodelistGenerator
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#' Coerce to a codelist with details
#'
#' @param x A codelist, codelist_with_details, concept_set_expression, or a candidate_codes.
#' @inheritParams cdmDoc
#' @param ... For extensibility
#'
#' @returns codelist
#' @export
#'
#' @examples
#' \donttest{
#' library(omock)
#' library(CDMConnector)
#'
#' # Creating CDM object
#' path <- downloadMockDataset(datasetName = "GiBleed",
#'                             path = NULL,
#'                             overwrite = NULL)
#' cdm <- mockCdmFromDataset(datasetName = "GiBleed")
#'
#' # Create codelist_with_details from a codelist
#' codelist <- getDrugIngredientCodes(cdm,
#'                                    name = "acetaminophen",
#'                                    nameStyle = "{concept_name}",
#'                                    type = "codelist")
#'
#' asCodelistWithDetails(codelist, cdm)
#'
#' # Create codelist from a candidate_codes
#' codelist <- getCandidateCodes(cdm,
#'                              keywords = "arthritis")
#'
#' asCodelistWithDetails(codelist)
#' }
asCodelistWithDetails <- function(x, cdm, ...){
  UseMethod("asCodelistWithDetails")
}

#' @export
#' @rdname asCodelistWithDetails
asCodelistWithDetails.codelist_with_details  <- function(x, ...){

  x |>
    omopgenerics::newCodelistWithDetails()

}

#' @export
#' @rdname asCodelistWithDetails
asCodelistWithDetails.codelist  <- function(x, cdm, ...){

  addDetails(x, cdm = cdm) |>
    omopgenerics::newCodelistWithDetails()

}

#' @export
#' @rdname asCodelistWithDetails
asCodelistWithDetails.candidate_codes  <- function(x, cdm, ...){
  x <- removeClass(x, "candidate_codes")
  list("candidate_codes" = x |>
         dplyr::select("concept_id", "concept_name",
                       "domain_id", "vocabulary_id", "standard_concept")
  ) |>
    omopgenerics::newCodelistWithDetails()
}

test_that("load route data", {
  expect_identical(colnames(doseFormToRoute),
                   c("dose_form_concept_id",
                     "dose_form_concept_name",
                     "route_category"))
})

---
title: "Generate a candidate codelist"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a04_GenerateCandidateCodelist}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%"
)
```

In this example we will create a candidate codelist for osteoarthritis, exploring how different search strategies may impact our final codelist. 
First, let's load the necessary packages and create a cdm reference using mock data.

```{r,  message=FALSE, warning=FALSE}
library(dplyr)
library(CodelistGenerator)

cdm <- mockVocabRef()
```

The mock data has the following hypothetical concepts and relationships:

```{r, echo=FALSE}
knitr::include_graphics("Figures/1.png")
```

## Search for keyword match 
We will start by creating a codelist with keywords match. Let's say that we want to find those codes that contain "Musculoskeletal disorder" in their concept_name:
```{r, echo=FALSE}
knitr::include_graphics("Figures/2.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition", 
  standardConcept = "Standard",
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

Note that we could also identify it based on a partial match or based on all combinations match.
```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)

getCandidateCodes(
  cdm = cdm,
  keywords = "Disorder musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)
```

Notice that currently we are only looking for concepts with `domain = "Condition"`. However, we can expand the search to all domains using `domain = NULL`.

`getCandidateCodes()` function will generate a table with class "candidate_codes", which contains an attribute with the details of the search strategy:
```{r, message=FALSE}
candidate_codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)

searchStrategy(candidate_codes)
```


## Include non-standard concepts
Now we will include standard and non-standard concepts in our initial search. By setting `standardConcept = c("Non-standard", "Standard")`, we allow the function to return, in the final candidate codelist, both the non-standard and standard codes that have been found.

```{r,echo=FALSE}
knitr::include_graphics("Figures/3.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  standardConcept = c("Non-standard", "Standard"),
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)
```

## Multiple search terms 
We can also search for multiple keywords simultaneously, capturing all of them with the following search:

```{r,echo=FALSE}
knitr::include_graphics("Figures/4.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = c(
    "Musculoskeletal disorder",
    "arthritis"
  ),
  domains = "Condition",
  standardConcept = c("Standard"),
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

## Add descendants
Now we will include the descendants of an identified code using `includeDescendants` argument
```{r,echo=FALSE}
knitr::include_graphics("Figures/5.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

Notice that now, in the column `found_from`, we can see that we have obtain `concept_id=1` from an initial search, and `concept_id_=c(2,3,4,5)` when searching for descendants of concept_id 1.

## With exclusions
We can also exclude specific keywords using the argument `exclude`

```{r, echo=FALSE}
knitr::include_graphics("Figures/6.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  exclude = c("Osteoarthrosis", "knee"),
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

When multiple words are added within a term (e.g., "knee osteoarthritis"), each word
will be searched independently, so that for example, "osteoarthritis of knee" is excluded:
```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  exclude = c("knee osteoarthritis"),
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

If we only want to exclude exact matching terms (without accounting for words boundaries) we need to add "/" at the beginning and
at the end of the term. Hence, using "knee osteoarthritis", "osteoarthritis of knee" **won't** be excluded. However, if we had 
"right knee osteoarthritis", it would be excluded.
```{r, message=FALSE}
# No exclusion:
getCandidateCodes(
    cdm = cdm,
    keywords = "Knee",
    domains = "Condition",
    exclude = NULL,
    standardConcept = c("Standard", "Non-standard"),
    includeDescendants = TRUE,
    searchInSynonyms = FALSE,
    searchNonStandard = FALSE,
    includeAncestor = FALSE
)

# Exclusion looking for terms:
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("knee osteoarthritis"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# Exclusion looking for partial matching terms (without word boundaries)
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/knee osteoarthritis/"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# Exclusion looking for partial matching terms (without word boundaries)
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/e osteoarthritis/"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

If we want to do exact matching (that means, to find the exact two words "knee osteoarthritis" in the concept name) we need to use "/\b" at the
beginning and at the end of the expression.
```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/\bKnee osteoarthritis/\b"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# We will now only search for "ee osteoarthritis" to show that 
# "knee osteoarthritis" won't be excluded:
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/\bee osteoarthritis/\b"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```
```

For example, if we look for 

Notice that, for example, if we wanted `keywords = "depression"` and `exclude = "ST depression"`, concepts like "poSTpartum depression" would be excluded. To avoid this,
we could use `exclude = "/ST depression/"`. Notice that, "poST depression" would also be excluded with this option.

Hence, there is another option to exclude exact matching terms accounting for words boundaries: adding "/\b" at the beginning and at the end of the term. For example, if we look for "/\bp osteoarthritis/\b", concepts like "hip osteoarthritis **won't** be excluded.



## Add ancestor
To include the ancestors one level above the identified concepts, we can use the argument `includeAncestor`
```{r, echo=FALSE}
knitr::include_graphics("Figures/7.png")
```

```{r, message=FALSE}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Osteoarthritis of knee",
  includeAncestor = TRUE,
  domains = "Condition",
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
)

codes
```

## Search using synonyms
We can also pick up codes based on their synonyms. For example, **Osteoarthrosis** has a synonym of **Arthritis**.
```{r, echo=FALSE}
knitr::include_graphics("Figures/8.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthrosis",
  domains = "Condition",
  searchInSynonyms = TRUE,
  standardConcept = "Standard",
  includeDescendants = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

Notice that if `includeDescendants = TRUE`, **Arthritis** descendants will also be included:
```{r,echo=FALSE}
knitr::include_graphics("Figures/9.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthrosis",
  domains = "Condition",
  searchInSynonyms = TRUE,
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```


## Search via non-standard
We can also pick up concepts associated with our keyword via non-standard search.
```{r,echo=FALSE}
knitr::include_graphics("Figures/10.png")
```

```{r, message=FALSE}
codes1 <- getCandidateCodes(
  cdm = cdm,
  keywords = "Degenerative",
  domains = "Condition",
  standardConcept = "Standard",
  searchNonStandard = TRUE,
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  includeAncestor = FALSE
)
codes1
```

Let's take a moment to focus on the `standardConcept` and `searchNonStandard` arguments to clarify the difference between them. `standardConcept` specifies whether we want only standard concepts or also include non-standard concepts in the final candidate codelist. `searchNonStandard` determines whether we want to search for keywords among non-standard concepts.

In the previous example, since we set `standardConcept = "Standard"`, we retrieved the code for **Osteoarthrosis** from the non-standard search. However, we did not obtain the non-standard code **degenerative arthropathy** from the initial search. If we allow non-standard concepts in the final candidate codelist, we would retireve both codes:

```{r,echo=FALSE}
knitr::include_graphics("Figures/11.png")
```

```{r, message=FALSE}
codes2 <- getCandidateCodes(
  cdm = cdm,
  keywords = "Degenerative",
  domains = "Condition",
  standardConcept = c("Non-standard", "Standard"),
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  includeAncestor = FALSE
)
codes2
```

---
title: "Benchmarking the CodelistGenerator R package"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a10_CodelistGeneratorBenchmark}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r, include = FALSE}
CDMConnector::requireEunomia("synpuf-1k", "5.3")
```

To check the performance of the CodelistGenerator package, we can use the benchmarkCodelistGenerator(). This function generates some hypothetical study codelists using various settings and times to check how long these analyses take.

We can start for example by benchmarking our mock data, which uses duckdb.

```{r, message=FALSE, warning=FALSE}
library(DBI)
library(duckdb)
library(dplyr)
library(CDMConnector)
library(CodelistGenerator)
library(visOmopResults)

# Connect to the database and create the cdm object
con <- dbConnect(duckdb(), 
                      eunomiaDir("synpuf-1k", "5.3"))
cdm <- cdmFromCon(con = con, 
                  cdmName = "Eunomia Synpuf",
                  cdmSchema   = "main",
                  writeSchema = "main", 
                  achillesSchema = "main")

timings <- benchmarkCodelistGenerator(cdm)

visOmopTable(timings,
             hide = c("variable_name", "variable_level", "strata_name", "strata_level"),
             groupColumn = "task")
```

---
title: "CodelistGenerator search options: codes for osteoarthritis"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a02_Candidate_codes_OA}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r,  message=FALSE, warning=FALSE,echo=FALSE}
library(DBI)
library(dplyr)
library(CodelistGenerator)
library(CDMConnector)
```


## Creating a codelist for osteoarthritis
For this example we are going to generate a candidate codelist for osteoarthritis, looking at the impact of alternative search strategies.

```{r, eval=FALSE}
library(DBI)
library(RPostgres)
```

```{r, eval=FALSE}
# postgres database connection details
serverDbi <- Sys.getenv("server")
user <- Sys.getenv("user")
password <- Sys.getenv("password")
port <- Sys.getenv("port")
host <- Sys.getenv("host")

db <- dbConnect(RPostgres::Postgres(),
  dbname = serverDbi,
  port = port,
  host = host,
  user = user,
  password = password
)

# name of vocabulary schema
vocabularyDatabaseSchema <- "vocabulary"

# create cdm reference
cdm <- CDMConnector::cdm_from_con(
  con = db,
  cdm_schema = vocabularyDatabaseSchema
)
```


## Search strategies
### Condition domain, without searching synonyms, with exclusions, without including descendants or ancestor
```{r,  message=FALSE, warning=FALSE,echo=FALSE}
oaCodes1 <- readRDS(system.file("optionsData01.RData", 
                                            package = "CodelistGenerator"))
```

To start we will search for "osteoarthritis", while excluding "post-infection" and "post-traumatic", but without searching synonyms, without searching via non-standard codes, and without including descendants or the direct ancestor of the included concepts.

```{r, eval=FALSE}
oaCodes1 <- getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthritis",
  domains = "Condition",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  exclude = c(
    "post-infection",
    "post-traumatic"
  ),
  includeDescendants = FALSE,
  includeAncestor = FALSE
)
```

What is the candidate codelist?
```{r,  message=FALSE, warning=FALSE }  
oaCodes1 %>% 
  glimpse()
```


### Including descendants
```{r,  message=FALSE, warning=FALSE,echo=FALSE}
oaCodes2 <- readRDS(system.file("optionsData02.RData", 
                                            package = "CodelistGenerator"))
```

Now we will also include the descendants of included concepts.
```{r, eval=FALSE}
oaCodes2 <- getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthritis",
  domains = "Condition",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  exclude = c(
    "post-infection",
    "post-traumatic"
  ),
  includeDescendants = TRUE,
  includeAncestor = FALSE
)
```

What new codes do we pick up?
```{r,  message=FALSE, warning=FALSE }  
newCodes1To2 <- compareCodelists(oaCodes1, oaCodes2) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To2 %>% 
  glimpse()
```

### Including observation domain
```{r,  message=FALSE, warning=FALSE,echo=FALSE}
oaCodes3 <- readRDS(system.file("optionsData03.RData", 
                                            package = "CodelistGenerator"))
```

Now we will search the observation domain as well as the condition domain.
```{r, eval=FALSE}
oaCodes3 <- getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthritis",
  domains = c("Condition", "Observation"),
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  exclude = c(
    "post-infection",
    "post-traumatic"
  ),
  includeDescendants = FALSE,
  includeAncestor = FALSE
)
```

What new codes do we pick up?
```{r,  message=FALSE, warning=FALSE }  
newCodes1To3 <- compareCodelists(oaCodes1, oaCodes3) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To3 %>% 
  glimpse()
```

### Search synonyms
```{r,  message=FALSE, warning=FALSE,echo=FALSE}
oaCodes4 <- readRDS(system.file("optionsData04.RData", 
                                            package = "CodelistGenerator"))
```

Now we will search the concept synonym table to identify concepts to include.
```{r, eval=FALSE}
oaCodes4 <- getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthritis",
  domains = "Condition",
  searchInSynonyms = TRUE,
  searchNonStandard = FALSE,
  exclude = c(
    "post-infection",
    "post-traumatic"
  ),
  includeDescendants = FALSE,
  includeAncestor = FALSE
)
```

What new codes do we pick up?
```{r,  message=FALSE, warning=FALSE }  
newCodes1To4 <- compareCodelists(oaCodes1, oaCodes4) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To4 %>% 
  glimpse()
```

### Search via non-standard
```{r,  message=FALSE, warning=FALSE,echo=FALSE}
oaCodes5 <- readRDS(system.file("optionsData05.RData", 
                                            package = "CodelistGenerator"))
```

Now we will search the concept synonym table to identify concepts to include.
```{r, eval=FALSE}
oaCodes5 <- getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthritis",
  domains = "Condition",
  searchInSynonyms = FALSE,
  searchNonStandard = TRUE,
  exclude = c(
    "post-infection",
    "post-traumatic"
  ),
  includeDescendants = FALSE,
  includeAncestor = FALSE
)
```

What new codes do we pick up?
```{r,  message=FALSE, warning=FALSE }  
newCodes1To5 <- compareCodelists(oaCodes1, oaCodes5) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To5 %>% 
  glimpse()
```

### Include ancestor
```{r,  message=FALSE, warning=FALSE,echo=FALSE}
oaCodes8 <- readRDS(system.file("optionsData07.RData", 
                                            package = "CodelistGenerator"))
```

Now we include the direct ancestor of included terms.
```{r, eval=FALSE}
oaCodes8 <- getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthritis",
  domains = "Condition",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  exclude = c(
    "post-infection",
    "post-traumatic"
  ),
  includeDescendants = FALSE,
  includeAncestor = TRUE
)
```

What new codes do we pick up?
```{r,  message=FALSE, warning=FALSE }  
newCodes1To8 <- compareCodelists(oaCodes1, oaCodes8) %>%
  filter(codelist == "Only codelist 2") %>%
  select(-"codelist")

newCodes1To8 %>% 
  glimpse()
```

---
title: "Generating vocabulary based codelists for medications"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a05_GenerateVocabularyBasedCodelist}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN
)

CDMConnector::requireEunomia("synpuf-1k", "5.3")
```

In this vignette, we will explore how to generate codelists for medications using the OMOP CDM vocabulary tables. To begin, let's load the necessary packages and create a cdm reference using Eunomia synthetic data.
```{r,  message=FALSE, warning=FALSE}
library(DBI)
library(duckdb)
library(dplyr)
library(CDMConnector)
library(CodelistGenerator)

# Connect to the database and create the cdm object
con <- dbConnect(duckdb(), 
                 eunomiaDir("synpuf-1k", "5.3"))
cdm <- cdmFromCon(con = con, 
                  cdmName = "Eunomia Synpuf",
                  cdmSchema = "main",
                  writeSchema = "main",
                  achillesSchema = "main")
```

## Ingredient based codelists

The `getDrugIngredientCodes()` function can be used to generate the medication codelists based on ingredient codes. 

We can see that we have many drug ingredients for which we could create codelists.
```{r}
availableDrugIngredients(cdm) |> glimpse()
```

We will likely be interested in some specific drug ingredients of interest. Say for example we would like a codelist for acetaminophen then we can get this easily enough.
```{r}
acetaminophen_codes <- getDrugIngredientCodes(
  cdm = cdm,
  name = c("acetaminophen")
)

acetaminophen_codes
```

Notice that either the concept name or the concept ID can be specified to find the relevant codes.
```{r}
acetaminophen_codes <- getDrugIngredientCodes(
  cdm = cdm,
  name = 1125315
)

acetaminophen_codes
```

Instead of getting back all concepts for acetaminophen, we can use the `ingredientRange` argument to return only concepts associated with acetaminophen and at least one more drug ingredient (i.e. combination therapies). Here instead of returning a codelist with only the concept IDs, we will get them with details so that we can see concept names.

```{r}
acetaminophen_two_or_more_ingredients <- getDrugIngredientCodes(
  cdm = cdm,
  name = "acetaminophen",
  ingredientRange = c(2, Inf),
  type = "codelist_with_details"
)

acetaminophen_two_or_more_ingredients

acetaminophen_two_or_more_ingredients[[1]] |> 
  pull("concept_name") |> 
  head(n = 5) # Only the first five will be shown
```

Or we could instead only return concepts associated with acetaminophen and no other drug ingredient.

```{r}
acetaminophen_one_ingredient <- getDrugIngredientCodes(
  cdm = cdm,
  name = "acetaminophen",
  ingredientRange = c(1, 1),
  type = "codelist_with_details"
)

acetaminophen_one_ingredient

acetaminophen_one_ingredient[[1]] |> 
  pull("concept_name") |> 
  head(n = 5) # Only the first five will be shown
```

### Restrict to a specific dose form
Perhaps we are just interested in a particular dose form. We can see that there are many available.
```{r}
availableDoseForms(cdm) |> glimpse()
```

We can choose one or more of these to restrict to a particular dose form when finding our relevant codes. Here, for example, we only include codes with a dose form of injection. 

```{r}
acetaminophen_injections <- getDrugIngredientCodes(
  cdm = cdm,
  name = "acetaminophen",
  doseForm = "injection",
  type = "codelist_with_details"
)

acetaminophen_injections[[1]] |> 
  pull("concept_name") |> 
  head(n = 5) 
```

### Restrict to a specific dose unit

Similarly, we can might also want to restrict to a specific dose unit. Again we have a number of options available in our vocabularies.

```{r}
availableDoseUnits(cdm) |> glimpse()
```

Here we'll just include codes with a dose unit of milligram.

```{r}
acetaminophen_miligram <- getDrugIngredientCodes(
  cdm = cdm,
  name = "acetaminophen",
  doseUnit = "milligram",
  type = "codelist_with_details"
)

acetaminophen_miligram[[1]] |> 
  pull("concept_name") |> 
  head(n = 5) 
```

### Restrict to a specific route

Lastly, we can restrict to a specific route category. We can see we again have a number of options.

```{r}
availableRouteCategories(cdm) |> glimpse()
```

Here we'll include only concepts with a route category of inhalable.

```{r}
acetaminophen_inhalable <- getDrugIngredientCodes(
  cdm = cdm,
  name = "acetaminophen",
  routeCategory = "inhalable",
  type = "codelist_with_details"
)

acetaminophen_inhalable[[1]] |> 
  pull("concept_name") |> 
  head(n = 5) 
```

### Search multiple ingredients
The previous examples have focused on single drug ingredient, acetaminophen. We can though specify multiple ingredients, in which case we will get a codelist back for each.

```{r}
acetaminophen_heparin_codes <- getDrugIngredientCodes(
  cdm = cdm,
  name = c("acetaminophen", "heparin")
  )

acetaminophen_heparin_codes
```



And if we don´t specify an ingredient, we will get a codelist for every drug ingredient in the vocabularies!

## ATC based codelists

Analogous to `getDrugIngredientCodes()`, `getATCCodes()` can be used to generate a codelist based on a particular ATC class. 

With ATC we have five levels of the classification which we could be interested in. The first level is the broadest while the fifth is the narrowest.

```{r}
availableATC(cdm, level = c("ATC 1st")) |> glimpse()
availableATC(cdm, level = c("ATC 2nd")) |> glimpse()
availableATC(cdm, level = c("ATC 3rd")) |> glimpse()
availableATC(cdm, level = c("ATC 4th")) |> glimpse()
availableATC(cdm, level = c("ATC 5th")) |> glimpse()
```

In this example, we will produce an ATC level 1 codelist based on Alimentary Tract and Metabolism Drugs.

```{r}
atc_codelist <- getATCCodes(
  cdm = cdm,
  level = "ATC 1st",
  name = "alimentary tract and metabolism"
)

atc_codelist
```

Similarly as with `getDrugIngredientCodes()`, we can use `nameStyle` to specify the name of the elements in the list, `type` argument to obtain a codelist with details, the `doseForm` argument to restrict to specific dose forms, the `doseUnit` argument to restrict to specific dose unit, and the `routeCategory` argument to restrict to specific route categories. 

---
title: "CodelistGenerator options: examples with a with mock vocabulary"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{03_Options_for_CodelistGenerator}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Mock vocabulary database
```{r,  message=FALSE, warning=FALSE,echo=FALSE}
library(DBI)
library(dplyr)
library(CodelistGenerator)
library(CDMConnector)
```

Let´s say we have a mock vocabulary database with these hypothetical concepts and relationships.

```{r,echo=FALSE}
knitr::include_graphics("mock_db_fig1.png")
```

```{r,  message=FALSE, warning=FALSE,echo=FALSE, results='hide'}
cdm <- mockVocabRef(backend = "data_frame")
```

## Search for exact keyword match 
```{r,echo=FALSE}
knitr::include_graphics("mock_db_fig2.png")
```

To find "Musculoskeletal disorder" we can search for that like so 
```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  includeDescendants = FALSE,
)

codes %>% 
  glimpse()
```

Note, we would also identify it based on a partial match
```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal",
  domains = "Condition",
  includeDescendants = FALSE
)

codes %>% 
  glimpse()
```

## Add descendants
```{r,echo=FALSE}
knitr::include_graphics("mock_db_fig7.png")
```

To include descendants of an identified code, we can set includeDescendants to TRUE
```{r}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  includeDescendants = TRUE
) %>% 
  glimpse()
```

## Multiple search terms 
We can also search for multiple keywords at the same time, and would have picked these all up with the following search
```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = c(
    "Musculoskeletal disorder",
    "arthritis",
    "arthrosis"
  ),
  domains = "Condition",
  includeDescendants = FALSE
)

codes %>% 
  glimpse()
```

## Add ancestor
```{r, echo=FALSE}
knitr::include_graphics("mock_db_fig5.png")
```

To include the ancestors one level above the identified concepts we can set includeAncestor to TRUE

```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Osteoarthritis of knee",
  includeAncestor = TRUE,
  domains = "Condition"
)

codes %>% 
  glimpse()
```

## Searches with multiple words
```{r, echo=FALSE}
knitr::include_graphics("mock_db_fig4.png")
```

We can also find concepts with multiple words even if they are in a different order. For example, a search for "Knee osteoarthritis" will pick up "Osteoarthritis of knee".

```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Knee osteoarthritis",
  domains = "Condition",
  includeDescendants = TRUE
)

codes %>% 
  glimpse()
```


## With exclusions
```{r, echo=FALSE}
knitr::include_graphics("mock_db_fig5.png")
```

We can also exclude specific terms 

```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "arthritis",
  exclude = "Hip osteoarthritis",
  domains = "Condition"
)

codes %>% 
  glimpse()
```

## Search using synonyms
```{r, echo=FALSE}
knitr::include_graphics("mock_db_fig6.png")
```

We can also pick up codes based on their synonyms. In this case "Arthritis" has a synonym of "Osteoarthrosis" and so a search of both the primary name of a concept and any of its associated synonyms would pick up this synonym and it would be included.

```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthrosis",
  domains = "Condition",
  searchInSynonyms = TRUE
)

codes %>% 
  glimpse()
```

## Search via non-standard
Or we could have also picked up "Osteoarthrosis" by searching via non-standard.

```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = c("arthritis", "arthropathy"),
  domains = "Condition",
  searchNonStandard = TRUE
)

codes %>% 
  glimpse()
```

## Search for both standard and non-standard concepts

```{r, echo=FALSE}
knitr::include_graphics("mock_db_fig8.png")
```

We can also include non-standard codes in our results like so

```{r}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = c(
    "Musculoskeletal disorder",
    "arthritis",
    "arthropathy",
    "arthrosis"
  ),
  domains = "Condition",
  standardConcept = c("Standard", "Non-standard")
)

codes %>% 
  glimpse()
```

---
title: "Generate a candidate codelist"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a04_GenerateCandidateCodelist}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  out.width = "100%"
)
```

In this example we will create a candidate codelist for osteoarthritis, exploring how different search strategies may impact our final codelist. 
First, let's load the necessary packages and create a cdm reference using mock data.

```{r,  message=FALSE, warning=FALSE}
library(dplyr)
library(CodelistGenerator)

cdm <- mockVocabRef()
```

The mock data has the following hypothetical concepts and relationships:

```{r, echo=FALSE}
knitr::include_graphics("Figures/1.png")
```

## Search for keyword match 
We will start by creating a codelist with keywords match. Let's say that we want to find those codes that contain "Musculoskeletal disorder" in their concept_name:
```{r, echo=FALSE}
knitr::include_graphics("Figures/2.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition", 
  standardConcept = "Standard",
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

Note that we could also identify it based on a partial match or based on all combinations match.
```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)

getCandidateCodes(
  cdm = cdm,
  keywords = "Disorder musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)
```

Notice that currently we are only looking for concepts with `domain = "Condition"`. However, we can expand the search to all domains using `domain = NULL`.

`getCandidateCodes()` function will generate a table with class "candidate_codes", which contains an atribute with the details of the search strategy:
```{r, message=FALSE}
candidate_codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal",
  domains = "Condition",
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)

searchStrategy(candidate_codes)
```


## Include non-standard concepts
Now we will include standard and non-standard concepts in our initial search. By setting `standardConcept = c("Non-standard", "Standard")`, we allow the function to return, in the final candidate codelist, both the non-standard and standard codes that have been found.

```{r,echo=FALSE}
knitr::include_graphics("Figures/3.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  standardConcept = c("Non-standard", "Standard"),
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  includeAncestor = FALSE
)
```

## Multiple search terms 
We can also search for multiple keywords simultaneously, capturing all of them with the following search:

```{r,echo=FALSE}
knitr::include_graphics("Figures/4.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = c(
    "Musculoskeletal disorder",
    "arthritis"
  ),
  domains = "Condition",
  standardConcept = c("Standard"),
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

## Add descendants
Now we will include the descendants of an identified code using `includeDescendants` argument
```{r,echo=FALSE}
knitr::include_graphics("Figures/5.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

Notice that now, in the column `found_from`, we can see that we have obtain `concept_id=1` from an initial search, and `concept_id_=c(2,3,4,5)` when searching for descendants of concept_id 1.

## With exclusions
We can also exclude specific keywords using the argument `exclude`

```{r, echo=FALSE}
knitr::include_graphics("Figures/6.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  exclude = c("Osteoarthrosis", "knee"),
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

When multiple words are added within a term (e.g., "knee osteoarthritis"), each word
will be searched independently, so that for example, "osteoarthritis of knee" is excluded:
```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Musculoskeletal disorder",
  domains = "Condition",
  exclude = c("knee osteoarthritis"),
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

If we only want to exclude exact matching terms (without accounting for words boundaries) we need to add "/" at the beginning and
at the end of the term. Hence, using "knee osteoarthritis", "osteoarthritis of knee" **won't** be excluded. However, if we had 
"rightknee osteoarthritis", it would be excluded.
```{r, message=FALSE}
# No exclusion:
getCandidateCodes(
    cdm = cdm,
    keywords = "Knee",
    domains = "Condition",
    exclude = NULL,
    standardConcept = c("Standard", "Non-standard"),
    includeDescendants = TRUE,
    searchInSynonyms = FALSE,
    searchNonStandard = FALSE,
    includeAncestor = FALSE
)

# Exclusion looking for terms:
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("knee osteoarthritis"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# Exclusion looking for partial matching terms (without word boundaries)
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/knee osteoarthritis/"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# Exclusion looking for partial matching terms (without word boundaries)
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/e osteoarthritis/"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

If we want to do exact matching (that means, to find the exact two words "knee osteoarthritis" in the concept name) we need to use "/\b" at the
beginning and at the end of the expression.
```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/\bKnee osteoarthritis/\b"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)

# We will now only search for "ee osteoarthritis" to show that 
# "knee osteoarthritis" won't be excluded:
getCandidateCodes(
  cdm = cdm,
  keywords = "Knee",
  domains = "Condition",
  exclude = c("/\bee osteoarthritis/\b"),
  standardConcept = c("Standard", "Non-standard"),
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```
```

For example, if we look for 

Notice that, for example, if we wanted `keywords = "depression"` and `exclude = "ST depression"`, concepts like "poSTpartum depression" would be excluded. To avoid this,
we could use `exclude = "/ST depression/"`. Notice that, "poST depression" would also be excluded with this option.

Hence, there is another option to exclude exact matching terms accounting for words boundaries: adding "/\b" at the beginning and at the end of the term. For example, if we look for "/\bp osteoarthritis/\b", concepts like "hip osteoarthritis **won't** be excluded.



## Add ancestor
To include the ancestors one level above the identified concepts, we can use the argument `includeAncestor`
```{r, echo=FALSE}
knitr::include_graphics("Figures/7.png")
```

```{r, message=FALSE}
codes <- getCandidateCodes(
  cdm = cdm,
  keywords = "Osteoarthritis of knee",
  includeAncestor = TRUE,
  domains = "Condition",
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
)

codes
```

## Search using synonyms
We can also pick up codes based on their synonyms. For example, **Osteoarthrosis** has a synonym of **Arthritis**.
```{r, echo=FALSE}
knitr::include_graphics("Figures/8.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthrosis",
  domains = "Condition",
  searchInSynonyms = TRUE,
  standardConcept = "Standard",
  includeDescendants = FALSE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```

Notice that if `includeDescendants = TRUE`, **Arthritis** descendants will also be included:
```{r,echo=FALSE}
knitr::include_graphics("Figures/9.png")
```

```{r, message=FALSE}
getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthrosis",
  domains = "Condition",
  searchInSynonyms = TRUE,
  standardConcept = "Standard",
  includeDescendants = TRUE,
  searchNonStandard = FALSE,
  includeAncestor = FALSE
)
```


## Search via non-standard
We can also pick up concepts associated with our keyword via non-standard search.
```{r,echo=FALSE}
knitr::include_graphics("Figures/10.png")
```

```{r, message=FALSE}
codes1 <- getCandidateCodes(
  cdm = cdm,
  keywords = "Degenerative",
  domains = "Condition",
  standardConcept = "Standard",
  searchNonStandard = TRUE,
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  includeAncestor = FALSE
)
codes1
```

Let's take a moment to focus on the `standardConcept` and `searchNonStandard` arguments to clarify the difference between them. `standardConcept` specifies whether we want only standard concepts or also include non-standard concepts in the final candidate codelist. `searchNonStandard` determines whether we want to search for keywords among non-standard concepts.

In the previous example, since we set `standardConcept = "Standard"`, we retrieved the code for **Osteoarthrosis** from the non-standard search. However, we did not obtain the non-standard code **degenerative arthropathy** from the initial search. If we allow non-standard concepts in the final candidate codelist, we would retireve both codes:

```{r,echo=FALSE}
knitr::include_graphics("Figures/11.png")
```

```{r, message=FALSE}
codes2 <- getCandidateCodes(
  cdm = cdm,
  keywords = "Degenerative",
  domains = "Condition",
  standardConcept = c("Non-standard", "Standard"),
  searchNonStandard = FALSE,
  includeDescendants = FALSE,
  searchInSynonyms = FALSE,
  includeAncestor = FALSE
)
codes2
```

---
title: "Code lists for medications"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a04_codelists_for_medications}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

```{r,  message=FALSE, warning=FALSE,echo=FALSE}
library(DBI)
library(dplyr)
library(CodelistGenerator)
```

## Creating a codelists for acetaminophen
For this vignette we are going to create codelists for identifying a acetaminophen, looking in the drug domain.

## Connecting to a database with the OMOP CDM vocabularies
```{r, eval=FALSE}
# postgres database connection details
serverDbi <- Sys.getenv("server")
user <- Sys.getenv("user")
password <- Sys.getenv("password")
port <- Sys.getenv("port")
host <- Sys.getenv("host")

db <- DBI::dbConnect(RPostgres::Postgres(),
  dbname = serverDbi,
  port = port,
  host = host,
  user = user,
  password = password
)

# name of vocabulary schema
vocabularyDatabaseSchema <- "vocabulary"
```

## Codelist from "acetaminophen" and its descendants
First we can follow the approach of identifying high-level codes and include all their descendants.

```{r, eval=FALSE}
library(dplyr)
library(CodelistGenerator)
```

```{r, eval=FALSE}
acetaminophen1 <- getCandidateCodes(
  cdm = cdm,
  keywords = "acetaminophen",
  domains = "drug",
  standardConcept = "standard",
  includeDescendants = TRUE
)
```

```{r,  message=FALSE, warning=FALSE, echo=FALSE}
acetaminophen1 <- readRDS(system.file("medData01.RData", 
                                            package = "CodelistGenerator"))
```

```{r,  message=FALSE, warning=FALSE }  
acetaminophen1 %>% dplyr::glimpse()
```

---
title: "Introduction to CodelistGenerator"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{01_Introduction_to_CodelistGenerator}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

## Creating a code list for dementia
For this example we are going to generate a candidate codelist for dementia, only looking for codes in the condition domain. Let's first load some libraries

```{r,  message=FALSE, warning=FALSE,echo=FALSE}
library(DBI)
library(dplyr)
library(CodelistGenerator)
library(CDMConnector)
```

## Connect to the OMOP CDM vocabularies
CodelistGenerator works with a `cdm_reference` to the vocabularies tables of the OMOP CDM using the CDMConnector package. 

```{r, eval=FALSE}
# example with postgres database connection details
db <- DBI::dbConnect(RPostgres::Postgres(),
  dbname = Sys.getenv("server"),
  port = Sys.getenv("port"),
  host = Sys.getenv("host"),
  user = Sys.getenv("user"),
  password = Sys.getenv("password")
)

# create cdm reference
cdm <- CDMConnector::cdm_from_con(
  con = db,
  cdm_schema = Sys.getenv("vocabulary_schema")
)
```


## Check version of the vocabularies
It is important to note that the results from CodelistGenerator will be specific to a particular version of the OMOP CDM vocabularies. We can see the version of the vocabulary being used like so
```{r, eval=FALSE}
getVocabVersion(cdm = cdm)
```

```{r,  message=FALSE, warning=FALSE,echo=FALSE}
vocabVersion <- load(system.file("introVocab.RData", 
                 package = "CodelistGenerator"))
vocabVersion
```

## A code list from "Dementia" (4182210) and its descendants
The simplest approach to identifying potential codes is to take a high-level code and include all its descendants. 


```{r, eval=FALSE}
codesFromDescendants <- tbl(
  db,
  sql(paste0(
    "SELECT * FROM ",
    vocabularyDatabaseSchema,
    ".concept_ancestor"
  ))
) %>%
  filter(ancestor_concept_id == "4182210") %>%
  select("descendant_concept_id") %>%
  rename("concept_id" = "descendant_concept_id") %>%
  left_join(tbl(db, sql(paste0(
    "SELECT * FROM ",
    vocabularyDatabaseSchema,
    ".concept"
  )))) %>%
  select(
    "concept_id", "concept_name",
    "domain_id", "vocabulary_id"
  ) %>%
  collect()
```

```{r,  message=FALSE, warning=FALSE,echo=FALSE}
codesFromDescendants <- readRDS(system.file("introData01.RData", 
                                            package = "CodelistGenerator"))
```

```{r,  message=FALSE, warning=FALSE }  
codesFromDescendants %>% 
  glimpse()
```

This looks to pick up most relevant codes. But, this approach misses codes that are not a descendant of 4182210. For example, codes such as "Wandering due to dementia" (37312577; https://athena.ohdsi.org/search-terms/terms/37312577) and "Anxiety due to dementia" (37312031; https://athena.ohdsi.org/search-terms/terms/37312031) are not picked up.

## Generating a candidate code list using CodelistGenerator
To try and include all such terms that could be included we can use CodelistGenerator.

First, let's do a simple search for a single keyword of "dementia", including descendants of the identified codes.
```{r, eval=FALSE }
dementiaCodes1 <- getCandidateCodes(
  cdm = cdm,
  keywords = "dementia",
  domains = "Condition",
  includeDescendants = TRUE
)
```

```{r,  message=FALSE, warning=FALSE,echo=FALSE}
dementiaCodes1 <- readRDS(system.file("introData02.RData", 
                                            package = "CodelistGenerator"))
```

```{r,  message=FALSE, warning=FALSE }
dementiaCodes1%>% 
  glimpse()
```


## Comparing code lists
What is the difference between this code list and the one from 4182210 and its descendants?
```{r,  eval=FALSE }
codeComparison <- compareCodelists(
  codesFromDescendants,
  dementiaCodes1
)
```

```{r,  message=FALSE, warning=FALSE,echo=FALSE}
codeComparison <- readRDS(system.file("introData03.RData", 
                                            package = "CodelistGenerator"))
```

```{r,  message=FALSE, warning=FALSE } 
codeComparison %>%
  group_by(codelist) %>%
  tally()
```

What are these extra codes picked up by CodelistGenerator?
```{r,  message=FALSE, warning=FALSE }
codeComparison %>%
  filter(codelist == "Only codelist 2") %>% 
  glimpse()
```

## Review mappings from non-standard vocabularies
Perhaps we want to see what ICD10CM codes map to our candidate code list. We can get these by running

```{r,  message=FALSE, warning=FALSE,echo=FALSE}
icdMappings <- readRDS(system.file("introData04.RData", 
                                            package = "CodelistGenerator"))
```

```{r,  eval=FALSE }
icdMappings <- getMappings(
  cdm = cdm,
  candidateCodelist = dementiaCodes1,
  nonStandardVocabularies = "ICD10CM"
)
```

```{r,  message=FALSE, warning=FALSE }
icdMappings %>% 
  glimpse()
```


```{r,  message=FALSE, warning=FALSE,echo=FALSE}
readMappings <- readRDS(system.file("introData05.RData", 
                                            package = "CodelistGenerator"))
```

```{r,  eval=FALSE }
readMappings <- getMappings(
  cdm = cdm,
  candidateCodelist = dementiaCodes1,
  nonStandardVocabularies = "Read"
)
```

```{r,  message=FALSE, warning=FALSE }
readMappings %>% 
  glimpse()
```

---
title: "Exploring the OMOP CDM vocabulary tables"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a02_ExploreCDMvocabulary}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  chunk_output_type: console
---

```{r, include = FALSE}
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN
)
```

```{r, include = FALSE}
CDMConnector::requireEunomia("synpuf-1k", "5.3")
```

In this vignette, we will explore the functions that help us delve into the vocabularies used in our database. These functions allow us to explore the different vocabularies and concepts characteristics.

First of all, we will load the required packages and a eunomia database.
```{r, warning=FALSE, message=FALSE}
library(DBI)
library(dplyr)
library(CDMConnector)
library(CodelistGenerator)

# Connect to the database and create the cdm object
con <- dbConnect(duckdb::duckdb(), 
                 eunomiaDir("synpuf-1k", "5.3"))
cdm <- cdmFromCon(con = con, 
                  cdmName = "Eunomia Synpuf",
                  cdmSchema   = "main",
                  writeSchema = "main", 
                  achillesSchema = "main")
```

Note that we have included [achilles tables](https://github.com/OHDSI/Achilles) in our cdm reference, which are used for some of the analyses.

## Vocabulary characteristics

We can first start by getting the vocabulary version of our CDM object:
```{r}
vocabularyVersion(cdm)
```

And the available vocabularies, which correspond to the column *vocabulary_id* from the concept table:
```{r}
availableVocabularies(cdm)
```

## Domains

We can also explore the domains that our CDM object has, which is the column *domain_id* from the concept table:
```{r}
availableDomains(cdm)
```

or restrict the search among *standard* concepts:
```{r}
availableDomains(cdm,
                 standardConcept = "Standard")
```

## Concept class

We can further explore the different classes that we have (reported in *concept_class_id* column from the concept table):
```{r}
availableConceptClassIds(cdm)
```

Or restrict the search among *non-standard* concepts with *condition* domain:
```{r}
availableConceptClassIds(cdm, 
                         standardConcept = "Non-standard", 
                         domain = "Condition")
```

## Relationships

We can also explore the different relationships that are present in our CDM:
```{r}
availableRelationshipIds(cdm)
```

Or narrow the search among *standard* concepts with domain *observation*:
```{r}
availableRelationshipIds(cdm,
                         standardConcept1 = "Standard",
                         standardConcept2 = "Standard",
                         domains1 = "Observation",
                         domains2 = "Observation")
```

---
title: "Types of Codelist"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a03_TypesOfCodelist}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN
)
```

In this vignette we will introduce the three different types of codelist that CodelistGenerator uses: codelist, codelist with details, and concept set. First of all, we will upload the packages required for the demonstration.

```{r}
library(omopgenerics, warn.conflicts = FALSE)
library(dplyr, warn.conflicts = FALSE)
library(CodelistGenerator)
```

## Codelist

Codelist format is the simplest of the three formats. To create one, we just need a named list where each element contains a vector of concept IDs. You can see an example below on how to create one:

```{r}
codelist <- list("codes1" = c(1L, 2L, 3L),
                 "codes2" = c(4L, 5L, 10L))
codelist <- newCodelist(codelist)

codelist 
```

Notice that the codelist has two specific attributes, the name of the codelists that contains and the class:
```{r}
attributes(codelist)
```

### Create an empty codelist 

We can create an empty codelist using omopGenerics package:

```{r}
empty_codelist  <- emptyCodelist()

empty_codelist
```

## Codelist with details

Codelist with details contains a tibble in each element. To create one, we need a named list where each element contains a tibble with at least the column *concept_id*.

```{r}
codelist_with_details <- list("codes1" = tibble("concept_id" = c(1L, 2L, 3L)),
                              "codes2" = tibble("concept_id" = c(4L, 5L, 10L)))
codelist_with_details <- newCodelistWithDetails(codelist_with_details)

codelist_with_details
```

Notice that this allows us to add more information of each one of the codes:

```{r}
codelist_with_details <- list(
  "codes1" = tibble("concept_id" = c(1L, 2L, 3L),
                    "concept_name" = c("Musculoskeletal disorder", "Osteoarthrosis", "Arthritis"),
                    "domain_id" = c("Condition", "Condition", "Condition"),
                    "vocabulary_id" = c("SNOMED","SNOMED","SNOMED"),
                    "standard_concept" = c("S","S","S")),
  "codes2" = tibble("concept_id" = c(4L, 5L, 10L),
                    "concept_name" = c("Osteoarthritis of knee", "Osteoarthritis of hip", "Adalimumab"),
                    "domain_id" = c("Condition", "Condition", "Drug"),
                    "vocabulary_id" = c("SNOMED","SNOMED","RxNorm"),
                    "standard_concept" = c("S","S","S")))
codelist_with_details <- newCodelistWithDetails(codelist_with_details)

codelist_with_details
```

The codelist with details also has two specific attributes:
```{r}
attributes(codelist_with_details)
```

### Create an empty codelist with details

omopGenerics package also has a function to create an empty codelist with details.

```{r}
empty_codelist_with_details <- emptyCodelistWithDetails()

empty_codelist_with_details
```

## Concept set expression

A concept set expression contains a tibble with the logic describing how this concepts will be included or excluded. It usually follows the JSON file structure. To create one, we need a named list with a tibble containing the columns: *concept_id*, *excluded*, *descendants*, and *mapped*:

```{r}
concept_set_expression <- list(
  "codes1" = tibble("concept_id" = c(1L, 2L, 3L),
                    "excluded" = c(FALSE, FALSE, FALSE),
                    "descendants" = c(TRUE, FALSE, FALSE),
                    "mapped" = c(TRUE, TRUE, TRUE)),
  "codes2" = tibble("concept_id" = c(4L, 5L, 10L),
                    "excluded" = c(FALSE, FALSE, FALSE),
                    "descendants" = c(FALSE, FALSE, FALSE),
                    "mapped" = c(TRUE, TRUE, TRUE))
)

concept_set_expression <- newConceptSetExpression(concept_set_expression)

concept_set_expression
```

You can also see the attributes:
```{r}
attributes(concept_set_expression)
```

### Create an empty concept set expression

We can also use omopGenerics package to create an empty concept set expression:

```{r}
empty_concept_set_expression <- emptyConceptSetExpression()

empty_concept_set_expression
```

### Concept set expression from a JSON file

CodelistGenerator contains a function that allows us to read a JSON file and transform it to a concept set expression (or other types of codelists). See the vignette **Extract codelists from JSON files** for further information.

## Change to different codelist types

There are several functions in CodelistGenerator that will allow us to go from a codelist to a codelist with details.

### Codelist/Concept set expression to codelist with details

Notice that to go from a codelist or concept_set_expression to a codelist_with_details, we will need to provide the cdm, as the details of the concepts will be extracted from the *concept* table.

```{r}
cdm <- mockVocabRef()
codelist_to_codelist_with_details <- asCodelistWithDetails(codelist, cdm)
codelist_to_codelist_with_details

codelist_to_concept_set_expression <- asCodelistWithDetails(codelist, cdm)
codelist_to_concept_set_expression

```

### Codelist/Codelist with details to concept set expression

```{r}

codelist_to_concept_set_expression <- asConceptSetExpression(codelist)
codelist_to_concept_set_expression

codelist_with_details_to_concept_set_expression <- asConceptSetExpression(codelist_with_details)
codelist_with_details_to_concept_set_expression
```
### Codelist with details/Concept set expression to codelist
```{r}
codelist_with_details_to_codelist <- asCodelist(codelist_with_details)
codelist_with_details_to_codelist

```

---
title: "Manipulate codelists"
output: rmarkdown::html_vignette
vignette: >
 %\VignetteIndexEntry{a08_ManipulateCodelists}
 %\VignetteEngine{knitr::rmarkdown}
 %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
  NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
  
  knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN)
```
  
```{r, include = FALSE}
  CDMConnector::requireEunomia("synpuf-1k", "5.3")
```
  
  ## Introduction: Manipulate codelists
  This vignette introduces a set of functions designed to manipulate and explore codelists within an OMOP CDM. Specifically, we will learn how to:
  
  -   **Subset a codelist** to keep only codes meeting a certain criteria.
  -   **Stratify a codelist** based on attributes like dose unit or route of administration.
  -   **Add or exclude** concepts from a codelists.
  -   **Compare two codelists** to identify shared and unique concepts.
  
  First of all, we will load the required packages and connect to a mock database.
```{r, warning=FALSE, message=FALSE}
library(DBI)
library(duckdb)
library(dplyr)
library(CDMConnector)
library(CodelistGenerator)

# Download mock database
requireEunomia(datasetName = "synpuf-1k", cdmVersion = "5.3")

# Connect to the database and create the cdm object
con <- dbConnect(duckdb(), eunomiaDir("synpuf-1k", "5.3"))
cdm <- cdmFromCon(con = con, 
cdmName = "Eunomia Synpuf",
cdmSchema   = "main",
writeSchema = "main", 
achillesSchema = "main")
```

We will start by generating a codelist for *acetaminophen* using `getDrugIngredientCodes()`
```{r, warning=FALSE, message=FALSE}
acetaminophen <- getDrugIngredientCodes(cdm,
                                        name = "acetaminophen",
                                        nameStyle = "{concept_name}",
                                        type = "codelist")
```

### Subsetting a Codelist
Subsetting a codelist will allow us to reduce a codelist to only those concepts that meet certain conditions.

#### Subset by Domain
We will now subset to those concepts that have `domain = "Drug"`. Remember that, to see the domains available in your codelist, you can use `associatedDomains()`.
```{r, warning=FALSE, messages=FALSE}
acetaminophen_drug <- subsetOnDomain(acetaminophen, 
                                     cdm, 
                                     domain = "Drug")

acetaminophen_drug
```
We can use the `negate` argument to exclude concepts with a certain domain:

```{r, warning=FALSE, messages=FALSE}
acetaminophen_no_drug <- subsetOnDomain(acetaminophen, 
                                        cdm, 
                                        domain = "Drug", 
                                        negate = TRUE)

acetaminophen_no_drug
```

#### Subset on vocabulary
We will now subset the codelist to only include concepts from RxNorm vocabulary. You can also use `associatedVocabularies()` to explore the vocabularies available in your codelist.
```{r, warning=FALSE, messages=FALSE}
acetaminophen_rxnorm <- subsetOnVocabulary(acetaminophen_drug, 
                                           cdm, 
                                           c("RxNorm"))
acetaminophen_rxnorm
```

#### Subset on Dose Unit
We will now filter to only include concepts with specified dose units. Remember that you can use `associatedDoseUnits()` to explore the dose units available in your codelist.
```{r, warning=FALSE, messages=FALSE}
acetaminophen_mg_unit <- subsetOnDoseUnit(acetaminophen_rxnorm, 
                                          cdm, 
                                          c("milligram", "unit"))
acetaminophen_mg_unit
```
As before, we can use argument `negate = TRUE` to exclude instead.

#### Subset on ingredient range
We can now subset on those drugs with 3 to 30 ingredients:
```{r, warning=FALSE, messages=FALSE}
acetaminophen_ingredient <- subsetOnIngredientRange(acetaminophen_drug, 
                                                cdm,
                                                ingredientRange = c(3, 30))

acetaminophen_ingredient
```
Notice that `negate = TRUE` would keep all those concepts with less than 3 ingredients or more than 30 (without including those with 3 or 30 ingredients).

#### Subset on route category
We will now subset to those concepts that do not have an "unclassified_route" or "transmucosal_rectal". See `associatedRouteCategories()` to explore route categories available in your codelist. 
```{r, warning=FALSE, messages=FALSE}
acetaminophen_route <- subsetOnRouteCategory(acetaminophen_mg_unit, 
                                             cdm, 
                                             c("transmucosal_rectal","unclassified_route"), 
                                             negate = TRUE)
acetaminophen_route
```

#### Subset on dose forms
We will now subset to those concepts with specific dose forms. See `associatedDseForms()` to explore dose forms available in your codelist. 
```{r, warning=FALSE, messages=FALSE}
# First, check which dose forms are available in our codelist
acetaminophen_drug |> 
  associatedDoseForms(cdm)
acetaminophen_oral <- subsetOnDoseForm(acetaminophen_drug, 
                                            cdm, 
                                            c("Oral Solution","Oral Capsule"))
acetaminophen_oral 
```

### Stratify codelist
Instead of filtering, stratification allows us to split a codelist into subgroups based on defined vocabulary properties.

#### Stratify by Dose Unit
```{r, warning=FALSE, messages=FALSE}
acetaminophen_doses <- stratifyByDoseUnit(acetaminophen, cdm, keepOriginal = TRUE)

acetaminophen_doses
```

#### Stratify by Route Category
```{r, warning=FALSE, messages=FALSE}
acetaminophen_routes <- stratifyByRouteCategory(acetaminophen, cdm)

acetaminophen_routes
```

#### Stratify by Dose Form
```{r, warning=FALSE, messages=FALSE}
acetaminophen_dose_forms <- stratifyByDoseForm(acetaminophen, cdm)

acetaminophen_dose_forms
```

## Add or remove concepts from a codelist
We can also add specific concepts to our codelist. For example, we will add the
ingredient "acetaminophen" to all our codelists:
```{r, warning=FALSE, messages=FALSE}
acetaminophen_routes1 <- addConcepts(acetaminophen_routes, 
                                     cdm,
                                     concepts = c(1125315L))
acetaminophen_routes1
```

Or we can add acetaminophen + descendants, and only to some of the codelists
```{r, warning=FALSE, messages=FALSE}
x <- getDescendants(cdm = cdm, conceptId = c(1125315L))
acetaminophen_routes2 <- addConcepts(acetaminophen_routes, 
                                     cdm,
                                     concepts = x$concept_id, 
                                     codelistName = "acetaminophen_unclassified_route_category")
acetaminophen_routes2
```

And similarly, we can exclude specific concepts and their descendants from our codelist:
```{r, warning=FALSE, messages=FALSE}
acetaminophen_routes3 <- excludeConcepts(acetaminophen_routes, 
                                         cdm,
                                         concepts = x$concept_id, 
                                         codelistName = "acetaminophen_inhalable")
acetaminophen_routes3
```

Notice that in this case, the codelist "acetaminophen_inhalable" is removed as there are no elements left after the exclusion of the code 35873016 and its descendants.

### Codelist construction
Notice that all the functions introduced previously are "pipeble", allowing for a tidy and clear codelist construction:
```{r, warning=FALSE, messages=FALSE}
acetaminophen <- getDrugIngredientCodes(cdm,
                                        name = "acetaminophen",
                                        nameStyle = "{concept_name}",
                                        type = "codelist")

new_codelist <- acetaminophen |>
  addConcepts(cdm, 
              concepts = c(1L, 2L, 3L)) |>
  subsetOnDomain(cdm,
                 domain = "Drug") |>
  stratifyByDoseUnit(cdm = cdm) |>
  excludeConcepts(cdm,
                  concepts = c(1127898)) 

new_codelist
```

### Compare codelists
Now we will compare two codelists to identify overlapping and unique codes. 
```{r, warning=FALSE, messages=FALSE}
acetaminophen <- getDrugIngredientCodes(cdm, 
                                        name = "acetaminophen", 
                                        nameStyle = "{concept_name}",
                                        type = "codelist_with_details")
hydrocodone <- getDrugIngredientCodes(cdm, 
                                      name = "hydrocodone", 
                                      doseUnit = "milligram", 
                                      nameStyle = "{concept_name}",
                                      type = "codelist_with_details")
```

Compare the two sets:
```{r}
comparison <- compareCodelists(acetaminophen,
                               hydrocodone)

comparison |> glimpse()

comparison |> filter(codelist == "Both")
```

---
title: "Codelist diagnostics"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a09_RunCodelistDiagnostics}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")

knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN)
```

```{r, include = FALSE}
CDMConnector::requireEunomia("synpuf-1k", "5.3")
```

This vignette presents a set of functions to explore the use of codes in a codelist. We will cover the following key functions:

-   `summariseAchillesCodeUse()`: Summarises the code use using ACHILLES tables.
-   `summariseCodeUse()`: Summarises the code use in patient-level data.
-   `summariseOrphanCodes()`: Identifies orphan codes related to a codelist using ACHILLES tables.
-   `summariseCohortCodeUse()`: Evaluates codelist usage within a cohort.

Let's start by loading the required packages, connecting to a mock database, and generating a codelist for example purposes. We'll use `getCandidateCodes()` to find our codes.

```{r, message=FALSE, warning=FALSE}
library(DBI)
library(duckdb)
library(dplyr)
library(CDMConnector)
library(CodelistGenerator)
library(CohortConstructor)
library(omopgenerics)

# Connect to the database and create the cdm object
con <- dbConnect(duckdb(), 
                      eunomiaDir("synpuf-1k", "5.3"))
cdm <- cdmFromCon(con = con, 
                  cdmName = "Eunomia Synpuf",
                  cdmSchema   = "main",
                  writeSchema = "main", 
                  achillesSchema = "main")

# Create a codelist for depression
depression <- getCandidateCodes(cdm,
                                keywords = "depression")
depression <- newCodelist(list("depression" = depression$concept_id))
```

## Running diagnostics for a codelist

### Summarise code use using ACHILLES tables

This function uses ACHILLES summary tables to count the number of records and persons associated with each concept in a codelist. Notice that it requires that ACHILLES tables are available in the CDM.

```{r, message=FALSE, warning=FALSE}
achilles_code_use <- summariseAchillesCodeUse(depression, 
                                              cdm, 
                                              countBy = c("record", "person"))
```

From this, we will obtain a [summarised result](https://darwin-eu.github.io/omopgenerics/articles/summarised_result.html) object. We can easily visualise the results using `tableAchillesCodeUse()`:

```{r, message=FALSE, warning=FALSE}
tableAchillesCodeUse(achilles_code_use,
                     type = "gt")
```

Notice that concepts with zero counts will not appear in the result table.

### Summarise code use using patient-level data

This function performs a similar task as above but directly queries patient-level data, making it usable even if ACHILLES tables are not available. It can be configured to stratify results by concept (`byConcept`), by year (`byYear`), by sex (`bySex`), or by age group (`byAgeGroup`). We can further specify a specific time period (`dateRange`).

```{r, message=FALSE, warning=FALSE}
code_use <- summariseCodeUse(depression,
                             cdm,
                             countBy = c("record", "person"),
                             byYear  = FALSE,
                             bySex   = FALSE,
                             ageGroup =  list("<=50" = c(0,50), ">50" = c(51,Inf)),
                             dateRange = as.Date(c("2010-01-01", "2020-01-01")))

tableCodeUse(code_use, type = "gt")
```

### Identify orphan codes

Orphan codes are concepts that might be related to our codelist but that have not been included. It can be used to ensure that we have not missed any important concepts. Notice that this function uses ACHILLES tables.

`summariseOrphanCodes()` will look for descendants (via *concept_descendants* table), ancestors (via *concept_ancestor* table), and concepts related to the codes included in the codelist (via *concept_relationship* table). Additionally, if the cdm contains PHOEBE tables (*concept_recommended* table), they will also be used.

```{r, message=FALSE, warning=FALSE}
orphan <- summariseOrphanCodes(depression, cdm)
tableOrphanCodes(orphan, type = "gt")
```

## Run diagnostics within a cohort

You can also evaluate how the codelist is used within a specific cohort. First, we will define a cohort using the `conceptCohort()` function from CohortConstructor package.

```{r, message=FALSE, warning=FALSE}
cdm[["depression"]] <- conceptCohort(cdm, 
                                     conceptSet = depression, 
                                     name = "depression")
```

Then, we can summarise the code use within this cohort:

```{r, message=FALSE, warning=FALSE}
cohort_code_use <- summariseCohortCodeUse(cdm,
                                          cohortTable = "depression",
                                          countBy = c("record", "person"))
tableCohortCodeUse(cohort_code_use)
```

### Summarise code use at cohort entry

Use the `timing` argument to restrict diagnostics to codes used at the entry date of the cohort.

```{r, message=FALSE, warning=FALSE}
cohort_code_use <- summariseCohortCodeUse(cdm,
                                          cohortTable = "depression",
                                          countBy = c("record", "person"),
                                          timing = "entry")
tableCohortCodeUse(cohort_code_use)
```

### Cohort code use with a different codelist

By default we'll get cohort code use for the codes that were used for creating the cohort. But we could change this to another cohort. Here we'd get counts for anxiety codes that occur on the same day as entry into the depression cohort.

```{r, message=FALSE, warning=FALSE}
anxiety <- getCandidateCodes(cdm,
                             keywords = "anxiety")
anxiety <- newCodelist(list("anxiety" = anxiety$concept_id))

cohort_code_use <- summariseCohortCodeUse(cdm,
                                          cohortTable = "depression",
                                          x = anxiety,
                                          countBy = c("record", "person"),
                                          timing = "entry")
tableCohortCodeUse(cohort_code_use)
```

### Stratify cohort code use

You can also stratify cohort code use results by year (`byYear`), by sex (`bySex`), or by age group (`byAgeGroup`):

```{r, message=FALSE, warning=FALSE}
cohort_code_use <- summariseCohortCodeUse(cdm = cdm,
                                          cohortTable = "depression",
                                          countBy = c("record", "person"),
                                          byYear = FALSE,
                                          bySex = TRUE,
                                          ageGroup = NULL)
tableCohortCodeUse(cohort_code_use)
```

---
title: "Types of Codelist"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{a03_TypesOfCodelist}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
NOT_CRAN <- identical(tolower(Sys.getenv("NOT_CRAN")), "true")
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = NOT_CRAN
)
```

In this vignette we will introduce the three different types of codelist that CodelistGenerator uses: codelist, codelist with details, and concept set. First of all, we will upload the packages required for the demonstration.

```{r}
library(omopgenerics, warn.conflicts = FALSE)
library(dplyr, warn.conflicts = FALSE)
library(CodelistGenerator)
```

## Codelist

Codelist format is the simplest of the three formats. To create one, we just need a named list where each element contains a vector of concept IDs. You can see an example below on how to create one:

```{r}
codelist <- list("codes1" = c(1L, 2L, 3L),
                 "codes2" = c(4L, 5L, 10L))
codelist <- newCodelist(codelist)

codelist 
```

Notice that the codelist has two specific attributes, the name of the codelists that contains and the class:
```{r}
attributes(codelist)
```

### Create an empty codelist 

We can create an empty codelist using omopgenerics package:

```{r}
empty_codelist  <- emptyCodelist()

empty_codelist
```

## Codelist with details

Codelist with details contains a tibble in each element. To create one, we need a named list where each element contains a tibble with at least the column *concept_id*.

```{r}
codelist_with_details <- list("codes1" = tibble("concept_id" = c(1L, 2L, 3L)),
                              "codes2" = tibble("concept_id" = c(4L, 5L, 10L)))
codelist_with_details <- newCodelistWithDetails(codelist_with_details)

codelist_with_details
```

Notice that this allows us to add more information of each one of the codes:

```{r}
codelist_with_details <- list(
  "codes1" = tibble("concept_id" = c(1L, 2L, 3L),
                    "concept_name" = c("Musculoskeletal disorder", "Osteoarthrosis", "Arthritis"),
                    "domain_id" = c("Condition", "Condition", "Condition"),
                    "vocabulary_id" = c("SNOMED","SNOMED","SNOMED"),
                    "standard_concept" = c("S","S","S")),
  "codes2" = tibble("concept_id" = c(4L, 5L, 10L),
                    "concept_name" = c("Osteoarthritis of knee", "Osteoarthritis of hip", "Adalimumab"),
                    "domain_id" = c("Condition", "Condition", "Drug"),
                    "vocabulary_id" = c("SNOMED","SNOMED","RxNorm"),
                    "standard_concept" = c("S","S","S")))
codelist_with_details <- newCodelistWithDetails(codelist_with_details)

codelist_with_details
```

The codelist with details also has two specific attributes:
```{r}
attributes(codelist_with_details)
```

### Create an empty codelist with details

omopgenerics package also has a function to create an empty codelist with details.

```{r}
empty_codelist_with_details <- emptyCodelistWithDetails()

empty_codelist_with_details
```

## Concept set expression

A concept set expression contains a tibble with the logic describing how this concepts will be included or excluded. It usually follows the JSON file structure. To create one, we need a named list with a tibble containing the columns: *concept_id*, *excluded*, *descendants*, and *mapped*:

```{r}
concept_set_expression <- list(
  "codes1" = tibble("concept_id" = c(1L, 2L, 3L),
                    "excluded" = c(FALSE, FALSE, FALSE),
                    "descendants" = c(TRUE, FALSE, FALSE),
                    "mapped" = c(TRUE, TRUE, TRUE)),
  "codes2" = tibble("concept_id" = c(4L, 5L, 10L),
                    "excluded" = c(FALSE, FALSE, FALSE),
                    "descendants" = c(FALSE, FALSE, FALSE),
                    "mapped" = c(TRUE, TRUE, TRUE))
)

concept_set_expression <- newConceptSetExpression(concept_set_expression)

concept_set_expression
```

You can also see the attributes:
```{r}
attributes(concept_set_expression)
```

### Create an empty concept set expression

We can also use omopgenerics package to create an empty concept set expression:

```{r}
empty_concept_set_expression <- emptyConceptSetExpression()

empty_concept_set_expression
```

### Concept set expression from a JSON file

CodelistGenerator contains a function that allows us to read a JSON file and transform it to a concept set expression (or other types of codelists). See the vignette **Extract codelists from JSON files** for further information.

## Change to different codelist types

There are several functions in CodelistGenerator that will allow us to go from a codelist to a codelist with details.

### Codelist/Concept set expression to codelist with details

Notice that to go from a codelist or concept_set_expression to a codelist_with_details, we will need to provide the cdm, as the details of the concepts will be extracted from the *concept* table.

```{r}
cdm <- mockVocabRef()
codelist_to_codelist_with_details <- asCodelistWithDetails(codelist, cdm)
codelist_to_codelist_with_details

codelist_to_concept_set_expression <- asCodelistWithDetails(codelist, cdm)
codelist_to_concept_set_expression

```

### Codelist/Codelist with details to concept set expression

```{r}

codelist_to_concept_set_expression <- asConceptSetExpression(codelist)
codelist_to_concept_set_expression

codelist_with_details_to_concept_set_expression <- asConceptSetExpression(codelist_with_details)
codelist_with_details_to_concept_set_expression
```
### Codelist with details/Concept set expression to codelist
```{r}
codelist_with_details_to_codelist <- asCodelist(codelist_with_details)
codelist_with_details_to_codelist

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{domainDoc}
\alias{domainDoc}
\title{Helper for consistent documentation of \code{domain}.}
\arguments{
\item{domain}{Character vector with one or more of the OMOP CDM domains.
The results will be restricted to the given domains. Check the available
domains in the database by running 'availableDomains()', or 'associatedDomains()'
to explore the domains associated with a codelist. If NULL, all supported domains
are included: Condition, Drug, Procedure, Device, Observation, and Measurement.}
}
\description{
Helper for consistent documentation of \code{domain}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{.optionsDoc}
\alias{.optionsDoc}
\title{Helper for consistent documentation of \code{.options}.}
\arguments{
\item{.options}{Named list with additional formatting options.
visOmopResults::tableOptions() shows allowed arguments and
their default values.}
}
\description{
Helper for consistent documentation of \code{.options}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vocabUtilities.R
\name{vocabularyVersion}
\alias{vocabularyVersion}
\title{Get the available version of the vocabulary used in the cdm}
\usage{
vocabularyVersion(cdm)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}
}
\value{
The vocabulary version being used in the cdm.
}
\description{
Get the available version of the vocabulary used in the cdm
}
\examples{
\donttest{
library(CodelistGenerator)
cdm <- mockVocabRef()
vocabularyVersion(cdm = cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableDoseUnits.R
\name{availableDoseUnits}
\alias{availableDoseUnits}
\title{Get available dose units}
\usage{
availableDoseUnits(cdm, standardConcept = "Standard")
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{standardConcept}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}
}
\value{
A character vector with available routes.
}
\description{
Get the available dose units
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all dose units available in the CDM
availableDoseUnits(cdm = cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetOnDoseForm.R
\name{subsetOnDoseForm}
\alias{subsetOnDoseForm}
\title{Subset a codelist to only those codes from a particular domain.}
\usage{
subsetOnDoseForm(x, cdm, doseForm, negate = FALSE)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{doseForm}{Dose form/s. See \code{availableDoseForms()} to explore available dose forms
in your codelist.}

\item{negate}{If FALSE, only concepts with the dose form specified will be
returned. If TRUE, concepts with the dose form specified will be excluded.}
}
\value{
The codelist with only those concepts associated with the dose form
(if negate = FALSE) or the codelist without those concepts associated with
the dose form (if negate = TRUE).
}
\description{
Subset a codelist to only those codes from a particular domain.
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef()

codelist <- newCodelist(list("codes" = c(10L,20L,21L)))

# Dose forms present in our codelist:
codelist |> associatedDoseForms(cdm)

codes <- subsetOnDoseForm(
              x = codelist,
              cdm = cdm,
              doseForm = "Injection")
codes

codes |> associatedDoseForms(cdm)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{levelICD10Doc}
\alias{levelICD10Doc}
\title{Helper for consistent documentation of \code{level}.}
\arguments{
\item{level}{Can be either "ICD10 Chapter", "ICD10 SubChapter",
"ICD10 Hierarchy", or "ICD10 Code".}
}
\description{
Helper for consistent documentation of \code{level}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{ingredientRangeDoc}
\alias{ingredientRangeDoc}
\title{Helper for consistent documentation of \code{ingredientRange}.}
\arguments{
\item{ingredientRange}{Used to restrict descendant codes to those
associated with a specific number of drug ingredients. Must be a vector of
length two with the first element the minimum number of ingredients allowed
and the second the maximum. A value of c(2, 2) would restrict to only
concepts associated with two ingredients.}
}
\description{
Helper for consistent documentation of \code{ingredientRange}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{groupColumnStrataDoc}
\alias{groupColumnStrataDoc}
\title{Helper for consistent documentation of \code{groupColumn}.}
\arguments{
\item{groupColumn}{Variables to use as group labels. Allowed columns are:
"cdm_name", "codelist_name", "standard_concept_name", "standard_concept_id",
"estimate_name", "source_concept_name", "source_concept_id", "domain_id". If
results are stratified, "year", "sex", "age_group" can also be used.
These cannot be used in header.}
}
\description{
Helper for consistent documentation of \code{groupColumn}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{headerDoc}
\alias{headerDoc}
\title{Helper for consistent documentation of \code{header}.}
\arguments{
\item{header}{A vector specifying the elements to include in the header. The
order of elements matters, with the first being the topmost header.
The header vector can contain one of the following variables: "cdm_name",
"codelist_name", "domain_id", "standard_concept_name", "standard_concept_id",
"estimate_name", "standard_concept", "vocabulary_id".
Alternatively, it can include other names to use as overall header labels.}
}
\description{
Helper for consistent documentation of \code{header}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCandidateCodes.R
\name{getCandidateCodes}
\alias{getCandidateCodes}
\title{Perform a systematic search to identify a candidate codelist using the OMOP
CDM vocabulary tables.}
\usage{
getCandidateCodes(
  cdm,
  keywords,
  exclude = NULL,
  domains = "Condition",
  vocabularyId = NULL,
  standardConcept = "Standard",
  searchInSynonyms = FALSE,
  searchNonStandard = FALSE,
  includeDescendants = TRUE,
  includeAncestor = FALSE
)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{keywords}{Character vector of terms to search for.
\itemize{
\item The search performed is broad, matching the provided keywords as substrings
anywhere within concept names. For example, \code{keywords = c("sep")} will
find "sepsis", "aseptic necrosis", and "acquired cardiac septal defect"
(along with many more).
\item Where more than one word is given, all combinations of those words will be
identified. For example, \code{keywords = c("knee osteoarthritis")} will identify
a concept with the name "osteoarthritis of knee".
\item Multiple keywords can be provided. For example,
\code{keywords = c("knee osteoarthritis", "hip osteoarthritis")} would identify
both "osteoarthritis of knee" and "osteoarthritis of hip"
}}

\item{exclude}{Character vector of words to identify concepts to exclude.
For example, \code{getCandidateCodes(cdm,  keywords = "septic", exclude = "aseptic",  domains = "condition")}
would remove concepts "aseptic" when searching for concepts with "septic" in
their name.
\itemize{
\item When one term contains multiple words (e.g., "knee osteoarthritis"),
each word will be search individually, so that "osteoarthritis of knee" would also be excluded.
If you only want to exclude partial matching terms, please add "/" at the beginning and the end of each term
(e.g., \verb{"/knee osteoarthritis/}"). Notice that, with this options,
concepts like "right knee osteoarthritis" will also be excluded (as this is a partial match), but
"osteoarthritis of knee" won't be excluded. Different terms can have different rules (e.g.,
c("hip osteoarthritis", "/knee osteoarthritis/")).
\item With multiple words, if we want exact matches accounting for word boundaries, we need to use
\verb{/\\b} at the beginning and at the end of each expression. In the previous example, using
\code{"/bknee osteoarthritis/\\b"}, "right knee osteoarthritis" won't be excluded, but
"History of knee osteoarthritis" will be excluded.
}}

\item{domains}{Character vector with one or more of the OMOP CDM domain for
which to search within. If NULL, all domains are included in the search. Use
\code{availableDomains(cdm = cdm)} to identify available domains to search within.}

\item{vocabularyId}{Character vector with one or more vocabulary ID to
restrict results to. If NULL, all vocabularies will be included in the
search. Use \code{availableVocabularies(cdm)} to identify available vocabularies
to restrict search to.}

\item{standardConcept}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}

\item{searchInSynonyms}{Either TRUE or FALSE. If TRUE the code will also
search using both the primary name in the concept table and synonyms from
the concept synonym table.}

\item{searchNonStandard}{Either TRUE or FALSE. If TRUE the code will also
search via non-standard concepts.}

\item{includeDescendants}{Either TRUE or FALSE. If TRUE descendant concepts
of identified concepts will be included in the candidate codelist. If FALSE
only direct mappings from ICD-10 codes to standard codes will be returned.}

\item{includeAncestor}{Either TRUE or FALSE.
If TRUE the direct ancestor concepts of identified concepts
will be included in the candidate codelist.}
}
\value{
A "candidate_codes" object. This includes a tibble with the potential
codes of interest, along with an attribute containing the search strategy.
}
\description{
Based on the given search strategy, this function identifies a set of codes
that may represent a clinical event of interest in data mapped to the OMOP
CDM. These codes can then be considered for creating a study phenotype.
}
\examples{
\donttest{
library(CodelistGenerator)
cdm <- mockVocabRef()
getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthritis"
 )
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{xDocCohort}
\alias{xDocCohort}
\title{Helper for consistent documentation of \code{x} where input can be codelist or cohort.}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details. If set to NULL, the codelist associated with the cohort will be extracted}
}
\description{
Helper for consistent documentation of \code{x} where input can be codelist or cohort.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableRelationshipIds.R
\name{associatedRelationshipIds}
\alias{associatedRelationshipIds}
\title{Get all relationships types that exist in the OMOP vocabulary \code{concept_relationship}
table for a given set of concepts in a codelist.}
\usage{
associatedRelationshipIds(
  x,
  cdm,
  standardConcept1 = c("Standard", "Non-standard", "Classification"),
  standardConcept2 = c("Standard", "Non-standard", "Classification"),
  domains1 = NULL,
  domains2 = NULL
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{standardConcept1}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}

\item{standardConcept2}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}

\item{domains1}{Character vector with one or more of the OMOP CDM domain. If NULL, all domains are considered.}

\item{domains2}{Character vector with one or more of the OMOP CDM domain. If NULL, all domains are considered.}
}
\value{
A character vector with unique concept relationship values.
}
\description{
Get all relationships types that exist in the OMOP vocabulary \code{concept_relationship}
table for a given set of concepts in a codelist.
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Create codelist
codelist <- newCodelist(list("codes1" = c(8479L, 4117795L, 44022939L),
                             "codes2" = c(8480L, 8600L, 8481L, 4189167L, 40371897L)))

# You can optionally restrict to only relationships between concepts that are
# "Standard" and "Non-standard". For example:
relationships <- associatedRelationshipIds(x = codelist,
                                           cdm = cdm,
                                           standardConcept1 = "Standard",
                                           standardConcept2 = "Non-standard")
relationships

# It returns the relationships between concepts where:
#  - concept_id_1 is 'Standard'
#  - concept_id_2 is 'Non-standard'
# Similarly, we can obtain the relationships restricting by domain:
relationships <- associatedRelationshipIds(x = codelist,
                                           cdm = cdm,
                                           domains1 = c("Drug", "Condition"),
                                           domains2 = c("Drug", "Condition"))
relationships
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asCodelistWithDetails.R
\name{asCodelistWithDetails}
\alias{asCodelistWithDetails}
\alias{asCodelistWithDetails.codelist_with_details}
\alias{asCodelistWithDetails.codelist}
\alias{asCodelistWithDetails.candidate_codes}
\title{Coerce to a codelist with details}
\usage{
asCodelistWithDetails(x, cdm, ...)

\method{asCodelistWithDetails}{codelist_with_details}(x, ...)

\method{asCodelistWithDetails}{codelist}(x, cdm, ...)

\method{asCodelistWithDetails}{candidate_codes}(x, cdm, ...)
}
\arguments{
\item{x}{A codelist, codelist_with_details, concept_set_expression, or a candidate_codes.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{...}{For extensibility}
}
\value{
codelist
}
\description{
Coerce to a codelist with details
}
\examples{
\donttest{
library(omock)
library(CDMConnector)

# Creating CDM object
path <- downloadMockDataset(datasetName = "GiBleed",
                            path = NULL,
                            overwrite = NULL)
cdm <- mockCdmFromDataset(datasetName = "GiBleed")

# Create codelist_with_details from a codelist
codelist <- getDrugIngredientCodes(cdm,
                                   name = "acetaminophen",
                                   nameStyle = "{concept_name}",
                                   type = "codelist")

asCodelistWithDetails(codelist, cdm)

# Create codelist from a candidate_codes
codelist <- getCandidateCodes(cdm,
                             keywords = "arthritis")

asCodelistWithDetails(codelist)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMappings.R
\name{getMappings}
\alias{getMappings}
\title{Show mappings from non-standard vocabularies to standard.}
\usage{
getMappings(
  candidateCodelist,
  cdm = NULL,
  nonStandardVocabularies = c("ATC", "ICD10CM", "ICD10PCS", "ICD9CM", "ICD9Proc",
    "LOINC", "OPCS4", "Read", "RxNorm", "RxNorm Extension", "SNOMED")
)
}
\arguments{
\item{candidateCodelist}{Dataframe.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{nonStandardVocabularies}{Character vector.}
}
\value{
Tibble with the information of potential standard to non-standard
mappings for the codelist of interest.
}
\description{
Show mappings from non-standard vocabularies to standard.
}
\examples{
\donttest{
cdm <- CodelistGenerator::mockVocabRef()
codes <- CodelistGenerator::getCandidateCodes(
  cdm = cdm,
  keywords = "osteoarthritis"
)
CodelistGenerator::getMappings(
  cdm = cdm,
  candidateCodelist = codes,
  nonStandardVocabularies = "READ"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableConceptClassIds.R
\name{associatedConceptClassIds}
\alias{associatedConceptClassIds}
\title{Get the concept classes associated with a codelist}
\usage{
associatedConceptClassIds(x, cdm, standardConcept = "Standard", domain = NULL)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{standardConcept}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}

\item{domain}{Character vector with one or more of the OMOP CDM domains.
The results will be restricted to the given domains. Check the available
domains in the database by running 'availableDomains()', or 'associatedDomains()'
to explore the domains associated with a codelist. If NULL, all supported domains
are included: Condition, Drug, Procedure, Device, Observation, and Measurement.}
}
\value{
The concept classes
}
\description{
Get the concept classes associated with a codelist
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmFromDataset(datasetName = "GiBleed")

# Get concept_class_ids in a codelist
x <- newCodelist(list("codes1" = c(1118088L, 40213201L, 35208414L),
                      "codes2" = c(1557272L, 4336464L, 4295880L)))
associatedConceptClassIds(x,
                          cdm,
                          standardConcept = "Standard")

# Notice that this corresponds to the information provided by `concept_class_id`
# column in the `concept` table
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{tableStyleDoc}
\alias{tableStyleDoc}
\title{Helper for consistent documentation of \code{style}.}
\arguments{
\item{style}{A character string or custom R code to define the visual formatting
of the table. This argument can be provided in two ways: (1) Pre-defined Styles
(Character String): Use a name for a built-in style (e.g., "darwin"). See
visOmopResults::tableStyle() for available options. (2) Custom Code (Advanced):
Supply a block of custom R code. This code must be specific to the table type.
See visOmopResults::tableStyleCode() for structural examples.}
}
\description{
Helper for consistent documentation of \code{style}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{bySexDoc}
\alias{bySexDoc}
\title{Helper for consistent documentation of \code{bySex}.}
\arguments{
\item{bySex}{TRUE or FALSE. If TRUE code use will be summarised by sex.}
}
\description{
Helper for consistent documentation of \code{bySex}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDescendants.R
\name{getDescendants}
\alias{getDescendants}
\title{Get descendant codes for a given concept}
\usage{
getDescendants(cdm, conceptId, withAncestor = FALSE)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{conceptId}{concept_id to search}

\item{withAncestor}{If TRUE, return column with ancestor. In case of multiple
ancestors, concepts will be separated by ";".}
}
\value{
The descendants of a given concept id.
}
\description{
Get descendant codes for a given concept
}
\examples{
\donttest{
library(CodelistGenerator)
cdm <- mockVocabRef()
getDescendants(cdm = cdm, conceptId = 1)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableDoseForms.R
\name{associatedDoseForms}
\alias{associatedDoseForms}
\title{Get the dose forms associated with drug concepts in a codelist}
\usage{
associatedDoseForms(x, cdm)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}
}
\value{
The dose forms available for drug concepts.
}
\description{
Get the dose forms associated with drug concepts in a codelist
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all dose forms available in a codelist
codelist <- newCodelist(list("codes1" = c(194152L, 1830279L, 40558872L),
                             "codes2" = c(44022939L)))
associatedDoseForms(x = codelist, cdm = cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classCodeSearch.R
\name{exportCodeSearch}
\alias{exportCodeSearch}
\title{Export a \code{code_search} object into an Excel spreadsheet}
\usage{
exportCodeSearch(codeSearch, file, path = getwd())
}
\arguments{
\item{codeSearch}{A \code{code_search} object.}

\item{file}{Name of the Excel spreadsheet to be created.}

\item{path}{Path where to create the Excel file.}
}
\value{
The \code{code_search} object is exported to a file.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}
This function will be moved to omopgenerics (and re-exported by
CodelistGenerator) once stable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summariseOrphanCodes.R
\name{summariseOrphanCodes}
\alias{summariseOrphanCodes}
\title{Find orphan codes related to a codelist using achilles counts and, if
available, PHOEBE concept recommendations}
\usage{
summariseOrphanCodes(
  x,
  cdm,
  domain = c("condition", "device", "drug", "measurement", "observation", "procedure",
    "visit")
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{domain}{Character vector with one or more of the OMOP CDM domains.
The results will be restricted to the given domains. Check the available
domains in the database by running 'availableDomains()', or 'associatedDomains()'
to explore the domains associated with a codelist. If NULL, all supported domains
are included: Condition, Drug, Procedure, Device, Observation, and Measurement.}
}
\value{
A summarised result containing the frequency of codes related
to (but not in) the codelist.
}
\description{
Find orphan codes related to a codelist using achilles counts and, if
available, PHOEBE concept recommendations
}
\examples{
\donttest{
library(CodelistGenerator)

cdm <- mockVocabRef("database")
codes <- getCandidateCodes(cdm = cdm,
                          keywords = "Musculoskeletal disorder",
                          domains = "Condition",
                          includeDescendants = FALSE)
codelist <- omopgenerics::newCodelist(list("msk" = codes$concept_id))
orphan_codes <- summariseOrphanCodes(x = codelist,
                                      cdm = cdm)

orphan_codes
CDMConnector::cdmDisconnect(cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summariseCodeUse.R
\name{summariseCodeUse}
\alias{summariseCodeUse}
\title{Summarise code use in patient-level data.}
\usage{
summariseCodeUse(
  x,
  cdm,
  countBy = c("record", "person"),
  byConcept = TRUE,
  byYear = FALSE,
  bySex = FALSE,
  ageGroup = NULL,
  dateRange = as.Date(c(NA, NA)),
  useSourceCodes = FALSE
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{countBy}{Either "record" for record-level counts or "person" for
person-level counts.}

\item{byConcept}{TRUE or FALSE. If TRUE code use will be summarised by
concept.}

\item{byYear}{TRUE or FALSE. If TRUE code use will be summarised by year.}

\item{bySex}{TRUE or FALSE. If TRUE code use will be summarised by sex.}

\item{ageGroup}{If not NULL, a list of ageGroup vectors of length two.}

\item{dateRange}{Two dates. The first indicating the earliest cohort start
date and the second indicating the latest possible cohort end date. If NULL
or the first date is set as missing, the earliest observation_start_date in
the observation_period table will be used for the former. If NULL or the
second date is set as missing, the latest observation_end_date in the
observation_period table will be used for the latter.}

\item{useSourceCodes}{Whether the codelist provided contains source codes (TRUE) or standard codes (FALSE).}
}
\value{
A tibble with count results overall and, if specified, by strata.
}
\description{
Summarise code use in patient-level data.
}
\examples{
\dontrun{
library(omopgenerics)
library(CodelistGenerator)
con <- DBI::dbConnect(duckdb::duckdb(),
                      dbdir = CDMConnector::eunomiaDir())
cdm <- CDMConnector::cdmFromCon(con,
                                cdmSchema = "main",
                                writeSchema = "main")
acetiminophen <- c(1125315,  1127433, 40229134,
                  40231925, 40162522, 19133768,  1127078)
poliovirus_vaccine <- c(40213160)
cs <- newCodelist(list(acetiminophen = acetiminophen,
                       poliovirus_vaccine = poliovirus_vaccine))
results <- summariseCodeUse(cs,cdm = cdm)
results
CDMConnector::cdmDisconnect(cdm)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/benchmarkCodelistGenerator.R
\name{benchmarkCodelistGenerator}
\alias{benchmarkCodelistGenerator}
\title{Run benchmark of codelistGenerator analyses}
\usage{
benchmarkCodelistGenerator(cdm)
}
\arguments{
\item{cdm}{a CDM reference object}
}
\value{
a tibble with time taken for different analysis
}
\description{
Run benchmark of codelistGenerator analyses
}
\examples{
\donttest{
library(CodelistGenerator)

cdm <- mockVocabRef()

timings <- benchmarkCodelistGenerator(cdm)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/mockVocabRef.R
\name{mockVocabRef}
\alias{mockVocabRef}
\title{Generate example vocabulary database}
\usage{
mockVocabRef(backend = "data_frame")
}
\arguments{
\item{backend}{'database' (duckdb) or 'data_frame'.}
}
\value{
cdm reference with mock vocabulary.
}
\description{
Generate example vocabulary database
}
\examples{
\donttest{
library(CodelistGenerator)
cdm <- mockVocabRef()
cdm
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetOnDoseUnit.R
\name{subsetOnDoseUnit}
\alias{subsetOnDoseUnit}
\title{Subset a codelist to only those with a particular dose unit.}
\usage{
subsetOnDoseUnit(x, cdm, doseUnit, negate = FALSE)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{doseUnit}{Only codes with the specified dose unit
will be returned. If NULL, descendant codes will be returned regardless
of dose unit Use 'availableDoseUnits()' to see the available dose units,
or 'associatedDoseUnits()' to see the associated dose forms in a codelist.}

\item{negate}{If FALSE, only concepts with the dose unit specified will be
returned. If TRUE, concepts with the dose unit specified will be excluded.}
}
\value{
The codelist with only those concepts associated with the
dose unit (if negate = FALSE) or codelist without those concepts associated with the
dose unit(if negate = TRUE).
}
\description{
Subset a codelist to only those with a particular dose unit.
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef()
codes <- subsetOnDoseUnit(x = newCodelist(list("codes" = c(20,21))),
                          cdm = cdm,
                          doseUnit = c("milligram"))

codes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetOnDomain.R
\name{subsetOnDomain}
\alias{subsetOnDomain}
\title{Subset a codelist to only those codes from a particular domain.}
\usage{
subsetOnDomain(x, cdm, domain, negate = FALSE)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{domain}{Character vector with one or more of the OMOP CDM domains.
The results will be restricted to the given domains. Check the available
domains in the database by running 'availableDomains()', or 'associatedDomains()'
to explore the domains associated with a codelist. If NULL, all supported domains
are included: Condition, Drug, Procedure, Device, Observation, and Measurement.}

\item{negate}{If FALSE, only concepts with the domain specified will be
returned. If TRUE, concepts with the domain specified will be excluded.}
}
\value{
The codelist with only those concepts associated with the domain
(if negate = FALSE) or the codelist without those concepts associated with
the domain (if negate = TRUE).
}
\description{
Subset a codelist to only those codes from a particular domain.
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef()
codes <- subsetOnDomain(
              x = newCodelist(list("codes" = c(10,13,15))),
              cdm = cdm,
              domain = "Drug")
codes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classCodeSearch.R
\name{emptyCodeSearch}
\alias{emptyCodeSearch}
\title{Empty code search object}
\usage{
emptyCodeSearch(searchStrategy = NULL)
}
\arguments{
\item{searchStrategy}{The search strategy used to generate the candidate
codes.}
}
\value{
An empty \code{code_search} object.
}
\description{
Empty code search object
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{hideDoc}
\alias{hideDoc}
\title{Helper for consistent documentation of \code{hide}.}
\arguments{
\item{hide}{Table columns to exclude, options are:  "cdm_name",
"codelist_name", "domain_id", "standard_concept_name", "standard_concept_id",
"estimate_name", "standard_concept", "vocabulary_id". These cannot be used in
header or groupColumn.}
}
\description{
Helper for consistent documentation of \code{hide}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{tableDoc}
\alias{tableDoc}
\title{Helper for consistent documentation of \code{table}.}
\arguments{
\item{table}{cdm table of interest.}
}
\description{
Helper for consistent documentation of \code{table}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{typeTableDoc}
\alias{typeTableDoc}
\title{Helper for consistent documentation of \code{type}.}
\arguments{
\item{type}{Type of desired formatted table. By default (type = NULL), it will
create a 'gt' table. To see supported formats use visOmopResults::tableType().}
}
\description{
Helper for consistent documentation of \code{type}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratifyByVocabulary.R
\name{stratifyByVocabulary}
\alias{stratifyByVocabulary}
\title{Subset a codelist to only those codes from a particular domain.}
\usage{
stratifyByVocabulary(
  x,
  cdm,
  nameStyle = "{codelist_name}_{vocabulary}",
  keepOriginal = FALSE
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{nameStyle}{Naming of the new codelists, use \code{{codelist_name}} to
include the codelist name and \code{{vocabulary}} to include the vocabulary name.}

\item{keepOriginal}{Whether to keep the original codelist (TRUE) or just return
the stratified ones (FALSE).}
}
\value{
The codelist with the required stratifications, as different elements
of the list.
}
\description{
Subset a codelist to only those codes from a particular domain.
}
\examples{
\donttest{
library(CodelistGenerator)

cdm <- mockVocabRef()

codes <- stratifyByVocabulary(
  x = newCodelist(list("codes" = c(10L, 13L, 15L))),
  cdm = cdm,
  keepOriginal = TRUE
)

codes
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{countByDoc}
\alias{countByDoc}
\title{Helper for consistent documentation of \code{countBy}.}
\arguments{
\item{countBy}{Either "record" for record-level counts or "person" for
person-level counts.}
}
\description{
Helper for consistent documentation of \code{countBy}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratifyByDoseForm.R
\name{stratifyByDoseForm}
\alias{stratifyByDoseForm}
\title{Stratify a codelist by dose form.}
\usage{
stratifyByDoseForm(
  x,
  cdm,
  nameStyle = "{codelist_name}_{dose_form}",
  keepOriginal = FALSE
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{nameStyle}{Naming of the new codelists, use \code{{codelist_name}} to
include the codelist name and \code{{dose_form}} to include the dose form name.}

\item{keepOriginal}{Whether to keep the original codelist (TRUE) or just return
the stratified ones (FALSE).}
}
\value{
The codelist with the required stratifications, as different elements
of the list.
}
\description{
Stratify a codelist by dose form.
}
\examples{
\donttest{
library(CodelistGenerator)

cdm <- mockVocabRef()

codes <- newCodelist(list("codes" = c(10L, 20L, 21L)))
new_codes <- stratifyByDoseForm(x = codes,
                                cdm = cdm,
                                keepOriginal = TRUE)
new_codes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{levelATCDoc}
\alias{levelATCDoc}
\title{Helper for consistent documentation of \code{level}.}
\arguments{
\item{level}{ATC level. Can be one or more of "ATC 1st", "ATC 2nd",
"ATC 3rd", "ATC 4th", and "ATC 5th".}
}
\description{
Helper for consistent documentation of \code{level}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableDoseForms.R
\name{availableDoseForms}
\alias{availableDoseForms}
\title{Get the dose forms for drug concepts}
\usage{
availableDoseForms(cdm)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}
}
\value{
The dose forms available for drug concepts.
}
\description{
Get the dose forms for drug concepts
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all domains available in the CDM
availableDoseForms(cdm = cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratifyByDomain.R
\name{stratifyByDomain}
\alias{stratifyByDomain}
\title{Stratify a codelist by domain category.}
\usage{
stratifyByDomain(
  x,
  cdm,
  nameStyle = "{codelist_name}_{domain}",
  keepOriginal = FALSE
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{nameStyle}{Naming of the new codelists, use \code{{codelist_name}} to
include the codelist name and \code{{domain}} to include the domain name.}

\item{keepOriginal}{Whether to keep the original codelist (TRUE) or just return
the stratified ones (FALSE).}
}
\value{
The codelist with the required stratifications, as different elements
of the list.
}
\description{
Stratify a codelist by domain category.
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef()
codes <- newCodelist(list("concepts_1" = c(20L,21L,22L),
                         "concepts_2" = c(10L,13L,21L)))
new_codes <- stratifyByDomain(x = codes,
                             cdm = cdm,
                             keepOriginal = TRUE)
new_codes
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetOnRouteCategory.R
\name{subsetOnRouteCategory}
\alias{subsetOnRouteCategory}
\title{Subset a codelist to only those with a particular route category}
\usage{
subsetOnRouteCategory(x, cdm, routeCategory, negate = FALSE)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{routeCategory}{Only codes with the specified route will be
returned. If NULL, descendant codes will be returned regardless of route
category. Use 'availableRouteCategories()' to find the available route categories in
the database, and 'associatedRouteCategories()' to get drug routs associated with
a codelist.}

\item{negate}{If FALSE, only concepts with the routeCategory specified will
be returned. If TRUE, concepts with the routeCategory specified will be excluded.}
}
\value{
The codelist with only those concepts associated with the
specified route categories (if negate is FALSE) or the codelist without those
concepts associated with the specified route categories (if negate is TRUE).
}
\description{
Subset a codelist to only those with a particular route category
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef()
codes <- subsetOnRouteCategory(
              x = newCodelist(list("codes" = c(20,21))),
              cdm = cdm,
              routeCategory = "topical")
codes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{codelistNameDoc}
\alias{codelistNameDoc}
\title{Helper for consistent documentation of \code{codelistNameDoc}}
\arguments{
\item{codelistName}{Name or names of codelist in x. If NULL, all codelist present
in x will be considered.}
}
\description{
Helper for consistent documentation of \code{codelistNameDoc}
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{keepOriginalDoc}
\alias{keepOriginalDoc}
\title{Helper for consistent documentation of \code{keepOriginal}.}
\arguments{
\item{keepOriginal}{Whether to keep the original codelist (TRUE) or just return
the stratified ones (FALSE).}
}
\description{
Helper for consistent documentation of \code{keepOriginal}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableVocabularies.R
\name{associatedVocabularies}
\alias{associatedVocabularies}
\title{Get the vocabularies associated with a codelist}
\usage{
associatedVocabularies(x, cdm, standardConcept = "Standard", domain = NULL)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{standardConcept}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}

\item{domain}{Character vector with one or more of the OMOP CDM domains.
The results will be restricted to the given domains. Check the available
domains in the database by running 'availableDomains()', or 'associatedDomains()'
to explore the domains associated with a codelist. If NULL, all supported domains
are included: Condition, Drug, Procedure, Device, Observation, and Measurement.}
}
\value{
Names of available vocabularies.
}
\description{
Get the vocabularies associated with a codelist
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all vocabularies from a codelist
codelist <- newCodelist(list("codes1" = c(35604877L, 35604394L),
                             "codes2" = c(4214687L)))
associatedVocabularies(cdm = cdm,
                      x = codelist)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{byConceptDoc}
\alias{byConceptDoc}
\title{Helper for consistent documentation of \code{byConcept}.}
\arguments{
\item{byConcept}{TRUE or FALSE. If TRUE code use will be summarised by
concept.}
}
\description{
Helper for consistent documentation of \code{byConcept}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{doseUnitDoc}
\alias{doseUnitDoc}
\title{Helper for consistent documentation of \code{doseUnit}.}
\arguments{
\item{doseUnit}{Only codes with the specified dose unit
will be returned. If NULL, descendant codes will be returned regardless
of dose unit Use 'availableDoseUnits()' to see the available dose units,
or 'associatedDoseUnits()' to see the associated dose forms in a codelist.}
}
\description{
Helper for consistent documentation of \code{doseUnit}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableRouteCategories.R
\name{associatedRouteCategories}
\alias{associatedRouteCategories}
\title{Get drug routes associated with a codelist}
\usage{
associatedRouteCategories(x, cdm)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}
}
\value{
A character vector with all available routes.
}
\description{
Get the dose form categories available in the database (see
https://doi.org/10.1002/pds.5809) for more details on how routes
were classified).
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Create a codelist
codelist <- newCodelist(list("codes1" = c(194152L, 1830279L, 40558872L),
                             "codes2" = c(44022939L)))

# Get all route categories available in a codelist
route_categories <- associatedRouteCategories(codelist, cdm)
route_categories
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{includeDescendantsDoc}
\alias{includeDescendantsDoc}
\title{Helper for consistent documentation of \code{includeDescendants}.}
\arguments{
\item{includeDescendants}{Either TRUE or FALSE. If TRUE descendant concepts
of identified concepts will be included in the candidate codelist. If FALSE
only direct mappings from ICD-10 codes to standard codes will be returned.}
}
\description{
Helper for consistent documentation of \code{includeDescendants}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableRelationshipIds.R
\name{availableRelationshipIds}
\alias{availableRelationshipIds}
\title{Get available relationships between concepts}
\usage{
availableRelationshipIds(
  cdm,
  standardConcept1 = "Standard",
  standardConcept2 = "Standard",
  domains1 = "Condition",
  domains2 = "Condition"
)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{standardConcept1}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}

\item{standardConcept2}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}

\item{domains1}{Character vector with one or more of the OMOP CDM domain. If NULL, all domains are considered.}

\item{domains2}{Character vector with one or more of the OMOP CDM domain. If NULL, all domains are considered.}
}
\value{
A character vector with unique concept relationship values.
}
\description{
Get available relationships between concepts
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all relationship ids in the CDM between `Condition` and `Standard` concepts.
availableRelationshipIds(cdm = cdm,
                         standardConcept1 = "Standard",
                         standardConcept2 = "Standard",
                         domains1 = "Condition",
                         domains2 = "Condition")

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{standardConceptDoc}
\alias{standardConceptDoc}
\title{Helper for consistent documentation of \code{standardConcept}.}
\arguments{
\item{standardConcept}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}
}
\description{
Helper for consistent documentation of \code{standardConcept}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDrugIngredientCodes.R
\name{getDrugIngredientCodes}
\alias{getDrugIngredientCodes}
\title{Get descendant codes of drug ingredients}
\usage{
getDrugIngredientCodes(
  cdm,
  name = NULL,
  nameStyle = "{concept_code}_{concept_name}",
  doseForm = NULL,
  doseUnit = NULL,
  routeCategory = NULL,
  ingredientRange = c(1, Inf),
  type = "codelist"
)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{name}{Names of ingredients of interest. For example, c("acetaminophen",
"codeine"), would result in a list of length two with the descendant
concepts for these two particular drug ingredients. Users can also specify the
concept ID instead of the name (e.g., c(1125315, 42948451)) using a numeric vector.}

\item{nameStyle}{Name style to apply to returned list. Can be one of
\code{"{concept_code}"},\code{"{concept_id}"}, \code{"{concept_name}"}, or a combination (i.e.,
\code{"{concept_code}_{concept_name}"}).}

\item{doseForm}{Only codes with the specified dose form
will be returned. If NULL, descendant codes will be returned regardless
of dose form. Use 'availableDoseForms()' to see the available dose forms in the database,
or 'associatedDoseForms()' to see the associated dose forms in a codelist.}

\item{doseUnit}{Only codes with the specified dose unit
will be returned. If NULL, descendant codes will be returned regardless
of dose unit Use 'availableDoseUnits()' to see the available dose units,
or 'associatedDoseUnits()' to see the associated dose forms in a codelist.}

\item{routeCategory}{Only codes with the specified route will be
returned. If NULL, descendant codes will be returned regardless of route
category. Use 'availableRouteCategories()' to find the available route categories in
the database, and 'associatedRouteCategories()' to get drug routs associated with
a codelist.}

\item{ingredientRange}{Used to restrict descendant codes to those
associated with a specific number of drug ingredients. Must be a vector of
length two with the first element the minimum number of ingredients allowed
and the second the maximum. A value of c(2, 2) would restrict to only
concepts associated with two ingredients.}

\item{type}{Can be "codelist" or "codelist_with_details".}
}
\value{
Concepts with their format based on the type argument.
}
\description{
Get descendant codes of drug ingredients
}
\examples{
\donttest{
library(CodelistGenerator)
cdm <- mockVocabRef()
getDrugIngredientCodes(cdm = cdm,
                       name = "Adalimumab",
                       nameStyle = "{concept_name}")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableDrugIngredients.R
\name{availableDrugIngredients}
\alias{availableDrugIngredients}
\title{Get the names of all available drug ingredients}
\usage{
availableDrugIngredients(cdm, standardConcept = "Standard")
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{standardConcept}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}
}
\value{
A vector containing the concept names for all ingredient level codes
found in the concept table of cdm.
}
\description{
Get the names of all available drug ingredients
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all drug ingredients available in the CDM for standard concepts
availableDrugIngredients(cdm = cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{minimumCountDoc}
\alias{minimumCountDoc}
\title{Helper for consistent documentation of \code{minimumCount}.}
\arguments{
\item{minimumCount}{Any codes with a frequency under this will be removed.}
}
\description{
Helper for consistent documentation of \code{minimumCount}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetOnIngredientRange.R
\name{subsetOnIngredientRange}
\alias{subsetOnIngredientRange}
\title{Subset a codelist to only those codes with a range of number of ingredients}
\usage{
subsetOnIngredientRange(x, cdm, ingredientRange, negate = FALSE)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{ingredientRange}{Used to restrict descendant codes to those
associated with a specific number of drug ingredients. Must be a vector of
length two with the first element the minimum number of ingredients allowed
and the second the maximum. A value of c(2, 2) would restrict to only
concepts associated with two ingredients.}

\item{negate}{If FALSE, only concepts with the ingredient range specified will be
returned (both limits included). If TRUE, concepts with number of ingredients outside
the range will be returned.}
}
\value{
The codelist with only those concepts associated with the domain
(if negate = FALSE) or the codelist without those concepts associated with
the domain (if negate = TRUE).
}
\description{
Subset a codelist to only those codes with a range of number of ingredients
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef()
codes <- subsetOnIngredientRange(
              x = newCodelist(list("codes" = c(10L, 13L))),
              cdm = cdm,
              ingredientRange = c(2, 10))
codes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{hideStrataDoc}
\alias{hideStrataDoc}
\title{Helper for consistent documentation of \code{hide}.}
\arguments{
\item{hide}{Table columns to exclude, options are: "cdm_name",
"codelist_name", "year", "sex", "age_group", "standard_concept_name",
"standard_concept_id", "estimate_name", "source_concept_name",
"source_concept_id", "domain_id". If results are stratified, "year", "sex",
"age_group" can also be used. These cannot be used in header or groupColumn.}
}
\description{
Helper for consistent documentation of \code{hide}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asConceptSetExpression.R
\name{asConceptSetExpression}
\alias{asConceptSetExpression}
\alias{asConceptSetExpression.codelist}
\alias{asConceptSetExpression.codelist_with_details}
\title{Coerce to a concept set expression}
\usage{
asConceptSetExpression(x)

\method{asConceptSetExpression}{codelist}(x)

\method{asConceptSetExpression}{codelist_with_details}(x)
}
\arguments{
\item{x}{A codelist, codelist_with_details, concept_set_expression, or a candidate_codes.}
}
\value{
codelist
}
\description{
Coerce to a concept set expression
}
\examples{
\donttest{
library(omock)
library(CDMConnector)

# Creating CDM object
cdm <- mockCdmFromDataset(datasetName = "GiBleed")

# Create concept_set_expression from a codelist
codelist <- getDrugIngredientCodes(cdm,
                                   name = "acetaminophen",
                                   nameStyle = "{concept_name}",
                                   type = "codelist")

asConceptSetExpression(codelist)

# Create concept_set_expression from a codelist_with_details
codelist <- getDrugIngredientCodes(cdm,
                                   name = "acetaminophen",
                                  nameStyle = "{concept_name}",
                                   type = "codelist_with_details")

asConceptSetExpression(codelist)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classCodeSearch.R
\name{searchStrategy}
\alias{searchStrategy}
\title{Get the search strategy used to create a certain code_search}
\usage{
searchStrategy(codeSearch)
}
\arguments{
\item{codeSearch}{A \code{code_search} object.}
}
\value{
A search strategy tibble.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}
This function will be moved to omopgenerics (and re-exported by
CodelistGenerator) once stable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CodelistGenerator-package.R
\docType{package}
\name{CodelistGenerator-package}
\alias{CodelistGenerator}
\alias{CodelistGenerator-package}
\title{CodelistGenerator: Identify Relevant Clinical Codes and Evaluate Their Use}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Generate a candidate code list for the Observational Medical Outcomes Partnership (OMOP) common data model based on string matching. For a given search strategy, a candidate code list will be returned.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://darwin-eu.github.io/CodelistGenerator/}
  \item Report bugs at \url{https://github.com/darwin-eu/CodelistGenerator/issues}
}

}
\author{
\strong{Maintainer}: Edward Burn \email{edward.burn@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0002-9286-1128}{ORCID})

Authors:
\itemize{
  \item Marta Alcalde-Herraiz \email{marta.alcaldeherraiz@ndorms.ox.ac.uk} (\href{https://orcid.org/0009-0002-4405-1814}{ORCID})
  \item Martí Català \email{marti.catalasabate@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0003-3308-9905}{ORCID})
  \item Xihang Chen \email{xihang.chen@ndorms.ox.ac.uk} (\href{https://orcid.org/0009-0001-8112-8959}{ORCID})
  \item Nuria Mercade-Besora \email{nuria.mercadebesora@ndorms.ox.ac.uk} (\href{https://orcid.org/0009-0006-7948-3747}{ORCID})
  \item Mike Du \email{mike.du@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0002-9517-8834}{ORCID})
  \item Danielle Newby \email{danielle.newby@ndorms.ox.ac.uk} (\href{https://orcid.org/0000-0002-3001-1478}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{typeBroadDoc}
\alias{typeBroadDoc}
\title{Helper for consistent documentation of \code{type}.}
\arguments{
\item{type}{Can be "codelist", "codelist_with_details" or
"concept_set_expression".}
}
\description{
Helper for consistent documentation of \code{type}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{xDoc}
\alias{xDoc}
\title{Helper for consistent documentation of \code{x}.}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}
}
\description{
Helper for consistent documentation of \code{x}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{cdmDoc}
\alias{cdmDoc}
\title{Helper for consistent documentation of \code{cdm}.}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}
}
\description{
Helper for consistent documentation of \code{cdm}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{doseFormToRoute}
\alias{doseFormToRoute}
\title{Table showing the route category associated with each dose form.}
\format{
A data frame
\describe{
\item{dose_form_concept_id}{Concept ID of each dose form}
\item{dose_form_concept_name}{Concept name of each dose form}
\item{route_category}{Route category associated to the dose form}
}
}
\usage{
doseFormToRoute
}
\description{
Table showing the route category associated with each dose form.
}
\examples{
\donttest{
library(CodelistGenerator)
doseFormToRoute
}
}
\keyword{data}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/addConcepts.R
\name{addConcepts}
\alias{addConcepts}
\title{Add concepts to a codelist}
\usage{
addConcepts(x, cdm, concepts, codelistName = NULL)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{concepts}{Concepts_ID to add}

\item{codelistName}{Name or names of codelist in x. If NULL, all codelist present
in x will be considered.}
}
\value{
A codelist
}
\description{
Add concepts to a codelist
}
\examples{
\donttest{
library(omock)
library(CDMConnector)

# Creating CDM object
cdm <- mockCdmFromDataset(datasetName = "GiBleed")

# Creating codelist
codelist <- getDrugIngredientCodes(cdm,
                                   nameStyle = "{concept_name}")

# Add a concept to all the codelists:
codelist$acetaminophen
codelist <- codelist |>
 addConcepts(cdm, concepts = c(1L))
codelist$acetaminophen

# Add a concept to a specific codelist
codelist$amiodarone
codelist <- codelist |>
  addConcepts(cdm, concepts = c(2L), codelistName = "amiodarone")
codelist$amiodarone

# See function: `excludeConcepts()` for details on how to remove specific concepts
# from a codelist
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getATCCodes.R
\name{getATCCodes}
\alias{getATCCodes}
\title{Get the descendant codes of Anatomical Therapeutic Chemical (ATC) classification codes}
\usage{
getATCCodes(
  cdm,
  level = c("ATC 1st"),
  name = NULL,
  nameStyle = "{concept_code}_{concept_name}",
  doseForm = NULL,
  doseUnit = NULL,
  routeCategory = NULL,
  type = "codelist"
)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{level}{ATC level. Can be one or more of "ATC 1st", "ATC 2nd",
"ATC 3rd", "ATC 4th", and "ATC 5th".}

\item{name}{ATC name of interest. For example, c("Dermatologicals",
"Nervous System"), would result in a list of length two with the descendant
concepts for these two particular ATC groups.}

\item{nameStyle}{Name style to apply to returned list. Can be one of
\code{"{concept_code}"},\code{"{concept_id}"}, \code{"{concept_name}"}, or a combination (i.e.,
\code{"{concept_code}_{concept_name}"}).}

\item{doseForm}{Only codes with the specified dose form
will be returned. If NULL, descendant codes will be returned regardless
of dose form. Use 'availableDoseForms()' to see the available dose forms in the database,
or 'associatedDoseForms()' to see the associated dose forms in a codelist.}

\item{doseUnit}{Only codes with the specified dose unit
will be returned. If NULL, descendant codes will be returned regardless
of dose unit Use 'availableDoseUnits()' to see the available dose units,
or 'associatedDoseUnits()' to see the associated dose forms in a codelist.}

\item{routeCategory}{Only codes with the specified route will be
returned. If NULL, descendant codes will be returned regardless of route
category. Use 'availableRouteCategories()' to find the available route categories in
the database, and 'associatedRouteCategories()' to get drug routs associated with
a codelist.}

\item{type}{Can be "codelist" or "codelist_with_details".}
}
\value{
Concepts with their format based on the type argument
}
\description{
Get the descendant codes of Anatomical Therapeutic Chemical (ATC) classification codes
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Create a codelist with 1st level ATC codes available in the CDM
codelist <- getATCCodes(cdm = cdm,
                        level = "ATC 1st")
codelist

# Tune the name of the generated codelists
codelist <- getATCCodes(cdm = cdm,
                        level = "ATC 1st",
                        nameStyle = "{concept_name}_{concept_code}")
codelist

# Search for a specific ATC name of interest
codelist <- getATCCodes(cdm = cdm,
                        level = "ATC 2nd",
                        name = "immunostimulants")
codelist

# Restrict concepts to specific dose forms, dose units, or route categories.
# Remember that you can use `availableDoseForm()`, `availableDoseUnit()` and
# `availableRouteCategory()` to explore your codelist.
codelist <- getATCCodes(cdm = cdm,
                        level = "ATC 2nd",
                        doseForm = NULL,
                        doseUnit = NULL,
                        routeCategory = NULL,)
codelist

# You can also create directly a codelist_with_details using the argument `type`
codelist <- getATCCodes(cdm = cdm,
                        level = "ATC 1st",
                        type = "codelist_with_details")
codelist
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stratifyByBrand.R
\name{stratifyByBrand}
\alias{stratifyByBrand}
\title{Stratify a codelist by brand category.}
\usage{
stratifyByBrand(
  x,
  cdm,
  nameStyle = "{codelist_name}_{brand}",
  keepOriginal = FALSE
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{nameStyle}{Naming of the new codelists, use \code{{codelist_name}} to
include the codelist name and \code{{brand}} to include the brand name.}

\item{keepOriginal}{Whether to keep the original codelist (TRUE) or just return
the stratified ones (FALSE).}
}
\value{
The codelist with the required stratifications, as different elements
of the list.
}
\description{
Stratify a codelist by brand category.
}
\examples{
\donttest{
library(CodelistGenerator)

cdm <- mockVocabRef()
codes <- newCodelist(list(
  concepts_1 = c(20L, 21L, 22L),
  concepts_2 = c(10L, 13L, 21L)
))

new_codes <- stratifyByBrand(x = codes,
                             cdm = cdm,
                             keepOriginal = TRUE)
new_codes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableDrugIngredients.R
\name{associatedDrugIngredients}
\alias{associatedDrugIngredients}
\title{Get the names of drug ingredients associated with codelist}
\usage{
associatedDrugIngredients(x, cdm, standardConcept = "Standard")
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{standardConcept}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}
}
\value{
A vector containing the concept names for all ingredient level codes
found in the concept table of cdm.
}
\description{
Get the names of drug ingredients associated with codelist
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all drug ingredients associated with a codelist
codelist <- newCodelist(list("codes1" = c(37498042L),
                             "codes2" = c( 42899580L, 35741956L)))
associatedDrugIngredients(x = codelist, cdm = cdm,
                         standardConcept = c("Standard", "Non-standard"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compareCodelists.R
\name{compareCodelists}
\alias{compareCodelists}
\title{Compare overlap between two sets of codes}
\usage{
compareCodelists(codelist1, codelist2)
}
\arguments{
\item{codelist1}{Output of getCandidateCodes or a codelist}

\item{codelist2}{Output of getCandidateCodes.}
}
\value{
Tibble with information on the overlap of codes in both codelists.
}
\description{
Compare overlap between two sets of codes
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create a CDM object
downloadMockDataset(datasetName = "GiBleed",
                    path = NULL,
                    overwrite = NULL)
cdm <- mockCdmFromDataset(datasetName = "GiBleed")

# Compare two candidate_codes object
codes1 <- getCandidateCodes(
  cdm = cdm,
  keywords = "Arthritis",
  domains = "Condition",
  includeDescendants = TRUE)

codes2 <- getCandidateCodes(
  cdm = cdm,
  keywords = c("osteo"),
  domains = "Condition",
  includeDescendants = TRUE)

# compareCodelists(
#   codelist1 = codes1,
#   codelist2 = codes2
# )

# Compare two codelists
acetaminophen <- getDrugIngredientCodes(cdm,
                                        name = "acetaminophen",
                                        nameStyle = "{concept_name}",
                                        type = "codelist")

hydrocodone <- getDrugIngredientCodes(cdm,
                                      name = "hydrocodone",
                                      nameStyle = "{concept_name}",
                                      type = "codelist")
compareCodelists(
  codelist1 = acetaminophen,
  codelist2 = hydrocodone)
# Notice that concept_name = NA as `codelist` class does not store this information
# for each concept.

# Compare two codelists_with_details
acetaminophen <- getDrugIngredientCodes(cdm,
                                        name = "acetaminophen",
                                        nameStyle = "{concept_name}",
                                        type = "codelist_with_details")

hydrocodone <- getDrugIngredientCodes(cdm,
                                      name = "hydrocodone",
                                      nameStyle = "{concept_name}",
                                      type = "codelist_with_details")
compareCodelists(
  codelist1 = acetaminophen,
  codelist2 = hydrocodone
)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/codesFromConceptSet.R
\name{codesFromCohort}
\alias{codesFromCohort}
\title{Get concept ids from JSON files containing cohort definitions}
\usage{
codesFromCohort(path, cdm, type = c("codelist"))
}
\arguments{
\item{path}{Path to a file or folder containing JSONs of cohort definitions.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{type}{Can be "codelist", "codelist_with_details" or
"concept_set_expression".}
}
\value{
Named list with concept_ids for each concept set.
}
\description{
Get concept ids from JSON files containing cohort definitions
}
\examples{
\donttest{
library(CodelistGenerator)
cdm <- mockVocabRef("database")
x <- codesFromCohort(cdm = cdm,
                     path =  system.file(package = "CodelistGenerator",
                     "cohorts_for_mock"))
x
CDMConnector::cdmDisconnect(cdm)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetToCodesInUse.R
\name{subsetToCodesInUse}
\alias{subsetToCodesInUse}
\title{Filter a codelist to keep only the codes being used in patient records}
\usage{
subsetToCodesInUse(
  x,
  cdm,
  minimumCount = 0L,
  table = c("condition_occurrence", "device_exposure", "drug_exposure", "measurement",
    "observation", "procedure_occurrence", "visit_occurrence")
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{minimumCount}{Any codes with a frequency under this will be removed.}

\item{table}{cdm table of interest.}
}
\value{
The filtered codelist with only the codes used in the database
}
\description{
Filter a codelist to keep only the codes being used in patient records
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef("database")
codes <- getCandidateCodes(cdm = cdm,
                           keywords = "arthritis",
                           domains = "Condition",
                           includeDescendants = FALSE)
x <- subsetToCodesInUse(newCodelist(list("cs1" = codes$concept_id,
                               "cs2" = 999)),
                                cdm = cdm)

x
CDMConnector::cdmDisconnect(cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{ageGroupDoc}
\alias{ageGroupDoc}
\title{Helper for consistent documentation of \code{ageGroup}.}
\arguments{
\item{ageGroup}{If not NULL, a list of ageGroup vectors of length two.}
}
\description{
Helper for consistent documentation of \code{ageGroup}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/asCodelist.R
\name{asCodelist}
\alias{asCodelist}
\alias{asCodelist.codelist}
\alias{asCodelist.codelist_with_details}
\alias{asCodelist.concept_set_expression}
\alias{asCodelist.candidate_codes}
\title{Coerce to a codelist}
\usage{
asCodelist(x, ...)

\method{asCodelist}{codelist}(x, ...)

\method{asCodelist}{codelist_with_details}(x, ...)

\method{asCodelist}{concept_set_expression}(x, cdm, ...)

\method{asCodelist}{candidate_codes}(x, ...)
}
\arguments{
\item{x}{A codelist, codelist_with_details, concept_set_expression, or a candidate_codes.}

\item{...}{For extensibility}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}
}
\value{
codelist
}
\description{
Coerce to a codelist
}
\examples{
\donttest{
library(omock)
library(CDMConnector)

# Creating CDM object
cdm <- mockCdmFromDataset(datasetName = "GiBleed")

# Create codelist from a codelist_with_details
codelist <- getDrugIngredientCodes(cdm,
                                   name = "acetaminophen",
                                   nameStyle = "{concept_name}",
                                   type = "codelist_with_details")

asCodelist(codelist)

# Create codelist from a candidate_codes
codelist <- getCandidateCodes(cdm,
                              keywords = "arthritis")

asCodelist(codelist)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summariseCohortCodeUse.R
\name{summariseCohortCodeUse}
\alias{summariseCohortCodeUse}
\title{Summarise code use among a cohort in the cdm reference}
\usage{
summariseCohortCodeUse(
  cdm,
  cohortTable,
  x = NULL,
  cohortId = NULL,
  timing = "any",
  countBy = c("record", "person"),
  byConcept = TRUE,
  byYear = FALSE,
  bySex = FALSE,
  ageGroup = NULL,
  useSourceCodes = FALSE,
  codelistType = "index event"
)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{cohortTable}{A cohort table from the cdm reference.}

\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()},
\code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.
If set to NULL, the codelist associated with the cohort will be used.}

\item{cohortId}{A vector of cohort IDs to include}

\item{timing}{When to assess the code use relative cohort dates. This can
be "any"(code use any time by individuals in the cohort) or  "entry" (code
use on individuals' cohort start date).}

\item{countBy}{Either "record" for record-level counts or "person" for
person-level counts.}

\item{byConcept}{TRUE or FALSE. If TRUE code use will be summarised by
concept.}

\item{byYear}{TRUE or FALSE. If TRUE code use will be summarised by year.}

\item{bySex}{TRUE or FALSE. If TRUE code use will be summarised by sex.}

\item{ageGroup}{If not NULL, a list of ageGroup vectors of length two.}

\item{useSourceCodes}{Whether the codelist provided contains source codes
(TRUE) or standard codes (FALSE).}

\item{codelistType}{The reason for the codelist. Can be "index event",
"inclusion criteria", or "exit criteria". Only apply when x is NULL and using
codes associated with a cohort.}
}
\value{
A tibble with results overall and, if specified, by strata
}
\description{
Summarise code use among a cohort in the cdm reference
}
\examples{
\dontrun{
library(CodelistGenerator)
library(duckdb)
library(DBI)
library(CDMConnector)
con <- dbConnect(duckdb(),
                 dbdir = eunomiaDir())
cdm <- cdmFromCon(con,
                 cdmSchema = "main",
                 writeSchema = "main")
cdm <- generateConceptCohortSet(cdm = cdm,
                  conceptSet = list(a = 260139,
                                    b = 1127433),
                  name = "cohorts",
                  end = "observation_period_end_date",
                  overwrite = TRUE)

results_cohort_mult <-
summariseCohortCodeUse(omopgenerics::newCodelist(list(cs = c(260139,19133873))),
                      cdm = cdm,
                      cohortTable = "cohorts",
                      timing = "entry")

results_cohort_mult
CDMConnector::cdmDisconnect(cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summariseAchillesCodeUse.R
\name{summariseAchillesCodeUse}
\alias{summariseAchillesCodeUse}
\title{Summarise code use from achilles counts.}
\usage{
summariseAchillesCodeUse(x, cdm, countBy = c("record", "person"))
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{countBy}{Either "record" for record-level counts or "person" for
person-level counts.}
}
\value{
A tibble with summarised counts.
}
\description{
Summarise code use from achilles counts.
}
\examples{
\donttest{
library(CodelistGenerator)
cdm <- mockVocabRef("database")
oa <- getCandidateCodes(cdm = cdm, keywords = "osteoarthritis")
codelist <- omopgenerics::newCodelist(list(oa = oa$concept_id))
result_achilles <- summariseAchillesCodeUse(codelist, cdm = cdm)
result_achilles
CDMConnector::cdmDisconnect(cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{doseFormDoc}
\alias{doseFormDoc}
\title{Helper for consistent documentation of \code{doseForm}.}
\arguments{
\item{doseForm}{Only codes with the specified dose form
will be returned. If NULL, descendant codes will be returned regardless
of dose form. Use 'availableDoseForms()' to see the available dose forms in the database,
or 'associatedDoseForms()' to see the associated dose forms in a codelist.}
}
\description{
Helper for consistent documentation of \code{doseForm}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsetOnVocabulary.R
\name{subsetOnVocabulary}
\alias{subsetOnVocabulary}
\title{Subset a codelist to only those codes from a particular vocabulary.}
\usage{
subsetOnVocabulary(x, cdm, vocabulary, negate = FALSE)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{vocabulary}{Vocabulary to subset with (i.e., SNOMED)}

\item{negate}{If FALSE, only concepts with the vocabulary specified will be
returned. If TRUE, concepts with the vocabulary specified will be excluded.}
}
\value{
The codelist with only those concepts associated with the vocabulary
(if negate = FALSE) or the codelist without those concepts associated with
the vocabulary (if negate = TRUE).
}
\description{
Subset a codelist to only those codes from a particular vocabulary.
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef()
codes <- subsetOnVocabulary(
              x = newCodelist(list("codes" = c(1L,13L,15L))),
              cdm = cdm,
              vocabulary = "SNOMED")
codes
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tableOrphanCodes.R
\name{tableOrphanCodes}
\alias{tableOrphanCodes}
\title{Format the result of summariseOrphanCodes into a table}
\usage{
tableOrphanCodes(
  result,
  type = NULL,
  header = c("cdm_name", "estimate_name"),
  groupColumn = character(),
  hide = character(),
  style = NULL,
  .options = list()
)
}
\arguments{
\item{result}{A \verb{<summarised_result>} with results of the type
"orphan_codes".}

\item{type}{Type of desired formatted table. By default (type = NULL), it will
create a 'gt' table. To see supported formats use visOmopResults::tableType().}

\item{header}{A vector specifying the elements to include in the header. The
order of elements matters, with the first being the topmost header.
The header vector can contain one of the following variables: "cdm_name",
"codelist_name", "domain_id", "standard_concept_name", "standard_concept_id",
"estimate_name", "standard_concept", "vocabulary_id".
Alternatively, it can include other names to use as overall header labels.}

\item{groupColumn}{Variables to use as group labels. Allowed columns are:
"cdm_name", "codelist_name", "domain_id", "standard_concept_name",
"standard_concept_id", "estimate_name", "standard_concept", "vocabulary_id".
These cannot be used in header.}

\item{hide}{Table columns to exclude, options are:  "cdm_name",
"codelist_name", "domain_id", "standard_concept_name", "standard_concept_id",
"estimate_name", "standard_concept", "vocabulary_id". These cannot be used in
header or groupColumn.}

\item{style}{A character string or custom R code to define the visual formatting
of the table. This argument can be provided in two ways: (1) Pre-defined Styles
(Character String): Use a name for a built-in style (e.g., "darwin"). See
visOmopResults::tableStyle() for available options. (2) Custom Code (Advanced):
Supply a block of custom R code. This code must be specific to the table type.
See visOmopResults::tableStyleCode() for structural examples.}

\item{.options}{Named list with additional formatting options.
visOmopResults::tableOptions() shows allowed arguments and
their default values.}
}
\value{
A table with a formatted version of the summariseOrphanCodes
result.
}
\description{
Format the result of summariseOrphanCodes into a table
}
\examples{
\donttest{
library(CodelistGenerator)
library(omopgenerics)
cdm <- mockVocabRef("database")
codes <- getCandidateCodes(cdm = cdm,
keywords = "Musculoskeletal disorder",
domains = "Condition",
includeDescendants = FALSE)

orphan_codes <- summariseOrphanCodes(x = newCodelist(list("msk" = codes$concept_id)),
cdm = cdm)

tableOrphanCodes(orphan_codes)

CDMConnector::cdmDisconnect(cdm)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{routeCategoryDoc}
\alias{routeCategoryDoc}
\title{Helper for consistent documentation of \code{routeCategory}.}
\arguments{
\item{routeCategory}{Only codes with the specified route will be
returned. If NULL, descendant codes will be returned regardless of route
category. Use 'availableRouteCategories()' to find the available route categories in
the database, and 'associatedRouteCategories()' to get drug routs associated with
a codelist.}
}
\description{
Helper for consistent documentation of \code{routeCategory}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/codesFromConceptSet.R
\name{codesFromConceptSet}
\alias{codesFromConceptSet}
\title{Get concept ids from JSON files containing concept sets
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}}
\usage{
codesFromConceptSet(path, cdm, type = c("codelist"))
}
\arguments{
\item{path}{Path to a file or folder containing JSONs of concept sets.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{type}{Can be "codelist", "codelist_with_details" or
"concept_set_expression".}
}
\value{
Named list with concept_ids for each concept set.
}
\description{
Get concept ids from JSON files containing concept sets
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create a CDM object
cdm <- mockCdmReference()

# Load JSON files
x <- codesFromConceptSet(cdm = cdm,
                         path =  system.file(package = "CodelistGenerator",
                                             "concepts_for_mock"))
x

# Load JSON files as codelist_with_details
x <- codesFromConceptSet(cdm = cdm,
                         path =  system.file(package = "CodelistGenerator",
                                             "concepts_for_mock"),
                         type = "codelist_with_details")
x

# Load JSON files as concept_set_expression
x <- codesFromConceptSet(cdm = cdm,
                         path =  system.file(package = "CodelistGenerator",
                                             "concepts_for_mock"),
                         type = "concept_set_expression")
x

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reexports.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{cohortCodelist}
\alias{importCodelist}
\alias{importConceptSetExpression}
\alias{exportCodelist}
\alias{exportConceptSetExpression}
\alias{newCodelist}
\alias{newCodelistWithDetails}
\alias{newConceptSetExpression}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{omopgenerics}{\code{\link[omopgenerics]{cohortCodelist}}, \code{\link[omopgenerics]{exportCodelist}}, \code{\link[omopgenerics]{exportConceptSetExpression}}, \code{\link[omopgenerics]{importCodelist}}, \code{\link[omopgenerics]{importConceptSetExpression}}, \code{\link[omopgenerics]{newCodelist}}, \code{\link[omopgenerics]{newCodelistWithDetails}}, \code{\link[omopgenerics]{newConceptSetExpression}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableRouteCategories.R
\name{availableRouteCategories}
\alias{availableRouteCategories}
\title{Get available drug routes}
\usage{
availableRouteCategories(cdm)
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}
}
\value{
A character vector with all available routes.
}
\description{
Get the dose form categories available in the database (see
https://doi.org/10.1002/pds.5809) for more details on how routes
were classified).
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all domains available in the CDM
availableRouteCategories(cdm = cdm)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableATC.R
\name{availableATC}
\alias{availableATC}
\title{Get the names of all available Anatomical Therapeutic Chemical (ATC) classification codes}
\usage{
availableATC(cdm, level = c("ATC 1st"))
}
\arguments{
\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{level}{ATC level. Can be one or more of "ATC 1st", "ATC 2nd",
"ATC 3rd", "ATC 4th", and "ATC 5th".}
}
\value{
A vector containing the names of ATC codes for the chosen level(s)
found in the concept table of cdm.
}
\description{
Get the names of all available Anatomical Therapeutic Chemical (ATC) classification codes
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get ATC 1st level classification codes
availableATC(cdm, level = "ATC 1st")

# Get all ATC classification codes
availableATC(cdm, level = c("ATC 1st", "ATC 2nd", "ATC 3rd", "ATC 4th", "ATC 5th"))
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/availableDomains.R
\name{associatedDomains}
\alias{associatedDomains}
\title{Get the domains associated with a codelist}
\usage{
associatedDomains(x, cdm, standardConcept = "Standard")
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{cdm}{A cdm reference to an OMOP CDM dataset. If data is held within a
database, the vocabulary tables should be in the same schema as the clinical
tables (person, observation period, and so on).}

\item{standardConcept}{Character vector with one or more of "Standard",
"Classification", and "Non-standard". These correspond to the flags used
for the standard_concept field in the concept table of the cdm.}
}
\value{
A vector with the domains of the cdm.
}
\description{
Get the domains associated with a codelist
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create CDM object
cdm <- mockCdmReference()

# Get all domains available in a codelist
codelist <- newCodelist(list("codes1" = c(194152L, 1830279L, 40558872L),
                             "codes2" = c(44022939L)))
associatedDomains(x = codelist, cdm = cdm,
                 standardConcept = c("Non-standard", "Standard"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/documentation_helper.R
\name{nameStyleDoc}
\alias{nameStyleDoc}
\title{Helper for consistent documentation of \code{nameStyle}.}
\arguments{
\item{nameStyle}{Name style to apply to returned list. Can be one of
\code{"{concept_code}"},\code{"{concept_id}"}, \code{"{concept_name}"}, or a combination (i.e.,
\code{"{concept_code}_{concept_name}"}).}
}
\description{
Helper for consistent documentation of \code{nameStyle}.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intersectCodelists.R
\name{intersectCodelists}
\alias{intersectCodelists}
\title{Generate a codelist from the intersection of different codelists. The generated
codelist will come out in alphabetical order.}
\usage{
intersectCodelists(x, keepOriginal = FALSE)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{keepOriginal}{Whether to keep the original codelist (TRUE) or just return
the stratified ones (FALSE).}
}
\value{
A codelist
}
\description{
Generate a codelist from the intersection of different codelists. The generated
codelist will come out in alphabetical order.
}
\examples{
\donttest{
library(CodelistGenerator)
library(omock)

# Create a CDM object
cdm <- mockCdmReference()

# Intersect two codelists
codelist <- newCodelist(list("mood" = c(37110496L, 4226696L, 4304866L),
                             "manic" = c(37110496L, 4226696L)))

intersectCodelists(codelist, keepOriginal = TRUE)


# Intersect two codelists_with_details
codelist <- asCodelistWithDetails(codelist, cdm)

intersectCodelists(codelist, keepOriginal = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/classCodeSearch.R
\name{newCodeSearch}
\alias{newCodeSearch}
\title{Create a new \code{code_search} object}
\usage{
newCodeSearch(codes, searchStrategy)
}
\arguments{
\item{codes}{A tibble with concept ids. It must contain "search_id",
"concept_id", "found_from", "vocabulary_name", "concept_name", "domain_id",
"vocabulary_id", "concept_class_id", "standard_concept", "concept_code",
"valid_start_date", "valid_end_date" and "invalid_reason" as columns.}

\item{searchStrategy}{A tibble with the search strategy used to derive the
codes. It must contain "search_id", "strategy_name" and "strategy_value" as
columns.}
}
\value{
A \code{code_search} object.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#experimental}{\figure{lifecycle-experimental.svg}{options: alt='[Experimental]'}}}{\strong{[Experimental]}}
This function will be moved to omopgenerics (and re-exported by
CodelistGenerator) once stable.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/unionCodelists.R
\name{unionCodelists}
\alias{unionCodelists}
\title{Generate a codelist from the union of different codelists. The generated codelist
will come out in alphabetical order.}
\usage{
unionCodelists(
  x,
  newCodelistName = NULL,
  keepOriginal = FALSE,
  codelistsToJoin = names(x)
)
}
\arguments{
\item{x}{A codelist, codelist_with_details, or a concept_set. See \code{newCodelist()}, \code{newCodelistWithDetails()}, \code{newConceptSetExpression()} functions for more details.}

\item{newCodelistName}{Character vector with the name of the new codelist. If
NULL all codelists names will be combined.}

\item{keepOriginal}{Whether to keep the original codelist (TRUE) or just return
the stratified ones (FALSE).}

\item{codelistsToJoin}{Character vector with the names of the codelists to be
unioned.}
}
\value{
A codelist
}
\description{
Generate a codelist from the union of different codelists. The generated codelist
will come out in alphabetical order.
}
\examples{
\donttest{
library(CodelistGenerator)
cdm <- mockVocabRef()
getDrugIngredientCodes(cdm,
                        nameStyle = "{concept_name}") |>
unionCodelists()
}
}
